g_dinamic_cssjs.push("jusaas/js/UniTable.js");

/*
 修改日期 09-22 18:39
 --------------------------------------------------------------------------------------------------------------------------------------------------------------
 MCSSTable类概述：
 1、用于快速根据MCSS数据对象配置生成HTML表格
 2、需要显示表格的网页，只要简单几行代码即可ajax方式生成table。
 3、只需通过适当的配置，生成的table就具有浏览、翻页、删除、编辑、搜索等功能。
 4、通过这个类的接口，还可以给在第一个栏和最后一栏添加额外的代码，例如在每一行的最后显示某个按钮。
 --------------------------------------------------------------------------------------------------------------------------------------------------------------
 使用范例:
 此范例会生成一个带有增删改简单搜索翻页的图标式按钮表格
 var field_dealing=[{'loginuser':myshow},{'tasktype':myshow2}];
 var mcssTable=new MCSSTable({tableid:"mcss_table",modelid:modelid,showfirst:true,last_td_name:'操作',toolbar:"add,edit,del,search",
 special_field_show:field_dealing,last_td_func:last_td_func,sql:"select * from table1s",showRecordCount:true,showRecordActionAtLast:true,actionStyle:"ICON",
 pageposition:"rightdown"});
 mcssTable.run();
 --------------------------------------------------------------------------------------------------------------------------------------------------------------
 Params参数说明：
 tableid:HTML的Table的id
 modelid:MCSS的模型ID
 homeUrl:方法路径前缀，在TP中一般写成"__APP__"即可
 selectType:radio:单选按钮,checkbox：复选框
 toolbar:显示在操作按钮的按钮。如果无此参数，则自动取模型的actions属性
 showtitle:显示标题
 showfirst:数据在表格第一列前增加新的列，一般用于储存“打开、编辑、选择”等按钮。如果选择此参数，则first_td_func也是必须的，否则列头与下面的内容对不齐
 showRowActionBtn:是否隐藏行按钮，默认是true
 sql:如果传递了sql参数，获取数据时将以该sql语句代替模型的sql属性
 filter:执行sql的条件
 special_field_show:自定义表格上字段内容展现形式
 showRecordActionAtLast:true表示把记录操作按钮显示在每行最后，否则显示在第一栏
 showRecordCount:true表示直接把记录总数显示出来
 pageposition:翻页按钮位置，如果为'rightdown'，则显示在右下角，否则显示在工具栏
 first_td_name:额外在第一列添加的操作列的名称
 first_td_func：额外增加的第一列的内容的产生方法，是一个function名
 last_td_name:额外添加在最后一列的名称
 last_td_func：额外增加的最后一列的内容的产生方法，是一个function名
 beforeAddRow:添加每行记录前的方法，可以用来添加额外的td或tr，如分组显示
 afterAddRow(mctable,tr_html,rowData):添加每行记录后的方法，可以用来添加额外的td或tr，如分组显示(此方法暂无)
 addGroupInfo(mctable,rowData):添加每行记录后的方法，可以用来添加额外的td或tr，如分组显示(此方法暂无)
 style:simple/normal  风格:简约/标准 默认为标准风格
 exporturl：执行特殊导出时可以添加此参数跳入自定义的Action中
 afterLoadRows：数据加载完后执行的方法
 actionStyle:按钮的展现形式 TEXT为文字按钮 ICON为图片按钮
 viewtype:默认的显示模式。tableedit:表格编辑，calendar：日历，table：列表。空值表示列表
 hideToolbar:true表示不创建工具栏
 lang:传入的语言类型,cn/en
 defaultvalue：设置打开表单的默认值,格式: pid:'123',name:'jack'
 currentpage:当前页
 searchword:搜索关键词
 width:列表宽度
 pagerows:列表默认显示一页的条数
 hideLastTd:隐藏最后一列
 hideLastTr:隐藏最后一行
 deleteRow:删除一行,参数为删除的rowids和mcsstable
 confirmword:自定义的确定删除语言,如”您真的要删除吗?”
 popupFormID:弹出列表的模型id
 popupFieldID: 弹出列表的所选字段id
 showConfirmButton:弹出列表显示确定按钮
 impurl:
 refObject:
 params.afterDeleteRow:自定义删除后方法
 params.diffLineCss==false:鼠标移动到行上必不有样式效果
 params.userAction:用户自定义加入记录操作栏的方法
 params.afterGetModel:获得模型后的事件，可用于动态修改模型的属性
 params.view:表格表现形式。'table'或空表示常规的列表；‘tableedit’表示全表编辑，‘celledit’表示单元格编辑
 params.view_id:视图的id
 params.beforeAddNew
 params.exportMenus：额外的导出菜单项目
 params.afterRunAutoform:打开新增窗口创建了autoform后事件
 params.afterUpdateDataToRow:把data数据更新到对应的行（tr）上后的事件
 params.afterAppendRow:在view或tableedit模式下，新增行后的事件
 params.notcelledit:在params.view=='celledit' 时，告诉mcsstable不要使用celledit的的字段列表，如‘name,status’
 params.getAutoId:true表示新增记录是赋予自动id
 params.SpecialFieldActionAfterChange(mcsstable,dom,recordid,fieldid):在表格中修改字段值后触发的事件。在用TableEditor中使用
 params.rowhover:false表示不需要“鼠标移动到行上显示特殊样式”
 params.noClickTD:true表示不需要单击td的事件。单击时会修改当前行的背景色
 params.canAppendRow:在编辑状态下，是否允许新增记录。
 params.createSpecialFieldInputer:自定义创建表格的单元格编辑框的方法。范例给data字段指定了自定义方法：var field_createSpecialFieldInputer={'data':datafield_createSpecialFieldInputer};
 params.showShortcutKey:是否启用操作快捷键
 params.defaultOrderby:排序
 params.pageoptions:每页记录数选项，如"10,20,50"
 params.showAlpha：是否显示标题行顶部的字母
 params.showLineNum：是否显示行号列
 params.showRowActionMenu：是否显示行操作菜单列
 params.style_hideLeftRightBorder：是否隐藏单元格的左右边框
 params.tableLayout:默认为“fixed”
 params.beforeSaveForm:打开表单，单击表单的保存按钮时执行的方法。如果
 this.getRow(recordid):根据记录id获得行（tr）
 var MCSSTables=new Array;//多个MCSSTable对象管理
 MCSSTables定义在common.js中
 */
function mcsstable_updateMCSSTables(id,data)
{
    //如果已经存在则更新
    for(var i=0;i<MCSSTables.length;i++){
        if (MCSSTables[i].id==id)
        {
            MCSSTables[i].data=data;
            return;
        }
    }
    //如果不存在则添加
    MCSSTables.push({"id":id,data:data});
}
function mcsstable_getMCSSTable(id){
    if (typeof(id)=='object')
        return id;
    var r = "";
    for(var i=0;i<MCSSTables.length;i++){
        if (MCSSTables[i].id==id || MCSSTables[i].id==id+"_table" || MCSSTables[i].id+"_data"==id || MCSSTables[i].id+"_fixRow"==id || MCSSTables[i].id+"_fixCol"==id || MCSSTables[i].id+"_fixRowCol"==id)
        {
            r = MCSSTables[i].data;
            break;
        }
    }
    return r;
}

//创建MCSSTable对象的更简单的方式,这个方法逐渐淘汰
function newMCSSTable(id,modelid,params)
{
    if (!params)
        params={};
    return new MCSSTable(id, modelid,params);
}

function MCSSTable(id,modelid,params) {
    this.typename='MCSSTable';
    //alert("看到这行表明对象被创建了。");
    if (!document.getElementById(id))
    {
        alert("创建表格失败。因为ID为"+id+"的对象不存！");
        return;
    }

    this.id=id;
    this.modelid=modelid;
    if (!params)
        params={};
    if (params.showfirst==undefined)
        params.showfirst=true;
    this.params=params;

    //常用的固定的对象id写在这里
    this.tableid=id+"_table";//即将创建的table的id
    this.pagebarid=this.tableid+"_action_pagebar";//翻页条的div的id
    this.rowaction_col_id=this.tableid+"_rowaction_col";
    this.table_toolbar_id=this.tableid+"_tabletoolbar";
    if (this.params.table_toolbar_id)
        this.table_toolbar_id=this.params.table_toolbar_id;
    this.tablecontainer_id=this.tableid+"_tablecontainer";
    this.tableheader_id=this.tableid+"_tableheader";
    this.tabledata_id=this.tableid+"_data";
    this.searchinput_Id=this.tableid+"_searchinput";
    this.datatableGrandFather_id=this.tableid+"_datatableGrandFather";
    this.selected_border_css="solid 2px #a4c0f8";


    //赋值给MAIN全局，以备main调用修改样式。可能还有其他好办法，暂时先这样
    uni_global.main_tablecontainer_id=this.tablecontainer_id;//赋值给MAIN全局，以备main调用修改样式。
    mian_table_toolbar_id=this.table_toolbar_id;

    this.params.tableid=this.tableid;//即将创建的table的id。为了兼容历史
    this.params.modelid=modelid;
    this.params.canAppendRow=true;
    if (!this.params.autoAddEmptyRow) {this.params.autoAddEmptyRow=false ;}
    if (!this.params.tableLayout) {this.params.tableLayout='fixed' ;}

    if (!this.params.showColHeaderOnScroll) {this.params.showColHeaderOnScroll=false ;}

    mcsstable_updateMCSSTables(this.tableid,this);

    this.fields = new Array();
    this.first_td_func=params.first_td_func;
    this.last_td_func=params.last_td_func;

    this.currentpage=1;
    if (params.currentpage)
        this.currentpage=params.currentpage;

    this.totalpages=-1;//总页数
    this.pagerows=10;//每页记录数
    this.searchword='';
    if (params.searchword)
        this.searchword=params.searchword;
    this.filter=params.filter;//数据过滤器，允许程序动态修改
    this.view="table";
    if (params.view)
        this.view=params.view;//编辑模式

    this.lang='cn';//语言，cn或en
    this.nullgroupname="--";//空组的替代字符
    if (params.lang)
        this.lang=params.lang;
    if (!params.homeUrl)
        params.homeUrl=g_homeurl;
    if (params.showRecordCount==undefined)
        params.showRecordCount=true;

    //解析url中的过滤条件。“查看相关记录”列类型需要这个
    var filterFromUrl=common_getParam('filter');
    if (filterFromUrl){
        if (params.filter)
            params.filter="("+params.filter+") and ("+filterFromUrl+")";
        else
            params.filter="("+filterFromUrl+")";
    }

    //如果没有传入下列url，则采用默认的url
    this.getDataListUrl=params.homeUrl+g_systempath+"/getData";
    this.getModelUrl=params.homeUrl+g_systempath+"/getModelData";
    if (params.getDataListUrl)
        this.getDataListUrl=params.getDataListUrl;
    if (params.getModelUrl)
        this.getModelUrl=params.getModelUrl;

    if (params.homeUrl.substring(params.homeUrl.length)!="/")
    {
        params.homeUrl+="/";
    }
    if(!params.showSaveAndAddButton)
        this.showSaveAndAddButton=params.showSaveAndAddButton;

    var divParent=$("#"+this.id).get(0);
    if (divParent){
        if (divParent.style.height){
            if (divParent.style.height!="auto"){
                this.params.height=parseInt( divParent.style.height);
            }
        }
    }

    this.modeldata;
    this.keyfield='id';
    this.actions = "";
    this.rooturl=g_rooturl;
    this.view_id=this.params.view_id;//视图id
    this.mcss_theme=getCookie("mcss_theme");
    this.fieldData;//不知道哪里用到
    this.selectedRowsID='';//已选记录的id列表，用逗号分隔
    this.last_groupfield_value="[-dfsafdsaf324342314fdsafdsfa-]";

    //开始创建表格
    var _this=this;

    //run方式用于初始化。第一次创建时会执行run，创建后也可以用过run重新创建表格。既然是重新创建，因此一些默认参数应该在这里初始化
    this.run = function(func) {

        this.tableid=this.id+"_table";
        this.params.tableid=this.tableid;
        var h="";
        if (this.params.table_toolbar_id)
        {
        }
        else
            h+="<div id='"+this.table_toolbar_id+"' modelid='"+this.modelid+"' onclick='mcsstable_clearFloatToolBar()' style='height:2.0em' class='mcsstable_toolbar'></div>"; //工具栏

        h+="<div id='"+this.tablecontainer_id+"' class='table-container pull-left' >"
            +'<table id="'+this.tableid+'" style="overflow-x: hidden; overflow-y: hidden; table-layout: fixed; height: 320px; " class="mcsstable"></table>';
            +"</div><div id='sidebar-right'></div>";
        $("#"+this.id).html(h).addClass("mcsstable-container");


        this.body=document.getElementById(this.id);
        this.tablecontainer=document.getElementById(this.tablecontainer_id);


        this.table=$("#"+this.tableid).get(0);
        //$(this.table).empty();
        $("#"+this.pagebarid).remove();

        //add by sunliangchao 设置表格的位置为相对定位
        this.tablecontainer.style.overflow="hidden";
        this.table.style.overflow="hidden";
        this.tablecontainer.style.height="100%";
        this.tablecontainer.style.width="100%";
        //document.body.style.overflow="hidden";
        //added by sunliangchao

        this.table.parentNode.table=this.table;
        this.table.parentNode.parentNode.table=this.table;

        var divSelBack = document.createElement("div");
        divSelBack.id=this.tableid+"_selback";
        divSelBack.style.position="absolute";
        divSelBack.style.border="none";
        divSelBack.style.pointerEvents="none";

        var divSelParent = document.createElement("div");
        divSelParent.id=this.tableid+"_selbackparent";
        divSelParent.style.position="absolute";
        divSelParent.style.border="none";
        divSelParent.style.pointerEvents="none";
        this.table.parentNode.parentNode.appendChild(divSelBack);
        divSelBack.appendChild(divSelParent);

        this.divSelBack=divSelBack;
        this.divSelParent=divSelParent;

        this.loadTableJS();
        if (!this.runWhenFinished)
            this.runWhenFinished=func; //第一次创建了表格并加载了数据后执行的方法
        this.mcssData=null;
        this.cellEditor=null;
        this.setGroupField('');
        this.orderFieldid='';
        this.advancedFilter ='';
        this.autoFilter ='';
        this.searchword ='';
        this.order = "";//表格最终的排序依据
        this.recordCount=0;//当前这些取到的记录数量，用于判断翻页
        this.uniTableStyle = new UniTableStyle(this);//生成McssTable的样式类
        $(this.getAction('groupby')).val('');


        //alert("控制器方法路径是："+this.getModelUrl);
        if (this.params.model)
        {
            this.init(this,this.params.model);
        }
        else
        {
            var _this=this;
            var params={modelid:this.params.modelid,usermodel_recordid:this.params.usermodel_recordid,
                model_ownerid:this.params.model_ownerid,view_id:this.params.view_id,version_id:this.params.version_id};
            this.getModelDataAndDo(params,this.init);
        }
    }
    //刷新模型数据
    this.refreshModelData=function(){
        var params={modelid:this.params.modelid,usermodel_recordid:this.params.usermodel_recordid,
            model_ownerid:this.params.model_ownerid,view_id:this.params.view_id,version_id:this.params.version_id};
        this.getModelDataAndDo(params,this.setModelData);
    }
    //给mcsstable的modeldata赋值
    this.setModelData=function(mcsstable,model){
        mcsstable.modeldata = model;

        model.fixFields=[];
        model.dataFields=[];
        var sumFields="";
        mcsstable.fields=new Array();

        for(var i=0;i<model.fields.length;i++){
            var fd=model.fields[i];
            if(fd.show_sum) {
                mcsstable.hasSummaryField=true;
                if (sumFields=="")
                    sumFields+=(fd.id);
                else
                    sumFields+=("," + fd.id);
            }
            if (fd.fixCol && fd.fixCol!="false"){
                if (fd.groupby && fd.groupby=="true" && !mcsstable.groupFieldid){
                    mcsstable.setGroupField(fd.id);
                }
                if (fd.isvisible+"" != "false"){
                    model.fixFields.push(fd);
                    mcsstable.fields.push(fd.id);
                }
            }else{
                if (fd.isvisible+"" != "false") {
                    model.dataFields.push(fd);
                    mcsstable.fields.push(fd.id);
                }
            }
        }
        mcsstable.sumFields=sumFields;

    }
    //得到模型结构数据执行的第一个方法、创建表格
    this.init=function(mcsstable,model)
    {
        //sunliangchao 临时添加模拟固定列
        //model.fields[0].fixCol=true;
        //model.fields[1].fixCol=true;
        //model.fields[3].show_sum=true;
        //上两句临时添加，下面的必须保留；
        var _this=mcsstable;
        //_this.modeldata=model;
        _this.modelDealer=new MCSSModel(model,{mcsstable:_this});//模型操作类
        //_this.modelDealer.loadJS();
        _this.modelDealer.parseModelDefaultValue(_this.params.defaultValue);
        _this.setModelData(_this,model);

        //var model = this.modeldata;

        var data=model;
        if (!data)
        {
            $("#"+_this.id).attr('modelid',_this.modelid);
            $("#"+_this.id).html("<span style='color:red'>ERROR:模型结构错误("+_this.modelid+")</span>");
            return;
        }
        var err=common_getAccessError(data);
        if (err)
        {
            $("#"+_this.id).html(err);
            return;
        }

        if (_this.params.beforeCreateTable)
        {
            _this.params.beforeCreateTable(_this);
        }

        if (_this.params.afterGetModel)
        {
            _this.params.afterGetModel(_this.modeldata,_this);
        }
        //判断是否有视图
        var view_id=mcsstable.view_id;
        if (!view_id)
            view_id=mcsstable_getFirstShareView(mcsstable);
        if (!view_id)
            view_id=mcsstable.modeldata.defaultView;

        if (view_id && view_id!='DEFAULT')
        {
            mcsstable.openView(view_id);
        }
        else
        {
            mcsstable_createTable(data,params.tableid);
        }
    }

    //根据模型结构初始化表格结构，不包括记录数据 ffff
    this.createTableStruture=function(mcsstable,model)
    {
        var _this=mcsstable;
        var data=model;
        if (!data)
        {
            $("#"+_this.id).attr('modelid',_this.modelid);
            $("#"+_this.id).html("<span style='color:red'>ERROR:模型结构错误("+_this.modelid+")</span>");
            return;
        }
        var err=common_getAccessError(data);
        if (err)
        {
            $("#"+_this.id).html(err);
            return;
        }
        _this.modelDealer.parseModelDefaultValue(_this.params.defaultValue);

        if (_this.params.afterGetModel)
        {
            _this.params.afterGetModel(_this.modeldata,_this);
        }
        if (_this.params.beforeCreateTable)
        {
            _this.params.beforeCreateTable(_this);
        }

    }
    this.editCell=function(td){
        if  (!(this.params.project_fulldata && this.params.project_fulldata.shareoption && this.params.project_fulldata.shareoption.indexOf("[notaddrecord]")>-1))
        {
            this.cellEditor.mcssCreateEditor(td);
            var input = $(td).find('input,textarea[fieldid],select[fieldid]');
            input.focus();
        }
    }
    this.undoList=[];
    this.addUndoList=function (undoObject,undoFun,undoParams,redoObject,redoFun,redoParams,tableState)
    {
        var cmd={};
        if (tableState)
        {
            cmd.tableState=tableState;
        }
        else if (tableState!=false) {
            cmd.tableState={scrollLeft:this.table.parentNode.parentNode.scrollLeft,scrollTop:this.table.parentNode.parentNode.scrollTop};
            if (this.table.currentRow) cmd.tableState.rowIndex = this.table.currentRow.rowIndex;
            if (this.divSel && this.divSel.td ) {
                cmd.tableState.rowIndex=this.divSel.td.parentNode.rowIndex;
                cmd.tableState.cellIndex=this.getCellIndex(this.divSel.td);//.cellIndex;
            }
        }
        cmd.undoFun = undoFun;
        cmd.undoParams = undoParams;
        cmd.redoFun = redoFun;
        cmd.redoParams = redoParams;
        cmd.undoObject=undoObject;
        cmd.redoObject=redoObject;
        this.undoList.push(cmd);
        this.redoList=[];

    }
    this.undo = function(){

        if (this.divSel){
            if (this.divSel.td){
                var input=$(this.divSel.td).find('input,div[fieldid],select[fieldid]').get(0);
                if (input && input.oldvalue != input.value){
                    input.newvalue=input.value;
                    if (input.tagName=="DIV") input.newvalue=input.innerText;
                    input.value=input.oldvalue;
                    return;
                }
            }
        }

        if (this.undoList.length<1) return;
        var undocmd=this.undoList[this.undoList.length-1];
        if (undocmd.undoFun)
        {
            var p0=null;
            var p1=null;
            var p2=null;
            var p3=null;
            var p4=null;
            var p5=null;
            var p6=null;
            var p7=null;

            if (undocmd.undoParams.length>0) p0=undocmd.undoParams[0];
            if (undocmd.undoParams.length>1) p1=undocmd.undoParams[1];
            if (undocmd.undoParams.length>2) p2=undocmd.undoParams[2];
            if (undocmd.undoParams.length>3) p3=undocmd.undoParams[3];
            if (undocmd.undoParams.length>4) p4=undocmd.undoParams[4];
            if (undocmd.undoParams.length>5) p5=undocmd.undoParams[5];
            if (undocmd.undoParams.length>6) p6=undocmd.undoParams[6];
            if (undocmd.undoParams.length>7) p7=undocmd.undoParams[7];
            if (undocmd.undoObject){
                undocmd.undoObject[undocmd.undoFun](p0,p1,p2,p3,p4,p5,p6,p7);
            }else{
                undocmd.undoFun(p0,p1,p2,p3,p4,p5,p6,p7);
            }

        }
        this.undoList.pop();

        this.redoList.push(undocmd);
        if (undocmd.tableState){
            if (undocmd.tableState.scrollTop) this.tableData.parentNode.parentNode.scrollTop = undocmd.tableState.scrollTop;
            if (undocmd.tableState.scrollLeft) this.tableData.parentNode.parentNode.scrollLeft = undocmd.tableState.scrollLeft;

            if (this.currentRow && this.currentRow.rowIndex != undocmd.tableState.rowIndex)
                this.setCurrentRow(this.tableFixCol.rows[undocmd.tableState.rowIndex]);
            if (undocmd.tableState.rowIndex && undocmd.tableState.cellIndex){
                var td=this.getRow(undocmd.tableState.rowIndex).cells[undocmd.tableState.cellIndex];
                if (td && td.parentNode) setFocusToTd(td);
            }
        }
    }
    this.redoList=[];
    this.redo=function(){

        if (this.divSel){
            if (this.divSel.td){
                var input=$(this.divSel.td).find('input,div[fieldid],select[fieldid]').get(0);
                if (input && input.newvalue != undefined && input.newvalue != input.value){
                    input.oldvalue=input.value;
                    if (input.tagName=="DIV") input.oldvalue=input.innerText;
                    input.value=input.newvalue;
                    return;
                }
            }
        }

        if (this.redoList.length<1) return;
        var redocmd=this.redoList[this.redoList.length-1];

        if (redocmd.redoFun)
        {
            var p0=null;
            var p1=null;
            var p2=null;
            var p3=null;
            var p4=null;
            var p5=null;
            var p6=null;
            var p7=null;

            if (redocmd.redoParams.length>0) p0=redocmd.redoParams[0];
            if (redocmd.redoParams.length>1) p1=redocmd.redoParams[1];
            if (redocmd.redoParams.length>2) p2=redocmd.redoParams[2];
            if (redocmd.redoParams.length>3) p3=redocmd.redoParams[3];
            if (redocmd.redoParams.length>4) p4=redocmd.redoParams[4];
            if (redocmd.redoParams.length>5) p5=redocmd.redoParams[5];
            if (redocmd.redoParams.length>6) p6=redocmd.redoParams[6];
            if (redocmd.redoParams.length>7) p7=redocmd.redoParams[7];

            if (redocmd.redoObject){
                redocmd.redoObject[redocmd.redoFun](p0,p1,p2,p3,p4,p5,p6,p7);
            }else{
                redocmd.redoFun(p0,p1,p2,p3,p4,p5,p6,p7);
            }

        }
        this.redoList.pop();
        this.undoList.push(redocmd);

        if (redocmd.tableState){
            if (redocmd.tableState.scrollTop) this.tableData.parentNode.parentNode.scrollTop= redocmd.tableState.scrollTop;
            if (redocmd.tableState.scrollLeft) this.tableData.parentNode.parentNode.scrollLeft= redocmd.tableState.scrollLeft;
            if (this.currentRow && this.currentRow.rowIndex != redocmd.tableState.rowIndex)
                this.setCurrentRow(this.tableFixCol.rows[redocmd.tableState.rowIndex]);
            if (redocmd.tableState.rowIndex && redocmd.tableState.cellIndex)
                setFocusToTd(this.getRow(redocmd.tableState.rowIndex).cells[redocmd.tableState.cellIndex]);
        }
    }
    //设置单元格
    this.setCellData=function(recordid,fieldid,value){

        var recordtype="UPDATE";
        var newData={mcss_recordtype:recordtype,recordid:recordid,fieldid:fieldid,value:value};
        this.mcssData.editNewData(newData);
        this.mcssData.save(null,{onlyonefield:fieldid,refObject:this,alert:false});

        var cell=this.getCellById(recordid,fieldid);
        if (cell==null) return;

        var record = this.mcssData.getRow(recordid);
        var field = this.modelDealer.getField(fieldid);

        if (this.view=="celledit"){
            var str = mcsstable_getFieldDisplay(field,value,false,record,this);
            cell.innerHTML=str;
            cell.inEdit=false;
        }
    }
    //根据recordid,fieldid得到单元格
    this.getCellById=function (recordid,fieldid){
        for(var r=0;r<this.tableData.rows.length;r++)
        {
            if (this.tableFixCol.rows[r].getAttribute("recordid")==recordid)
            {
                row=this.getRow(r);
                for(var c=1;c<row.cells.length;c++)
                {
                    if (row.cells[c].id==fieldid)
                    {
                        cell = row.cells[c];
                        return cell;
                    }
                }
            }
        }
        return null;
    }
    this.getRowById=function (recordid){
        for(var r=0;r<this.tableData.rows.length;r++)
        {
            if (this.tableFixCol.rows[r].getAttribute("recordid")==recordid)
            {
                row=this.getRow(r);
                return row;
            }
        }
        return null;
    }
    //从表格中，缓存中和数据库中立刻删除行
    this.deleteRow=function(recordid){

        var urlpath = g_homeurl+"/System/Model/deleterows";
        if (this.modeldata.delurl)
            urlpath = g_homeurl+"/"+this.modeldata.delurl;

        var row=null;
        var rowd=null;
        for(var r=0;r<this.tableFixCol.rows.length;r++)
        {
            if (this.tableFixCol.rows[r].getAttribute("recordid")==recordid)
            {
                row=this.tableFixCol.rows[r];
                rowd=this.tableData.rows[r];
                break;
            }
        }

        if (this.view=='tableedit')
        {
            var record={mcss_recordtype:"DELETE",recordid:recordid};
            this.mcssData.deleteRow(record);
            this.mcssData.save();
        }
        else if(this.view=='celledit')
        {
            var pid="";
            if (this.params.project_fulldata)
                pid=this.params.project_fulldata.project_record.id;
            $.post(urlpath,	{modelid:this.modelid,rowIds:recordid,projectid:pid},function(data){});
            this.mcssData.unloadRow(recordid);
        }
        if(row){
            row.remove();
            rowd.remove();
            this.resortLineIndex();
        }

    }
    //在表格中缓存中和数据库中立即追加或插入行
    this.addRow=function(rowData,dataIndex){

        var rowIndex=1;
        for(var r=1;r<this.tableFixCol.rows.length;r++)
        {
            if (this.tableFixCol.rows[r].dataIndex==dataIndex-1)
            {
                rowIndex=this.tableFixCol.rows[r].rowIndex+1;
                break;
            }
        }
        mcsstable_insertEmptyrow(this.table,rowIndex,rowIndex+1,rowIndex);

        var row=this.tableFixCol.rows[rowIndex];

        var mcssData=this.mcssData;
        var fields=mcssData.modeldata.fields;
        var recordid=rowData.id;
        var newData={mcss_recordtype:"NEW",recordid:recordid,fieldid:this.keyfield,value:recordid};
        mcssData.editNewData(newData);
        for(var i=0;i<fields.length;i++)
        {
            if (fields[i].id!=this.keyfield && fields[i].id!='SYS_ADDIP' )//处理字段默认值，都要生成一条记录更新到数据组中
            {
                newData={mcss_recordtype:"NEW",recordid:recordid};
                newData.fieldid=fields[i].id;
                newData.value=rowData[newData.fieldid];
                mcssData.editNewData(newData);
            }
        }

        this.mcssData.save(null, { refObject: this,alert:false });

        mcsstable_onNewRow(row,rowData,rowData.id);
        this.resortLineIndex();

    }
    //更新一行数据
    this.updaterowData=function(rowData){

        var recordid=rowData.id;

        this.mcssData.updateRow(recordid,rowData);
        this.updateRowByData(recordid,rowData);

        var mcssData=this.mcssData;
        var fields=mcssData.modeldata.fields;
        var recordid=rowData.id;
        var newData={mcss_recordtype:"UPDATE",recordid:recordid,fieldid:this.keyfield,value:recordid};
        mcssData.editNewData(newData);
        for(var i=0;i<fields.length;i++)
        {
            if (fields[i].id!=this.keyfield && fields[i].id!='SYS_ADDIP' )//处理字段默认值，都要生成一条记录更新到数据组中
            {
                newData={mcss_recordtype:"UPDATE",recordid:recordid};
                newData.fieldid=fields[i].id;
                newData.value=rowData[newData.fieldid];
                mcssData.editNewData(newData);
            }
        }

        this.mcssData.save(null, { refObject: this,alert:false });

    }
    this.getCol=function(fieldid){
        var objCol={};
        objCol.fixCol=false;
        var col =$(this.tableFixRow).find("col[fieldid='"+fieldid+"']").get(0);
        if (!col) {
            objCol.fixCol=true;
            col =$(this.tableFixRowCol).find("col[fieldid='"+fieldid+"']").get(0);
        }
        var th =$(this.tableFixRow).find("th[fieldid='"+fieldid+"']").get(0);
        if (!th) th =$(this.tableFixRowCol).find("th[fieldid='"+fieldid+"']").get(0);
        var colData =$(this.tableData).find("col[fieldid='"+fieldid+"']").get(0);
        if (!colData) colData =$(this.tableFixCol).find("col[fieldid='"+fieldid+"']").get(0);
        var colTotal =$(this.tableTotal).find("td[id='"+fieldid+"']").get(0);
        if (!colTotal) colTotal =$(this.tableFixTotal).find("td[id='"+fieldid+"']").get(0);

        objCol.fieldid=fieldid;
        objCol.id=fieldid;
        objCol.style=col.style;
        objCol.colTitle=col;
        objCol.colData=colData;
        objCol.colTotal=colTotal;
        objCol.th=th;
        objCol.mcsstable=this;

        objCol.getWidth = function(){
            return parseInt( this.colTitle.style.width);
        }

        objCol.setWidth=function(iwidth){
            if (this.colTitle) this.colTitle.style.width=iwidth+"px";
            if (this.colData) this.colData.style.width=iwidth+"px";
            if (this.colTotal) this.colTotal.style.width=iwidth+"px";
            if (this.th) this.th.style.width=iwidth+"px";

            this.mcsstable.adjustColWidth();
        }
        objCol.getDisplay=function(){
            return this.colTitle.style.display;
        }
        objCol.setDisplay=function(disp){
            if (this.colTitle) this.colTitle.style.display=disp;
            if (this.colData) this.colData.style.display=disp;
            if (this.colTotal) this.colTotal.style.display=disp;
            if (this.th) this.th.style.display=disp;

            var tds = $(this.mcsstable.table).find("td[id="+fieldid+"]");
            if (disp=="none")
                tds.hide();
            else
                tds.show();

            this.mcsstable.adjustColWidth();

        }
        objCol.getTitle=function(){
            if (this.th) return this.th.innerText;
        }
        return objCol;

    }
    //在某列的前后插入一列
    this.insertCol=function(colInfo,beforeCol,position){

        var bCol=beforeCol;
        if (!position) position="left";
        var col = "<col fieldid='"+colInfo.id+"' style=\"width:100px\">";
        var h =this.createOneHeader(colInfo,"__lastgroup__","__grouptext__");
        //默认在左侧插入
        if(position=="left"){
            $(bCol.colTitle).before(col);
            $(bCol.colData).before(col);
            $(bCol.th).before(h);
        }
        else{
            $(bCol.colTitle).after(col);
            $(bCol.colData).after(col);
            $(bCol.th).after(h);
        }
        var table2=this.tableData;
        var table3=this.tableTotal;

        if (bCol.fixCol){
            table2=this.tableFixCol;
            table3=this.tableFixTotal;
            colInfo.fixCol=true;

            var insertIndex;
            for(insertIndex=0;insertIndex<this.modeldata.fixFields.length;insertIndex++){
                if (this.modeldata.fixFields[insertIndex]['id']==bCol.id) break;
            }
            if(position=="right")
                insertIndex+=1;

            this.modeldata.fixFields.splice(insertIndex,0,colInfo);

            $.post(g_homeurl+"/System/ModelMaker/setColumnFixed",{fieldid:colInfo.id,modelid:this.params.modelid,usermodel_recordid:this.params.usermodel_recordid,fixed:true,groupby:false},function(data){});

        }else{

            var insertIndex;
            for(insertIndex=0;insertIndex<this.modeldata.dataFields.length;insertIndex++){
                if (this.modeldata.dataFields[insertIndex]['id']==bCol.id) break;
            }
            if(position=="right")
                insertIndex+=1;

            this.modeldata.dataFields.splice(insertIndex,0,colInfo);
        }

        //在表格中插入对应单元格
        var tdArr = $(table2).find("td[id="+bCol.fieldid+"]");

        for(var i=0;i<tdArr.length;i++){
            var tdStr = tdArr.eq(i);
            var recordid = tdStr.parent().attr("recordid");
            var str = mcsstable_createTd(this,colInfo,"2",recordid,"","");
            if(position=="left")
                tdStr.before(str);
            else
                tdStr.after(str);
        }
        //在合计列插入对应单元格
        if (table3){
            var tdc = $(table3).find("td[id="+bCol.fieldid+"]");
            var str="<td id='"+ colInfo.id +"' style='width:100px'></td>"
            if(position=="left")
                tdStr.before(str);
            else
                tdStr.after(str);
        }

        this.modelDealer.insertField(bCol.id,position,colInfo);
        this.adjustColWidth();
        return this.getCol(colInfo.id);

    }
    this.deleteCol=function(fieldid){

        this.modelDealer.deleteField(fieldid);

        var col = this.getCol(fieldid);

        if (col.fixCol)
            var fields=this.modeldata.fixFields;
        else
            var fields=this.modeldata.dataFields;

        for(var i=0;i<fields.length;i++){
            if (fields[i].id==fieldid){
                fields.splice(i,1);
                break;
            }
        }

        $(col.colTitle).remove();
        $(col.colData).remove();
        $(col.th).remove();

        $(this.tableData).find("td[id='"+fieldid+"']").remove();
        $(this.tableFixCol).find("td[id='"+fieldid+"']").remove();

        this.adjustColWidth();
    }
    //撤销滚动用
    this.ScrollTo=function(left,top)
    {

        var panel = this.table.parentNode.parentNode;
        this.undoscroll=true;
        if (panel.scrollLeft != left) panel.scrollLeft=left;
        if (panel.scrollTop != top) panel.scrollTop=top;

    }
    this.showTip = function (td,tip,seconds,color)
    {
        if(!seconds) seconds=3;
        if (!color) color="#000";
        errcssLeft=$(td).position().left+20;
        errcssTop=$(td).position().top-30;
        if (this.divTip){
            var divTip = this.divTip;
        }else{
            var divTip = document.createElement("div");
            divTip.style.display="block";
            divTip.style.borderRadius="5px";
            divTip.setAttribute("class","err_info");
            divTip.style["-webkit-transition"] ="left 0.25s ease-out,top 0.25s ease-out,opacity 1.5s ease-out";
            divTip.style["-moz-transition"] ="left 0.25s ease-out,top 0.25s ease-out,opacity 1.5s ease-out";
            divTip.style["transition"] ="left 0.25s ease-out,top 0.25s ease-out,opacity 1.5s ease-out";
            document.body.appendChild(divTip);
            this.divTip = divTip;
        }
        divTip.style.color=color;
        divTip.style.pointerEvents="none";
        divTip.setAttribute("title",tip);
        divTip.innerHTML="<span>"+tip+"</span><span class='dec'><s class='dec1'>◆</s><s class='dec2'>◆</s></span>";
        divTip.style.opacity="1"
        divTip.style.left=errcssLeft+"px";
        divTip.style.top = errcssTop + "px";
        if (this.idclear) window.clearTimeout(this.idclear);
        this.idclear = window.setTimeout(function(){
            divTip.style.opacity="0";
        },seconds * 1000);
    }
    //移动行位置
    this.moveRowSortIndex=function(cutFromIndex,cutToIndex,pasteIndex)
    {
        for(var r=cutFromIndex;r<=cutToIndex;r++)
        {
            this.tableFixCol.rows[r].setAttribute("iscut",'true');
        }

        mcsstable_paste(null,this.tableid,{tr:$(this.tableFixCol.rows[pasteIndex])});

    }
    
    this.scale=function(sc){
        
        var scl=1;
        if (!sc) {
            if (!this.scaleTo)
                this.scaleTo=1;
            scl=this.scaleTo;
        }else{
            if (sc>2) sc=2;
            if (sc<0.2) sc=0.2; 
            if (sc>0.95 && sc<1.05 ) sc=1;
            scl=sc;
            //this.scaleTo=sc;
        }
        if (!sc) return;
        
        var mcsstable=this;
        mcsstable.table.style.width=mcsstable.table.baseWidth/scl+"px";
        mcsstable.table.parentNode.style.width=mcsstable.table.baseWidth*10+'px';
        //mcsstable.table.parentNode.parentNode.style.width=mcsstable.table.style.width;
        mcsstable.table.style.height=mcsstable.table.baseHeight/scl + "px";
        if (mcsstable.hasSummaryField)
            mcsstable.table.totalHeight=30;
        else
            mcsstable.table.totalHeight=0;
        
        mcsstable.tableData.parentNode.parentNode.style.width=(parseFloat(mcsstable.table.style.width) -mcsstable.table.fixColWidth )+"px";
        
        $('#col_Data').get(0).style.width=mcsstable.tableData.parentNode.parentNode.style.width;
        mcsstable.table.rows[0].cells[1].style.width=mcsstable.tableData.parentNode.parentNode.style.width;
        
        mcsstable.tableFixCol.parentNode.style.height= (parseFloat(mcsstable.table.style.height)-mcsstable.table.titleHeight-mcsstable.table.totalHeight)+"px";
        mcsstable.table.rows[1].style.height=mcsstable.tableFixCol.parentNode.style.height;
        mcsstable.tableData.parentNode.parentNode.style.height=mcsstable.tableFixCol.parentNode.style.height;
        
        mcsstable.table.style["-webkit-transform-origin"]="left top";
        mcsstable.table.style["-webkit-transform"]="scale("+ scl +","+scl+")";
        mcsstable.divSelBack.style["-webkit-transform-origin"]="left top";
        mcsstable.divSelBack.style["-webkit-transform"]="scale("+ scl +","+scl+")";
        
        mcsstable.divSelBack.style.width=mcsstable.table.parentNode.style.width;
        mcsstable.divSelBack.style.height=mcsstable.table.style.height;
        
        window.setTimeout(function(){
            if (mcsstable.tableData.offsetHeight<mcsstable.tableData.parentNode.parentNode.offsetHeight){
                var h=100+mcsstable.tableData.parentNode.parentNode.offsetHeight-mcsstable.tableData.offsetHeight;
                var ifrom = mcsstable.maxDataIndex;
                var ito=ifrom+parseInt (h/mcsstable.tableData.rowHeight);
                mcsstable_appendEmptyrow(mcsstable.table,ifrom,ito);
            }
        },100);
        
    }
    this.canEditCell=function(){

        var shareoption=this.params.project_fulldata.shareoption;
        //如果是新增的记录
        if (shareoption && shareoption.indexOf("[editablefields:")>-1 && shareoption.indexOf("[canedit_allfields_mycreated]")>-1)
        {
            var recordid=$(td.parentNode).attr("recordid");
            var adduser=this.mcssData.getFieldValue(recordid,"SYS_ADDUSER");
            if (adduser==uni_global.loginuser)
                return true;
        }
        return null;
    }
    //设置表格样式并创建工具栏
    this.setStyleAndToolbar=function(modeldata)
    {
        var mcsstable=this;
        var tableid=mcsstable.tableid;
        var data=modeldata;
        $("#"+tableid).addClass("mcsstable");
        
        
        if (mcsstable.params.style)
            $("#"+tableid).attr("style",mcsstable.params.style);
        //if (mcsstable.params.width)
        //	$("#"+tableid).css("width",mcsstable.params.width);
        if (mcsstable.modeldata && mcsstable.modeldata.viewtype)
            mcsstable.viewtype=mcsstable.modeldata.viewtype;
        if (data.keyfield && data.keyfield!='')
            mcsstable.keyfield=data.keyfield;
        if (data.pagerows)
            mcsstable.pagerows=data.pagerows;
        var params=mcsstable.params;
        if (params.pagerows)
            mcsstable.pagerows=params.pagerows;

        /*数据库分表*///（暂不处理）写入openurl，到时候跟踪一下，此方法目前不常用
        if (data.openurl=="default") {
            data.openurl = g_systempath+"/viewRecord/model/"+params.modelid+"/tablename/"+data.tablename;
        }

        if(!data.delurl || data.delurl=="default"){
            data.delurl = g_systempath+"/deleterows";
        }
        if (mcsstable.modeldata)
        {
            if(data.openurl)
                mcsstable.modeldata.openurl=data.openurl;
            if(data.editurl)
                mcsstable.modeldata.editurl=data.editurl;
            if(data.delurl)
                mcsstable.modeldata.delurl=data.delurl;
        }
        mcsstable.setDefualtGroupField();
        //	if(!params.style && params.style!='NONE' && params.hideToolbar!=true)
        //		mcsstable_createToolbar(tableid);
    }

    //创建列标题行
    this.createHeader=function(modeldata)
    {
        var mcsstable=this;
        var tableid=mcsstable.tableid;
        var str="<tr><td style='border:none;'><div><table id='"+tableid+"_fixRowCol'></table></div></td><td style='border:none;width:auto;'><div style='border:none;overflow:hidden;width:200px;'><table id='"+tableid+"_fixRow'></table></div></td></tr>";
        str+="<tr>";
        str+="<td class='tabledata-td' style='border:none;'><div style='overflow:hidden;border:none' onscroll='mcsstable_onFixTableScroll(event);' ><table id='"+tableid+"_fixCol' ></table><div style='height:3000px'></div></div></td>";
        str+="<td class='tabledata-td' style='border:none;'><div id='"+this.datatableGrandFather_id+"' style='overflow:auto;border:none;width:100%;height:100%' onscroll='mcsstable_onPanelScroll(event);' ><div style='border:none'><table id='"+tableid+"_data'></table></div></div></td>";
        str+="</tr>";

        mcsstable.table.innerHTML=("<tbody>"+str+"</tbody>");

        var tableData=$("#"+mcsstable.tabledata_id).get(0);
        var tableFixCol=$("#"+tableid+"_fixCol").get(0);
        var tableFixRow=$("#"+tableid+"_fixRow").get(0);
        var tableFixRowCol=$("#"+tableid+"_fixRowCol").get(0);

        mcsstable.table.rows[0].setAttribute("id" , mcsstable.tableheader_id);
        mcsstable.table.rows[0].setAttribute("modelid" , mcsstable.params.modelid);


        tableData.parentNode.parentNode.table=tableData;
        tableData.parentNode.parentNode.mcsstable=mcsstable;
        tableData.parentNode.parentNode.tableid=tableid;
        tableData.parentNode.table=tableData;
        tableData.parentNode.tableid=tableid;

        tableFixCol.parentNode.tableid=tableid;

        tableData.mcsstable=mcsstable;
        tableFixCol.mcsstable=mcsstable;
        tableFixRow.mcsstable=mcsstable;
        tableFixRowCol.mcsstable=mcsstable;

        mcsstable.table.mcsstable=mcsstable;
        
        
        mcsstable.tableData=tableData;
        mcsstable.tableFixCol=tableFixCol;
        mcsstable.tableFixRow=tableFixRow;
        mcsstable.tableFixRowCol=tableFixRowCol;

        
        var el = mcsstable.table;//document.getElementsByTagName("canvas")[0];
        el.addEventListener("touchstart", table_touchStart, true);
        el.addEventListener("touchmove", table_touchMove, true);
        el.addEventListener("touchend", table_touchEnd, true);
        
        if (!uexWidgetOne.getPlatform()) {
            mcsstable.tableData.onclick=mcsstable_OnMouseUp;
            mcsstable.tableFixCol.onclick=mcsstable_OnMouseUp;
        }
        
        mcsstable.tableData.addEventListener("keydown",mcsstable_OnKeyDown,false);
        
        tableData.style.tableLayout="fixed";
        tableFixCol.style.tableLayout="fixed";
        tableFixRow.style.tableLayout="fixed";
        tableFixRowCol.style.tableLayout="fixed";

        //mcsstable.table.style.borderCollapse="collapse";
        tableData.parentNode.style.border="none";
        tableFixCol.parentNode.style.border="none";
        tableFixRow.parentNode.style.border="none";

        tableData.style.border="none";
        tableFixCol.style.border="none";
        tableFixRow.style.border="none";
        tableFixRowCol.style.border="none";

        tableData.style.marginTop="-1px";
        tableData.style.marginLeft="-1px";
        tableFixCol.style.marginTop="-1px";
        tableFixRow.style.marginLeft="-1px";

        tableFixRow.className="mcsstable_header";
        tableFixRow.style.tableLayout="fixed";

        var model = mcsstable.modeldata;
        if (!model)
            return;
        model.fixFields=[];
        model.dataFields=[];
        var sumFields="";
        mcsstable.fields=new Array();
        
        var idedituser=-1;
        var fdedituser=null;
        for(var i=0;i<model.fields.length;i++){
            var fd=model.fields[i];
            //add by sunliangchao 移动端默认标题修改
            if (fd.name=="点此设标题") fd.name="点此设置列";
            if (fd.name=="" && fd.isvisible==undefined ) fd.isvisible="false";
            if (fd.id=='SYS_EDITUSER') {
                fd.isvisible='true';
                fd.name='修改人';
                fd.style='width:150px;background:#eee';
                fdedituser=fd;
                idedituser=i;
            }
            if (fd.id=='SYS_EDITTIME'){
                fd.style='width:150px;background:#eee';
            }
        }
        if (idedituser>=0){ 
            model.fields.splice(idedituser,1);
            model.fields.push(fdedituser);
        }
        for(var i=0;i<model.fields.length;i++){
            var fd=model.fields[i];
            //add by sunliangchao 移动端默认标题修改
            if (fd.name=="点此设标题") fd.name="点此设置列";
            if (fd.name=="" && fd.isvisible==undefined ) fd.isvisible="false";
            if(fd.show_sum) {
                mcsstable.hasSummaryField=true;
                if (sumFields=="")
                    sumFields+=(fd.id);
                else
                    sumFields+=("," + fd.id);
            }
            if (fd.type=="image"){
                mcsstable.hasImageCol=true;
            }
            if (fd.fixCol && fd.fixCol!="false"){
                if (fd.groupby && fd.groupby=="true" && !mcsstable.groupFieldid){
                    mcsstable.setGroupField(fd.id);
                }
                if (fd.isvisible+"" != "false"){
                    model.fixFields.push(fd);
                    mcsstable.fields.push(fd.id);
                }
            }else{

                if (fd.isvisible+"" != "false" ) {
                    model.dataFields.push(fd);
                    mcsstable.fields.push(fd.id);
                }
            }
        }

        mcsstable.sumFields=sumFields;

        var tableid=mcsstable.tableid;
        var table=$("#" + tableid);
        var tableFixRow=$("#" + tableid+"_fixRow");
        var tableFixRowCol=$("#" + tableid+"_fixRowCol");
        var tableFixCol=$("#" + tableid+"_fixCol");
        var tableData=$("#" + mcsstable.tabledata_id);
        var data=modeldata;
        var params=this.params;

        table.get(0).style.tableLayout="fixed";
        if (this.params.hideToolbar!=true){
            mcsstable_createToolbar(tableid);
        }else{
            $("#" + mcsstable.table_toolbar_id).hide();
        }
        var dataFields = data.dataFields;

        cols="";
        var style1="";
        var sth="<tr>";
        var lastgroup='';
        for(var i = 0;i < dataFields.length;i++){
            style='width:100px';
            if(dataFields[i]['style']){
                style=dataFields[i]['style'];
                style=setStyleWidth(style);
            }
            cols+="<col fieldid='"+dataFields[i].id+"' style='"+style+"' />";
            sth+=this.createOneHeader(dataFields[i],lastgroup,dataFields[i].group);
            lastgroup=dataFields[i].group;
        }
        sth+="</tr>";
        //cols+="<col style='width:auto' />";//在最后添加了额外的宽度设为auto的<col>的办法解决最后一列宽度总是不完美的问题
        tableFixRow.append(cols);
        tableFixRow.append(sth);
        tableData.append(cols);

        var html="<tr>";
        var strCols="";
        var fixColWidth=0;
        //显示第一栏
        if (params.showLineNum)
        {
            fixColWidth=35;
            strCols+="<col style='width:35px'/>";
            html+="<th class='mcsstable_th_line' style='width:34px'>";
            html+="<div class='col-alpha icon-export' id='main_showFormStyle' style='display:none' onclick='tableset_showFormStyle(this,\""+tableid+"\")'></div>";
            if (params.showTableSetting)
                html+="<div class='icon-arrow-down2' id='a_mcsstable_tablesetting' style='padding-top:10px;width:100;height:100%' all_setting_tag='close' onclick='mcsstable_tablesetting(this)'></div>";
            html+="</th>";
        }
        if (params.showRowActionMenu){
            fixColWidth+=25;
            html+="<th style='width:25px'></th>";
            strCols+="<col style='width:25px'/>";
        }
        if (params.showfirst && mcsstable.view=='celledit' )//显示第一栏
        {
            if (params.showRecordMenu==false || params.showRowActionBtn==false)
            {
                fixColWidth+=25;
                html += "<th class='mcsstable_th_actions' style='width:25px'>";
                strCols+="<col style='width:25px'/>";

            }else{
                fixColWidth+=75;
                html += "<th class='mcsstable_th_actions' style='width:75px'>";
                strCols+="<col style='width:75px'/>";
            }
            if (params.first_td_name)
                html += params.first_td_name;
            html += "</th>";

        }

        var fixFields = data.fixFields;
        var lastgroup='';
        for(var i = 0;i < fixFields.length;i++){
            var style = fixFields[i]['style'];
            style=setStyleWidth(style);
            fixColWidth+=getStyleWidth(style);
            strCols+="<col fieldid='"+ fixFields[i]['id'] +"' style='"+ style +"'/>";
            html+=this.createOneHeader(fixFields[i],lastgroup,fixFields[i].group);
        }
        if (fixColWidth>0) fixColWidth+=2;
        html+="</tr>";
        tableFixRowCol.append(strCols);
        tableFixCol.append(strCols);
        tableFixRowCol.append(html);
        tableFixRowCol.get(0).style.width=fixColWidth-2+"px";
        tableFixRowCol.get(0).style.tableLayout="fixed";
        tableFixRowCol.get(0).parentNode.style.width=fixColWidth+"px";
        tableFixRowCol.get(0).className="mcsstable_header";
        //add by sunliangchao
        $(mcsstable.table).append("<colgroup><col id='col_fixCol' style='width:"+ fixColWidth +"px'><col id='col_Data' style='width:auto'></colgroup>")

        mcsstable.table.rows[0].cells[0].style.width=fixColWidth+"px";
        mcsstable.table.rows[1].cells[0].style.width=fixColWidth+"px";
        mcsstable.table.fixColWidth=fixColWidth;

        tableFixRow.find(".tablefieldgroup").each(function(i,item){
            if (item.innerHTML=='-')
            {
                //$(item).css("color","white");
                $(item.parentNode).css("border-left","none");
                $(item.parentNode).prev().css("border-right","none");
            }
            else
                $(item.parentNode).css("border-right","none");
        })

        mcsstable.adjustColWidth();

        if (mcsstable.params.autoWidthField)
            mcsstable.setFieldAutoWidth(mcsstable.params.autoWidthField);

        if (mcsstable.params.afterCreateHeader)
            mcsstable.params.afterCreateHeader(mcsstable);

        //调整左上角的高度
        mcsstable.tableFixRowCol.style.height=mcsstable.tableFixRow.offsetHeight+"px";
        mcsstable.tableFixRow.style.height=mcsstable.tableFixRow.offsetHeight+"px";
        mcsstable.table.rows[0].style.height=mcsstable.tableFixRow.offsetHeight+"px";

        if (!mcsstable.params.height || mcsstable.params.height!="auto")
        {
            var divHome=$("#"+mcsstable.id).get(0);

            if (mcsstable.params.height)
                var h1=mcsstable.params.height;
            else if (divHome.style.height)
                var h1= divHome.offsetHeight;
            else
                var h1=document.documentElement.clientHeight-$("#"+mcsstable.id).get(0).offsetTop;

            var containerHeight=h1;
            var tableHeight=h1;	//-52
            //(10 是滚动条宽度，50是 超级表格logo条的高度，2是底边距)
        }else{
            var h1=document.documentElement.clientHeight-$("#"+mcsstable.tablecontainer_id).get(0).offsetTop;
            var containerHeight=h1;
            var tableHeight=h1;	//-52
            //(10 是滚动条宽度，50是 超级表格logo条的高度，2是底边距)
        }
        containerHeight=containerHeight;
        if (this.params.hideToolbar!=true){
            tableHeight=containerHeight-33;
        }else{
            tableHeight=containerHeight;
        }
        //$("#"+mcsstable.tablecontainer_id).css('height',containerHeight+'px');
        $("#"+mcsstable.tablecontainer_id).css({'height':tableHeight+'px'});
        mcsstable.table.style.height=tableHeight+'px';
        
        //保存基本的宽高
        mcsstable.table.baseHeight= tableHeight;
        mcsstable.table.baseWidth=mcsstable.table.offsetWidth;
        mcsstable.table.titleHeight=mcsstable.tableFixRow.offsetHeight;
        var dTableHeight=tableHeight-mcsstable.tableFixRow.offsetHeight-2;
        if (mcsstable.hasSummaryField) dTableHeight=dTableHeight-30;

        mcsstable.tableFixCol.parentNode.style.height=dTableHeight+'px';
        mcsstable.tableData.parentNode.parentNode.style.height=mcsstable.tableFixCol.parentNode.style.height;
        mcsstable.table.rows[1].style.height=dTableHeight+"px";

        if(params.fieldAction=='menu'  || !params.fieldAction)
            mcsstable.setHeaderChangeWidth(mcsstable.params.tableid);

// 		if (uni_global.isMobile())//手机屏幕
// 			$("#"+mcsstable.pagebarid).css("float","left");

        mcsstable_setShortcutKey(mcsstable);
    }

    //当可见列数太少时（小于10吧）把可见的最后一列的width改为auto，多余10时，设为100px；
    this.adjustColWidth=function()
    {
        var tableFixRow=this.tableFixRow;
        var tableFixCol=this.tableFixCol;
        var tableFixRowCol = this.tableFixRowCol;
        var tableData=this.tableData;

        var cs = $(tableFixRow).find("col");
        if(tableData.rows.length>0) var cds=tableData.rows[0].cells;
        var tbWidth=0;
        for(var c=0;c<cs.length;c++){
            if (cs[c].style.width){
                if (cs[c].style.display!="none"){
                    var iw=parseInt(cs[c].style.width);
                    if (isNaN(iw)) iw=100;
                    tbWidth+=iw;
                }
            }else{
                if (cs[c].style.display!="none"){
                    cs[c].style.width="100px";
                    tbWidth+=100;
                }
            }
            if (cds){
                if (cds[c]) cds[c].style.width=cs[c].style.width;
            }
        }

        tableFixRow.className="mcsstable_header";
        tableFixRow.style.width=tbWidth+"px";
        tableFixRow.style.tableLayout="fixed";
        tableFixRow.parentNode.style.width=(tbWidth+100)+"px";
        tableFixRow.parentNode.style.overflow="visible";
        tableData.parentNode.style.width = (tbWidth+1)+"px"
        tableData.style.width=tbWidth+"px";

        var fixColWidth=0;

        var cs = $(tableFixRowCol).find("col");

        for(var c=0;c<cs.length;c++){
            if (cs[c].style.width){
                if (cs[c].style.display!="none"){
                    var iw=parseInt(cs[c].style.width);
                    if (isNaN(iw)) iw=100;
                    fixColWidth+=iw;
                }
            }else{
                if (cs[c].style.display!="none"){
                    cs[c].style.width="100px";
                    fixColWidth+=100;
                }
            }
        }
        if (fixColWidth>0) fixColWidth+=2;
        tableFixRowCol.style.width=fixColWidth-2+"px";
        tableFixRowCol.parentNode.style.width=fixColWidth+"px";
        tableFixRowCol.parentNode.parentNode.style.width=fixColWidth+"px";
        tableFixCol.style.width=fixColWidth+"px";
        tableFixCol.parentNode.parentNode.style.width=fixColWidth+"px";

        //ssssss  ckj
        if (this.params.width=='auto')
        {
            w=this.body.offsetWidth-40;
            $(this.tableFixRow).css('width',(w+1)+"px");
            $(this.tableData).css('width',w+"px");
            $(this.tableData.parentNode).css('width',w+"px");
        }
    }

    //调试试验 可删除 ckj
    this.getEveryWidth=function()
    {
        var s="";
        var obj=this.tableData;
        for(var i=0;i<10;i++)
        {
            s+="\n\r id:"+obj.id+",width:"+obj.offsetWidth;
            obj=obj.parentNode;
        }
        alert(s);
    }

    this.createOneHeader=function(v,lastgroup,grouptext)
    {
        var params=this.params;

        var s="";
        if (v.prop && (v.prop).indexOf("NOTORDER")!=-1)
        {
            s += "<th tag='fieldtitle' id='" + v.id +" ";
        }
        else
        {
            if (!v.name)
                v.name="";
            if(lang[v.name])
                v.name=lang[v.name];
            s += "<th tag='fieldtitle' id='" + v.id + "' fieldid='" + v.id + "'";

            var style1="";
            if(v.style)
                style1+=v.style;//自定义宽度存放在v.style中
//			if (lastgroup==v.group)
//				style1+=";border-left:none;";
            if (params.style_hideLeftRightBorder)
                style1 +=";border-left:none;border-right:none;";
            s+="style='"+style1+"'";//自定义宽度存放在v.style 中

        }
        s+=" >";
        if (params.showAlpha)
        {
            s+="<div id='colmove' class='col-alpha' style='z-index:1;position:absolute;display:none;width:100px;height:8px;margin-top:0px;'";
            s+="></div>";
        }
        if (v.group)
        {
            if (lastgroup!=v.group)
            {
                if (v.group.length>10)
                    grouptext=v.group.substr(0,9)+"...";
                else
                    grouptext=v.group;
                s+="<div class='tablefieldgroup' title='"+v.group+"'>"+grouptext+"</div>";
            }
            else
            {
                grouptext="-";
                s+="<div class='tablefieldgroup-hide' title='"+v.group+"'>"+grouptext+"</div>";
            }
        }
        s+="<div id='colwidth' class='mcsstable-th-changewidth' style='position:absolute;margin-left:120px;'></div>";
        if(params.fieldAction=='menu'  || !params.fieldAction)
        {
            s+="<div id='colname' tabindex=0 class='mcsstable_title_background mcsstable_colname' onclick='ding_mcsstable_header_alpha_click(this,event,true)' ontouchend='ding_mcsstable_header_alpha_click(this,event,true)'";
            if (v.group)
                s+="' style='height:100%;min-height:35px;border-top:dashed 1px gray' ";
            else
                s+="' style='height:100%;min-height:35px;' ";
            s+="title='"+ v.name +"' >"+v.name+"</div>";
        }
        else
        {
            s+="<span title='"+v.name+"'";
            s+=">"+v.name+"</span>";
        }
        s+="</th>";
        return s;
    }
    //标上行号
    this.resortLineIndex=function()
    {
        if (!this.params.showLineNum) return;
        if (this.tableData.fromRow==undefined) this.tableData.fromRow=0;
        var n=1;
        for(i=0;i<this.tableFixCol.rows.length;i++)
        {
            if (!$(this.tableFixCol.rows[i]).hasClass("rowgroup") && !this.tableFixCol.rows[i].getAttribute("groupname"))
            {
                this.tableFixCol.rows[i].cells[0].innerHTML=n + this.tableData.fromRow ;
                this.tableFixCol.rows[i].dataIndex = n + this.tableData.fromRow;
                n++;
            }
        }
    }

    this.getJPOS=function(){
        var JPos = {};//一个介绍对象
        (function($){
            $.getAbsPos = function(pTarget){//js的获得相对位置方法
                var x_ = y_ = 0;
                if(pTarget.style.position != "absolute"){
                    while(pTarget.offsetParent){
                        x_ += pTarget.offsetLeft;
                        y_ += pTarget.offsetTop;
                        pTarget = pTarget.offsetParent;
                    }
                }
                x_ += pTarget.offsetLeft;
                y_ += pTarget.offsetTop;
                return {x:x_,y:y_};
            }
            $.getEventPos = function(evt){
                var _x,_y;
                evt = JEvent.getEvent(evt);
                if(evt.pageX || evt.pageY){
                    _x = evt.pageX;
                    _y = evt.pageY;
                }else if(evt.clientX || evt.clientY){
                    _x = evt.clientX + (document.body.scrollLeft || document.documentElement.scrollLeft) - (document.body.clientLeft || document.documentElement.clientLeft);
                    _y = evt.clientY + (document.body.scrollTop || document.documentElement.scrollTop) - (document.body.clientTop || document.documentElement.clientTop);
                }else{
                    return $.getAbsPos(evt.target);
                }
                return {x:_x,y:_y};
            }
        })(JPos);
        //=======================================================================================
        var JEvent = {};
        (function($){
            $.getEvent = function(evt){
                evt = window.event || evt;
                if(!evt){
                    var fun = JEvent.getEvent.caller;
                    while(fun != null){
                        evt = fun.arguments[0];
                        if(evt && evt.constructor == Event)
                            break;
                        fun = fun.caller;
                    }
                }
                return evt;
            }
            $.doEvent = function(fun){
                var args = arguments;
                return function(){
                    return fun(args);
                }
            }
        })(JEvent);
        //=======================================================================================
        return JPos;
    }
    this.setHeight=function(height){
        height=parseInt(height);
        var ptop=this.table.parentNode.offsetTop;
        var pheight=height-ptop-2;
        this.table.parentNode.style.height=pheight+"px";
        this.table.parentNode.style.height=pheight+"px";
        
        var dtop=this.table.rows[1].cells[1].offsetTop;
        var bheight=0;
        if (this.table.rows.length>2) bheight=30;
        var dheight=pheight-dtop-bheight-4;
        this.tableData.parentNode.parentNode.style.height=dheight+"px";
        this.tableFixCol.parentNode.style.height=dheight+"px";
        this.table.rows[1].style.height=dheight+"px";
        this.divSelBack.style.height=dheight+"px";
    }
    //通过拖拽调整列宽度
    this.setHeaderChangeWidth=function(tableid){
        var mcsstable = mcsstable_getMCSSTable(tableid);
        var JPos = this.getJPOS();
        var colIndex;
        var tableFixRow = mcsstable.tableFixRow;
        var tableFixRowCol = mcsstable.tableFixRowCol;
        var table1=null;
        var table2=null;
        var table3=null;
        var cols = null;//tableFixRow.getElementsByTagName("COL");
        var cols2=null;
        var colSel=null;
        var dragMask = document.createElement("DIV");
        var mAWidth = mAHeight = 1200;
        document.body.insertBefore(dragMask,document.body.lastChild);
        dragMask.style.cssText = "width:" + mAWidth + "px;height:" + mAHeight + "px;position:absolute;display:none;z-index:999999;";
        var colWidth;
        var mask_mousemove = function(evt){
            if(document.selection){//IE ,Opera
                if(document.selection.empty)
                    document.selection.empty();//IE
                else{//Opera
                    document.selection = null;
                }
            }else if(window.getSelection){//FF,Safari
                window.getSelection().removeAllRanges();
            }
            var oPos = JPos.getAbsPos(this);
            var mPos = JPos.getEventPos(evt);
            var x = mPos.x - oPos.x - mAWidth / 2;
            var tmpX = parseInt(colWidth) + x;
            dragMask.style.left = mPos.x - mAWidth / 2 + "px";
            colWidth = tmpX >= 10 ? tmpX + "px" : "10px";
            cols[colIndex].style.width = colWidth;
            cols2[colIndex].style.width = colWidth;

            if (table3)
                table3.rows[0].cells[colIndex].style.width=colWidth;

            var iwidth=0;
            for(var c=0;c<cols.length;c++){
                iwidth+=parseInt(cols[c].style.width);
            }
            table1.style.width=iwidth+"px";
            table2.style.width=table1.style.width;
            if (table1==tableFixRowCol) table1.parentNode.parentNode.style.width=iwidth+"px";
            if (table2.parentNode.parentNode.tagName=="TD") table2.parentNode.parentNode.style.width=iwidth+"px";
            if (table2.parentNode.parentNode.parentNode.tagName=="TD") table2.parentNode.parentNode.parentNode.style.width=iwidth+"px";
            if (table3)
            {
                table3.style.width=table1.style.width;
                if (table1==tableFixRowCol)
                    table3.parentNode.parentNode.style.width=iwidth+"px";
            }
            table1.style.width=iwidth+"px";
            dragMask.style.cursor = 'e-resize';
            //if(!document.all)
            // dataTable.parentNode.style.width = dataTable.parentNode.clientWidth + x + "px";//这一句为处理FF时用。
        }
        var mask_mouseup = function(evt){
            $("body").removeAttr("onselectstart");
            mcsstable.column_width_array = new Array();

            mcsstable.column_width_array[$(colSel).attr("fieldid")] = colSel.offsetWidth+"px";

            tableset_saveColumnWidth(tableid);
            dragMask.style.display = "none";

        }
        dragMask.onmousemove = mask_mousemove;
        dragMask.onmouseup = mask_mouseup;
        var cell_mousedown = function(evt){
            $("body").attr("onselectstart","return false");
            colIndex = this.colIndex;
            colSel=this.parentNode;
            table1=this.parentNode.parentNode.parentNode.parentNode;

            if (table1==tableFixRow){
                table2=mcsstable.tableData;
                table3=mcsstable.tableTotal;
            }else if(table1==tableFixRowCol){
                table2=mcsstable.tableFixCol;
                table3=mcsstable.tableFixTotal;
            }
            cols = table1.getElementsByTagName("COL");
            cols2=table2.getElementsByTagName("COL");
            colWidth = cols[colIndex].style.width;
            var mPos = JPos.getEventPos(evt);

            if (mcsstable.divSel){
                mcsstable.divSel.style.display="none";
            }

            with(dragMask.style){
                left = mPos.x - mAWidth / 2  + "px";
                top = mPos.y - mAHeight / 2 + "px";
                display = "";
            }
        }
        var i,o;
        for(i = 0;i<tableFixRow.rows[0].cells.length;i++){
            o=tableFixRow.rows[0].cells[i];
            var div = $(o).find("div[class='mcsstable-th-changewidth']").get(0);
            if(div){
                div.colIndex = i;
                div.onmousedown = cell_mousedown;
            }
        }
        for(i = 0;i<tableFixRowCol.rows[0].cells.length;i++){
            o=tableFixRowCol.rows[0].cells[i];
            var div = $(o).find("div[class='mcsstable-th-changewidth']").get(0);
            if(div){
                div.colIndex = i;
                div.onmousedown = cell_mousedown;
            }
        }
    }
    //设置某列的宽度
    this.setColWidth=function(colIndex,width)
    {
        $(this.table).find('col').eq(colIndex).css('width',width);
        $(this.table.rows[0].cells[colIndex]).css('width',width);
    }
    //根据行列号，返回TD单元格
    this.getCell=function(row,cell){
        if (row>=this.tableData.rows.length) return null;
        if (this.tableData.rows.length<1) return null;
        var fixcells=this.tableFixCol.rows[0].cells;

        if (cell<fixcells.length)
            return this.tableFixCol.rows[row].cells[cell];
        else
            return this.tableData.rows[row].cells[cell-fixcells.length];
    }
    //得到TD的列序号
    this.getCellIndex=function (td){
        if (td.parentNode.parentNode.parentNode==this.tableFixCol ||  td.parentNode.parentNode.parentNode==this.tableFixRowCol)
            return td.cellIndex;
        else
            return td.cellIndex+this.tableFixCol.rows[0].cells.length;
    }
    //把两个table的row拼起来组成一个row，方便需要时调用，该方法兼容性好，但执行效率较低，
    this.getRow=function(rowIndex){
        if (rowIndex>=this.tableFixCol.rows.length) return null;
        var row={id:this.tableFixCol.rows[rowIndex].getAttribute("id"),recordid:this.tableFixCol.rows[rowIndex].getAttribute("id")};
        row.rowIndex=rowIndex;
        //模拟cells
        row.cells=[];
        for(var i=0;i<this.tableFixCol.rows[rowIndex].cells.length;i++){
            row.cells.push(this.tableFixCol.rows[rowIndex].cells[i]);
        }
        for(var i=0;i<this.tableData.rows[rowIndex].cells.length;i++){
            row.cells.push(this.tableData.rows[rowIndex].cells[i]);
        }
        //模拟get/set/Attribute两个函数
        row.mcsstable=this;
        row.getAttribute=function(att){
            return this.mcsstable.tableFixCol.rows[rowIndex].getAttribute(att);
        }
        row.setAttribute=function(att,val){
            this.mcsstable.tableFixCol.rows[rowIndex].setAttribute(att,val);
            this.mcsstable.tableData.rows[rowIndex].setAttribute(att,val);
        }
        return row;
    }
    //根据文字内容自适应行高
    this.fixRowHeight=function(r){
        if (!r){
            if (this.tableFixCol && this.tableData){
                for(var i=0;i<this.tableData.rows.length;i++){
                    var r=i;
                    this.tableFixCol.rows[r].style.height=this.tableData.rows[r].offsetHeight+"px";
                    if (this.modeldata.fixFields && this.modeldata.fixFields.length>0){
                        this.tableData.rows[r].style.height=this.tableFixCol.rows[r].offsetHeight+"px";
                        this.tableFixCol.rows[r].style.height=this.tableData.rows[r].offsetHeight+"px";
                    }
                }
            }
        }else{
            if (this.tableFixCol && this.tableData){
                if (this.tableFixCol.rows.length>r){
                    this.tableFixCol.rows[r].style.height=this.tableData.rows[r].offsetHeight+"px";
                    if (this.modeldata.fixFields && this.modeldata.fixFields.length>0){
                        this.tableData.rows[r].style.height=this.tableFixCol.rows[r].offsetHeight+"px";
                        this.tableFixCol.rows[r].style.height=this.tableData.rows[r].offsetHeight+"px";
                    }
                }
            }
        }
    }

    //从后台获取模型，然后执行指定方法
    this.getModelDataAndDo=function(params,dosome)
    {
        var _this=this;
        
        function modelReturn(modeldata){
            _this.modelDealer=new MCSSModel(modeldata);//模型操作类

            _this.setModelData(_this, modeldata);
 
            if (_this.params.afterGetModel)
                _this.params.afterGetModel(modeldata,_this);

            _this.modelDealer.parseModelDefaultValue(_this.params.defaultValue);
            localStorage.setItem("modeldata_"+_this.params.usermodel_recordid+ '_' + _this.params.view_id , JSON.stringify(modeldata));
            localStorage.setItem("modeldata_time_"+_this.params.usermodel_recordid+ '_' + _this.params.view_id ,new Date().format('yyyy-MM-ddTHH:mm:ss'));
            dosome(_this,modeldata);            
        }
        
        if (!params)
            params={modelid:this.params.modelid,usermodel_recordid:this.params.usermodel_recordid,
                model_ownerid:this.params.model_ownerid,view_id:this.view_id,version_id:this.params.version_id};
        
        var dt=localStorage.getItem("modeldata_time_"+_this.params.usermodel_recordid+ '_' + _this.params.view_id );
        var mdtime=new Date(Date.parse((dt)));
        var md=false;
   
        var tdiff=(new Date().getTime()- mdtime.getTime())/60000 ;
        if (tdiff<10)
        {
            md=localStorage.getItem("modeldata_"+ params.usermodel_recordid + '_' + params.view_id ,"");
            if (md!="") md=jQuery.parseJSON(md);
        }
        if (md && md!="" && md!='null'){
            modelReturn(md);
        }else{
            $.getJSON(this.getModelUrl,params,modelReturn);
        }
    }
    //从后台获取表格记录数据，然后执行指定方法
    this.getRecordsAndDo=function(params,dosome)
    {
        if(params.fromRow && parseInt(params.fromRow)<0)
            return;
        // params.fromRow = 0;
        var p={
            modelid:params.modelid,
            filter:params.filter,
            orderby:params.orderby,
            page:params.page,
            pagerows:params.pagerows,
            view_id:params.view_id,
            searchword:params.searchword,
            sql:params.sql,
            fromRow:params.fromRow,
            rowCount:params.rowCount,
            getRecordCount:params.getRecordCount,
            usermodel_recordid:params.usermodel_recordid,
            model_ownerid:params.model_ownerid,version_id:params.version_id};
        p.filter=uni_cleanSQL_JSON(p.filter);
        p.sql=uni_cleanSQL_JSON(p.sql);
        p.searchword=uni_cleanSQL_JSON(p.searchword);

        var _this=this;

        $.post(this.getDataListUrl,p,function(data, textStatus, jqXHR){
            //alert("控制器方法返回的记录数组是："+data);
            if (textStatus=='success'){
                dosome(_this,data);
            }
            else
            {
                alert("读取数据错误！");
            }
        },'json');
    }

    this.loadTableJS=function()
    {
        // if(!this.params.fieldAction || this.params.fieldAction=='menu')
        // mcss_importJS("vendor/dingtalk/jusaas/js/tableset.js");
        //mcss_importJS("jusaas/js/MCSSInputer.js");
        //if (this.params.view=='celledit' || this.params.view=='tableedit')
        //	mcss_importJS("jusaas/js/MCSSTableEditor.js");

    }

    this.getFieldValue=function(id,fieldname)
    {
        if (this.data)
        {
            for(var i=0;i<this.data.length;i++)
            {
                if (this.data[i].id==id)
                {
                    //return this.data[i].
                }
            }
        }
        return '';
    }
    var _this=this;
    this.setFieldValueToTD=function(recordid,fieldid,value,readonly){

        var row=$(_this.table).find("tr[recordid='"+recordid+"']")
        var td = $(row).find("td[id='"+ fieldid +"']");

        var input=$(td).find('input,div[fieldid]');
        td.readonly=true;
        td.get(0).readonly=true;
        var rowData=_this.mcssData.getRow(recordid,fieldid);

        fields=_this.modeldata["fields"];
        for(var j=0;j<fields.length;j++)
        {
            if (fields[j].id==fieldid)
                break;
        }

        mcdom.popup("该单元格已被他人修改为:" + value ,{seconds:3,obj:td.get(0),hideheader:true,fadeOut:true});
        rowData[fieldid]=value;
        var s=mcsstable_createTd(_this,fields[j],"",recordid,rowData,"");

        if (readonly) {

            if (input.get(0)){
                input.val(value);
                input.get(0).style.display='none';
            }
        }
        td.get(0).outerHTML=s;
        if (!readonly){
            mcsstable.cellEditor.clickCell(td);
            var input=$(td).find('input,div[fieldid]');
            if (input.get(0) && input.get(0).tagName=='INPUT')
            {
                input.val(value);
            }
        }

    }

    this.reload=function()
    {
        mcsstable_reload(this.tableid);
    }
    this.showAllActions=function(yesno)
    {
        if($("#"+this.params.tableid+"_moreact_btn").html()){
            this.showAction("del");
            this.showAction("tableedit");
            return;
        }
        if (yesno==false)
            $("#"+this.table_toolbar_id).children().hide();
        else
            $("#"+this.table_toolbar_id).children().show();
    }
    this.showAction=function(actions,show)
    {
        var actionArr = actions.split(',');
        for(var i = 0;i < actionArr.length;i++){
            var action = actionArr[i];
            $("#"+this.tableid+"_action_"+action).show();
            if (show==false)
                $("#"+this.tableid+"_action_"+action).hide();
        }
    }
    this.removeActions=function(actions)
    {
        var actionArr = actions.split(',');
        for(var i = 0;i < actionArr.length;i++){
            $("#"+this.tableid+"_action_"+actionArr[i]).remove();
        }
    }
    //获得工具栏dom对象
    this.getToolBar=function()
    {
        return document.getElementById(this.table_toolbar_id);
    }
    //获得指定按钮dom对象,action例子:add表示新增按钮，edit表示编辑按钮
    this.getAction=function(action)
    {
        return document.getElementById(this.tableid+"_action_"+action);
    }
    //显示／隐藏所有分组的按钮
    this.getShowHideGroupBtn=function(tableid)
    {
        return "<a id='"+tableid+"_expandAllGroup'  class='icon-stack' onclick='mcsstable_expandAllGroup(this);' title='隐藏/显示所有组' ></a>";
    }

    this.showDataToCell=function(tr,recordid,fieldid,value)
    {
        var field=this.modelDealer.getField(fieldid);
        if (field)
        {
            var rowData=this.mcssData.getRow(recordid,fieldid);
            var s=mcsstable_getFieldDisplay(field,value,this.params.special_field_show,rowData,this) ;
            $(tr).find("td[id='"+fieldid+"']").html(s);
        }
    }
    //根据一条数据记录，把其值更新到tr行上
    //data:一个json对象
    this.updateRowByData=function(recordid,data)
    {
        var trf=$(this.tableFixCol).find("tr[recordid='"+recordid+"']").get(0);
        var trd=$(this.tableData).find("tr[recordid='"+recordid+"']").get(0);
        if (!trf) return;
        for(f in data)
        {
            this.showDataToCell(trf,recordid,f,data[f]);
            this.showDataToCell(trd,recordid,f,data[f]);
        }
        if (this.params.afterUpdateDataToRow)
            this.params.afterUpdateDataToRow(trf,recordid,data,this);
    }

    //根据指定字段和方向，修改字段排序依据参数
    this.setFieldOrder=function(fieldid,updown)
    {
        this.orderFieldid=fieldid;
        this.field_order_updown=updown;
        mcsstable_rows(this.tableid);
    }

    //因为用户动作，如修改分组依据，单击排序字段等，确定最终的排序依据(即确定this.order的值),包括分组排序和列排序或者两者组合
    this.setOrder=function()
    {
        this.order='';
        if (this.groupFieldid)
        {
            if (this.group_order_updown==undefined)
                this.group_order_updown="";
            if (this.order)
                this.order+=",";
            this.order+=this.groupFieldid+" "+this.group_order_updown;
        }
        if (this.orderFieldid)
        {
            if (this.field_order_updown==undefined)
                this.field_order_updown="";
            if (this.order)
                this.order+=",";
            this.order+=this.orderFieldid+" "+this.field_order_updown;
        }
        else
        {
            if (this.params.defaultOrderby)
            {
                if (this.order)
                    this.order+=",";
                this.order+=" "+this.params.defaultOrderby;
            }
        }
    }


    //校验每个输入值
    this.verifyInputer=function()
    {
        var _this=this;
        var all=$(this.table).find("input");
        var inputer;
        var pass=true;
        for(var i=0;i<all.length;i++)
        {
            inputer=all[i];
            var f=_this.modelDealer.getField($(inputer).attr('fieldid'));
            if (f)
            {
                var err=inputer_verify(inputer,f,{when:'save',owner:this});
                if (err && err !=true)
                {
                    inputer_showErr(inputer,err,f);
                    $(inputer).focus();
                    pass=false;
                }
            }
        }
        return pass;
    }
    //表内编辑时把值更新到字段上。showup:是否显示在输入框上
    this.editField=function(recordid,fieldid,value,showup)
    {
        this.mcssData.setFieldValue(recordid,fieldid,value);
        if(showup==undefined || showup===true)
            this.cellEditor.showFieldValueInCell(recordid,fieldid);
    }

    //设置groupFieldid和groupField对象
    this.setGroupField=function(fieldid)
    {
        if (!this.modeldata)
            return;
        this.groupField=null;
        //this.groupOrder="";
        if (fieldid=="NONEGROUP")
            this.groupFieldid='';
        else
        {
            this.groupFieldid=fieldid;
            this.groupField=this.modelDealer.getField(fieldid);
        }
    }

    //设置初始化时的默认分组对象
    this.setDefualtGroupField=function()
    {
        if (this.modeldata && this.modeldata.groupby=='y' && this.modeldata.orderby)
        {
            var fields1=this.modeldata.orderby.split(",");
            if (fields1.length>1){
                var fields2=fields1[0].split(' ');
                this.setGroupField(fields2[0]);
                if (fields2.length==2)
                    this.group_order_updown=fields2[1];
                //this.groupOrder=this.modeldata.orderby;
                var fields3=fields1[1].split(' ');
                this.orderFieldid=fields3[0];
                if (fields3.length==2)
                    this.field_order_updown=fields3[1];
            }
        }
    }

    //获得左右指定了像素宽度的列的宽度之和，不包括未指定宽度的列
    this.getAllHeaderWidth=function()
    {
        var w=0;
        var fields=this.modeldata.fields;
        for(var i=0;i<fields.length;i++)
        {
            //if (fields[i].style)
        }
    }
    //是否有输入值且未保存的
    this.ifDataChanged=function()
    {
        return (this.mcssData && this.mcssData.state=='CHANGED' && this.mcssData.inputedByKey==true);
    }

    this.getAddUrl=function()
    {
        var tmp=this.modeldata.addurl;
        if (!tmp || tmp=="default")
            tmp=g_systempath+"/addRecord";
        if (tmp)
            tmp+="/modelid/"+this.params.modelid;
        var url=this.params.homeUrl+tmp;
        if (this.params.defaultValue)
        {
            url+="/defaultValue/"+this.params.defaultValue;
        }
        return url;
    }
    //获得共享视图的id列表
    this.getShareViews=function()
    {
        var shareviews="";
        if (this.params.project_fulldata && this.params.project_fulldata.shareviews && this.params.project_fulldata.project_record.SYS_ADDUSER!=loginuser)
            shareviews=","+this.params.project_fulldata.shareviews+",";
        return shareviews;
    }
    //获取保存过的查询名称
    this.loadViewList=function(mcsstable,defaultView,alwaysFetchNew)
    {
        if (!mcsstable)
            mcsstable=this;
        var viewaction=mcsstable.getAction('view');
        if (viewaction)
        {
            var h="";
            var select_id=viewaction.id;
            var shareviews=this.getShareViews();
            if (!shareviews || shareviews.indexOf(",DEFAULT,")>-1)
                h+="<option value='DEFAULT'>全部数据</option>";
            $(viewaction).html(h);
            if (!defaultView)
                defaultView=this.modeldata.defaultView;

            if (mcsstable.params.project_fulldata)
                var owner=mcsstable.params.project_fulldata.project_record.SYS_ADDUSER;
            if (!alwaysFetchNew && this.params.viewlist)
            {
                this.loadViewDataToList(mcsstable,this.params.viewlist,select_id,defaultView);
            }
            else
            {
                var _this=this;

                $.getJSON(g_homeurl+"/System/Model/getViewList",{modelid:mcsstable.modelid,usermodel_recordid:mcsstable.params.usermodel_recordid,owner:owner,version_id:mcsstable.params.version_id},function(data)
                {
                    _this.params.viewlist=data;
                    _this.loadViewDataToList(mcsstable,data,select_id,defaultView);
                });
            }
        }
    }

    this.loadViewDataToList=function(mcsstable,views,select_id,defaultView)
    {
        var h="",ifadd;
        //判断表格的共享视图
        var shareviews=this.getShareViews();
        for(i=0;i<views.length;i++)
        {
            ifadd=1;
            if (shareviews)
            {
                if (shareviews.indexOf(","+views[i].id+",")==-1)
                    ifadd=0;
            }
            if (ifadd)
            {
                h+="<option value='"+views[i].id+"' ";
                if (views[i].id==defaultView)
                    h+=" selected=true";
                h+=">"+views[i].name+"</option>";
            }
        }
        $("#"+select_id).append(h);
        mcsstable_createSysViewList(mcsstable);
        
        var sels=$("#"+select_id).find('option');
        var viewStr="";
        for(var i=0;i<sels.length;i++){
            var opt=sels.eq(i);
            viewStr+="<div class='weui_actionsheet_cell' title='"+opt.get(0).value +"'>" + opt.get(0).text+"</div>";
        }
        mcsstable.viewlist_HtmlString=viewStr;
        
//		if (!defaultView)
//			this.setOneDefaultView(mcsstable)
        if (mcsstable.params.afterLoadViewList)
            mcsstable.params.afterLoadViewList(mcsstable);
    }

    /*
     //如果没有默认视图，则指定第一个作为默认视图
     this.setOneDefaultView=function()
     {
     var options=$(this.getAction("view")).find("option");
     for(var i=0;i<options.length;i++)
     {
     if (options[i].value!="SAVEASVIEW" && options[i].value!="MANAGEVIEW")
     {
     options[i].checked=true;
     break;
     }
     }
     }
     */


    //打开视图
    this.openView=function(id)
    {
        this.view_id=id;
        var _this = this;
        $(this.table).html("");
        $.post(g_homeurl+"/Oa/StatView/statview_getViewType",{view_id:id},function(data){
            if(data=='stat')
                _this.getStatViewDataAndShow();
            else
                _this.getModelDataAndDo(null,_this.createHeaderAndGetViewRecords);
        })
    }
    //从后台获取统计视图对应的数据，并绘制表格
    this.getStatViewDataAndShow=function(params)
    {
        mcss_importJS("jusaas/js/UniStatView.js");
        var _this=this;
        var viewDataUrl =g_homeurl+"/System/Model/statview_getModelData";
        if (!params)
            params={modelid:this.params.modelid,usermodel_recordid:this.params.usermodel_recordid,
                model_ownerid:this.params.model_ownerid,view_id:this.view_id,version_id:this.params.version_id};
        $.getJSON(viewDataUrl,params,function(modeldata){
            _this.modeldata=modeldata;
            _this.isStatView = true;
            _this.modelDealer=new MCSSModel(modeldata);//模型操作类
            if (_this.params.afterGetModel)
                _this.params.afterGetModel(modeldata,_this);
            _this.modelDealer.parseModelDefaultValue(_this.params.defaultValue);
            if(_this.params.model_ownerid!=loginuser)
                g_statview_canEditTag = false;
            // if(_this.view =="" ||_this.view=="table")
            // g_statview_canEditTag = false;
            if(modeldata.viewTypeShowAll=='yes')
                g_statview_ListShow = true;
            else
                g_statview_ListShow = false;
            _this.mcsstable_createToolbarStat(_this,modeldata);
            g_statview_VisibleFields = modeldata.fields;
            g_statview_searchWord = modeldata.view_searchword;
            g_statview_query = modeldata.view_query;
            g_statview_currentView = _this.view_id;
            g_statview_everPage = parseInt(modeldata.viewPagerows);
            statview_tableId = _this.tableid;
            statview_projectid = _this.params.usermodel_recordid;
            try{
                statview_init_allFields(modeldata);
                statview_createStatView(modeldata.viewFields);
            }
            catch(e){
                alert("程序错误，因为程序文件加载未完成。一般情况下刷新页面即可解决。");
            }

        })
    }
    this.mcsstable_createToolbarStat=function(mcsstable,modeldata){
        if(mcsstable.params.afterCreateToolbar){
            type='stat';
            mcsstable.params.afterCreateToolbar(mcsstable,type);
        }
        var tableid = mcsstable.tableid;
        var window_height = $(window).height();
        $("#" + tableid+"_tablecontainer").find('table').css({'height':20+'px'});
        $("#" + tableid+"_tablecontainer").addClass('mcsstable').css({'height':(window_height-165)+'px','overflow':'auto'});
    }
    //根据模型结构创建列标题行，然后后台获得视图信息，并更新数据行
    this.createHeaderAndGetViewRecords=function(mcsstable,modeldata)
    {
        mcsstable.setStyleAndToolbar(modeldata);
        mcsstable_setIconField(mcsstable);

        var _this=mcsstable;
        _this.isStatView = false;
        $.getJSON(g_homeurl+"/System/Model/openView",{'id':mcsstable.view_id},function(view){


            _this.advancedFilter =view.query;
            _this.view_searchword =view.searchword;
            _this.autoFilter =view.autofilter;
            var groupby=view.groupby.split(" ");
            _this.setGroupField(groupby[0]);
            $(_this.getAction('groupby')).val(groupby[0]);
            if (groupby.length>1)
                _this.group_order_updown=groupby[1];
            var sortby=view.sortby.split(" ");
            _this.orderFieldid=sortby[0];
            if (sortby.length>1)
                _this.field_order_updown=sortby[1];

            //分组的字段默认为固定列
            if (_this.groupFieldid){
                var fs=modeldata.fields;
                for(var i=0;i<fs.length;i++){
                    if (fs[i].id==_this.groupFieldid){
                        fs[i].fixCol=true;
                    }
                }
            }
            //add by sunliangchao

            mcsstable.createHeader(modeldata);
            if (mcsstable.runWhenFinished)
                mcsstable.runWhenFinished(mcsstable);
            mcsstable_loaddatarows(_this.tableid);
        });
    }
    //设置列移动的事件
    this.setTdMoveFunc=function(row)
    {
        var table = this.tableFixRow;
        var divid = this.tableid+"_movingbar";

        var rows = this.tableFixRow.rows;
        var rowdatas= this.tableData.rows;

        $(row).find("th[tag='fieldtitle']").mouseover(function(){
            //var td = $(this).find("td[tr_isselect='true']");
            if($("#"+divid).attr("id") && $(table).find("td[tr_isselect='true']").length==0){
                var cellIndex = $(this).get(0).cellIndex;
                var check = $(table).find("tr").eq(0).find("th").eq(cellIndex).attr('th_isselect');
                if(check )
                    return;
                $(this).attr("ismove",true);
                var cellIndex = $(this).get(0).cellIndex;
                for(var i = 0;i < rowdatas.length;i++){
                    $(rowdatas[i]).find("th,td").eq(cellIndex).css("border-right","2px solid black");
                }
                for(var i = 0;i < rows.length;i++){
                    $(rows[i]).find("th,td").eq(cellIndex).css("border-right","2px solid black");
                }
            }
        })
        $(row).find("th[tag='fieldtitle']").mouseout(function(){
            var cellIndex = $(this).get(0).cellIndex;
            var check = $(table).find("th").eq(cellIndex).attr('th_isselect');
            if(check )
                return;
            if($("#"+divid).attr("id") && $(table).find("td[tr_isselect='true']").length==0){
                $(this).removeAttr("ismove");
                var cellIndex = $(this).get(0).cellIndex;
                for(var i = 0;i < rows.length;i++){
                    $(rows[i]).find("th,td").eq(cellIndex).css("border-right","1px solid #ccc");
                }
                for(var i = 0;i < rowdatas.length;i++){
                    $(rowdatas[i]).find("th,td").eq(cellIndex).css("border-right","1px solid #ccc");
                }
            }
        })
        $(row).find("th[tag='fieldtitle']").mouseup(function(){
            var cellIndex = $(this).get(0).cellIndex;
            var check = $(table).find("th").eq(cellIndex).attr('th_isselect');
            if(check )
                return;
            if($("#"+divid).attr("id") && $(table).find("td[tr_isselect='true']").length==0){
                $(this).removeAttr("ismove");
                for(var i = 0;i < rows.length;i++){
                    $(rows[i]).find("th,td").eq(cellIndex).css("border-right","1px solid #ccc");
                }
                for(var i = 0;i < rowdatas.length;i++){
                    $(rowdatas[i]).find("th,td").eq(cellIndex).css("border-right","1px solid #ccc");
                }
                if($(table).find("th[th_isselect='true']").length > 1){
                    mcdom.alert("多列暂不支持拖动",'提示','info','fadeout');return;
                }
                mcsstable_moveColumnPosition(this,table);
            }
        })
    }
    //设置行移动的事件
    this.setTrMoveFunc=function(row)
    {
        var table = this.table;
        var divid = table.id+"_movingbar";
        $(row).mouseover(function(){
            //var td = $(this).find("td[tr_isselect='true']");
            if($("#"+divid).attr("id") && $(table).find("th[th_isselect='true']").length==0){
                $(this).attr("ismove",true);
                $(this).css("border-top","2px solid black");
            }
        })
        $(row).mouseout(function(){
            if($("#"+divid).attr("id") && $(table).find("th[th_isselect='true']").length==0){
                $(this).removeAttr("ismove");
                $(this).css("border-top","");
            }
        })
        $(row).mouseup(function(){
            if($("#"+divid).attr("id") && $(this).attr("ismove") && $(table).find("th[th_isselect='true']").length==0){
                var tds = $(table).find("td[tr_isselect='true']");
                for(var i = 0;i < tds.length;i++){
                    $(tds[i]).parent().attr("iscut",true);
                }
                mcsstable_paste(this,table.id,{tr:this});
                $(this).removeAttr("ismove");
                $(this).css("border-top","");
                mcsstable_clearSelect_td(table);
            }
        })
    }

    //设置当前行
    this.setCurrentRow=function(row)
    {
        if (row)
        {
            if (this.currentRow && this.currentRow.parentNode){
                //this.currentRow.className="";
                if (!$(row).attr("groupname")){
                    if(this.tableFixCol.rows[this.currentRow.rowIndex])
                        this.tableFixCol.rows[this.currentRow.rowIndex].className="";
                    if(this.tableData.rows[this.currentRow.rowIndex])
                        this.tableData.rows[this.currentRow.rowIndex].className="";
                }
                //this.currentRow.cells[0].className="linenumber";
            }
            this.currentRow=row;
            if (!$(row).attr("groupname")){
                //row.className='tr_edit';
                if(this.tableFixCol.rows[row.rowIndex])
                    this.tableFixCol.rows[row.rowIndex].className="tr_edit";
                if(this.tableData.rows[row.rowIndex])
                    this.tableData.rows[row.rowIndex].className="tr_edit";
                //row.cells[0].className="linenumber_edit";
            }
            document.currentRow=row;
            $(".unitable_currentrow").removeClass('unitable_currentrow');
            //$(row).find('td').eq(0).addClass("unitable_currentrow");
        }
        else
        {
            if (this.currentRow && this.currentRow.parentNode){
                if (!$(this.currentRow).attr("groupname")){
                    if(this.tableFixCol.rows[this.currentRow.rowIndex])
                        this.tableFixCol.rows[this.currentRow.rowIndex].className="";
                    if(this.tableData.rows[this.currentRow.rowIndex])
                        this.tableData.rows[this.currentRow.rowIndex].className="";
                }
            }
            this.currentRow=null;
            document.currentRow=null;
        }
    }

    this.save=function()
    {
        $(this.getAction("save")).click();
    }
    this.removeEditActions=function()
    {
        this.removeActions('del,add,tableedit,batchedit,import,styleedit_show');
        $("#"+this.getActionId("view")+"_viewactions").remove();
        $(this.table).find(".mcsstable_record_del,.mcsstable_record_edit,.mcsstable_record_add,.mcsstable_record_cut,.mcsstable_record_paste").remove();
    }
    //设置字段你的宽度
    this.setFieldWidth=function(fieldid,width)
    {
        $(this.tableFixRow).find("col[fieldid='"+fieldid+"']").css('width',width);
        $("#"+this.tabledata_id).find("col[fieldid='"+fieldid+"']").css('width',width);
        $("#"+this.tabledata_id).find("col[fieldid='"+fieldid+"']").css('width',width);
    }
    this.getFieldWidth=function(fieldid)
    {
        return $("#"+this.tabledata_id).find("col[fieldid='"+fieldid+"']").css('width');
    }
    //设置某个字段宽度为auto，这样才能把表格剩余的空间填满
    this.setFieldAutoWidth=function(fieldid)
    {
        var w1=this.tablecontainer.offsetWidth-document.getElementById(this.tabledata_id).offsetWidth;
        if (w1>0)
        {
            var w2=this.getFieldWidth(fieldid);
            w2=(parseFloat(w2)+w1-75)+"px";
            this.setFieldWidth(fieldid,w2);
        }
        //this.setEveryWidth();
    }

    this.setEveryWidth=function()
    {
        w=this.body.offsetWidth-30;
        $(this.tableFixRow).css('width',w+"px");
        $(this.tableData).css('width',w+"px");
        $(this.tableData.parentNode).css('width',w+"px");
        $(this.tableData.parentNode.parentNode).css('width',w+"px");
    }

}

//拖动列
function mcsstable_moveColumnPosition(column,table){
    var mcsstable = mcsstable_getMCSSTable(table.id);

    var th = $(mcsstable.tableFixRow).find("th[th_isselect='true']").eq(0);
    if(th.size()==0)
    {
        return;
    }
    if (th.get(0)==column) return;
    //mcsstable_createTempMoveTd(column,table);

    var moveIndex = $(th).get(0).cellIndex;
    var positionIndex = parseInt($(column).get(0).cellIndex)+parseInt(1);

    //移动th
    var tr = $(th).parent();
    var thiscol=tr.find('th').eq(moveIndex).get(0);
    var nextcol=tr.find('th').eq(positionIndex).get(0);
    if (!thiscol || thiscol==nextcol) return;

    if(nextcol)
        thiscol.parentNode.insertBefore(thiscol,nextcol);
    else
        thiscol.parentNode.appendChild(thiscol);

    //移动col
    var thiscol=$(table).find('col').eq(moveIndex).get(0);
    var nextcol=$(table).find('col').eq(positionIndex).get(0);
    if (!thiscol || thiscol==nextcol) return;

    if(nextcol)
        thiscol.parentNode.insertBefore(thiscol,nextcol);
    else
        thiscol.parentNode.appendChild(thiscol);


    var rows = $(table.mcsstable.tableData).find("tr");
    for(var i = 1;i < rows.length;i++){
        var thiscol=$(rows[i]).find("td").eq(moveIndex).get(0);
        var nextcol=$(rows[i]).find('td').eq(positionIndex).get(0);
        if(nextcol)
            thiscol.parentNode.insertBefore(thiscol,nextcol);
        else
            thiscol.parentNode.appendChild(thiscol);
    }

    //移动col
    var thiscol=$(mcsstable.tableData).find('col').eq(moveIndex).get(0);
    var nextcol=$(mcsstable.tableData).find('col').eq(positionIndex).get(0);
    if (!thiscol || thiscol==nextcol) return;

    if(nextcol)
        thiscol.parentNode.insertBefore(thiscol,nextcol);
    else
        thiscol.parentNode.appendChild(thiscol);

    mcsstable_saveColumnDisplay(mcsstable);
    //$(table).find("th[istemp='true'],td[istemp='true']").remove();
    //add by gang 拖动列后，调整宽度
    mcsstable.setHeaderChangeWidth(mcsstable.params.tableid);
    mcsstable_clearSelect_td(table);
}


function ding_mcsstable_header_alpha_click(obj,event,onlyPopMenu){
    //重命名调用方法prompt无效（当前值不能写入）
    //插入列调用方法toast无效（列名称）
    //$(".actionSheet_wrap").fadeOut(100);
    var col_name = $(obj).html();
    var table=mcdom.getObjByTagName(obj,"TABLE");
    var th=mcdom.getObjByTagName(obj,"TH");

    var cellIndex = obj.parentNode.cellIndex;
    var tableid=table.id;
    var mcsstable=mcsstable_getMCSSTable(tableid);

    if (mcsstable.view!="celledit" && mcsstable.view!="tableedit") return;
    clearSelStyle(mcsstable);
    th.style.backgroundColor='rgb(186, 225, 245)';
    var current_col = $(table).find("th").eq(cellIndex).get(0);
    var fieldid=$(current_col).attr('fieldid');
    if(is_share && is_share=='share'){
        ding_sortColumn(fieldid,tableid);
        return;
    }

    if (document.body && document.documentElement)
        var height3=document.body.clientHeight>document.documentElement.clientHeight?document.body.clientHeight:document.documentElement.clientHeight;
    else
        height3=document.documentElement.clientHeight;
    var current_link=window.location.href;
    if(current_link.indexOf("/")>-1){
        var current_link=current_link.split("/");
        var m_index=current_link.length-1;
        if(current_link[m_index]=='share')
            return;
    }

    var menu_htm="";
    var isadmin = mcsstable.params.model_ownerid==loginuser;
    
    if (isadmin){
        menu_htm+="<div class='weui_actionsheet_cell ding_rename'>重命名</div>";
        menu_htm+="<div class='weui_actionsheet_cell ding_insert'>添加列</div>";
        menu_htm+="<div class='weui_actionsheet_cell ding_delete'>删除列</div>";
    }
    
    menu_htm+="<div class='weui_actionsheet_cell ding_sortasc' >升序</div>";
    menu_htm+="<div class='weui_actionsheet_cell ding_sortdesc' >降序</div>";
    
    if (isadmin) menu_htm+="<div class='weui_actionsheet_cell ding_property'>列设置</div>";
    
    menu_htm+="<div class='weui_actionsheet_action'></div>";
    //menu_htm+="<div class='weui_actionsheet_cell' id='actionsheet_cancel' onclick='actionsheetfadeout()'>取消</div>";
    //$(" #new_create_table").css("z-index","1");
    //$('body').append(menu_htm);
    
    dom_showPopupMenu(menu_htm,obj,{position:"bottom"});
    
    
    localStorage.setItem("modeldata_"+mcsstable.params.usermodel_recordid+ '_' + mcsstable.params.view_id , "");
    
    $(".ding_b_option,.ding-cover").click(function(){
        $('.ding_b_menu').remove();
        $(".ding-cover").remove();
    });
    $(".ding_rename").click(function(){
        ding_renameColumn(fieldid,tableid,col_name,obj);
        $.post(g_homeurl+"/Oa/Project/addlog",{log:"app_log_unitable_renameCol"},null);
    });
    $(".ding_insert").click(function(){
        $.post(g_homeurl+"/Oa/Project/addlog",{log:"app_log_unitable_addCol"},null);
        ding_insertColumn(fieldid,tableid);
    });
    $(".ding_delete").click(function(){
        $.post(g_homeurl+"/Oa/Project/addlog",{log:"app_log_unitable_deleteCol"},null);
        tableset_deleteColumn(fieldid,tableid);
    });
    $(".ding_sortasc").click(function(){
        tableset_setFieldOrder(fieldid,tableid,"asc");
    });
    $(".ding_sortdesc").click(function(){
        tableset_setFieldOrder(fieldid,tableid,"desc");
    });
    $(".ding_property").click(function(){
        $.post(g_homeurl+"/Oa/Project/addlog",{log:"app_log_unitable_setCol"},null);
        ding_talbeset_setColumnProperty(fieldid,tableid);
    });
}


function ding_renameColumn(fieldid,tableid,col_name,obj){
    var label_style=" style='font-size: 16px;' ";
    var input_style=" style='height: 34px;' ";
    var hr_style=" style=' margin: 5px 0;' ";
    var btn_style="padding: 5px;text-align: center;width: 115px;position: relative;display: inline-block;font-size: 16px;";

    var btnid='popupconfirmbottom'+randomString(5);
    
    
    function fun_renamecol(table_name){
    if(table_name){
        var t_title = "";
        t_title =table_name;
        var mcsstable = mcsstable_getMCSSTable(tableid);
        var modelid = mcsstable.params.modelid;
        var usermodel_recordid = mcsstable.params.usermodel_recordid;

        $.post(g_homeurl+"/System/ModelMaker/setPropertyOfField",{fieldid:fieldid,modelid:modelid,usermodel_recordid:mcsstable.params.usermodel_recordid,fieldname:t_title},function(data) {
            mcsstable.refreshModelData();
            $(obj).html(t_title);
        });
    }
    }
    //var table_name=prompt("请为该列输入一个新名称");
    
    mcdom.input(fun_renamecol,col_name,"请为该列输入一个新名称");
  

}
function ding_insertColumn(fieldid,tableid){
    
    function fun_insertnewcol(column_name){
        if(column_name){
            var t_title = "";
            t_title = column_name;
            var mcsstable = mcsstable_getMCSSTable(tableid);
            var modelid = mcsstable.params.modelid;
            var usermodel_recordid = mcsstable.params.usermodel_recordid;
            var param = {mcsstable:mcsstable,fieldid:fieldid,modelid:modelid,usermodel_recordid:usermodel_recordid,name:t_title,position:'left'};
            tableset_quickInsertColumn(param);
    
        }        
    }
    mcdom.input(fun_insertnewcol,"新建列","请为新插入的列起一个名字");

    //actionsheetfadeout();

}
function ding_talbeset_setColumnProperty(fieldid,tableid){
    var mcsstable = mcsstable_getMCSSTable(tableid);
    var modelid = mcsstable.params.modelid;
    var usermodel_recordid = mcsstable.params.usermodel_recordid;
    var param = {mcsstable:mcsstable,fieldid:fieldid,modelid:modelid,usermodel_recordid:usermodel_recordid};

    var th = $("th[fieldid='"+fieldid+"']").get(0);
    var table=mcdom.getObjByTagName(th,"TABLE");
    var cur_field = mcsstable.modelDealer.allField[fieldid];
    

    localStorage.setItem("col_set_fieldid",param.fieldid);
    localStorage.setItem("col_set_modelid",param.modelid);
    localStorage.setItem("col_set_usermodel_recordid",param.usermodel_recordid);
    localStorage.setItem("col_set_position",param.position);
    localStorage.setItem("col_set_type","update");
    localStorage.setItem("col_set_cur_field",JSON.stringify(cur_field));
    uexWindow.open('col_set', '0', 'col_set.html', '5', '', '', 0, 1000);
}


//保存拖动的列
function mcsstable_saveColumnDisplay(mcsstable){
    var usermodel_recordid = mcsstable.params.usermodel_recordid;
    var modelid = mcsstable.params.modelid;
    var url = g_homeurl+"/System/ModelMaker/setColumnWidth";
    var column_order = "";
    var arr = $(mcsstable.table).find("th[fieldid!='']");
    for(var i = 0;i < arr.length;i++){
        if(!$(arr[i]).attr("fieldid"))
            continue;
        if(column_order)
            column_order+=',';
        column_order+=$(arr[i]).attr("fieldid");
    }
    if (mcsstable.view=='tableedit' || mcsstable.view=='celledit')
    {
        $.post(url,{modelid:modelid,usermodel_recordid:usermodel_recordid,order:column_order},function(data){
            //mcdom.alert('保存成功','提示','info','fadeout');
            mcsstable.modelDealer.swapField(column_order);
            $(mcsstable.getAction('save')).css('border-color','');

        })
    }
}
//拖动时生成临时列
function mcsstable_createTempMoveTd(column,table){
    var cellIndex = $(column).get(0).cellIndex;
    var rows = $(table.mcsstable.tableData).find("tr");
    var html="<th istemp='true' style='width:60px;'>Temp</th>";
    $(table.mcsstable.tableFixRow.rows[0]).find("th").eq(cellIndex).after(html);
    for(var i = 1;i < rows.length;i++){
        html = "<td istemp='true' style='width:60px;'>Temp</td>";
        $(rows[i]).find("td").eq(cellIndex).after(html);
    }
}
//交换节点
function mcsstable_swapNode(node1,node2){
    //获取父结点
    var _parent=node1.parentNode;
    //获取两个结点的相对位置
    var _t1=node1.nextSibling;
    var _t2=node2.nextSibling;
    //将node2插入到原来node1的位置
    if(_t1)_parent.insertBefore(node2,_t1);
    else _parent.appendChild(node2);
    //将node1插入到原来node2的位置
    if(_t2)_parent.insertBefore(node1,_t2);
    else _parent.appendChild(node1);
}
//加载列表视图
function mcsstable_createSysViewList(mcsstable)
{
    var shareviews=mcsstable.getShareViews();
    var h="";

    h+="<optgroup label='常用视图' id='"+mcsstable.tableid+"_view'>";
    if (!shareviews || shareviews && shareviews.indexOf(",SHOW_EDIT_TODAY,")>-1)
    {
        h+="<option value='SHOW_EDIT_TODAY' tag='USEROPTION' ";
        if (mcsstable.view_id=="SHOW_EDIT_TODAY")
            h+=" selected='true'";
        h+=">今天的数据</option>";
    }
    if (!shareviews || shareviews && shareviews.indexOf(",SHOW_EDIT_YESTODAY,")>-1)
    {
        h+="<option value='SHOW_EDIT_YESTODAY' tag='USEROPTION' ";
        if (mcsstable.view_id=="SHOW_EDIT_YESTODAY")
            h+=" selected='true'";
        h+=">昨天的数据</option>";
    }
    /*
     if (!shareviews || shareviews && shareviews.indexOf(",SHOW_CREATE_MY,")>-1)
     {
     h+="<option value='SHOW_CREATE_MY' tag='USEROPTION' ";
     if (mcsstable.view_id=="SHOW_CREATE_MY")
     h+=" selected='true'";
     h+=">"+lang.mycreatedrecord+"</option>";
     }*/

    if (!shareviews || shareviews && shareviews.indexOf(",SHOW_UPDATE_MY,")>-1)
    {
        h+="<option value='SHOW_UPDATE_MY' tag='USEROPTION' ";
        if (mcsstable.view_id=="SHOW_UPDATE_MY")
            h+=" selected='true'";
        h+=">我的数据</option>";
    }

    h+="</optgroup>";
    $(mcsstable.getAction('view')).append(h).change(function(){
        if (mcsstable.mcssData.state=='CHANGED'){
            mcdom.alert(lang.pleasesaveatfirst,'','','fadeout','','',{seconds:5});
            this.value='';
            return;
        }

        var v=this.value,filter = "",user = getCookie("mcss_loginuser"),today = uni_global.today;

        var dd = new Date();
        dd.setDate(dd.getDate()-1);//获取AddDayCount天后的日期
        var y = dd.getFullYear();
        var m = dd.getMonth()+1;//获取当前月份的日期
        if (m<10) m ="0"+"m";
        var d = dd.getDate();
        if (d<10) d="0"+d;

        var yestoday = y+"-"+m+"-"+d;

        if (v=='SHOW_CREATE_MY'){
            filter = "SYS_ADDUSER = '"+user+"'";
        }else if (v=='SHOW_EDIT_NEW'){
            filter = "SYS_EDITTIME > '"+mcsstable.lastReadtime+"'";
        }else if (v=='SHOW_UPDATE_MY'){
            filter = "SYS_EDITUSER = '"+user+"'";
        }else if (v=='SHOW_CREATE_TODAY'){
            filter = "SYS_ADDUSER = '"+user+"' and to_days(SYS_EDITTIME) = to_days('"+today+"')";
        }else if (v=='SHOW_UPDATE_TODAY'){
            filter = "SYS_EDITUSER = '"+user+"' and to_days(SYS_EDITTIME) = to_days('"+today+"')";
        }else if (v=='SHOW_EDIT_TODAY'){
            filter = "left(SYS_EDITTIME,10) = '"+today+"'";
        }else if (v=='SHOW_EDIT_YESTODAY'){
            filter = "left(SYS_EDITTIME,10) = '"+yestoday+"'";
        }else if (v=='SHOW_MYUPDATE_YESTODAY'){
            filter = "left(SYS_EDITTIME,10) = '"+yestoday+"' and SYS_EDITUSER = '"+user+"'";
        }else{
            return;
        }

        mcsstable.advancedFilter = "";
        mcsstable.autoFilter = filter;//"";
        mcsstable_loaddatarows(mcsstable.tableid);
    });

}

//单击列标题排序
function mcsstable_changeFieldOrder(obj,tableid)
{
    var mcsstable=mcsstable_getMCSSTable(tableid);
    $(obj).parent().children().attr("class","order_default");
    var updown='';
    if($(obj).data("orderdirection")=="desc")
    {
        $(obj).data("orderdirection","asc");
        $(obj).attr("class","order_desc");
        updown="desc";
    }
    else
    {
        $(obj).data("orderdirection","desc");
        $(obj).attr("class","order_asc")
        updown="asc";
    }
    mcsstable.setFieldOrder($(obj).attr('fieldid'),updown);
    mcsstable_loaddatarows(tableid);
}

//修改分组的排序方向，倒序还是正序。注意不是单击标题时修改排序
//this.group_order_updown:分组排序方向，asc或desc
function mcsstable_setOrderDirection(tableid,e){
    var mcsstable=mcsstable_getMCSSTable(tableid);
    if (mcsstable.ifDataChanged()){
        mcdom.alert(lang.pleasesaveatfirst,'','','fadeout','','',{seconds:5});
        this.value='';
        return;
    }

    if(mcsstable.groupFieldid){
        if(!mcsstable.group_order_updown || mcsstable.group_order_updown=='asc')
        {
            mcsstable.group_order_updown='desc';
            $(e).removeClass('icon-arrow-up').addClass('icon-arrow-down');
            $(e).attr('title','降序');
        }
        else{
            mcsstable.group_order_updown='asc';
            $(e).removeClass('icon-arrow-down').addClass('icon-arrow-up');
            $(e).attr('title','升序');
        }
        mcsstable_loaddatarows(tableid);
    }
    else
        mcdom.popup(lang.pleaseselectfield,{width:150,obj:e,fadeOut:true,seconds:3000});
}

//重新加载表格的模型和数据
function mcsstable_reload(tableid)
{
    var mcsstable=mcsstable_getMCSSTable(tableid);
    mcsstable.currentpage=1;
    mcsstable.tableData.parentNode.parentNode.scrollTop=0;
    mcsstable.tableData.parentNode.parentNode.scrollLeft=0;
    mcsstable.last_groupfield_value="[-fdsafdsafgyewyy532435dfsgfdg-]";//上一行分组字段的值

    mcsstable_loaddatarows(tableid);
}

function mcsstable_expandAllGroup(e)
{
    var o=$("#a_mcsstable_tablesetting").get(0);
    var table=mcdom.getObjByTagName(o,"TABLE");
    var table1=table.mcsstable.tableFixCol;
    var table2=table.mcsstable.tableData;
    if ($("#a_mcsstable_tablesetting").attr("groupvisible")=="NO")
    {
        $(table1).find("tr").show();
        $(table2).find("tr").show();
        $("#a_mcsstable_tablesetting").attr("groupvisible","YES");
    }
    else
    {
        $(table1).find("tr").hide();
        $(table1).find(".rowgroup").show();

        $(table2).find("tr").hide();
        $(table2).find(".rowgroup").show();

        $("#a_mcsstable_tablesetting").attr("groupvisible","NO");
    }
}
function mcsstable_getChangedDataInfo(mcsstable,callback){
    var url=g_homeurl+"/System/Model/getDataChangeInfo";
    $.getJSON(url,{projectid:mcsstable.params.usermodel_recordid},function(data){
        if (!data) {
            if (callback) callback(null);
            return;
        }
        if (data.rowCount==0) {
            if (callback) callback(null);
            return;
        }
        if (data.linkCount>0){
            mcsstable.params.shareLinkCount=data.linkCount;
            //mcsstable_refreshTeamInfo(mcsstable);
        }

        var msg={};
        var lastReadtime=data.lastReadtime;
        mcsstable.lastReadtime=data.lastReadtime;

        msg.message="自您上次查阅以后，一共有"+data.rowCount+"条数据更新，您想查看这些新数据吗？";
        window.currentTable=mcsstable;
        msg.command="mcsstable_loadChangedData(window.currentTable,'"+ lastReadtime +"');";
        msg.title="智能提醒";
        msg.SYS_ADDUSER="{SheetMsg}";
        msg.position="#mainsheet_table_action_refresh";

        if (callback) callback(msg);
    });
}
function mcsstable_refreshTeamInfo(mcsstable){
    if (!mcsstable) return;
    if (getCookie('mcsstable_team_panel_status')=="close") return;
    var ulist=$("#mcss-team-users")[0];
    if (ulist.childNodes.length<2) return;

    var url=g_homeurl+"/System/Model/getTeamMemberDataInfo";
    $.getJSON(url,{projectid:mcsstable.params.usermodel_recordid,filter:mcsstable.lastfilter},function(data){


        for(var j=0;j<ulist.childNodes.length;j++){
            var div = ulist.childNodes[j];
            var bfind=false;
            for(var i=0;i<data.length;i++){

                var user=data[i];

                if (div.childNodes[1].innerHTML == user.name && user.name!=""){

                    if (user.rowCount>0){
                        bfind=true;
                        div.firstChild.firstChild.style.color="#44f";
                        div.firstChild.style.color="#44f";

                        if (div.firstChild.childNodes.length>1){
                            var sp=div.firstChild.childNodes[1];
                        }
                        else{
                            var sp=document.createElement("span");
                            div.firstChild.appendChild(sp);
                        }

                        if (user.rowCount<10)
                            sp.innerHTML="("+ user.rowCount +")";
                        else
                            sp.innerHTML="(*)";

                        div.title= div.childNodes[2].innerHTML + "填写了" + user.rowCount + "条数据。";
                    }

                    break;
                }

            }
            if (bfind==false){
                if (div.firstChild.childNodes.length>0){
                    div.firstChild.firstChild.style.color="#999";
                    div.firstChild.style.color="#999";
                    div.title= div.childNodes[2].innerHTML + "还没有填写任何数据。";
                    if (div.firstChild.childNodes.length>1) div.firstChild.childNodes[1].innerHTML="";
                }
            }
        }
    });
}
function mcsstable_loadChangedData(mcsstable,ltime)
{
    mcsstable.autoFilter=" SYS_EDITTIME>'"+ltime+"' or SYS_ADDTIME>'"+ltime+"'";
    mcsstable_loaddatarows(mcsstable.tableid);
    //mcsstable.advancedFilter="";
}
//创建表体
function mcsstable_createTable(modeldata,tableid) {
    if (!modeldata)	return;
    var mcsstable=mcsstable_getMCSSTable(tableid);
    mcsstable.loaderror=true;
    mcsstable.createHeader(modeldata);
    if(mcsstable.params.autoAddEmptyRow==true)
        mcsstable_addEmptyrow(mcsstable.table.id,30);
    
    mcsstable_setIconField(mcsstable);
    mcsstable.view_searchword="";
    window.setTimeout(function(){
        mcsstable_loaddatarows(tableid); //加载记录数
    },100);
    mcsstable.setStyleAndToolbar(modeldata);

    /*
     var h="<div id='"+tableid+"_bottom' class='mcsstable_bottom'>";
     if (mcsstable.params.pageposition=='rightdown' || mcsstable.params.pageposition=='leftdown')
     h+=mcsstable_page(mcsstable.params.tableid);
     if (mcsstable.params.showAddBtnAtBottom && (mcsstable.view=="tableedit" || mcsstable.view=="celledit"))
     {

     h += "<div class='mcsstable-appendrow'>"
     +"<a id='"+mcsstable.getActionId('appendrow')+"' onclick=\"mcsstable_addOneRowToBottom(this,'"+tableid+"') \" class='btn btn-default btn-sm' title='快捷键：ctrl+g'>添加新行</a>"
     +"<a id='"+mcsstable.getActionId('appendrow')+"' onclick=\"mcsstable_addSomeRowsToBottomMenu(this,'"+tableid+"') \" class='btn btn-default btn-sm' title='一次性添加多行'>加5行</a>"
     +"</div>"
     }
     h+="</div>";
     $("#"+tableid+"_bottom").remove();
     $("#"+tableid).after(h);
     $("#" +tableid+"_pagerows").val(mcsstable.modeldata.pagerows);
     if (uni_global.isMobile())//手机屏幕
     $("#"+mcsstable.pagebarid).css("float","left");

     mcsstable_setShortcutKey(mcsstable);
     */
    //创建主页右侧面板服务
    if (mcsstable.params.afterCreateTable){
        mcsstable.params.afterCreateTable(mcsstable);
    }

    if (window.sysMessage){
        msg=window.sysMessage;
        if (msg.projectid==mcsstable.params.usermodel_recordid){
            window.mcsstable=mcsstable;
            showSysMessage(msg);
        }
    }
    if (mcsstable.runWhenFinished)
        mcsstable.runWhenFinished(mcsstable);
}


//设置保存快捷键
function mcsstable_setShortcutKey(mcsstable)
{
    $(document).unbind("keydown");

    $(document).keydown(function(event){
        if(event.keyCode==116){
            $(mcsstable.getAction("refresh")).click();
            return false;
        }
        //ctrl+s
        if (event.ctrlKey == true && event.keyCode == 83) {
            $(mcsstable.cellEditor.currentEditor).change();
            $(mcsstable.getAction('save')).click();
            return false;
        }

        // if(event.shiftKey == true && event.keyCode==40)
        // alert(event.keyCode);
        //ctrl+x
        if(event.ctrlKey == true && event.keyCode==88){

            //$("#"+mcsstable.table.id+"_tableheader").attr("cuttype","cut");
            mcsstable.cuttype="cut";
            mcsstable_setCopyBorder(mcsstable);
        }
        //ctrl+a
        if(event.ctrlKey == true && event.keyCode==65){
            mcsstable_selectAllRows(mcsstable.tableFixRow.rows[0].cells[0],mcsstable.tableid);
            return false;
        }
        //ctrl+z
        if((event.ctrlKey == true || mcsstable.table.macCtrl==true) && (!event.shiftKey && !mcsstable.table.macShift) && event.keyCode==90){
            mcsstable.undo();
            return false;
        }
        //ctrl+r
        if((event.ctrlKey == true || mcsstable.table.macCtrl==true) && event.keyCode==82){
            mcsstable.redo();
            return false;
        }
        //ctrl+shift+z
        if((event.ctrlKey == true || mcsstable.table.macCtrl==true) && (event.shiftKey || mcsstable.table.macShift) && event.keyCode==90){
            mcsstable.redo();
            return false;
        }
        //ctrl+i
        if ( (event.ctrlKey == true || mcsstable.macCtrl )&& event.keyCode == 73 ) {

            mcsstable_addnew(mcsstable.table.rows[0].cells[0],mcsstable.tableid,mcsstable.getAddUrl(),lang.add);
            return false;
        }
        //ctrl+d
        if ((event.ctrlKey == true && event.keyCode == 68) || event.keyCode==46 ){
            if (mcsstable.selectedRowsID){
                mcsstable_deleterows("",mcsstable.tableid)
            }
            else if (mcsstable.currentArea)
            {
                if (mcsstable.currentArea.beginRowIndex!=mcsstable.currentArea.endRowIndex || mcsstable.beginCellIndex!=mcsstable.endCellIndex){
                    mcsstable_DeleteArea(mcsstable);
                }
            }
            else{
                mcdom.popup("请选中要删除的记录",{fadeOut:true});
            }
            return false;
        }
    })
}
function mcsstable_DeleteArea(mcsstable){
    mcsstable_clearSelectData(mcsstable);
}
//用户回车保存修改名称

function mcsstable_editFieldName(event,tableid,obj)
{
    var mcsstable = mcsstable_getMCSSTable(tableid);
    var user = getCookie("mcss_loginuser");
    if(user!=mcsstable.params.project_fulldata.project_record.SYS_ADDUSER)
        return;
    var div = $(obj).find("div[class='mcsstable_colname']");
    if(event.keyCode=='13')
    {
        mcsstable_saveFieldName(div,tableid);
    }
}
//焦点消失的时候进行保存
function mcsstable_saveFieldName(div,tableid){
    div.title=div.innerText;
    tableset_renameField(tableid,div);
}


//用户点击是将列名变成可修改的方法

function mcsstable_editColumnTitle(obj,tableid)
{
    var mcsstable = mcsstable_getMCSSTable(tableid);
    if (!mcsstable.params.project_fulldata)
        return;

    if (mcsstable.divSel){
        mcsstable.divSel.style.display="none";
        mcsstable.divSel.divFill.style.display="none";
    }

    if(loginuser!=mcsstable.params.project_fulldata.project_record.SYS_ADDUSER)
    {
        $(obj).attr('contentEditable','false');
        //mcdom.alert("非项目创建者不允许修改结构",'提示','info','fadeout');
        return;
    }
    $(obj).attr('contentEditable','true');
    $(obj).focus();

    var title=obj.innerText;
    var req = /第.*?列/;

    if (title=="") return;

    if (document.selection) {
        var range = document.body.createTextRange();
        range.moveToElementText(obj.firstChild);
        range.select();
    } else if (window.getSelection) {
        var range = document.createRange();
        range.selectNode(obj.firstChild);
        window.getSelection().removeAllRanges();
        window.getSelection().addRange(range);
    }

}

//如果鼠标放到字段头上超过2秒，且字段是隐藏的（宽度小于5），则显示该字段
//var mouseonheader_start;
function mcsstable_headeronmouseover(e)
{
//	if (!mouseonheader_start)
//		 mouseonheader_start=1;
    if (parseFloat(e.style.width)<5)
        e.style.width='50.678px';

}
function mcsstable_headeronmouseout(e)
{
    if (e.style.width=='50.678px')
        e.style.width='3px';
}

//是否显示最后一栏，把行操作按钮放在最后一个单元格。现在很少用到
function ifShowLastTd(mcsstable)
{
    var params=mcsstable.params;
    var toolbar=','+params.toolbar+',';
    if  (!params.hideLastTd //既然指示隐藏，当然就不要显示了
        && //下面的指示表示需要操作栏
        (
            params.style=="simple"
            || params.showRecordActionAtLast
            || mcsstable.last_td_func
        )
        && //下面的条件表示操作栏中至少要有一样东西。
        (
            params.userAction || mcsstable.last_td_func || toolbar.indexOf(',open,') > -1 || toolbar.indexOf(',edit,') > -1 || toolbar.indexOf(',del,') > -1
        )
    )
        return true;
    return false;
}

//显示第一页数据
function mcsstable_loadfirstrow(tableid) {
    var mcsstable=mcsstable_getMCSSTable(tableid);
    mcsstable.currentpage=1;
    mcsstable_loaddatarows(tableid);
}

// 根据给你的数据组或从后台取得当前页的数据，并加载到表格中
function mcsstable_loaddatarows(tableid,fromRow) {

    var mcsstable=mcsstable_getMCSSTable(tableid);
    if (!mcsstable)	return;
    if (!fromRow ) fromRow=0;
    

    var params=mcsstable.params;
    var fields=mcsstable.fields;
    var modeldata=mcsstable.modeldata;
    var first_td_func=mcsstable.first_td_func;
    var last_td_func=mcsstable.last_td_func;
    mcsstable.last_groupfield_value="[-fdsafdsafgdh345454325fgdsgfdsgfdgfds-]"

    $(mcsstable.tableData).find("tr").remove();
    $(mcsstable.tableFixCol).find("tr").remove();

    var pagerows=30;
    if (params.pagerows)
    {
        if (params.pagerows<9){
            pagerows=params.pagerows;
        }else{
            if(params.pagerows>150)
                pagerows = 150;
        }
    }
    else
    {
        //通过表格的实际大小来判断应该加载的行数
        var pagerows= parseInt( mcsstable.table.offsetHeight / 25 ) + 1;
        if (pagerows<20)
            pagerows=50;

    }
    mcsstable.pagerows=pagerows;
    mcsstable.params.rowCount = params.pagerows;

    if (mcsstable.table.rows.length>0)
    {
        //$(mcsstable.table.rows[0]).after("<tr id='loading'><td colspan='"+mcsstable.table.rows[0].cells.length+"'>加载中...</td></tr>");
        mcsstable.table.colCount = fields.length+3;
        if(mcsstable.params.autoAddEmptyRow==true)
            mcsstable_addEmptyrow(mcsstable.table.id,mcsstable.pagerows);
    }

    mcsstable.params.fromRow = 0;
    mcsstable.table.parentNode.parentNode.table=mcsstable.table;

    if (params.data)//如果有给定的数据，就不从后台取
    {
        mcsstable.pagerows = params.data.length;
        mcsstable.params.rowCount = params.data.length;
        //对从参数传进来的数据，其数量可能超过模型设置的每页记录数，因此也要处理下
        var temprows=new Array();
        for(var i=(mcsstable.currentpage-1)*mcsstable.pagerows;i<(mcsstable.currentpage)*mcsstable.pagerows;i++)
        {
            if (params.data[i])
                temprows.push(params.data[i]);
        }

        mcsstable_calculateTotalRecordCount(mcsstable,params.data.length);
        mcsstable_loadRows(mcsstable,temprows);

        mcsstable.table.parentNode.parentNode.style.width=mcsstable.tablecontainer.clientWidth+"px";;
        mcsstable.table.parentNode.parentNode.style.height=mcsstable.tablecontainer.clientHeight+3+"px";
    }
    else
    {
        
        window.setTimeout(function(){
            if (mcsstable.loaderror==false) return;
            mcdom.confirm("由于网络原因，数据加载失败，点击确定重试？",function(b){
                if (b) mcsstable_loaddatarows(tableid,fromRow);
            });
        },15000);
        var sql="";
        //如果mcsstable类接收了sql参数，则把该参数传递给后台，以代替模型的sql属性。这样便于程序员在前台灵活组件sql语句，但要注意sql的字段要与模型字段列表匹配
        mcsstable.setOrder();
        if (params.sql)
            sql=params.sql;
        var searchword=mcsstable.searchword;
        if (mcsstable.view_searchword && searchword)
            searchword+="[AND]"+mcsstable.view_searchword;
        else
        if (mcsstable.view_searchword && searchword=="")
            searchword+=mcsstable.view_searchword;
        mcsstable.searchword=searchword;

        var lastfilter=mcsstable_getLastFilter(mcsstable);
        mcsstable.lastfilter=lastfilter;
        var p={modelid:mcsstable.modelid,filter:lastfilter,
            orderby:mcsstable.order.trim(),fromRow:fromRow,rowCount:params.rowCount,
            page:mcsstable.currentpage,pagerows:mcsstable.pagerows,view_id:mcsstable.view_id,
            searchword:searchword,sql:sql,getRecordCount:mcsstable.params.showRecordCount,
            usermodel_recordid:mcsstable.params.usermodel_recordid,
            model_ownerid:mcsstable.params.model_ownerid,version_id:mcsstable.params.version_id};
        mcsstable.getRecordsAndDo(p,function(mcsstable,data){
            
            mcsstable.loaderror=false;
            mcsstable_loadDataToRows(mcsstable,data);

            //移动到这里来否则视图中看不到合计行
            if (mcsstable.hasSummaryField)
            {
                var rowtol = mcsstable.table.insertRow(-1);
                rowtol.style.height="30px";
                var cellFix=rowtol.insertCell(-1);
                var cellData=rowtol.insertCell(-1);
                cellFix.style.border="none";
                cellData.style.border="none";

                createPageTotalInfo(mcsstable);
            }
            mcsstable.scale();
        });
    }

}

//ckj
function mcsstable_loadDataToRows(mcsstable,data)
{
    
    mcsstable_calculateTotalRecordCount(mcsstable,data.totalRecordCount);
    //add by sunliangchao 增加对表格开始行结束行、总行数的处理
    mcsstable.params.maxSort = data.maxSort;
    mcsstable.params.totalRecordCount = data.totalRecordCount;
    if (mcsstable.pagerows < 10) mcsstable.params.totalRecordCount = mcsstable.pagerows
    mcsstable.table.fromRow=0;
    mcsstable.tableData.fromRow=0;
    mcsstable_loadRows(mcsstable,data.rows);
    mcsstable.table.rowCount=data.rows.length;
    mcsstable.table.toRow=data.rows.length;

    //计算前20行的平均高度，以此高度乘以总行数作为理论高度
    var rows=mcsstable.table.rows;
    mcsstable.tableData.rowHeight=parseInt((mcsstable.tableData.offsetHeight ) / (mcsstable.tableData.rows.length) + 0.5 );
    if (mcsstable.hasImageCol) {
        if (mcsstable.tableData.rowHeight<120){
            mcsstable.tableData.rowHeight=120;
            mcsstable.tableFixCol.rowHeight=120;
        }
    }
    mcsstable.params.rowCount = parseInt(1500/mcsstable.tableData.rowHeight);
    if (rows[rows.length-1])
        mcsstable.table.colCount = rows[0].cells.length;
    mcsstable.tableData.dataRowCount = mcsstable.tableData.rows.length;
    //这里取消了增行按钮，在最末端添加5个空白行。
    var _total_count = parseInt (mcsstable.params.totalRecordCount);

    if (!(mcsstable.params.canAppendRow && (mcsstable.view=="tableedit" || mcsstable.view=="celledit")))
    {
        //删除空行的代码
        for(var i=mcsstable.tableData.rows.length-1;i>0;i--){
            var tr=mcsstable.tableData.rows[i];
            if (!tr.getAttribute("recordid") && !tr.getAttribute("groupname")){
                mcsstable.tableData.deleteRow(i);
                mcsstable.tableFixCol.deleteRow(i);
            }
        }
    }

    //if (mcsstable.params.tableLayout)
    //mcsstable.table.style.tableLayout=mcsstable.params.tableLayout;//add by sunliangchao 移动位置，否则safri下会有问题。

    var tableHeight = mcsstable.tableData.offsetHeight;

    var he=mcsstable.tableData.rowHeight*(_total_count+5) ;
    if (he<tableHeight) he=tableHeight;
    if (tableHeight < mcsstable.tableData.parentNode.parentNode.offsetHeight)
        he=tableHeight;

    mcsstable.tableData.parentNode.style.height= he +"px";
    //mcsstable.tableData.parentNode.parentNode.style.width=mcsstable.tablecontainer.clientWidth - mcsstable.tableFixCol.offsetWidth - 2 +"px";
    mcsstable.tableData.parentNode.parentNode.parentNode.style.width=mcsstable.tableData.parentNode.parentNode.style.width;
    //mcsstable.tableData.parentNode.style.overflow="hidden";//ckj
    //mcsstable.tableData.parentNode.style.overflowX="hidden";


    mcsstable.divSelParent.style.left="0px";
    mcsstable.divSelParent.style.top="0px";
    mcsstable.divSelParent.style.width= mcsstable.table.parentNode.style.width;
    mcsstable.divSelParent.style.height=mcsstable.table.parentNode.style.height;
    mcsstable.divSelParent.style.position="relative";
    //mcsstable.divSelParent.style.border="1px solid green";

    mcsstable.divSelBack.style.left=mcsstable.table.parentNode.offsetLeft + "px";
    mcsstable.divSelBack.style.top=mcsstable.table.parentNode.offsetTop + "px";
    mcsstable.divSelBack.style.width= mcsstable.table.clientWidth+"px";
    mcsstable.divSelBack.style.height=mcsstable.table.clientHeight+"px";;
    //mcsstable.divSelParent.style.border="1px solid red";
    mcsstable.divSelBack.style.overflow="hidden";

    var divHeight=mcsstable.tablecontainer.clientHeight;
    var tbar = document.getElementById(mcsstable.table_toolbar_id);
    if(tbar!==null){
        if (tbar.parentNode == mcsstable.tablecontainer) divHeight = divHeight - tbar.clientHeight-2;
    }
    mcsstable.table.parentNode.parentNode.style.height= divHeight + "px";

    var panel=mcsstable.table.parentNode.parentNode;
    mcsstable.oldScrollTop =0;
    panel.beiginTop=0;
    mcsstable.table.scrollFrom={top:0,left:0};

    mcsstable.tablecontainer.style.overflow="hidden";
    // document.body.style.overflow="hidden";
}
function mcsstable_onFixTableScroll(event)
{
    return;
    var panel = event.srcElement ? event.srcElement : event.target;
    var tableid=panel.tableid;
    var mcsstable=mcsstable_getMCSSTable(tableid);
    var panelData=mcsstable.tableData.parentNode.parentNode;

    if (panelData.scrollTop==panel.scrollTop)
        return;
    else
        panelData.scrollTop=panel.scrollTop

}
//add by sunliangchao 把拖动滚动条的事件处理从加载主数据的函数中移动出来。
function mcsstable_onPanelScroll(event)
{

    var panel = event.srcElement ? event.srcElement : event.target;
    var tableid=panel.tableid;
    var mcsstable=panel.mcsstable;

    //统计视图不处理懒加载
    if(mcsstable.isStatView) return;

    mcsstable.tableFixRow.parentNode.parentNode.scrollLeft=panel.scrollLeft;
    if (mcsstable.hasSummaryField && mcsstable.tableTotal)
        mcsstable.tableTotal.style.marginLeft="-"+(panel.scrollLeft+1)+"px";

    //if (mcsstable.divSel)  mcsstable.divSel.style.display='none';
    //处理选择框的容器
    if (mcsstable.divSelParent)
    {
        mcsstable.divSelParent.style.marginTop = - panel.scrollTop + "px";
        mcsstable.divSelParent.style.marginLeft = - panel.scrollLeft + "px";
    }
    if (mcsstable.tableFixCol.parentNode.scrollTop == panel.scrollTop) return;

    mcsstable.tableFixCol.parentNode.scrollTop = panel.scrollTop;

    mcsstable_clearFloatToolBar();
    if (mcsstable.pagerows < 10) return;

    var scrollTop, maxScroll, minScroll = 0;
    scrollTop = panel.scrollTop;
    maxScroll = panel.scrollHeight - panel.offsetHeight;
    panel.percent=scrollTop / maxScroll;
    //处理合计表格
    if (mcsstable.divSum)
    {
        mcsstable.divSum.firstChild.style.marginLeft="-" + panel.scrollLeft + "px";
    }
    //正式处理表格拖动的逻辑
    mcsstable_onvScroll(mcsstable,panel.percent);

    //通过定时器模拟拖动停止后的事件
    if (panel.itval) clearTimeout(panel.itval);
    var sctop = panel.scrollTop;
    panel.itval=window.setTimeout(function(){
        //判断只有位置发生改变了的时候，才去加载数据
        if (sctop == panel.scrollTop && panel.scrollTop != panel.beiginTop )
        {
            if (panel.itval) {
                window.clearTimeout(panel.itval);
                panel.itval=false;
            }
            if (mcsstable.bLoadData){
                panel.beiginTop=panel.scrollTop;
                mcsstable_onvScrollEnd(mcsstable,panel.percent);
            }
        }

    },200);

}
//add by sunliangchao
//滚动过程中触发该方法
function mcsstable_onvScroll(mcsstable,percent)
{
    //保障该方法在执行过程中不会再次被调用执行
    if (mcsstable.isdraging==true) return;
    mcsstable.isdraging=true;

    if (mcsstable.tableData.rows.length>1)
        mcsstable.tableData.rowHeight=parseInt(mcsstable.tableData.offsetHeight / mcsstable.tableData.rows.length + 0.5 );
    //根据滚动条的位置计算从哪一行开始加载数据
    var fromrow= parseInt( percent * (mcsstable.params.totalRecordCount-10))-20;
    if (fromrow<0) fromrow=0;
    if (mcsstable.hasImageCol){
        if (mcsstable.tableData.rowHeight<120){
            mcsstable.tableData.rowHeight=120;
            mcsstable.tableFixCol.rowHeight=120;
        }
    }
    rowcount=mcsstable.params.rowCount;
    var rows=mcsstable.tableData.rows;

    //取得最末的数据行数
    if (!rows[rows.length-2]) return;
    var imax =  rows[rows.length-1].dataIndex;//这可能是一个合计行或分组行
    if (!imax) imax = rows[rows.length-2].dataIndex;



    var rowHeight = mcsstable.tableData.rowHeight;
    //假设表格中有数据，则通过在表格中动态增山行来实现移动的效果
    if (mcsstable.tableData.dataRowCount >0)
    {
        //根据以下四个位置高度来计算表格应该增加或删除多少行。
        //ttop 表格所在位置的高度，
        //vtop 当前视图滚动条所指定的高度位置
        //tbottom 表格的底边所在的位置高度
        //vbottom 当前视图的底边所指示的位置高度
        var ttop = parseInt(mcsstable.tableData.style.marginTop);
        if (!ttop) ttop=0;
        var vtop = mcsstable.tableData.parentNode.parentNode.scrollTop;
        var tbottom = ttop + parseInt(mcsstable.tableData.clientHeight);
        var vbottom = mcsstable.tableData.parentNode.parentNode.scrollTop + parseInt( mcsstable.tableData.parentNode.parentNode.clientHeight)

        if (mcsstable.oldScrollTop > vtop ){
            while( ttop > vtop - 50)
            {
                if (ttop >0 ){
                    //console.log("上");
                    if (mcsstable.tableData.fromRow<=0) break;
                    mcsstable.tableData.fromRow--;
                    ttop -= rowHeight;
                    mcsstable_insertEmptyrow(mcsstable.table,mcsstable.tableData.fromRow+1,mcsstable.tableData.fromRow+2);
                    //在上部插入行的时候，可以适当删除底部的数据行
                    if ( tbottom > vbottom + 100)
                    {
                        mcsstable.tableData.deleteRow(mcsstable.tableData.rows.length-1);
                        mcsstable.tableFixCol.deleteRow(mcsstable.tableFixCol.rows.length-1);
                        imax--;
                        tbottom -= rowHeight;
                        mcsstable.tableData.dataRowCount--;
                        mcsstable.bLoadData=true;
                        if (mcsstable.tableData.dataRowCount<=0) {
                            findData=false;
                            break;
                        }
                    }

                }else{
                    ttop = 0;
                    break;
                }
            }

            if (mcsstable.tableData.fromRow<=0)
            {
                ttop = 0;
                mcsstable.tableData.fromRow = 0;
                var rows = mcsstable.tableData.rows;
                rows[0].dataIndex = 1;
                var rowfs=mcsstable.tableFixCol.rows;
                rowfs[0].dataIndex=1;
            }

        }else if (mcsstable.oldScrollTop < vtop )
        {
            while ( tbottom < vbottom + 100)
            {

                if ((!mcsstable.params.autoAddEmptyRow || mcsstable.view=='table') && imax >= mcsstable.params.totalRecordCount) break;
                imax++;
                //超过50行空行以后就不会增加更多的空行了
                if (imax > parseInt(mcsstable.params.totalRecordCount)+30){
                    break;
                }
                mcsstable_appendEmptyrow(mcsstable.table,imax,imax+1);
                tbottom += rowHeight;
                //在下部增行以后，可以适当删除上面的行
                if (ttop <  vtop - 50)
                {
                    //console.log("下_delete" );
                    ttop = ttop + rowHeight;
                    mcsstable.tableData.deleteRow(0);
                    mcsstable.tableFixCol.deleteRow(0);
                    mcsstable.tableData.fromRow++;
                    mcsstable.tableData.dataRowCount--;
                    mcsstable.bLoadData=true;
                    if (mcsstable.tableData.dataRowCount<=0) {
                        break;
                    }
                }

            }
        }

        mcsstable.tableData.style.marginTop=ttop+"px";
        mcsstable.tableFixCol.style.marginTop=ttop+"px";
        mcsstable.tableFixCol.parentNode.scrollTop = mcsstable.tableData.parentNode.parentNode.scrollTop;

    }
    
    //假设表格中没有数据，则通过更改列头的序号，来伪移动表格。
    if (mcsstable.tableData.dataRowCount<=0)
    {

        mcsstable.bLoadData=true ;
        //console.log("空");
        fromrow= parseInt( mcsstable.tableData.parentNode.parentNode.scrollTop/ mcsstable.tableData.rowHeight); //parseInt( percent * (mcsstable.params.totalRecordCount))-20;
        if (fromrow<0) fromrow=0;
        var rows=mcsstable.tableFixCol.rows;
        for (var r = 0; r < rows.length; r++) {
            if (rows.length > r) {
                rows[r].dataIndex=r + 1 + fromrow;
                if (mcsstable.params.showLineNum) rows[r].cells[0].innerHTML = r + 1 + fromrow ;
            }

        }
        mcsstable.tableData.fromRow=fromrow;

        mcsstable.tableData.style.marginTop = mcsstable.tableData.parentNode.parentNode.scrollTop+"px"; //fromrow * mcsstable.tableData.rowHeight + "px";
        mcsstable.tableFixCol.style.marginTop=mcsstable.tableData.style.marginTop;
        mcsstable.tableFixCol.parentNode.scrollTop = mcsstable.tableData.parentNode.parentNode.scrollTop;
    }

    mcsstable.oldScrollTop = parseInt(mcsstable.tableData.parentNode.parentNode.scrollTop);
    mcsstable.isdraging=false;

}
//add by sunliangchao
//滚动条滚动结束后加载数据
function mcsstable_onvScrollEnd(mcsstable,percent)
{
    var panel=mcsstable.tableData.parentNode.parentNode;
    scrollTop=panel.scrollTop;
    if (mcsstable.tableData.scrollFrom && !mcsstable.undoscroll ){
        mcsstable.addUndoList(mcsstable,"ScrollTo",[mcsstable.tableData.scrollFrom.left,mcsstable.tableData.scrollFrom.top],mcsstable,"ScrollTo",[panel.scrollLeft,panel.scrollTop],false);
        mcsstable.tableData.scrollFrom=null;
    }
    var fromrow=mcsstable.tableData.fromRow;
    var rows=mcsstable.tableFixCol.rows;

    //从表格中找到那些行是空行
    var dbRowIndex=-1;   //数据库中对应的行号
    var tableRowIndex=0; //Table中对应的行号
    var loadRowCount=0;  //需要加载的行数

    for(var r=0;r<rows.length;r++)
    {
        if (!rows[r].id && !rows[r].getAttribute("groupname"))
        {
            if(dbRowIndex==-1)
            {
                dbRowIndex = rows[r].dataIndex -1 ;
                tableRowIndex=r;
            }
            loadRowCount=loadRowCount+1;
        }else{
            if (dbRowIndex>0)break;
        }
    }

    //以下代码从loadrows中复制得来
    var params=mcsstable.params;
    var fields=mcsstable.fields;
    var modeldata=mcsstable.modeldata;
    var first_td_func=mcsstable.first_td_func;
    var last_td_func=mcsstable.last_td_func;
    if (params.pagerows)
        mcsstable.pagerows=params.pagerows;
    else
    {
        if (mcsstable.params.usermodel_recordid)
        {
            var pagerows = getCookie(mcsstable.params.usermodel_recordid+"_pagerows");
            if(pagerows)
                mcsstable.pagerows = pagerows;
        }
    }

    mcsstable.params.fromRow = dbRowIndex;

    mcsstable.tableFixCol.parentNode.scrollTop = mcsstable.tableData.parentNode.parentNode.scrollTop;
    var tableid=mcsstable.tableid;
    var sql="";
    //如果mcsstable类接收了sql参数，则把该参数传递给后台，以代替模型的sql属性。这样便于程序员在前台灵活组件sql语句，但要注意sql的字段要与模型字段列表匹配
    //mcsstable.setOrder();
    if (params.sql)
        sql=params.sql;
    var searchword=mcsstable.searchword;
    if (mcsstable.view_searchword && searchword)
        searchword+="[AND]"+mcsstable.view_searchword;
    else
    if (mcsstable.view_searchword && searchword=="")
        searchword+=mcsstable.view_searchword;
    //alert(parseInt(mcsstable.table-container )/25*2)
    var lastfilter=mcsstable_getLastFilter(mcsstable);
    var p={modelid:mcsstable.modelid,filter:lastfilter,
        orderby:mcsstable.order.trim(),fromRow:dbRowIndex ,rowCount:loadRowCount,
        page:mcsstable.currentpage,pagerows:mcsstable.pagerows,view_id:mcsstable.view_id,
        searchword:searchword,sql:sql,getRecordCount:mcsstable.params.showRecordCount,
        usermodel_recordid:mcsstable.params.usermodel_recordid,model_ownerid:mcsstable.params.model_ownerid,version_id:mcsstable.params.version_id};

    mcsstable.getRecordsAndDo(p,function(mcsstable,data){
        if (scrollTop!=panel.scrollTop) return;
        if (rows[tableRowIndex].dataIndex != (dbRowIndex+1)) return;
        mcsstable.tableData.scrollFrom={left:panel.scrollLeft,top:panel.scrollTop};
        mcsstable.tableData.undoscroll=false;
        mcsstable_loadMoreData(mcsstable,data,dbRowIndex,tableRowIndex)
    });
}
//add by sunliangchao 把重新加载数据的方法从时间处理中拆分出来。
function mcsstable_loadMoreData(mcsstable,data,dbRowIndex,tableRowIndex)
{

    mcsstable.bLoadData=true;
    var r=0;
    var rows=mcsstable.tableData.rows;

    //把返回的数据填充到空行
    for(var i=0;i<data.rows.length;i++)
    {
        r=i+tableRowIndex;
        var s =mcsstable_addonerow(data.rows[i],mcsstable,i+dbRowIndex+1);
        if (rows.length>r) {
            var bfind=false;
            //如果加载的行已经存在，则不覆盖
            if (mcsstable.params.totalRecordCount<30){
                for(var k=1;k<rows.length;k++)
                {
                    if (data.rows[i].id+"" == rows[k].getAttribute("recordid")+"")
                    {
                        r=k;
                        bfind=true;
                        break;
                    }
                }
            }
            //如果是当前正在编辑的行，也不覆盖
            var curRowIndex=-1;
            if (mcsstable.currentRow)
                curRowIndex=mcsstable.currentRow.rowIndex ;
            if (r!=curRowIndex && bfind==false){
                rows[r].outerHTML=s;
                rows[r].dataIndex = i+dbRowIndex+1;
                var sfix=mcsstable_addfixrow(data.rows[i],mcsstable,i+dbRowIndex+1);
                mcsstable.tableFixCol.rows[r].outerHTML = sfix;
                mcsstable.tableFixCol.rows[r].style.height=rows[r].offsetHeight+"px";
            }

        }
    }
    //add by sunliangchao 16-04-11 
    //if (mcsstable.params.afterLoadRows)
    //    mcsstable.params.afterLoadRows(mcsstable);
    window.setTimeout(function(){
        mcsstable.fixRowHeight();
        mcsstable.tableFixCol.parentNode.scrollTop = mcsstable.tableData.parentNode.parentNode.scrollTop;
    },500);

    if (mcsstable.view=='tableedit'){
        mcsstable_tableEdit(tableid);
    }
    if (r<rows.length-1)
    {
        for(var i = rows.length-1; i> r ; i--)
        {
            if (!rows[i].id && !rows[r].getAttribute("groupname"))
            {
                if (mcsstable.params.autoAddEmptyRow==true)
                    mcsstable_bindMouseEventOnEmptyRow(mcsstable, rows[i]);
                else
                    mcsstable.tableData.deleteRow(i);
            }
        }
    }
    //移除缓存中不再需要的数据
    for(var i=mcsstable.mcssData.data.length-1;i>=0;i--)
    {
        var rid=mcsstable.mcssData.data[i][mcsstable.keyfield];
        var bfind=false;
        $.each(rows, function(j, row){
            if (row.id==rid) bfind=true;
        });
        if (bfind==false) {
            mcsstable.mcssData.unloadRow(rid);
        }
    }
    mcsstable.tableData.dataRowCount = mcsstable.tableData.rows.length ;

    //处理缓存在内存中的数据
    mcsstable.mcssData.loadMoreRows(data.rows);
    if (mcsstable.cellEditor)
        mcsstable.cellEditor.mcssData=mcsstable.mcssData;


    //处理分组行
    mcsstable_addGroupRow(mcsstable,data.rows,tableRowIndex);
    //add by sunliangchao
    //表格的高度是估算的，如果用户的行高非常不统一，则高度需要重算一下
    if (mcsstable.tableData.rows.length>1)
        mcsstable.tableData.rowHeight=parseInt(mcsstable.tableData.offsetHeight / mcsstable.tableData.rows.length + 0.5 );
    if (mcsstable.hasImageCol){
        if (mcsstable.tableData.rowHeight<120){
            mcsstable.tableData.rowHeight=120;
            mcsstable.tableFixCol.rowHeight=120;
        }
    }
    var vh=parseInt(mcsstable.tableData.rowHeight) * (parseInt(mcsstable.params.totalRecordCount)+5);
    tableHeight=mcsstable.tableData.offsetHeight;
    if (tableHeight < mcsstable.tableData.parentNode.parentNode.offsetHeight)
        vh=tableHeight;

    mcsstable.tableData.parentNode.style.height= vh + "px";

    //重新生成索引
    //mcsstable.resortLineIndex();
    //绑定鼠标事件
    mcsstable_bindMouseEventOnRows(mcsstable.tableData.rows,mcsstable);
    //加载完成后的事件处理，甘特图需要用此事件来生成时间序列

    if (mcsstable.params.afterLoadRows)
    {
        mcsstable.params.afterLoadRows(mcsstable,'',true);
    }

    mcsstable.bLoadData=false;

}

//added by sunliangchao
//add by sunliangchao
//插入最简单的空白行

function mcsstable_insertEmptyrow(tb,r1,r2,beforeRow)
{
    var tableid=tb.id;
    var mcsstable=mcsstable_getMCSSTable(tableid);
    if (!beforeRow) beforeRow=0;
    var tableData=mcsstable.tableData;
    var tableFixRow=mcsstable.tableFixRow;
    var tableFixRowCol=mcsstable.tableFixRowCol;
    var tableFixCol=mcsstable.tableFixCol;

    if (tableData.rows.length>1) {
        var lastCells=tableData.rows[tableData.rows.length-1].cells;
    }else{
        var lastCells=tableFixRow.rows[0].cells;
    }

    for (var r = r2-1; r >= r1; r--) {
        var newRow = tableData.insertRow(beforeRow);
        newRow.dataIndex=r;
        newRow.style.height=tableData.rowHeight + "px";
        newRow.fillData = false;

        var icell=0;

        for (var c = icell; c < lastCells.length; c++) {
            if (lastCells){
                if (lastCells.length>c){
                    var cls = lastCells[c].className ;
                    var style = lastCells[c].getAttribute("style");
                    var id = lastCells[c].getAttribute("id");
                }
            }

            var ncell = newRow.insertCell(-1);
            if (cls) ncell.setAttribute("class",cls);
            ncell.setAttribute("tag","cell");
            if (style) ncell.setAttribute("style",style);
            if (id) ncell.setAttribute("id",id);
            if (r<=1){
                ncell.style.width=lastCells[c].offsetWidth+"px";
            }
        }
    }

    for (var r = r2-1; r >= r1; r--) {
        var lastCells=tableFixRowCol.rows[0].cells;
        var newRow = tableFixCol.insertRow(beforeRow);
        newRow.dataIndex=r;
        newRow.style.height=tableData.rowHeight + "px";
        var icell=0;
        if (mcsstable.params.showLineNum){
            var ncell = newRow.insertCell(-1);
            ncell.innerHTML = r;
            ncell.className="linenumber";
            if (r<=1) ncell.style.width=lastCells[icell].offsetWidth+"px";
            icell ++;
        }
        for (var c = icell; c < lastCells.length; c++) {
            if (lastCells){
                if (lastCells.length>c){
                    var cls = lastCells[c].className ;
                    var style = lastCells[c].getAttribute("style");
                    var id = lastCells[c].getAttribute("id");
                }
            }
            var ncell = newRow.insertCell(-1);
            ncell.setAttribute("tag","cell");
            if (cls) ncell.setAttribute("class",cls);
            if (style) ncell.setAttribute("style",style);
            if (id) ncell.setAttribute("id",id);
            if (r<=1) ncell.style.width=lastCells[c].offsetWidth+"px";

        }
    }

    window.setTimeout(function(){mcsstable.fixRowHeight();},500);

}
//追加空白行，与插入最简的空白行一致，只是从下端追加
function mcsstable_appendEmptyrow(tb,r1,r2)
{
    var tableid=tb.id;
    var mcsstable=mcsstable_getMCSSTable(tableid);
    var tableData=mcsstable.tableData;
    var tableFixRow=mcsstable.tableFixRow;
    var tableFixRowCol=mcsstable.tableFixRowCol;
    var tableFixCol=mcsstable.tableFixCol;

    if (tableData.rows.length>1) {
        var lastCells=tableData.rows[tableData.rows.length-1].cells;
    }else{
        var lastCells=tableFixRow.rows[0].cells;
    }

    for (var r = r1; r < r2; r++) {
        var newRow = tableData.insertRow(-1);
        newRow.dataIndex=r;
        newRow.style.height=tb.rowHeight + "px";
        newRow.fillData = false;

        var icell=0;

        for (var c = icell; c < lastCells.length; c++) {
            if (lastCells){
                if (lastCells.length>c){
                    var cls = lastCells[c].className ;
                    var style = lastCells[c].getAttribute("style");
                    var id = lastCells[c].getAttribute("id");
                }
            }

            var ncell = newRow.insertCell(-1);
            ncell.setAttribute("tag","cell");
            if (cls) ncell.setAttribute("class",cls);
            if (style) ncell.setAttribute("style",style);
            if (id) ncell.setAttribute("id",id);
            if (r<=1){
                ncell.style.width=lastCells[c].offsetWidth+"px";
            }
            ncell.style.background="";
        }
    }


    for (var r = r1; r < r2; r++) {
        var lastCells=tableFixRowCol.rows[0].cells;
        var newRow = tableFixCol.insertRow(-1);
        newRow.dataIndex=r;
        newRow.style.height=tb.rowHeight + "px";
        var icell=0;
        if (mcsstable.params.showLineNum){
            var ncell = newRow.insertCell(-1);
            ncell.innerHTML = r;
            ncell.className="linenumber";
            if (r<=1) ncell.style.width=lastCells[icell].offsetWidth+"px";
            icell ++;
        }
        for (var c = icell; c < lastCells.length; c++) {
            if (lastCells){
                if (lastCells.length>c){
                    var cls = lastCells[c].className ;
                    var style = lastCells[c].getAttribute("style");
                    var id = lastCells[c].getAttribute("id");
                }
            }
            var ncell = newRow.insertCell(-1);
            ncell.setAttribute("tag","cell");
            if (cls) ncell.setAttribute("class",cls);
            if (style) ncell.setAttribute("style",style);
            if (id) ncell.setAttribute("id",id);
            if (r<=1) ncell.style.width=lastCells[c].offsetWidth+"px";

        }
    }
    mcsstable.maxDataIndex=r2;
    window.setTimeout(function(){mcsstable.fixRowHeight();},500);

}
//added by sunliangchao

//设置智能搜索收集的数据.如果sessionStorage已经有值，直接用它赋值给智能诸如字段的data属性，否则从后台数据库读取
function mcsstable_setSmartData(mcsstable,filter){
    var tablename = mcsstable.modeldata.tablename;
    var fieldIds = mcsstable.modelDealer.smartselectFieldIds;
    var found=false;

    if(found==false && mcsstable.modelDealer.smartselectFields.length>0){
        /*数据库分表*///参数不明确，已添加 usermodel_recordid
        var _project_id = mcsstable.params.usermodel_recordid;
        $.getJSON(g_homeurl+"/System/Model/getSmartFieldData",{tablename:tablename,fields:fieldIds,filter:filter,usermodel_recordid:_project_id},function(data){
            mcsstable_smartDataSaveStorge(mcsstable,data);
        })
    }
}
//将智能选择字段的数据保存到seesionStorge
function mcsstable_smartDataSaveStorge(mcsstable,distinctData){
//	var fields=fieldIds.split(',');
//	for(var i=0;i<fields.length;i++)
    for(fieldid in  mcsstable.modelDealer.smartselectFields)
    {
        var field=mcsstable.modelDealer.smartselectFields[fieldid];
        var data1=distinctData[field.id];
        var data2=[];
        for(var j=0;j<data1.length;j++)
        {
            data2.push(data1[j][field.id]);
        }

        var totalArr=data2;
        if (field.data)
        {
            var list1=field.data.split(',');//模型字段的data属性值是后台处理过的项目
            totalArr=totalArr.concat(list1);//合并两个数组
        }
        field.data=totalArr.join();
        if (window.sessionStorage)
            window.sessionStorage["mcss_selectdata_"+field.id+"_"+field.name]=field.data;

    }
    return;
}

//获得mcsstble发给后台的所有过滤条件
function mcsstable_getLastFilter(mcsstable)
{
    var lastfilter="";
    if (mcsstable.params.filter)
    {
        if (lastfilter)
            lastfilter+=" and ";
        lastfilter+="("+mcsstable.params.filter+")";
    }
    if (mcsstable.advancedFilter)
    {
        if (lastfilter)
            lastfilter+=" and ";
        lastfilter+="("+mcsstable.advancedFilter+")";
    }
    if (mcsstable.autoFilter)
    {
        if (lastfilter)
            lastfilter+=" and ";
        lastfilter+="("+mcsstable.autoFilter+")";
    }
    //lastfilter=lastfilter.replace(/\\'/g, "<yh>");
    //lastfilter=lastfilter.replace(/\'/g, "<yh>");
    return lastfilter;
}

//根据模型获取到数据列表后事件处理
function afterGetTableData(data,parentObject)
{
    //parentObject在这里指mcsstable
    mcsstable_loadRows(parentObject,data);

}

//给计算表达式类型的字段计算值
function mcsstable_calculateRecords(mcsstable,records)
{
    var fields=mcsstable.modeldata.fields;
    for(var i=0;i<records.length;i++)
    {
        uni_calculateOneRecord(fields,records[i]);
    }
    return records;
}


//根据已获得的data加载到表格中,不包括表头
function mcsstable_loadRows(mcsstable,data)
{
    var tableid=mcsstable.tableid;
    //以上是错误处理
    mcsstable.mcssData=new MCSSData(data,mcsstable.modeldata,{refObject:mcsstable,afterDataChanged:mcsstable_afterDataChanged});
    data= mcsstable_calculateRecords(mcsstable,data);
    mcsstable.data=data;

    if (mcsstable.cellEditor) mcsstable.cellEditor.mcssData=mcsstable.mcssData;

    if (data && data.length>0)
    {
        if (mcsstable.viewtype=='calendar')
            mcsstable_loadCalendarRows(mcsstable);
        else
            mcsstable_createTableBody(tableid,data);

        var rows=$("#"+mcsstable.tabledata_id).get(0).rows;
        if (rows.length==0)
            return;
    }

    if (mcsstable.view=='tableedit')
        mcsstable_tableEdit(tableid);
    else if (mcsstable.view=='celledit'){
        mcsstable_cellEdit(tableid);
    }
    setCookie(mcsstable.tableid+"_action_total",data.length);

    mcsstable_setRecordIndex(tableid);
    if(!mcsstable.params.notFadeTo)
        $(mcsstable.table).find("tr:not(:first)").fadeTo("100",1);//动画效果

    $(mcsstable.getAction('shortcut')).show();
    if (mcsstable.table.rows.length>10)
    {
        $(mcsstable.getAction('pagebar')).show();
    }

    mcsstable_setSelectedRow(mcsstable);

    if (mcsstable.params.afterLoadRows)
        mcsstable.params.afterLoadRows(mcsstable);


    var fixRows=mcsstable.tableFixCol.rows;
    for(var r=0;r<fixRows.length;r++){
        fixRows[r].style.height=mcsstable.tableData.rows[r].offsetHeight+"px";
        if (mcsstable.tableData.rows[r].style.display=="none") fixRows[r].style.display="none";
    }
}

//值修改后，把保存按钮变成红色
function mcsstable_afterDataChanged(mcssdata,record)
{
    if (mcssdata.params.refObject.view=="tableedit")
        $(mcssdata.params.refObject.getAction("save")).css('border-color','#d9534f');
}

function mcsstable_InsertDataRow(mcsstable,rowdata,rowIndex){

    var currow= mcsstable.tableData.rows[rowIndex];
    var curFixRow= mcsstable.tableFixCol.rows[rowIndex];

    var dataIndex = mcsstable.tableData.rows[rowIndex].dataIndex;

    var recordid =  currow.getAttribute("recordid");

    var s =mcsstable_addonerow(rowdata,mcsstable.tableid,dataIndex);
    var rows=mcsstable.tableData.rows;
    var fixRows=mcsstable.tableFixCol.rows;

    if (rowIndex<rows.length-1) {
        if (recordid)
            $(currow).after(s);
        else
            currow.outerHTML=s;

    }

    var sfix=mcsstable_addfixrow(rowdata,mcsstable.tableid,dataIndex);
    if (rowIndex<fixRows.length) {
        if (recordid)
            $(curFixRow).after(sfix);
        else
            curFixRow.outerHTML=sfix;
    }
    if (recordid)
        mcsstable.resortLineIndex();

}

//创建表格的数据区域的tr
function mcsstable_createTableBody(tableid,data)
{
    var mcsstable=mcsstable_getMCSSTable(tableid);
    var s = "";
    var line=1;
    //modify by sunliangchao
    //为了提高效率，从追加数据改变为在空行里写数据
    var table=$("#"+tableid);
    //mcsstable_appendEmptyrow(mcsstable.table,1,data.length);

    var rows=mcsstable.tableData.rows;
    var fixRows=mcsstable.tableFixCol.rows;

    if (rows.length<data.length){
        mcsstable_addEmptyrow(tableid,data.length-rows.length);
    }
    for(var i=0;i<data.length;i++)
    {
        var rowData=data[i];

        var s =mcsstable_addonerow(data[i],mcsstable.tableid,i+1);
        if (i<rows.length) {
            rows[i].outerHTML=s;
            rows[i].dataIndex = i+1;
        }

        var sfix=mcsstable_addfixrow(data[i],mcsstable.tableid,i+1);
        if (i<fixRows.length) {
            fixRows[i].outerHTML=sfix;
            fixRows[i].dataIndex = i+1;
        }

    }

    //add by sunliangchao
    //把增加分组的代码移走作为单独的一个方法
    mcsstable_addGroupRow(mcsstable,data,0);

    mcsstable.resortLineIndex();
//	var t2=(new Date()).getTime()-t1;
//	alert(t2);
    mcsstable_bindMouseEventOnRows(mcsstable.table.rows,mcsstable);
    window.setTimeout(function(){mcsstable.fixRowHeight();},500);
}

function mcsstable_addGroupRow(mcsstable,data,fromrow)
{
    //处理分组显示
    var tableid=mcsstable.tableid;
    var rows=mcsstable.tableFixCol.rows;
    if(mcsstable.groupField){

        var ito=data.length;
        var fieldsList=mcsstable.modeldata["fields"];
        var fields=mcsstable.modeldata.fixFields;
        var ig=0;

        for(var i=0;i<ito;i++)
        {
            var rowIndex=i;
            var rowData=data[i];

            var thisGroup=rowData[mcsstable.groupFieldid];
            if (thisGroup==undefined || thisGroup==null)
                thisGroup="";

            if (thisGroup!=mcsstable.last_groupfield_value || i + fromrow==0 ) //开始一个新的分组行
            {
                var groupfield=mcsstable.groupField;
                var g=mcsstable_getFieldText(groupfield,thisGroup,rowData,mcsstable);
                if (!g)
                    g=mcsstable.nullgroupname;
                //创建分组名称

                var s="";
                mcsstable.groupname=g;
                s="<tr class='rowgroup' align='left' style='height:25px' groupname='"+thisGroup+"' >"
                    +"<td onmouseover='mcsstable_showGroupBtn(this)' onmouseout='mcsstable_hideGroupBtn(this)' id='"+mcsstable.tableid+"_first' class='mcsstable_groupname' colspan='"+( mcsstable.tableFixRowCol.rows[0].cells.length)+"'>"
                    +"<div class='expandgroup' style='float:left;overflow:hidden;white-space:nowrap;height:25px'>"
                    +"<div href='javascript:void(0)' onclick='mcsstable_slidenogroup(this)' tag='收起'>"
                    +"<span class='icon-arrow-down4'></span>"
                    +"</div>"
                    +"<div class='groupname'>"+g+"</div>";
                s+=" <div id='addnewtaskbtn' onclick='project_dealGroup(this,\""+rowData.tasktype+"\",\""+mcsstable.tableid+"\")'  title='添加' class='icon-plus' style='display:none' ></div>";
                s+=" <div id='edittaskbtn' onclick='mcsstable_changeGroupName(\""+mcsstable.tableid+"\",this)'   title='修改' class='icon-pencil' style='display:none' ></div>";
                s+="</div>";
                s+="</td></tr>";

                var sd="<tr class='rowgroup' style='height:25px' align='left' groupname='"+thisGroup+"' ><td class='mcsstable_groupname' colspan='"+( mcsstable.tableFixRow.rows[0].cells.length)+"'></td></tr>";
                var grow = mcsstable.tableFixCol.insertRow(i+ig+fromrow);
                var growData= mcsstable.tableData.insertRow(i+ig+fromrow);

                ig++;

                grow.outerHTML=s;
                grow.id="trgroup_" + thisGroup;
                grow.dataIndex = i+fromrow;
                growData.outerHTML=sd;
                growData.id="trgroup_" + thisGroup;
                growData.dataIndex = i+fromrow;

                mcsstable.last_groupfield_value= thisGroup;

                for(var j = 0;j<fieldsList.length;j++){
                    if(fieldsList[j].show_sum){
                        var linename =fieldsList[j].name;
                        var lineid =fieldsList[j].id;
                        if (rowData[lineid])
                        {
                            mcsstable_sumvalueByGroup=setArrayValue(mcsstable_sumvalueByGroup,tableid,lineid,parseFloat(rowData[lineid]));
                        }
                        else
                            mcsstable_sumvalueByGroup=setArrayValue(mcsstable_sumvalueByGroup,tableid,lineid,0);
                    }
                }
            }
            else //分组统计
            {
                for(var j = 0;j<fieldsList.length;j++){
                    if(fieldsList[j].show_sum){
                        var linename =fieldsList[j].name;
                        var lineid =fieldsList[j].id;
                        if (rowData[lineid])
                        {
                            var oldv=mcsstable_getArrayValue(mcsstable_sumvalueByGroup,tableid,lineid);
                            mcsstable_sumvalueByGroup=setArrayValue(mcsstable_sumvalueByGroup,tableid,lineid,oldv+ parseFloat(rowData[lineid]));
                        }
                    }
                }
                //有分组显示情况下显示字段合计
                if(mcsstable.groupFieldid && mcsstable.hasSummaryField)
                {
                    //if(!rowIndex)
                    //	rowIndex=mcsstable.mcssData.getRowIndex(rowData[mcsstable.keyfield]);
                    var nextGroup="";
                    if (data.length == rowIndex+1)
                    {
                        nextGroup="12vcv23g2432dvdvvcvcv";//第一个行时 这是故意写一个不会重复的名称
                    }
                    else
                    {
                        if (rowData.mcss_recordtype=="NEW")
                            nextGroup=mcsstable.last_groupfield_value;
                        else
                            nextGroup=data[rowIndex+1][mcsstable.groupFieldid];

                        if (nextGroup==undefined || nextGroup==null) nextGroup="";

                    }
                    if(mcsstable.last_groupfield_value != nextGroup)  //如果下一行是新组，则在当前行后，下一行前插入一条新行用来显示合计信息
                    {
                        s="<tr class='rowgroup' rowtype='groupsum' groupname='"+thisGroup+"' align='left'  >";
                        if (mcsstable.params.showLineNum)
                            s += "<td class='linenumber' style='width:35px' >合计</td>";
                        if (mcsstable.params.showRowActionMenu)//显示第一栏
                            s += "<td style='width:25px' ></td>";
                        if (mcsstable.params.showfirst)//显示第一栏
                        {
                            if (mcsstable.params.showRecordMenu==false || mcsstable.params.showRowActionBtn==false )
                                s += "<td style='width:25px' ></td>";
                            else
                                s += "<td style='width:75px' ></td>";
                        }
                        fields=mcsstable.modeldata.fixFields;
                        for ( var j = 0; j < fields.length; j++)
                        {
                            var sum="";
                            if(mcsstable.getFieldInfo(fields[j].id,"show_sum")){
                                var sum=mcsstable_getArrayValue(mcsstable_sumvalueByGroup,tableid,fields[j].id);
                                if (sum==null)
                                    sum='0';
                                else if(parseInt(sum)!=sum)
                                    sum =sum.toFixed(2);
                                sum="<span title='合计'>"+sum+"</span>";
                            }
                            s += "<td class='"+ mcsstable.tableid+"_"+fields[j].id+"'  style='text-align:right;padding:0 8px ;'  id='" + fields[j].id + "'>"+sum+"</td>";
                        }
                        var irow=i+ig+fromrow+1;
                        if (irow>=mcsstable.tableFixCol.rows.length) irow=-1;
                        var grow = mcsstable.tableFixCol.insertRow(irow);
                        grow.outerHTML=s;

                        s="<tr class='rowgroup' rowtype='groupsum' groupname='"+thisGroup+"' align='left'  >";
                        fields=mcsstable.modeldata.dataFields;
                        for ( var j = 0; j < fields.length; j++)
                        {
                            var sum="";
                            if(mcsstable.getFieldInfo(fields[j].id,"show_sum")){
                                var sum=mcsstable_getArrayValue(mcsstable_sumvalueByGroup,tableid,fields[j].id);
                                if (sum==null)
                                    sum='0';
                                else if(parseInt(sum)!=sum)
                                    sum =sum.toFixed(2);
                                sum="<span title='合计'>"+sum+"</span>";
                            }
                            s += "<td class='"+ mcsstable.tableid+"_"+fields[j].id+"'  style='text-align:right;padding:0 8px ;'  id='" + fields[j].id + "'>"+sum+"</td>";
                        }

                        var growData = mcsstable.tableData.insertRow(irow);
                        growData.outerHTML=s;
                        ig++;

                        grow.dataIndex = i+fromrow;
                    }
                }
            }
        }

        //add by sunliangchao 当表格向上拖拽的时候，删除出现的重复的分组行
        var nextRow=mcsstable.tableFixCol.rows[ito+ig+fromrow];
        if (nextRow  )
        {
            var gname=nextRow.getAttribute("groupname");
            if (gname ){
                if (gname == mcsstable.last_groupfield_value){
                    mcsstable.tableFixCol.deleteRow(ito+ig+fromrow);
                }
            }
        }
    }
}
//add by sunliangchao
//添加无任何数据的空白行
function mcsstable_addEmptyrow(tableid,count)
{
    var mcsstable=mcsstable_getMCSSTable(tableid);
    var table= mcsstable.tableData;
    tableid=mcsstable.tableid;

    var rows=mcsstable.tableData.rows;
    var rowindex = 1;
    if (rows.length>1)
        rowindex = rows[rows.length-1].dataIndex + 1 ;

    mcsstable_appendEmptyrow(table,rowindex,rowindex+count);

    for (r=rowindex;r<rowindex+count;r++){
        var row=mcsstable.tableData.rows[r-1];
        if (mcsstable.params.afterAppendRow)
        {
            mcsstable.params.afterAppendRow(mcsstable,row);
        }
    }
}
function mcsstable_bindMouseEventOnEmptyRow(mcsstable,tr)
{
    var tableid=mcsstable.tableid;
    tr.onmousemove=function(){mcsstable_row_onmouseover(tr,tableid);};
    tr.onmouseout=function(){mcsstable_row_onmouseout(tr,tableid);};

    tr.setAttribute("recordid",'');
    tr.setAttribute("id",'');
    var trfix=mcsstable.tableFixCol.rows[tr.rowIndex];
    trfix.onmousemove=function(){mcsstable_row_onmouseover(tr,tableid);};
    trfix.onmouseout=function(){mcsstable_row_onmouseout(tr,tableid);};
    trfix.setAttribute("recordid",'');
    trfix.setAttribute("id",'');

    var cells= trfix.cells;
    if (cells.length<3) return;
    var icell=0;

    if (mcsstable.params.showLineNum)
    {
        cells[icell].className = "linenumber";
        icell++;
    }
    if (mcsstable.params.showRowActionMenu){
        cells[icell].className = "rowmenu";
        cells[icell].innerHTML="<div tag='rowactionmenu' class='icon-arrow-down3' style='display:none' onclick='mcsstable_showRecordMenu(this);' tableid='"+tableid+"' recordid='' ></div>";
        icell++;
    }
    if (mcsstable.params.showfirst) icell++ ;
    var d="<div  class='tableditor_cover'"
        +" contentEditable='true' readonly='true' onfocus='editorCoverOnFocus(this,event);' onblur='editorCoverOnBlur(this);'   onkeydown='editorCoverOnkeydown(event);' "
        +"></div>";

    for(var r = icell;r<cells.length;r++)
    {
        cells[r].tag = "cell";
        if (cells[r].innerHTML==""){
            cells[r].innerHTML=d;
        }
    }
}

//add by sunliangchao
//空行点击的时候触发  mcsstable_clickEmptyRow
function mcsstable_emptyRowClick(td,e)
{

    var table=td.parentNode.parentNode.parentNode;
    var tableid=table.id;
    var mcsstable=mcsstable_getMCSSTable(tableid);
    var row=td.parentNode;
    if (!td.inEdit)
        mcsstable.editCell(td);

    /**
     * 功能模块： 功能限制
     * add by: Chenli
     */
    if(mcsstable.params.beforeClickEmptyRow){
        if(mcsstable.params.beforeClickEmptyRow(mcsstable,row.textContent)==false){
            return;
        }
    }
    /** end*/
    mcsstable.cellEditor.onNewRecord=mcsstable_onNewRecord;
}

//add by sunliangchao 从onnewRecord中拆分出来的方法
function mcsstable_onNewRow(row,rowData,recordid,currentField)
{
    if (!row || !row.parentNode)
        return;
    var table = row.parentNode.parentNode;
    var tableid = table.id;
    var mcsstable = mcsstable_getMCSSTable(tableid);
    mcsstable.mcssData.appendRow(rowData);
    //重新生成列后，当前选中的单元格会失效，需要重新指定
    if (mcsstable.divSel && mcsstable.divSel.td) {
        mcsstable.divSel.cellIndex = mcsstable.getCellIndex( mcsstable.divSel.td);//.cellIndex;
        mcsstable.divSel.rowIndex = mcsstable.divSel.td.parentNode.rowIndex;//.cellIndex;
    }

    var row=mcsstable.tableData.rows[row.rowIndex];
    row.setAttribute("recordid", recordid);
    row.setAttribute("id", recordid);
    //row.setAttribute("onclick", 'mcsstable_clickRow(this,\"' + tableid + '\");');
    row.setAttribute("onmouseover", 'mcsstable_row_onmouseover(this,\"' + tableid + '\")');
    row.setAttribute("onmouseout", 'mcsstable_row_onmouseout(this,\"' + tableid + '\")');

    var rowfix=mcsstable.tableFixCol.rows[row.rowIndex];
    rowfix.setAttribute("recordid", recordid);
    rowfix.setAttribute("id", recordid);
    //rowfix.setAttribute("onclick", 'mcsstable_clickRow(this,\"' + tableid + '\");');
    rowfix.setAttribute("onmouseover", 'mcsstable_row_onmouseover(this,\"' + tableid + '\")');
    rowfix.setAttribute("onmouseout", 'mcsstable_row_onmouseout(this,\"' + tableid + '\")');

    if (row && row.previousSibling && row.previousSibling.dataIndex)
        row.dataIndex=row.previousSibling.dataIndex+1;
    else
        row.dataIndex=1;
    var tdStyle = "", trStyle = "";
    if (rowData["SYS_FORMATE"]) {
        var cssArr = rowData["SYS_FORMATE"].split("~|~");
        if (cssArr.length == 2) {
            trStyle = cssArr[0];
            tdStyle = cssArr[1];
        } else {
            if (cssArr[0].indexOf("TR=|=") > -1)
                trStyle = cssArr[0];
            else
                tdStyle = cssArr[0];
        }
    }

    var tdArr;
    if (tdStyle)
        tdArr = tdStyle.split("-|-");
    var fieldsList = mcsstable.modeldata["fields"];
    var snfieldid = mcsstable.modelDealer.modeldata.SNFieldid
    var cells = $(row).find("td[tag='cell']");
    cells.each(function (i, cell) {
        for (var j = 0; j < fieldsList.length; j++) {
            if (fieldsList[j].id == cell.id) {
                if (cell.id != currentField && cell.id != snfieldid) {
                    var s = mcsstable_createTd(mcsstable, fieldsList[j], "", recordid, rowData, "");
                    cell.outerHTML = s;
                }
            }
        }
    })


    var icell = 0;
    if (mcsstable.params.showLineNum) {
        icell += 1;
    }

    if (mcsstable.params.showRowActionMenu) {
        var showOrHide = "none";
        if (mcsstable.params.alwaysShowRowActionMenu) showOrHide = "";
        s = "<td class='rowmenu'><div tag='rowactionmenu' class='icon-arrow-down3' style='display:" + showOrHide + "' onclick='mcsstable_showRecordMenu(this);' tableid='" + tableid + "' recordid='" + recordid + "' ></div></td>";
        rowfix.cells[icell].outerHTML = s;
        icell += 1;
    }
    var fvalue = "";
    if (rowData)
        fvalue = rowData[mcsstable.keyfield];
    if (mcsstable.params.showfirst)//显示第一栏
    {
        s = "<td class='mcsstable_td1 mcsstable-action-col'>";
        if (!mcsstable.params.showRecordActionAtLast) //打开和编辑按钮
        {
            s += mcsstable_getRecordAction(mcsstable, rowData);
        }

        if (mcsstable.first_td_func && rowData) {
            var td = mcsstable.first_td_func(rowData[mcsstable.keyfield], rowData);
            s += td;
        }
        s += "</td>";
        rowfix.cells[icell].outerHTML = s;
        //icell+=1;
    }
    //重新生成列后，当前选中的单元格会失效，需要重新指定
    if ( mcsstable.divSel && ! mcsstable.divSel.td.parentNode) {
        mcsstable.divSel.td= mcsstable.getCell(mcsstable.divSel.rowIndex,mcsstable.divSel.cellIndex);
    }

    if (mcsstable.modeldata.sortfieldid) {
        //这里的新行是只对空行增加的，可以直接从末尾追加，不需要参考上一行和下一行的序号
        //var iSort = mcsstable_setSortForNewRecord(mcsstable, row);
        var sortfield = mcsstable.modeldata.sortfieldid;
        mcsstable.params.maxSort = parseInt(mcsstable.params.maxSort) + 100;
        mcsstable.mcssData.setFieldValue(recordid,sortfield,mcsstable.params.maxSort);
        rowData[sortfield] = mcsstable.params.maxSort;
    }

    mcsstable_showNewRowActions(rowfix, mcsstable.table);

    mcsstable_bindMouseEventOnRows(row, mcsstable);

    mcsstable.params.totalRecordCount = parseInt(mcsstable.params.totalRecordCount) + 1;
    mcsstable_addEmptyrow(tableid, 1);
    var tb = parseInt(mcsstable.table.offsetTop) + parseInt(mcsstable.table.clientHeight) + 30;
    mcsstable.table.parentNode.style.height = tb + "px";

}

function mcsstable_onNewRecord(row, fieldid, value) {

    var table = row.parentNode.parentNode;
    var tableid = table.id;
    var mcsstable = mcsstable_getMCSSTable(tableid);
    var recordid = getTimeId();
    var rowData = mcsstable_createNewData(mcsstable, recordid);
    rowData[fieldid] = value;
    //新增行方法拆分出去
    mcsstable_onNewRow(row,rowData,recordid,fieldid);

    //为支持撤销
    var tableState={rowIndex:row.rowIndex,cellIndex:mcsstable.divSel.td.cellIndex};
    mcsstable.addUndoList(mcsstable,"deleteRow",[recordid],mcsstable,"addRow",[rowData,row.dataIndex],tableState);

    rowData[fieldid] = "";

    mcsstable_autoSetSnField(mcsstable,recordid);
    
    //记录多少人通过添加按钮添加，多少人直接通过数据行添加数据
    return recordid;

}
//把自动追加行和插入行的代码合并起来，把自动设置序列号字段的方法拆分出来。
function mcsstable_autoSetSnField(mcsstable,recordid){

    var rowData = mcsstable.mcssData.getRow(recordid);
    var snfieldid = mcsstable.modelDealer.modeldata.SNFieldid
    if (snfieldid) //有序列号字段的情况，需要异步处理
    {
        if (mcsstable.params.project_fulldata) {
            var defaultdata = mcsstable.modelDealer.getField(snfieldid).defaultdata;
            $.getJSON(uni_global.homeurl + "/System/Model/getExpressionValue", { fieldid: snfieldid, projectid: mcsstable.params.project_fulldata.project_record.id, data: defaultdata }, function (data) {
                var defaultdata = mcsstable.modelDealer.getField(snfieldid).defaultdata;
                //mcsstable.modelDealer.setFieldProp(snfieldid,'defaultdata',data.value);
                var sn = data.value;
                rowData[snfieldid] = sn;
                var fieldsList = mcsstable.modeldata["fields"];
                var strf = "";
                for (var j = 0; j < fieldsList.length; j++) {
                    if (fieldsList[j].id == snfieldid) {
                        strf = mcsstable_createTd(mcsstable, fieldsList[j], "", recordid, rowData, "");
                    }
                }
                var cell = mcsstable.getCellById(recordid,snfieldid);
                if (cell) cell.outerHTML = strf;

                rowData[snfieldid] = "";
                window.setTimeout(function () {
                    mcsstable.mcssData.setFieldValue(recordid, snfieldid, sn);
                    mcsstable.mcssData.save(null, { refObject: mcsstable });
                }, 200)
            });
        }
    }
}
//给任务甘特图的每一行绑定鼠标移动事件，目的是显示或隐藏第一格的的按钮
function mcsstable_bindMouseEventOnRows(rows,mcsstable)
{
    if(mcsstable.params.beforeBindMouseEventOnRows)
    {
        var ifconntinue=mcsstable.params.beforeBindMouseEventOnRows(mcsstable,rows);
        if (ifconntinue==false)
            return;
    }
}


//给记录序号排序
function mcsstable_setRecordIndex(tableid)
{
    var mcsstable=mcsstable_getMCSSTable(tableid);
    if (!mcsstable.modeldata)
        return;
    var fields=mcsstable.modeldata.fields;
    for(var i=0;i<fields.length;i++)
    {
        if (fields[i].type=='recordindex')
        {
            var ths=$("#"+tableid).find("tr").get(0).cells;
            for(var j=0;j<ths.length;j++)
            {
                if (ths[j].id==fields[i].id)
                {
                    $("#"+tableid).find("tr").each(function(k,item){
                        if (k>0 && !item.hasClass("rowgroup"))
                            item.cells[j].innerHTML=k;
                    })

                }
            }


        }
    }

}

/*
 function mcsstable_selectAll(self,tableid) {
 var mcsstable=mcsstable_getMCSSTable(tableid);
 var checkboxArr = document.getElementsByName(tableid + "_checkbox");
 for ( var i = 0; i < checkboxArr.length; i++) {
 if(self.checked){
 checkboxArr[i].checked = true;
 mcsstable_whencheckall(checkboxArr[i],tableid);
 }else{
 checkboxArr[i].checked = false;
 mcsstable.selectedRowsID='';
 }
 }
 }


 点击隐藏和显示
 */
function mcsstable_slidenogroup(obj){
    if($(obj).children('span').eq(0).attr('tag')=="收起"){
        $(obj).children('span').addClass("icon-arrow-down4").removeClass("icon-arrow-right6");
        $(obj).children('span').eq(0).attr('tag',"展开");
    }else{
        $(obj).children('span').removeClass("icon-arrow-down4").addClass("icon-arrow-right6");
        $(obj).children('span').eq(0).attr('tag',"收起");
    }
    var thisgroupindex=$(obj).parent().parent().parent().prop('rowIndex');
    var nextgroupindex=$(obj).parent().parent().parent().nextAll("[class='rowgroup']").eq(0).prop('rowIndex');
    var tableFixCol=$(obj).parent().parent().parent().parent().parent().get(0);
    if(!nextgroupindex) nextgroupindex=tableFixCol.rows.length;
    var tableData=tableFixCol.mcsstable.tableData;
    //$(obj).parent().parent().parent().nextAll('tr:lt('+(nextgroupindex-thisgroupindex-1)+')').toggle();
    for(var j=thisgroupindex+1;j<nextgroupindex;j++)
    {
        $(tableFixCol.rows[j]).toggle();
        $(tableData.rows[j]).toggle();
    }

}
/*
 创建table的一行（tr）的html代码。该方法被大量调用
 rowData：一条记录数据
 rowIndex：当前页第几条数据。允许为空。分组统计需要
 */

function mcsstable_addfixrow (rowData, tableid,rowIndex){
    var mcsstable=mcsstable_getMCSSTable(tableid);
    var isTempId='1',rowid='';
    if (rowData){
        rowid=recordid=rowData[mcsstable.keyfield];
        isTempId='';
    }
    else
    {
        rowid=getTimeId();
    }
    var recordid=rowid;
    var s="";
    s += "<tr recordid='"+recordid+"' isTempId='"+isTempId+"' id='"+rowid+"'  "; //去掉了class='mcsstable_tr"+line+"'
    s+=" onmouseover='mcsstable_row_onmouseover(this,\""+tableid+"\")' onmouseout='mcsstable_row_onmouseout(this,\""+tableid+"\")'";
    s+=">";
    if (mcsstable.params.showLineNum){
        s+="<td class='linenumber' style='width:35px'";

        s+=">"+rowIndex+"</td>";
    }
    if (mcsstable.params.showRowActionMenu){
        var showOrHide="none";
        if (mcsstable.params.alwaysShowRowActionMenu) showOrHide="";
        s+="<td class='rowmenu' style='width:25px' ><div tag='rowactionmenu' class='icon-arrow-down3' style='display:"+showOrHide+"' onclick='mcsstable_showRecordMenu(this);' tableid='"+tableid+"' recordid='"+recordid+"' ></div></td>";
    }
    if (mcsstable.params.showfirst)//显示第一栏
    {
        if (mcsstable.params.showRecordMenu==false || mcsstable.params.showRowActionBtn==false)
            s += "<td class='mcsstable-action-col' style='width:25px'>";
        else
            s += "<td class='mcsstable-action-col' style='width:75px'>";

        if (!mcsstable.params.showRecordActionAtLast) //打开和编辑按钮
        {
            s += mcsstable_getRecordAction(mcsstable,rowData);
        }
        var first_td_func=mcsstable.first_td_func;
        if (first_td_func && rowData) {
            var td = first_td_func(rowData[mcsstable.keyfield],rowData);
            s += td;
        }
        s += "</td>";
    }

    var sr=mcsstable_addonerow (rowData, tableid,rowIndex,mcsstable.modeldata.fixFields);
    s+=sr;
    s+="</tr>";
    return s;
}

function mcsstable_addonerow(rowData, tableid,rowIndex,fields)
{
    var mcsstable=mcsstable_getMCSSTable(tableid);
    if (!mcsstable.data)
        return "";
    tableid=mcsstable.tableid;
    var modeldata=mcsstable.modeldata;
    //var fields=mcsstable.fields;
    var first_td_func=mcsstable.first_td_func;
    var last_td_func=mcsstable.last_td_func;
    var params=mcsstable.params;

    var s = "";

    if (mcsstable.params.beforeAddRow)
    {
        s+=mcsstable.params.beforeAddRow(mcsstable,rowData);
    }
    var params=mcsstable.params;
    if (!mcsstable.modeldata)
        return;
    var fieldsList=mcsstable.modeldata.dataFields;
    if (fields) fieldsList=fields


    //以上处理分组

    //让行间隔出现不同的样式
    var line="1";
    if(rowIndex%2==0)
        line="2";
    if (params.diffLineCss==false)
        line="2";


    var isTempId='1',rowid='';
    if (rowData){
        rowid=recordid=rowData[mcsstable.keyfield];
        isTempId='';
    }
    else
    {
        rowid=getTimeId();
    }
    var recordid=rowid;

    mcsstable.lastRowId=rowid;
    var tdStyle = "",trStyle = "";
    if(rowData["SYS_FORMATE"]){
        var cssArr = rowData["SYS_FORMATE"].split("~|~");
        if(cssArr.length==2){
            trStyle = cssArr[0];
            tdStyle = cssArr[1];
        }else{
            if(cssArr[0].indexOf("TR=|=")>-1)
                trStyle = cssArr[0];
            else
                tdStyle = cssArr[0];
        }
    }
    if(trStyle){
        var styleArr = trStyle.split("=|=");
        trStyle = styleArr[1];
    }
    s += "<tr recordid='"+recordid+"' isTempId='"+isTempId+"' id='"+rowid+"'  "; //去掉了class='mcsstable_tr"+line+"'
    s += " onmouseover='mcsstable_row_onmouseover(this,\""+tableid+"\")' onmouseout='mcsstable_row_onmouseout(this,\""+tableid+"\")'";
    if(trStyle)
        s+=" style='"+trStyle+"'";
    s+=">";
    if (fields) s="";
    var tdArr;
    if(tdStyle)
        tdArr = tdStyle.split("-|-");
    for ( var j = 0; j < fieldsList.length; j++)
    {
        if (fieldsList[j].isvisible=="false")
            continue;
        var style = "";
        if(tdArr){
            for(var i = 0;i < tdArr.length;i++){
                var fieldArr = tdArr[i].split("=|=");
                if(fieldArr[0]==fieldsList[j].id){
                    style = fieldArr[1]
                    break;
                }
            }
        }
        s += mcsstable_createTd(mcsstable,fieldsList[j],line,recordid,rowData,style);
    }
    //把行操作按钮放在最后一个单元格。现在很少用到
    if (mcsstable.ifShowLastTd)
    {
        var td = '';
        if(last_td_func)
        {
            td = last_td_func(rowData[mcsstable.keyfield],rowData);
        }
        s += "<td class='mcsstable_td"+line+" recordaction' style='' id='"+rowid+"' >"+mcsstable_getRecordAction(mcsstable,rowData)+td+"</td>";
    }
    if (!fields) s += "</tr>";

    //removed by sunliangchao 把分组合计移动到addgrouprow函数中了

    //只有最后一行才显示本页合计(注意不是所有记录合计)
    //if (mcsstable.data.length == rowIndex+1 )
    //{
    //s+=createPageTotalInfo(mcsstable,params,line,fields,"",fieldsList,tableid);
    //}
    if (mcsstable.params.afterAddRow)
    {
        s=mcsstable.params.afterAddRow(mcsstable,s,rowData);
    }

    return s;
}
function mcsstable_showGroupBtn(obj){
    $(obj).find(".icon-pencil").show();
    $(obj).find(".icon-plus").show();
}
function mcsstable_hideGroupBtn(obj){
    $(obj).find(".icon-pencil").hide();
    $(obj).find(".icon-plus").hide();
}
function mcsstable_changeGroupName(tableid,obj){
    var mcsstable=mcsstable_getMCSSTable(tableid);
    mcdom.tempTableid = tableid;
    mcdom.tempOldGroupName = $(obj).prev().prev().text();
//	if (mcdom.tempOldGroupName==mcsstable.nullgroupname)
//		mcdom.tempOldGroupName="";
    mcdom.input(mcsstable_callBackChangeGroupName,mcdom.tempOldGroupName,{obj:obj,title:"输入名称"});
}
function mcsstable_callBackChangeGroupName(value){
    var tableid = mcdom.tempTableid;
    var mcsstable = mcsstable_getMCSSTable(tableid);
    value=uni_cleanSQL(value);
    $.post(g_homeurl+"/System/Model/modifyGroupName",{modelid:mcsstable.params.modelid,usermodel_recordid:mcsstable.params.usermodel_recordid,newName:value,group:mcsstable.groupFieldid,oldvalue:mcdom.tempOldGroupName},function(){
        //mcsstable_loaddatarows(tableid);
        var row=mcsstable.tableFixRowCol.rows[0];
        var gc=0;
        for(var c=0;c<row.cells.length;c++)
        {
            if(row.cells[c].getAttribute("id")==mcsstable.groupField.id)
            {
                gc=c;
                break;
            }
        }

        for(var r=0;r<mcsstable.tableFixCol.rows.length;r++)
        {
            var row=mcsstable.tableFixCol.rows[r];
            if (row.getAttribute("groupname")==mcdom.tempOldGroupName){

                row.setAttribute("groupname",value);
                $(row).find("span").get(1).innerHTML="<span>"+value+"</span>";
            }
            var cell=row.cells[gc];
            if (cell)
            {
                if (cell.innerText==mcdom.tempOldGroupName){
                    cell.innerHTML="<span>"+ value +"</span>";
                }
            }
        }

        mcdom.alert("修改成功",'提示','success','fadeout');
        mcdom.tempTableid = "";
        mcdom.tempOldGroupName = "";
    })
    return true;
}

function mcsstable_createTd(mcsstable,field,line,recordid,rowData,tdStyle){

    var tableid=mcsstable.tableid;
    var fieldtype=field.fieldtype;
    var type=field.type;
    var style=field.style;
    //去掉了class='mcsstable_td"+line
    var s= "<td  tag='cell'  id='" +field.id +"'";

    if (mcsstable.params.project_fulldata && mcsstable.params.project_fulldata.shareoption && mcsstable.params.project_fulldata.shareoption.indexOf("[editinnullcell]")>-1){
        if(rowData && rowData[field.id]!="" && rowData[field.id]!=0 && rowData[field.id]!=null)
        {
            s += " readonly=1 ";
        }
    }

    if(style && style.indexOf(':')==-1)
        s += " class='"+style+"'";
    var _style="" ;
    if (mcsstable.params.style_hideLeftRightBorder)
        _style+=";border-left:none;border-right:none;";
    if(style && style.indexOf(':')>-1){
        _style+= ";"+style;
    }
    else
        _style+= ";width:100px";
    if(tdStyle)
    {
        _style += ";"+tdStyle;
    }
    if(!_style || (_style && _style.indexOf("text-align")==-1))
    {
        var direction="";
        if (type=='recordindex' || type=='dropdown' || type=="date" || type=="radio" || type=="datetime" || type=="checkbox" || type=="checkboxlist" || type=="statueIcon" || type=="priorityIcon" || type=="colorIcon" || field.length<10)
            direction="center";
        else
        if (fieldtype=='float' || fieldtype=="int" || fieldtype=="real")
            direction="right";
        _style+= ";text-align:"+direction;
    }
    if (field.type=="image"){
        _style+=";height:120px";
    }
    s += " style='"+_style+"'";

    if (field.prop && (field.prop).indexOf("OPENRECORD")!=-1)
    {
        var modeldata=mcsstable.modeldata;
        var openurl = modeldata.openurl;
        if(openurl && (openurl.indexOf("/")>0))
        {
            s+=" onclick=\"mcsstable_openWindow('"+g_homeurl
                + "/"+openurl
                + "/"+mcsstable.keyfield+"/"
                + recordid
                + "','"
                + lang.open
                + "' ,'"+mcsstable.modeldata.open_window_style+"')\"";

        }
    }
    s+=" >";
    if (rowData)
    {
        var str=mcsstable_getFieldDisplay(field,rowData[field.id],mcsstable.params.special_field_show,rowData,mcsstable) ;
        var activecell=rowData[mcsstable.mcssData.activecellFieldId];//单元格额外信息
        if (activecell && activecell!='undefined')
        {
            try
            {
                var activecell=JSON.parse(activecell);
                if (activecell[field.id])
                {
                    var link=activecell[field.id]['link'];
                    if (link=='RECORD')
                        str="<a onclick='mcsstable_openAutoform(this,\""+tableid+"\",\""+recordid+"\",\"查看\",\"VIEW\")'>"+str+"</a>";
                    else if (link)
                        str="<a onclick='window.open(\""+link+"\");'>"+str+"</a>";
                }
            }
            catch(ex)
            {
                //alert(ex.message);
            }
        }
        s+=str;//"<span class='mcsstable_cellspan'>"+str+"</span>";
    }
    s+=	"</td>";
    return s;
}

function mcsstable_clickTD(e)
{
    var table=mcdom.getObjByTagName(e,'TABLE');
    var mcsstable=mcsstable_getMCSSTable(table.id);
    if (mcsstable.cellEditor)
    {
        mcsstable.cellEditor.setCurrentEditorStyle(e);
    }

    $(table.rows).removeClass("selected-row");
    $(table.rows).find('.selected-line-number').removeClass("selected-line-number");
    mcsstable.selectedRowsID="";
}
function mcsstable_FieldNameMouseover(obj,event)
{
    var scrollLeft = 0;
    obj=mcdom.getObjByTagName(obj,'TH');
    var table=mcdom.getObjByTagName(obj,'TABLE');
    var scrollLeft = table.parentNode.parentNode.scrollLeft;
    //不这样设置相关的控件会移位
    obj.style.textAlign="left";

    var obja = $(obj).find("#cola").get(0);
    if (obja){
        obja.style.opacity=1;
        obja.style.marginLeft=obj.offsetWidth - scrollLeft -20+"px";
        obja.style.marginTop = "14px";
    }
    var divn=$(obj).find("#colname").get(0);
    var divc=$(obj).find("#colmove").get(0);
    if (divc) {
        divc.style.display="";
        divc.style.width=obj.clientWidth-2+"px";
        divc.style.left=obj.offsetLeft- scrollLeft + table.parentNode.parentNode.offsetLeft +"px";
    }

    var diva=$(obj).find("#colwidth").get(0);
    if (diva){
        diva.style.marginLeft=obj.offsetWidth- scrollLeft-6+"px";
        diva.style.zIndex=1;
    }
}
function mcsstable_FieldNameMouseout(obj,event)
{
    var obja = $(obj).find("#cola").get(0);
    if (obja) obja.style.opacity=0;
    var diva=$(obj).find("#colmove").get(0);
    if (diva) diva.style.opacity=1;
}
//生成本页合计信息
//add by sunliangchao
function createPageTotalInfo(mcsstable)
{
    var sumHTML="<table class='mcsstablesum' style='table-layout:fixed;margin-left:-1px;width:"+mcsstable.tableData.offsetWidth+"px' ><tr rowtype='pagesum' align='left'>";
    var sumFix="<table class='mcsstablesum' style='table-layout:fixed;' ><tr rowtype='pagesum' align='left'>";
    if (!mcsstable.hasSummaryField) return;
    if (mcsstable.tableData.rows.length<2) return;

    var row = mcsstable.tableData.rows[0];
    if (row.getAttribute("class")=="rowgroup") row=mcsstable.tableData.rows[1];//在分组情况下，从后续的行开始合计。

    var cells=row.cells;
    for ( var j = 0; j < cells.length; j++)
    {
        sumHTML+="<td id='"+ cells[j].id +"' style='width:"+ cells[j].offsetWidth +"px'></td>"
    }
    var fixCells=mcsstable.tableFixCol.rows[row.rowIndex].cells;

    for ( var j = 0; j < fixCells.length; j++)
    {
        sumFix+="<td id='"+ fixCells[j].getAttribute('id') +"' style='width:"+ fixCells[j].offsetWidth +"px'></td>"
    }

    sumHTML+="</tr></table>";
    sumFix+="</tr></table>";

    mcsstable.table.rows[2].cells[1].innerHTML="<div>"+ sumHTML +"</div>";
    mcsstable.table.rows[2].cells[0].innerHTML="<div>"+ sumFix +"</div>";

    mcsstable.tableTotal=mcsstable.table.rows[2].cells[1].firstChild.firstChild;
    mcsstable.tableFixTotal=mcsstable.table.rows[2].cells[0].firstChild.firstChild;

    $.getJSON(g_homeurl+"/System/Model/getFieldSumValue",{sumFields:mcsstable.sumFields,modelid:mcsstable.modelid,projectid:mcsstable.params.project_fulldata.project_record.id,filter:mcsstable.filter,searchword:mcsstable.searchword}, function(data){

        var row = data.rows[0];
        var cells = $(mcsstable.table.rows[2]).find("td");
        for(var i=0;i<cells.length;i++)
        {
            var fid=cells[i].id;
            if (fid){
                var value="";
                if (row[fid]) value=row[fid];
                if (value){
                    if(parseInt(value)!=value)
                        value = Math.round(value*100)/100;
                    cells[i].innerHTML=value;
                }else
                    cells[i].innerHTML="";
            }
        }
    });
}

function mcsstable_getArrayValue(arr,tableid,field)
{
    for(var i=0;i<arr.length;i++)
    {
        if (arr[i].tableid==tableid && arr[i].field==field)
        {
            return arr[i].sumvalue;
        }
    }
    return null;
}
function setArrayValue(arr,tableid,field,value)
{
    var found=false;
    for(var i=0;i<arr.length;i++)
    {
        if (arr[i].tableid==tableid && arr[i].field==field)
        {
            arr[i].sumvalue=value;
            found=true;
        }
    }
    if (!found)
        arr.push({tableid:tableid,field:field,sumvalue:value});
    return arr;
}
function addItemToArray(arr,tableid,field,value)
{
    arr.push({tableid:tableid,field:field,sumvalue:value});
}

//创建记录操作（打开、编辑、删除）的html代码
function mcsstable_getRecordAction(mcsstable,v)
{
    var s="";
    var params=mcsstable.params;
    if(params.toolbar){
        var allactions=","+params.toolbar+",";
        var recordid="";
        if (v)
            recordid=v[mcsstable.keyfield];

        var modeldata=mcsstable.modeldata;
        var tableid=mcsstable.tableid;
        var openurl=modeldata.openurl;
        var showRecordActBtn="";
        var shareoptions=mcsstable.params.project_fulldata.shareoption;
        
        var canEdit=false;
        if (mcsstable.view!="celledit") 
            cellEdit=false;
        else if (recordid){
            if (mcsstable.params.model_ownerid == loginuser || shareoptions.indexOf('[editmyrecords]')<0 || (shareoptions.indexOf('[editmyrecords]')>=0 && mcsstable.mcssData.getFieldValue(recordid,'SYS_ADDUSER') == loginuser )){
                canEdit=true;
            }
        }
        
        if (params.showRecordMenu==false || params.showRowActionBtn==false)
        {
            showRecordActBtn='none';
        }

        if (params.showRowActionBtn==true){
            showRecordActBtn='';
        }
        if (allactions.indexOf(",open,")>-1)
        {
            s+="<a class='mcsstable_record_open icon-eye' style='margin-left:4px;display:"+showRecordActBtn+";'  tag='recordAction'  href='javascript:void(0)' ";
            if(openurl && (openurl.indexOf("/")>0))
            {
                openurl=g_homeurl+"/"+openurl+"/"+mcsstable.keyfield+"/"+recordid;
                s+=" onclick=\"mcsstable_showPopupForm(this,'"+tableid+"','"+openurl+"','"+lang.open+"','"+recordid+"','VIEW')\"";
            }
            else
                s+=" onclick='mcsstable_openAutoform(this,\""+tableid+"\",\""+recordid+"\",\"查看\",\"VIEW\")' ";
            s+=" title='查看记录详情'></a>";
        }
        if (mcsstable.view!='tableedit')
        {
            if (allactions.indexOf(",edit,")>-1)
            {
                var url = modeldata.editurl;
                //没有指定openurl或指定openurl但没有/号,说明openurl是一个modelid
                if (!url || url.indexOf("/")==-1)
                {
                    if (canEdit) s += "<a class='mcsstable_record_edit icon-pencil'  tag='recordAction' ontouchend=\"mcsstable_openAutoform(this,'"+tableid+"','"+recordid+"','编辑','EDIT')\" title='打开修改页面'></a>";
                }
                //else
                //{
                    //s+=mcsstable_getOpenActionHTML(mcsstable,url,recordid,lang.edit,lang.edit,lang.edit);
                //}
            }
        }
        if (allactions.indexOf(",del,")>-1) {
            s += "<a class='mcsstable_record_del icon-remove' style='display:"+showRecordActBtn+";'  tag='recordAction'  onClick=\"mcsstable_deleteOneRow(this,'"+tableid+"')\" title='删除当前记录'></a>";
        }
        //处理记录的“新增”按钮操作
        if (params.showRecordInsertAction && allactions.indexOf(",add,")>-1)
        {
            var url = modeldata.openurl;
            //没有指定openurl或指定openurl但没有/号,说明openurl是一个modelid
            if (!url || url.indexOf("/")==-1)
            {
                //if(mcsstable.modelid!='sys_usermodel')
                s+="<a class='mcsstable_record_add icon-plus' style='display:"+showRecordActBtn+";'  tag='recordAction' onclick='mcsstable_insertRow(this.parentNode.parentNode,\""+tableid+"\")'  title='在当前行前面插入新记录'></a>"
                    +"<a class='mcsstable_record_cut icon-scissors' style='display:"+showRecordActBtn+";' tag='recordAction' onclick='mcsstable_cut(this.parentNode.parentNode,\""+tableid+"\")'  title='剪切'></a>"
                    +"<a class='mcsstable_record_paste icon-paste' style='display:"+showRecordActBtn+";' tag='recordAction' onclick='mcsstable_paste(this.parentNode.parentNode,\""+tableid+"\")' title='粘贴'></a>";
            }
            else
            {
                s+=mcsstable_getOpenActionHTML(mcsstable,url,recordid,lang.add,lang.add,lang.add);
            }
        }

        if(params.userAction)
        {
            s = params.userAction(s,recordid,v,mcsstable);
        }
    }

    return s;
}


function mcsstable_showRecordMenu(e)
{
    var table=mcdom.getObjByTagName(e,"TABLE");
    var mcsstable = mcsstable_getMCSSTable(table.id);
    if (mcsstable.params.showRowMenu)
    {
        mcsstable.params.showRowMenu(e,mcsstable);
        return;
    }
    var tableid=$(e).attr('tableid');
    var recordid=$(e).attr('recordid');

    var row=$(table.rows[e.parentNode.parentNode.rowIndex]);
    var h = "";
    if (row.find(".mcsstable_record_open").size()>0)
        h+="<a id='record_menu_view'  title='查看当前记录' >查看</a>";
    if (row.find(".mcsstable_record_edit").size()>0)
        h+="<a id='record_menu_edit' title='修改当前记录' >修改</a>";
    if (row.find(".mcsstable_record_del").size()>0)
        h+="<a id='record_menu_del'   title='删除当前记录' >删除</a>";
    if (row.find(".mcsstable_record_add").size()>0)
        h+="<a id='record_menu_add'   title='在当前行前面插入记录' >插入</a>";
    if (row.find(".mcsstable_record_cut").size()>0)
        h+="<a id='record_menu_cut'   title='剪切当前行' >剪切</a>";
    if (row.find(".mcsstable_record_paste").size()>0)
        h+="<a id='record_menu_paste'   title='把作了剪切标记的行移动到当前行前面' >粘贴</a>";
    if(mcsstable.params.diy_method)
        h+="<a id='record_menu_diy'  title='"+mcsstable.params.diy_menubtn_hit+"'>"+mcsstable.params.diy_menubtn_val+"</a>";
    dom_showPopupMenu(h,e);

    var _e=e;
    $("#record_menu_view").click(function(){
        mcsstable_openAutoform(_e,tableid,recordid,"查看","VIEW");
    })
    $("#record_menu_edit").click(function(){
        mcsstable_openAutoform(_e,tableid,recordid,"编辑","EDIT");
    })
    $("#record_menu_del").click(function(){
        mcsstable_deleteOneRow(_e,tableid);
    })
    $("#record_menu_add").click(function(){
        var obj = $(_e).parent().next().find(".mcsstable_record_add").eq(0);
        $(obj).click();
    })
    $("#record_menu_cut").click(function(){
        var obj = $(e).parent().next().find(".mcsstable_record_cut").eq(0);
        $(obj).click();
    })
    $("#record_menu_paste").click(function(){
        var obj = $(_e).parent().next().find(".mcsstable_record_paste").eq(0);
        $(obj).click();
    })
    $("#record_menu_diy").click(function(){
        if(mcsstable.params.diy_method)
            mcsstable.params.diy_method(_e,tableid,recordid);
    })
}

function mcsstable_viewRecord(obj,recordObj)
{
    var tableid=$(recordObj).attr('tableid');
    var recordid=$(recordObj).attr('recordid');
    mcsstable_openAutoform(obj,tableid,recordid,"查看","VIEW");
}

//剪切一条记录
function mcsstable_cut(tr,tableid)
{
    var mcsstable = mcsstable_getMCSSTable(tableid);
    if($(tr).attr("iscut")){
        //$(tr).css({"border":""});
        $(tr).removeAttr("iscut");
    }else{
        //$(tr).css({"border":"2px dashed #ccc"});
        $(tr).attr("iscut","true");
    }
    //$(mcsstable.table).find("tr[iscut!='true']").find(".mcsstable_record_paste").show();
}
//粘贴记录到指定的位置
function mcsstable_paste(tr,tableid,params)
{
    var cutArr = $("tr[iscut='true']");
    var tr=$(tr);
    var mcsstable = mcsstable_getMCSSTable(tableid);
    var orderField = mcsstable.modeldata.sortfieldid;
    //if (obj) var tr = $(obj).parent().parent();//黏贴按钮所在的行
    if(params && params.tr)
        tr = params.tr;

    if(cutArr.length==0){
        mcdom.alert("没有要粘贴的数据",'','info','fadeout');return;
    }
    //支持撤销
    if (tr) {
        if (cutArr.get(0).rowIndex < tr.get(0).rowIndex)
            mcsstable.addUndoList(mcsstable,"moveRowSortIndex",[tr.get(0).rowIndex-1,tr.get(0).rowIndex+cutArr.length-2,cutArr.get(0).rowIndex],mcsstable,"moveRowSortIndex",[cutArr.get(0).rowIndex,cutArr.get(cutArr.length-1).rowIndex,tr.get(0).rowIndex]);
        else
            mcsstable.addUndoList(mcsstable,"moveRowSortIndex",[tr.get(0).rowIndex,tr.get(0).rowIndex+cutArr.length-1,cutArr.get(0).rowIndex+1],mcsstable,"moveRowSortIndex",[cutArr.get(0).rowIndex,cutArr.get(cutArr.length-1).rowIndex,tr.get(0).rowIndex]);
    }

    for(var i = 0;i < cutArr.length;i++){
        var rowData = mcsstable.mcssData.getRow($(cutArr[i]).attr("recordid"));
        if (mcsstable.groupFieldid)
            mcsstable.last_groupfield_value=rowData[mcsstable.groupFieldid];
        if (mcsstable.last_groupfield_value==undefined)
            mcsstable.last_groupfield_value="[-fdsafdsafgdh345454325fgdsgfdsgfdgfds-]";

        var h = mcsstable_addonerow(rowData,tableid);
        var s=mcsstable_addfixrow(rowData,tableid);
        $(mcsstable.tableData.rows[tr[0].rowIndex]).before(h);
        $(tr).before(s);

        var newRow=$(tr).prev().get(0);
        mcsstable.setTrMoveFunc(newRow);
        mcsstable.setCurrentRow(newRow);

        mcsstable_bindMouseEventOnRows($(tr).prev().get(0),mcsstable);
        var current = $(tr).prev().get(0);
        if(mcsstable.params.afterAppendRow)
            mcsstable.params.afterAppendRow(mcsstable,current);
        if (mcsstable.cellEditor)
            mcsstable.cellEditor.createRowEditor(current);

        $(mcsstable.tableData.rows[cutArr[i].rowIndex]).remove();
        $(cutArr[i]).remove();

        mcsstable_setSortForNewRecord(mcsstable,newRow);
    }
    if(mcsstable.view=="celledit"){
        mcsstable.mcssData.save(mcsstable.params.afterSave,{alert:true,refObject:mcsstable});
        $(mcsstable.table).find("tr[mcss_recordtype='NEW']").attr('mcss_recordtype',"");
    }
    mcsstable.resortLineIndex();
    mcsstable_showNewRowActions(newRow,mcsstable.table);
}

function mcsstable_addOneRowToBottom(obj,tableid)
{
    //chenli prof todo
    mcsstable_addSomeRowsToBottom(obj,tableid,1);
}
function mcsstable_addSomeRowsToBottomMenu(obj,tableid)
{
    //chenli prof todo
    var h = ""
        +"<a num='10' onclick='mcsstable_addSomeRowsToBottom(this,\""+tableid+"\",5)'>添加5行</a>"
        +"<a num='10' onclick='mcsstable_addSomeRowsToBottom(this,\""+tableid+"\",10)'>添加10行</a>"
        +"<a num='20' onclick='mcsstable_addSomeRowsToBottom(this,\""+tableid+"\",20)'>添加20行</a>"
        +"<a num='50' onclick='mcsstable_addSomeRowsToBottom(this,\""+tableid+"\",50)'>添加50行</a>"
    dom_showPopupMenu(h,obj);
}
function mcsstable_addSomeRowsToBottom(obj,tableid,num)
{
    var mcsstable=mcsstable_getMCSSTable(tableid);
    if (mcsstable_canAddNewRecord(mcsstable)==false)
        return;
    var obj=mcsstable.getAction('appendrow');

    if ($(obj).attr("isadding")=="yes")
        return;

    $(obj).attr("isadding","yes").html("正在添加...");
    setTimeout(	function(){
        num=parseInt(num);
        if (num>50){
            num=50;
            mcsstable.getAction("addrowcount").value=num;
        }
        for(var i=0;i<num;i++)
        {
            mcsstable_addNewEditRow(mcsstable);
        }
        mcdom.popup("成功添加了"+num+"行",{obj:obj,hideheader:true,fadeOut:true});
        $(obj).attr("isadding","no").html("添加新行");
    },500);
}
//在当前行前面插入新记录
function mcsstable_insertRow(tr,tableid)
{
    var mcsstable=mcsstable_getMCSSTable(tableid);
    if (mcsstable_canAddNewRecord(mcsstable)==false)
        return;
    var r=tr.rowIndex;
    mcsstable.setCurrentRow(tr);

    mcsstable_addNewEditRow(mcsstable,true);

    var cellIndex=null;
    mcsstable.fixRowHeight(r);
    if (mcsstable.divSel && mcsstable.divSel.td ) {

        cellIndex= mcsstable.divSel.td.cellIndex;
        var fcols=mcsstable.modeldata.fixFields.length;
        if (fcols>0)
        {
            var tr=mcsstable.tableFixCol.rows[mcsstable.currentRow.rowIndex];
            var td=tr.cells[tr.cells.length-fcols];
        }else{
            var tr=mcsstable.tableData.rows[mcsstable.currentRow.rowIndex];
            var td=tr.cells[0];
        }
        setFocusToTd(td);
        if (mcsstable.cellEditor) mcsstable.cellEditor.clickCell(td);
    }
}
//创建打开记录的按钮文字链接
function mcsstable_getOpenActionHTML(mcsstable,openurl,recordid,wintitle,title,content,usetext)
{
    var c='smallbut mcsstable_record_open';
    if (usetext)
        c="";
    var url=mcsstable.params.homeUrl
        + openurl
        + "/"+mcsstable.keyfield+"/"
        + recordid;
    var s = "<a class='"+c+"' onclick=\"mcsstable_openWindow('"+url
        + "','"
        + wintitle
        + "' ,'"+mcsstable.modeldata.open_window_style+"')\" title="
        + title + ">" + content + "&nbsp;&nbsp;</a>";
    return s;
}
//根据指定url打开新窗口,已经被mcsstable_openWindow代替
function mcsstable_showpopup(url,title,open_window_style){
    mcsstable_openWindow(url,title,open_window_style);
}

//根据指定url打开新窗口
function mcsstable_openWindow(url,title,open_window_style){
    if (open_window_style=='samewindow')
    {
        document.location.href=url;return;
    }
    else if (open_window_style=='newwindow'){
        window.open(url,"_blank");
    }
    else
    {
        if (open_window_style=='popup:middle' || open_window_style=='popup'){
            url+="/newwinow/true";
            //ShowIframe(url, 700,500,title);
            mcdom.showIframe(url,{width:"700",height:"500",title:title});
        }
        else if (open_window_style=='popup:large'){
            url+="/newwinow/true";
            //ShowIframe(url, 900,700,title);
            mcdom.showIframe(url,{width:"900",height:"700",title:title});
        }
        else {
            //ShowIframe(url, 700,400,title);
            mcdom.showIframe(url,{width:"700",height:"400",title:title});
        }
    }
}


/*
 删除指定的几个记录id的行
 表格记录删除
 包括多种情形：
 1、立即删除、保存后才删除
 2、删除一条、多条
 3、点击记录上的删除按钮、工具栏的删除按钮
 如果是缓存删除，不用确认，否则确认
 无论缓存删除还是删除真正删除，都是先删除页面rows，处理好mcssData数据；
 如果是真正删除，后台执行后要更新mcssData
 */
function mcsstable_deleteSelctedRows(mcsstable,recordids,params) {
    mcdom.confirm("您确定删除这一行吗？",function(b){
        if(b) fundel(mcsstable,recordids,params);
    },null);
 
}
function fundel(mcsstable,recordids,params) {
    
    if (typeof(mcsstable) == 'string')
        mcsstable = mcsstable_getMCSSTable(mcsstable);
    if (mcsstable.params.project_fulldata && mcsstable.params.project_fulldata.shareoption && mcsstable.params.project_fulldata.shareoption.indexOf("[notdelrecord]") > -1) {
        mcdom.alert("表格共享时被设置为不允许删除记录。", lang.del, 'info', 'fadeout', '', '', {potision: 'middle'});
        return;
    }
    var idArr = recordids.split(",");
    if (typeof(mcsstable) == 'string')
        mcsstable = mcsstable_getMCSSTable(mcsstable);
    for (var i = 0; i < idArr.length; i++) {
        if (!mcsstable.parseEditCondition(idArr[i])) {
            mcdom.alert('不允许删除', '提示', 'success', 'fadeout');
            $(mcsstable.table).find("tr[recordid='" + idArr[i] + "']").addClass('row_willbedeleted');
            return;
        }
    }

    //询问确认
    if (mcsstable.view != 'tableedit') {
        if (mcsstable.params.confirmword)
            alarmWord = mcsstable.params.confirmword;
        else {
            if (idArr.length > 1)
                var alarmWord = "确认要删除这 " + idArr.length + " 条记录吗？";
            else {
                $(mcsstable.table).find("tr[recordid='" + idArr[0] + "']").addClass('selected-row');
                var alarmWord = "确认要删除当前记录吗？";
            }
        }
    }
    //var hint = alarmWord;
    //if (!hint)
    //    hint = "确认删除记录吗？";
    for (var i = 0; i < idArr.length; i++) {
        var row = $(mcsstable.table).find("tr[id='" + idArr[i] + "']");
        var record = {mcss_recordtype: "DELETE", recordid: idArr[i]};
        if (mcsstable.params.beforeDeleteRow) {
            if (mcsstable.params.beforeDeleteRow(mcsstable, idArr[i]))
                continue;
        }
        var rowData = mcsstable.mcssData.getRow(idArr[i]);
        //为支持撤销
        //var tableState={rowIndex:row.rowIndex-1,cellIndex:mcsstable.divSel.td.cellIndex};

        mcsstable.addUndoList(mcsstable, "addRow", [rowData, row.get(0).dataIndex], mcsstable, "deleteRow", [idArr[i]]);

        if (mcsstable.view == 'tableedit') {
            if (mcsstable.cellEditor)
                mcsstable.cellEditor.mcssData.deleteRow(record);
            row.remove();
            $("#" + mcsstable.getActionId('save')).css("border", "1px solid red");

            //把刚刚删除的记录的i的从mcsstable.selectedRowsID中移走
            var arr1 = mcsstable.selectedRowsID.split(",");
            var ids = "";
            for (var j = 0; j < arr1.length; j++) {
                if (arr1[j] != idArr[i]) {
                    if (ids)
                        ids += ",";
                    ids += arr1[j];
                }
            }
            mcsstable.selectedRowsID = ids;
        }
        if (row && row.get(0)) {
            var rowIndex = row.get(0).rowIndex;
            if (i == idArr.length - 1) {
                if (rowIndex == mcsstable.table.rows.length)//如果删除的是最后一行
                {
                    mcsstable.setCurrentRow(mcsstable.table.rows[rowIndex - 1]);
                }
                else
                    mcsstable.setCurrentRow(mcsstable.table.rows[rowIndex]);
            }

        }
    }
    mcsstable_clearSelect_td(mcsstable.table);

    //从数据库删除指定记录
    if (mcsstable.view == '' || mcsstable.view == 'table' || mcsstable.view == 'celledit') {
        mcsstable_deleteRowsFromDB(mcsstable, recordids);
    }
    mcsstable.resortLineIndex();

    //var _height1 = mcsstable.table.parentNode.clientHeight;
    //mcsstable.table.parentNode.style.height= (_height1+30) + "px";

    mcsstable.params.totalRecordCount = parseInt(mcsstable.params.totalRecordCount) - idArr.length;
    window.setTimeout(function () {
        var tb = parseInt(mcsstable.table.offsetTop) + parseInt(mcsstable.table.clientHeight) + 60;
        mcsstable.table.parentNode.style.height = tb + "px";
    }, 300);
}

//工具栏上的删除按钮操作
function mcsstable_deleterows(e,tableid,recordid,params)
{
    var mcsstable=mcsstable_getMCSSTable(tableid);
    if (mcsstable.selectedRowsID=='')
    {
        //wx_alert('','请先勾选要操作的记录',fadeOut());
        mcdom.alert("请先勾选要操作的记录。",lang.del,'info','fadeout','','',{obj:e});
        return false;
    }
    mcsstable_deleteSelctedRows(mcsstable,mcsstable.selectedRowsID,params);
}

//记录行上的删除按钮
function mcsstable_deleteOneRow(e,tableid,params)
{
    var mcsstable = mcsstable_getMCSSTable(tableid);
    var recordid=$(mcdom.getObjByTagName(e,"TR")).attr('recordid');
    if (mcsstable.params.beforeDeleteRow)
    {
        if(mcsstable.params.beforeDeleteRow(mcsstable,recordid))
            return;
    }
    if (!recordid)
    {
        return false;
    }

    mcsstable_deleteSelctedRows(tableid,recordid,params);
}

//直接删除数据库中指定的记录，不询问
function mcsstable_deleteRowsFromDB(mcsstable,deleteIDs)
{
    var urlpath = g_homeurl+"/System/Model/deleterows";
    if (mcsstable.modeldata.delurl)
        urlpath = g_homeurl+"/"+mcsstable.modeldata.delurl;
    var pid="";
    if (mcsstable.params.project_fulldata)
        pid=mcsstable.params.project_fulldata.project_record.id;
    $.post(urlpath,	{modelid:mcsstable.modelid,rowIds:deleteIDs,projectid:pid},function(data){
        if (data.indexOf("ok")>-1)
        {
            //删除前台table的行，处理mcssData
            var idArr=deleteIDs.split(",");
            mcsstable.selectedRowsID="";

            for(var i=0;i<idArr.length;i++)
            {
                //不用加载所有数据的做法，但会导致判断最后一页出问题
                if (idArr[i]+""!=""){
                    var row=$(mcsstable.tableFixCol).find("tr[id='"+idArr[i]+"']");
                    var cells = row.get(0).cells;

                    for(var j=0;j<cells.length;j++)
                    {
                        cells[j].style.height=row.clientHeight+"px";
                    }
                    var cells = mcsstable.tableData.rows[row[0].rowIndex].cells;
                    for(var j=0;j<cells.length;j++)
                    {
                        cells[j].style.height=row.clientHeight+"px";
                    }
                    row.get(0).style.opacity="0";
                    setAction(row.get(0),0.1);
                }
            }

            window.setTimeout(function(){
                for(var i=0;i<idArr.length;i++)
                {
                    if (idArr[i]+""!=""){
                        var row=$(mcsstable.tableFixCol).find("tr[id='"+idArr[i]+"']");
                        var cells = row.get(0).cells;

                        for(var j=0;j<cells.length;j++)
                        {
                            cells[j].innerHTML="";
                        }
                        var cells = mcsstable.tableData.rows[row[0].rowIndex].cells;
                        for(var j=0;j<cells.length;j++)
                        {
                            cells[j].innerHTML="";
                        }
                    }
                }
            },200);
            window.setTimeout(function(){
                for(var i=0;i<idArr.length;i++)
                {
                    if (idArr[i]+""!=""){
                        var row=$(mcsstable.tableFixCol).find("tr[id='"+idArr[i]+"']");
                        var cells = row.get(0).cells;

                        for(var j=0;j<cells.length;j++)
                        {
                            cells[j].innerHTML="";
                            cells[j].style.height="0px";
                            setAction(cells[j],0.3);
                        }

                        var cells = mcsstable.tableData.rows[row[0].rowIndex].cells;
                        for(var j=0;j<cells.length;j++)
                        {
                            cells[j].innerHTML="";
                            cells[j].style.height="0px";
                            setAction(cells[j],0.3);
                        }
                        row.get(0).style.height="0px";
                        mcsstable.tableData.rows[row[0].rowIndex].style.height="0px";
                        setAction(row.get(0),0.3);
                        setAction(mcsstable.tableData.rows[row[0].rowIndex],0.3);
                    }
                }
            },210);
            window.setTimeout(function(){
                for(var i=0;i<idArr.length;i++)
                {
                    if (idArr[i]+""!=""){
                        var row=$(mcsstable.tableFixCol).find("tr[id='"+idArr[i]+"']");
                        var rowdata=mcsstable.tableData.rows[row[0].rowIndex];
                        row.remove();
                        $(rowdata).remove();
                        if(mcsstable.params.afterDeleteRow)
                            mcsstable.params.afterDeleteRow(mcsstable,idArr[i]);
                    }
                }
                mcsstable.resortLineIndex();

            },500);
            //wx_alert('','删除成功!',fadeOut());
            //mcdom.popup('删除成功！',{fadeOut:true,left:mcdom.getPosition(mcsstable.table).left+"px",top:'0px',hideheader:true});
        }
        else
        {
            alert(data);
        }
    });

}

function mcsstable_createToolbar(tableid){
    var mcsstable=mcsstable_getMCSSTable(tableid);
    var data=mcsstable.modeldata["fields"];
    var params=mcsstable.params;
    var modeldata=mcsstable.modeldata;
    var actions="";

    if (params.toolbar) {
        actions = params.toolbar;
    }
    else
        actions =modeldata.actions;
        
    if (!actions)
        actions='';

    if (actions=="all") {
        actions="view,open,add,edit,batchedit,tableedit,del,search,search2,filter,filter2," +
            "autowidth,groupby,import,export,viewtype,print";
    }
    params.toolbar=actions;
    mcsstable.actions=actions;

    
    if (actions) {
        var s = "";
        if (params.showtitle)
        {
            s+="<span id='"+mcsstable.id+"_action_title' class='mcssTable_title'>["+modeldata.title+"]</span>";
        }

        var tools = params.toolbar+',';
        var actionArr=tools.split(",");
        var action="",actionid='';
        if (params.showShareTools)
        {
            var n="";
            if (params.project_fulldata && params.project_fulldata.todayDiscussion>0){
                n=params.project_fulldata.todayDiscussion;
            }
            s += "<div class='uni-cooperation'>"
                    //+"<a id='"+mcsstable.tableid+"_action_share' onclick='main_sharePage(this,\""+tableid+"\")' class='btn btn-sm icon-share' title='把当前文件共享给他人查看或修改'>共享</a>"
                +"<a id='"+mcsstable.tableid+"_action_publish' onclick='mcsstable_displayPublish(this,\""+tableid+"\")' class='btn  btn-sm icon-share' style='font-size:14px' title='把这个表格发布、共享给他人' >发布共享</a>";
            +"<a id='"+mcsstable.tableid+"_action_export' onclick='main_exportSheet(this,\""+tableid+"\")' class='btn btn-sm icon-download' title='把表格导出到Excel文件' >导出</a>"
            +"<a id='"+mcsstable.tableid+"_action_comment' onclick='mcsstable_showDiscussion(this,\""+tableid+"\")' class='btn btn-sm icon-bubble'  title='显示或添加对当前文件的评论' >评论<span id='newdiscussioncount'>"+n+"</span></a>";
            s+="</div>";
        }

        for(var i=0;i<actionArr.length;i++)
        {
            action=actionArr[i];
            actionid=mcsstable.getActionId(action);
            if (action=="add") {
                //add 按钮添加
                var tmp=modeldata.addurl;
                if (!tmp || tmp=="default")
                    tmp=g_systempath+"/addRecord";
                if (tmp)
                    tmp+="/modelid/"+params.modelid;
                var url=params.homeUrl+tmp;
                if (params.defaultValue)
                {
                    url+="/defaultValue/"+params.defaultValue;
                }
                var url=mcsstable.getAddUrl();
                s += "<a id='"+actionid+"'  onclick=\"mcsstable_addnew(this,'"+tableid+"','"+url+"','"+lang.add+"') \" ";
                s+=mcsstable_getActionStyle(mcsstable,action,lang.add,'添加记录(ctrl+g)');
            }
            else if (action=="tableedit") {
                if (action=="tableedit" && (mcsstable.view=="tableedit" || mcsstable.view=="celledit"))
                {
                    var s1 = "<a id='"+mcsstable.getActionId('save')+"' onclick=\"mcsstable_save('"+tableid+"',this) \" ";
                    if (mcsstable.view!="tableedit")
                        s1+=" style='display:none'";
                    s1+=mcsstable_getActionStyle(mcsstable,lang.save,lang.save,'保存(ctrl+s)');
                    s+=s1;
                }
            }
            else if (action=="del" ) {
            }
            else if(action=="batchedit"){
                s += "<a  id='"+actionid+"' onclick='mcsstable_showBatchMenu(this,\""+tableid+"\",\""+params.modelid+"\")' class='btn  btn-sm' title='批量处理行记录' >行操作...</a>";
            }
            else if(action=="styleedit"){
                s += "<a id='"+actionid+"_show' onclick='mcsstable_showStyleEditorBar(this,\""+tableid+"\",\""+params.modelid+"\")' class='btn btn-sm' title='显示隐藏格式化按钮栏' >格式<span class='icon-arrow-right4'></span></a>";
                s += "<div id='"+actionid+"' class='styleeditorcainter' style='display:none'>";


                s += "<ul>"
                        // +"<li class='icon-scissors' title='剪切'></li>"
                        // +"<li class='icon-copy' title='复制'></li>"
                        // +"<li class='icon-paste' title='粘贴'></li>"
                    +"<li class='styleeditor styleeditor-Bold' csstype='bold' title='加粗' onclick='mcsstable_setBold(this,\""+tableid+"\")'></li>"
                    +"<li class='styleeditor styleeditor-Italic' csstype='italic' title='斜体' onclick='mcsstable_setItalic(this,\""+tableid+"\")'></li>"
                    +"<li class='styleeditor styleeditor-Underline' csstype='underline' title='下划线' onclick='mcsstable_setUnderline(this,\""+tableid+"\")'></li>"
//				+"<li class='styleeditor styleeditor-FontName' csstype='fontname' title='字体样式' onclick='mcsstable_setFontFace(this,\""+tableid+"\")'></li>"
                    +"<li class='styleeditor styleeditor-FontSize' csstype='fontsize' title='字号' onclick='mcsstable_setFontSize(this,\""+tableid+"\")'></li>"
                    +"<li class='styleeditor styleeditor-ForeColor' csstype='fontcolor' title='前景色' onclick='mcsstable_createFontColorSelector(this,\""+tableid+"\")'></li>"
                    +"<li class='styleeditor styleeditor-BackColor' csstype='backcolor' title='背景色' onclick='mcsstable_createBackgroundColorSelector(this,\""+tableid+"\")'></li>"
                    +"<li class='styleeditor styleeditor-AlignMode' csstype='alignmode' title='对齐方式' onclick='mcsstable_setFontPosition(this,\""+tableid+"\")'></li>"
                s +="</ul></div>";
            }
            else if (action=="refresh" ) {
                s += "<a id='"+actionid+"' onclick='mcsstable_refresh(\""+tableid+"\",\""+params.modelid+"\",this)' class='btn-sm uni-refresh' title='重新加载当前条件下的数据' >刷新</a>";
            }
            else if (action=='searchnow') {
            }
            else if (action=='view') {
                s+="<div style='position: absolute;right: 5px;top: 5px;text-align:center;color:#666; padding: 3px;width: 28px;height:25px;border:1px solid #ccc;' onclick='mcsstable_popupViewMenu(\""+params.tableid+"\",this)'>";
                s+="<image src='myImg/filter.png' style='width:1.3em;height:1.3em;margin-top:-1px;'></image>";
                s+="<select id='"+actionid+"' class='form-control input-sm btn-sm uni-viewlist' style='display:none;border:none;background-color:#f7f7f7;padding-left:5px;padding-top:px;width:100%;height:30px;' onchange=\"mcsstable_openQuery('"+actionid+"','"+params.tableid+"',this)\" title='打开、新建或管理视图'><option>--</option></select>";
                
                s+="</div>";
                //如果有高级搜索按钮，则加载高级搜索js代码
            }
            else if (action=='search2') {
            }
            else if (action=='search') {
                mcss_importJS("jusaas/js/mcsstable_advancesearch.js");
                s += "<div id='" + actionid+"' style='width:88%;margin-left:1%;margin-top:1px;;' class='uni-search'>"
                    +"<i id='"+actionid+"_searchbtn' style='color:#bbb' class='icon-search' title='回车搜索' onclick='mcsstable_search(\""+tableid+"\")'></i>"
                    +"<input id='"+mcsstable.searchinput_Id+"' type='search' style='width:100%;height:27px;font-size:0.9em !important' class='form-control input-sm' placeholder='"+lang.inputtosearch+"...' title='"+lang.search_title+"' onfocus='document.body.scrollLeft=0;' onkeydown='mcsstable_onsearchkeyup(event,\""+params.tableid+"\",this)' onchange='mcsstable_onsearchchange(\""+params.tableid+"\",this) ' onfocus='mcsstable_searchonfocus(\""+tableid+"\",this)' />"
                    +"<i id='"+actionid+"_clearbtn'  class='icon-cross' title='重置搜索' onclick='mcsstable_clearAndSearch(\""+tableid+"\",this)' style='display:none;width:23px;height:23px;position: relative;top:4px;margin-left: -24px;font-size:21px !important;'></i>";
                if (actions.indexOf('search2')>-1)
                    s+="<a id='"+mcsstable.getActionId("search2")+"' onclick='mcsstable_advancedSearch(\""+tableid+"\",this)' title='高级搜索' class='btn btn-sm' style='display:none;'>高级</a>";
                s+="</div>";
                //title='例1：北京：表示搜索包含“北京”字眼的记录\r\n例2：北京*公司：表示搜索同时包含“北京”和“公司”字眼的记录\r\n例3：北京*公司 网站：表示搜索同时包含“北京”和“公司”字眼，或者包含“网站”字眼的记录'
            }
            else if (action=='filter') {
                //等哪天想恢复再解锁
                //s += mcsstable.ceateFilter();
            }
            else if (action=='filter2') {
                mcss_importJS("jusaas/js/mcsstable_filterbar.js");
                s += "<a  id='"+actionid+"' onclick='mcsstable_createFilterBar(\""+params.tableid+"\")' ";
                s+=mcsstable_getActionStyle(mcsstable,action,'高级过滤器');
            }
            else if (action=='autowidth') {
                s += "<a  id='"+actionid+"' onclick='mcsstable_autowidth(\""+params.tableid+"\")'";
                s+=mcsstable_getActionStyle(mcsstable,action,lang.adjust_width,lang.adjust_width+":"+lang.nofoldlines+"/"+lang.compact);
            }
            else if (action=='export') {
                if (!params.showShareTools){
                    s += "<a  id='"+actionid+"' onclick='mcsstable_displayExportMenu(this,\""+params.tableid+"\")'";
                    s+=mcsstable_getActionStyle(mcsstable,action,lang.exporttofile,lang.ordertofile_title);
                }
            }
            else if (action=='print') {
                s += "<a  id='" +actionid+"' onClick='mcsstable_print(\""+params.tableid+"\")' ";
                s+=mcsstable_getActionStyle(mcsstable,action,lang.print,lang.print);
            }
            else if (action=='import') {
                //s += "<a  id='"+actionid+"' onClick='mcsstable_import(\""+params.tableid+"\")' ";
                //s+=mcsstable_getActionStyle(mcsstable,action,lang.mcssimport,lang.mcssimport_title);
            }
            else if (action=='groupby') {
                //s+=mcsstable_getRecordGroupList(mcsstable,action);//移动到在创建表头第一列时显示
            }
            else if (action=='page' ) //暂时不用，懒加载不支持。ckj
            {
                if (mcsstable.params.pageposition!='rightdown' && mcsstable.params.pageposition!='leftdown')
                    s += mcsstable_page(params.tableid);
            }
            else if (action=='viewtype') {
            }
            else
            {
                if (action && action!='open' && action!='edit')
                {
                    s += "<a  id='" + actionid+"' onClick='if (mcsstable_exeAction) mcsstable_exeAction(\""+action+"\",\""+mcsstable.tableid+"\")'";
                    s+=mcsstable_getActionStyle(mcsstable,action,action);
                }
            }
        }

        if (mcsstable.viewtype=="calendar")
            mcss_importJS("jusaas/calendar/McssCalendar.js");

        $("#" + mcsstable.table_toolbar_id).html(s);


        $("#" +tableid+"_pagerows").val(mcsstable.modeldata.pagerows);
    }



    if (mcsstable.view_id)
        var view_id=mcsstable.view_id;
    else
    {
        var view_id=mcsstable_getFirstShareView(mcsstable);
    }
    mcsstable.loadViewList(null,view_id);

    //创建页脚按钮
    if (mcsstable.params.showConfirmButton){
        var h="<a class='setSubmit2 btn-primary' href='javascript:void(0)' id='"+tableid+"_confirm' onclick='returnSelectedRows(\""+tableid+"\")'>确定所选</a>";
        $("#"+params.tableid).after(h);
    }

    if(mcsstable.params.afterCreateToolbar){
        mcsstable.params.afterCreateToolbar(mcsstable);
    }

}
//显示菜单
function mcsstable_displayPublish(e,tableid){
    var mcsstable=mcsstable_getMCSSTable(tableid);
    var h = ""
        +"<a id='"+mcsstable.tableid+"_action_webform' onclick='main_sharePage(this,\""+tableid+"\") ' title='发布共享链接，把当前文件共享给他人查看或修改' ><span class='icon-hyperlink'></span>共享表格数据：让他人查看或修改</a>"
        +"<a id='"+mcsstable.tableid+"_action_webform' onclick='main_publishForm(this,\""+tableid+"\")' title='发布表单链接，供他人添加数据' ><span class='icon-hyperlink'></span>发布空表单：让他人填写新数据</a>"
        +"<a id='"+mcsstable.tableid+"_action_webformSearch' onclick='main_publishSearch(this,\""+tableid+"\")'  title='发布搜索页面，供他人搜索数据' ><span class='icon-search'></span>共享搜索页面：让他人搜索表格数据</a>";
    dom_showPopupMenu(h,e,{width:'280px'});

    
    //记录四个菜单项的点击数 click_publish_share_menu,click_sharePage_menu,click_publishForm_menu,click_publishSearch_menu
}

//获得共享视图列表中的第一个视图
function mcsstable_getFirstShareView(mcsstable){
    var view_id="";
    var shareviews=mcsstable.getShareViews();
    if (shareviews)
    {
        view_id=shareviews.split(",")[1];
    }
    return view_id;
}



function mcsstable_searchonfocus(tableid,obj)
{
    //obj.style.width="200px";
    var mcsstable=mcsstable_getMCSSTable(tableid);
    mcsstable.showAction("search2");
}

//创建生成分组下拉列表的html代码
function mcsstable_getRecordGroupList(mcsstable,action)
{
    var s='',params=mcsstable.params;
    s +="<select  id='"+mcsstable.getActionId(action)+"' class='mcsstable_select form-control input-sm' title='"+lang.viewbygroup+"' onchange='mcsstable_setGroup(this,\""+params.tableid+"\")' >";
    if (mcsstable.modeldata.groupby=='y' && mcsstable.groupFieldid)
        s += "<option value='' >--"+lang.defaultgroup+"--</option>";
    else
        s += "<option value='' selected='true'>--"+lang.defaultgroup+"--</option>";
    $.each(mcsstable.modeldata.fields,function(i, v) {
        if (v.isvisible != "false")
        {
            if(lang[v.name] && isNaN(v.name))
                v.name=lang[v.name];
            if(v.name)
            {

                if (mcsstable.modeldata.groupby=='y' && mcsstable.groupFieldid==v.id)

                    s += "<option value='" + v.id + "' selected='true'>" + v.name + "</option>";

                else

                    s += "<option value='" + v.id + "'>" + v.name + "</option>";

            }
        }
    });
    s += "<option value='NONEGROUP'>--"+lang.nonegroup+"--</option>";
    s +="</select>"
        +"<a id='"+params.tableid+"_action_"+action+"_orderby'  class='icon-arrow-up' onclick='mcsstable_setOrderDirection(\""+params.tableid+"\",this)' id='"+params.tableid+"_action_sort'  title='升序'></a>"
        +mcsstable.getShowHideGroupBtn(mcsstable.tableid);
    return s;
}


//打开某个历史查询
function mcsstable_openQuery(select_id,tableid,e)
{
    var mcsstable=mcsstable_getMCSSTable(tableid);
    mcsstable.isStatView=false;
    if ($("#"+select_id).find("option:selected").attr('tag')!='USEROPTION')
    {
        var id=$("#"+select_id).val();
        if (id=='DEFAULT')
        {
            mcsstable.view_id='DEFAULT';
            mcsstable.run();
        }
        else if (id!='NONE')
        {
            mcsstable.openView(id);
        }
    }

    window.setTimeout(function(){ mcsstable_refreshTeamInfo(mcsstable);},1500);
}

function mcsstable_view_afterRun(unitable)
{
    $(unitable.table.rows[0].cells[0]).css("width","40px");
}


function mcsstable_view_afterLoadRows(mcsstable)
{
    $(mcsstable.table).find("td[id='isdefault']").find('input:checkbox').change(function(){
        if (this.checked==true)
        {
            var rows=mcsstable.mcssData.data;
            for(var i=0;i<rows.length;i++)
            {
                var recordid=rows[i][mcsstable.mcssData.keyfield];
                if (recordid!=$(this.parentNode.parentNode).attr('recordid'))
                {
                    mcsstable.editField(recordid,'isdefault',false,true);
                }
            }
            //this.checked=true;
        }
    })

//	var tr="<tr><td></td><td>默认视图</td><td></td><td><input type='checkbox' id='view_defaultview' onchange='mcsstable_publishDefaultview(this)'  /></td></tr>";
//	$(mcsstable.table).append(tr);

}

function mcsstable_publishDefaultview(e)
{
    if (e.id=='view_defaultview')
    {

    }
}
//单击视图整理表格的保存按钮后
function manageview_aftersave(mcssdata,mcsstable)
{
    var defaultView = $(mcsstable.table).find("td[id='isdefault']").find("input:checked").parent().parent().attr("recordid");
    mcsstable.loadViewList(mcsstable.params.refObject,defaultView,true);
    mcdom.closePopup();
}

//保存新视图
function mcsstable_saveasview(tableid,obj)
{
    mcdom.temp_tableid=tableid;
    var str="保存视图后，下次打开该视图后，将还原表格<br />当前的显示列、过滤条件、排序和分组。"
        +"<span id='a_viewhelp' title='什么是视图？' class='icon-question'  onclick='mcsstable_showViewHelp(this)'></span>"
        +"<br />请输入视图名称："
    //mcdom.input(mcsstable_saveViewToDB,"",{obj:obj,title:"保存为新视图",css:"width:400px;left:" + (document.clientWidth-800) + "px;" ,description:str});
    mcdom.input(mcsstable_saveViewToDB,"",{title:"保存为新视图",description:str,position:'middle'});

}

function mcsstable_showViewHelp(obj)
{
    common_showHelp(obj,"tzwcmvro",{width:"500px",height:'400px',openNewWindow:true});
}
//撤销执行的动作
function mcsstable_callreturn(tableid){
    var mcsstable = mcsstable_getMCSSTable(tableid);
    var actArr = mcsstable.callbackArr;

}

function mcsstable_saveViewToDB(name)
{
    if(!name){
        mcdom.alert('视图名称不能为空','提示','success','fadeout');
        return;
    }
    var mcsstable=mcsstable_getMCSSTable(mcdom.temp_tableid);
    var query=mcsstable.advancedFilter;
    if (!query)
        query="";
    var autofilter='';
    if (mcsstable.autoFilter)
        autofilter=mcsstable.autoFilter;
    var groupby=mcsstable.groupFieldid;
    if (groupby==undefined)
        groupby='';
    else
    if (mcsstable.group_order_updown)
        groupby+=" "+mcsstable.group_order_updown;
    var fields='',options='',isShowAll='';
    if(mcsstable.modeldata.viewTypeShowAll=='yes')
        isShowAll = 'yes';
    if(mcsstable.modeldata.viewType=='stat'){
        fields=statview_getStatFilter();
        options='stat';
    }else{
        var fid;
        var fds = mcsstable.fields;
        for(var i=0;i<fds.length;i++)
        {
            fid=fds[i];
            if (fid){
                if (fields) fields+=",";
                fields+=fid;
            }
        }
    }
    var sortby=mcsstable.orderFieldid;
    if (!sortby)
        sortby='';
    else
    if (mcsstable.field_order_updown)
        sortby+=" "+mcsstable.field_order_updown;
    var recordid='';
    if (mcsstable.params.project_fulldata)
        recordid=mcsstable.params.project_fulldata.project_record.id;
    name=uni_cleanSQL(name);
//	query = ' projectid = ' + recordid +' and (' +query +')';
    query=uni_cleanSQL(query);
    mcsstable.searchword=uni_cleanSQL(mcsstable.searchword);
    autofilter=uni_cleanSQL(autofilter);
    $.post(g_homeurl+"/System/Model/saveasview",{isShowAll:isShowAll,options:options,name:name,modelid:mcsstable.modelid,usermodel_recordid:recordid,query:query,sortby:sortby,groupby:groupby,fields:fields,searchword:mcsstable.searchword,autofilter:autofilter,owner:mcsstable.params.model_ownerid},function(new_id){
        if (new_id>0){
            mcdom.popup(lang.successfullysaved,{fadeOut:true,position:'middle'});
            mcsstable.loadViewList(null,new_id,true);
        }
        else if(new_id==-1){
            alert("该视图名已存在，请重新输入。");
            var str="保存视图后，下次打开该视图后，将还原表格<br />当前的显示列、过滤条件、排序和分组。"
                +"<span id='a_viewhelp' title='什么是视图？' class='icon-question'  onclick='mcsstable_showViewHelp(this)'></span>"
                +"<br />请输入视图名称："
            mcdom.input(mcsstable_saveViewToDB,name,{title:"保存为新视图",description:str,position:'middle'});
            return;
        }
        else
            mcdom.popup("保存失败!",{fadeOut:true});
    })
    return true;
}

//显示菜单
function mcsstable_displayExportMenu(e,tableid){
    var h = ""
        +"<a  onclick='mcsstable_export(\""+tableid+"\",\"xls\")' class='popupmenu_xls'>2003Excel(.xls)</a>"
        +"<a  onclick='mcsstable_export(\""+tableid+"\",\"xlsx\")' class='popupmenu_xlsx'>2007Excel(.xlsx)</a>"
        +"<a  onclick='mcsstable_export(\""+tableid+"\",\"csv\")' class='popupmenu_csv'>CSV"+lang.file+"(.csv)</a>";

    var mcsstable=mcsstable_getMCSSTable(tableid);
    if (mcsstable && mcsstable.params.exportMenus){
        h+=mcsstable.params.exportMenus;
    }
    dom_showPopupMenu(h,e,{needpicture123:true,width:'150px',position:'after'});

}

function mcsstable_removeActionMenu()
{
}
function mcsstable_getActionStyle(mcsstable,action,text,title)
{
    var s="";
    if (!title)
        title=text;
    if (mcsstable.params.actionStyle=='ICON')
    {
        s+=" class='actionicons "+action +"' title='"+title+"'>";
    }
    else
    {
        s+=" class='btn-sm' title='"+title+"'>";
        s+=text;
    }
    s+="</a> ";
    return s;
}

//字段合计值处理
var mcsstable_sumvalueByGroup=new Array();
var mcsstable_sumvalueAll=new Array();

//选择某个分组依据时
function mcsstable_setGroup(e,tableid){
    var mcsstable=mcsstable_getMCSSTable(tableid);
    if (mcsstable.ifDataChanged()){
        mcdom.alert(lang.pleasesaveatfirst,'','','fadeout','','',{seconds:5});
        this.value='';
        return;
    }

    if ($(e).find("option:selected").attr('tag')!='USEROPTION')
    {
        var fieldid=$("#"+tableid+"_action_groupby").val();
        mcsstable.setGroupField(fieldid);
        mcsstable_loaddatarows(tableid);
    }
}

//生成过滤器
MCSSTable.prototype.ceateFilter=function()
{
    var s = "<a onclick='mcsstable_fastSearchSet(this,\""+this.tableid+"\")' id='"+this.tableid+"_action_filter' class='labellink456 btn btn-default btn-sm  text_tableedit_btn' title='设置快速过滤列'>快速过滤</a>";
    return s;
}
//解析是否可编辑条件
MCSSTable.prototype.parseEditCondition=function(recordid)
{
    var record = this.mcssData.getRow(recordid);
    return  ((this.params.project_fulldata && this.params.project_fulldata.project_record.SYS_ADDUSER ==loginuser) ||  this.mcssData.canEditRecord(record));
}
//生成表格工具栏按钮的i的
MCSSTable.prototype.getActionId=function(action)
{
    return this.params.tableid+"_action_"+action;
}
//设置快速搜索页面
function mcsstable_fastSearchSet(obj,tableid){
    var mcsstable = mcsstable_getMCSSTable(tableid);
    var fields = mcsstable.modeldata.fields;
    var h = "<font>选择的列会在列菜单中增加“自动过滤”项</font><hr>";
    var fcount=0;
    for(var i=0;i<fields.length;i++)
    {
        if (fields[i].isvisible!='false')
            fcount++;
    }
    if (fcount>20)
        h+="<br><a style='float:right;' class='btn btn-primary btn-sm' title='确定' onclick='mcsstable_saveFastSearch(\""+tableid+"\")'>确定</a>";
    h+="<input type='checkbox' onclick='mcsstable_selectAllCheck(\"mcsstable_fastSearch\",this)'/> 全选 <br><br>";
    for(var i = 0;i < fields.length;i++){
        if(fields[i].isvisible!="false"){
            h+="<input class='mcsstable_fastSearch' type='checkbox' fieldid='"+fields[i].id+"'";
            if(fields[i].prop && fields[i].prop.indexOf('SHOWINFILTER')>-1)
                h+=" checked='checked'";
            h+=" />"+fields[i].name+"<br />";
        }
    }
    h+="<br><a style='float:right;' class='btn btn-primary btn-sm' title='确定' onclick='mcsstable_saveFastSearch(\""+tableid+"\")'>确定</a>";
    var pop=mcdom.showPopup(obj,h,"",null,null,"",270,"设置自动过滤字段");
    mcdom.moveToScreen(pop);
}
//保存快速搜索字段
function mcsstable_saveFastSearch(tableid){
    var mcsstable = mcsstable_getMCSSTable(tableid);
    var fieldids = "",arr = $(".mcsstable_fastSearch");
    for(var i = 0;i < arr.length;i++){
        if($(arr[i]).prop("checked")){
            if(fieldids)
                fieldids+=",";
            fieldids+=$(arr[i]).attr("fieldid");
        }
    }
    $.post(g_homeurl+"/System/ModelMaker/saveFastSearch",{fieldids:fieldids,modelid:mcsstable.params.modelid,usermodel_recordid:mcsstable.params.usermodel_recordid},function(){
        mcdom.closePopup();
        mcdom.alert('设置成功','提示','info','fadeout');
        mcsstable_setFastSearchProperty(fieldids,mcsstable);
    })
}

//为字段追加上属性
function mcsstable_setFastSearchProperty(fieldids,mcsstable){
    var arr = "";
    if(fieldids) arr = fieldids.split(",");

    var fields = mcsstable.modeldata.fields;
    for(var k = 0;k < fields.length;k++){
        if(fields[k].isvisible!="false"){
            mcsstable.modeldata.fields[k].prop = mcsstable_removeFastSearchProperty(mcsstable,k);
            for(var i = 0;i < arr.length;i++){
                if(arr[i] == fields[k].id){
                    if(mcsstable.modeldata.fields[k].prop)
                        mcsstable.modeldata.fields[k].prop+=";";
                    mcsstable.modeldata.fields[k].prop+="SHOWINFILTER";
                    break;
                }
            }
        }
    }
}
function mcsstable_removeFastSearchProperty(mcsstable,k){
    var prop = mcsstable.modeldata.fields[k].prop;
    if(prop && prop.indexOf("SHOWINFILTER")==-1 || !prop)
        return prop;
    var arr = prop.split(";");var str = "";
    for(var i = 0;i < arr.length;i++){
        if(arr[i]=="SHOWINFILTER")
            continue;
        if(str)
            str+=";";
        str+=prop[i];
    }
    return str;
}
//选择全部checkbox
function mcsstable_selectAllCheck(condition,obj){
    if($(obj).prop("checked"))
        $("."+condition).attr("checked","checked");
    else
        $("."+condition).removeAttr("checked");
}
//根据字段id获得其在表格中的顺序
MCSSTable.prototype.getFieldIndex=function(fieldId)
{
    var cells=this.table.rows[0].cells;
    for(var i=0;i<cells.length;i++)
    {
        if ($(cells[i]).attr('fieldid')==fieldId)
        {
            return i;
        }
    }
    return -1;
}

MCSSTable.prototype.getRow=function(recordid)
{
    var rows=$("#"+this.tableid).get(0).rows;
    for(var i=0;i<rows.length;i++)
    {
        if ($(rows[i]).attr('recordid')==recordid)
        {
            return rows[i];
        }
    }
    return null;
}

//下列列表过滤
function mcsstable_changeOption(obj,tableid)
{
    var mcsstable=mcsstable_getMCSSTable(tableid);
    if (mcsstable.ifDataChanged()){
        mcdom.alert(lang.pleasesaveatfirst,'','','fadeout','','',{seconds:5});
        this.value='';
        return;
    }

    var autoFilter="";
    if (obj.value=="ALL")
        autoFilter="";
    else
    {
        var arr=obj.value.split(":");
        autoFilter=arr[0]+"='"+arr[1]+"'";
    }
    mcsstable.autoFilter=autoFilter;//记录autoFilter对其它复杂过滤有用。例如在pm.js中

    mcsstable_loaddatarows(mcsstable.tableid);
}

function mcsstable_page(tableid){
    var s="";
    var mcss_table = mcsstable_getMCSSTable(tableid);
    var defaultPage = 10;
    var pagerow = mcss_table.modeldata.pagerows;
    if(pagerow && pagerow>=-1)
        defaultPage = pagerow;
    if (mcss_table.params.pageoptions)
        var pagerows=mcss_table.params.pageoptions.split(",");
    else
        var pagerows = [5,10,20,50,100,200,500,-1];
    s+="<a  class='icon-first123' id='"+mcss_table.getActionId('firstpage')+"' onclick='mcsstable_firstpage(this,\""+tableid+"\");'  style='display:none'";
    s+=mcsstable_getActionStyle(mcss_table,'firstpage','首页',lang.firstpage);

    s+="<a class='icon-previous123' id='"+mcss_table.getActionId('prepage')+"'  onclick='mcsstable_prepage(this,\""+tableid+"\")' ";
    s+=mcsstable_getActionStyle(mcss_table,'prepage','上页',lang.priorpage);

    s+="<input id='"+mcss_table.getActionId('currentpage')+"' class='form-control input-sm current-page' onkeyUp='mcsstable_gotoPagex("+'"'+tableid+'"'+",this.value,event)' title='请输入要跳转的页数，敲回车跳转' value='1'  style='display:none'/>";
    if (mcss_table.params.showRecordCount)
    {
        s+="<span  style='display:none'>/</span>";
        s+="<span id='"+mcss_table.getActionId('totalpage')+"' title='总页数'  style='display:none'>1</span>";
    }

    s+="<a class='nextpage' id='"+mcss_table.getActionId('nextpage')+"' onclick='mcsstable_nextpage(this,\""+tableid+"\")' ";
    s+=mcsstable_getActionStyle(mcss_table,'nextpage','下页',lang.nextpage);

    s+="<a class='icon-last123' id='"+mcss_table.getActionId('lastpage')+"' onclick='mcsstable_lastpage(this,\""+tableid+"\")'  style='display:none'";
    s+=mcsstable_getActionStyle(mcss_table,'lastpage','末页',lang.lastpage);

    s+="<select id='"+mcss_table.getActionId('pagerows')+"' class='form-control input-sm'  onChange='mcsstable_changepagerows(this,\""+tableid+"\")' title='每页记录数'  style='display:none'>";
    for(var i =0;i<pagerows.length;i++){
        if(pagerows[i]==defaultPage)
            s+="<option value='"+pagerows[i]+"' selected='true'>每页"+pagerows[i]+"</option> ";
        else if(pagerows[i]==-1)
            s+="";
        else
            s+="<option value='"+pagerows[i]+"'>每页"+pagerows[i]+"</option> ";
    }
    s+="</select>";
    s+="<a  id='"+mcss_table.getActionId('RecordCount')+"'  class='icon-bars'   style='display:none'";
    s+=" onClick='mcsstable_getTotalRows(\""+tableid+"\",this);'>";
    s+="</a>";

    s+="<a  title='显示更多翻页按钮' onclick='mcsstable_showAllPageBtn(this,\""+mcss_table.tableid+"\")' >...</a>";

    var cls='mcsstable-pagebar';
    var s1="<div id='"+mcss_table.pagebarid+"' ";
    s1+=" class='"+cls+"'";
    if (mcss_table.params.pageposition=="leftdown")
        s1+=" style='float:left'";
    s1+=" >" + s + "</div>";
    return s1;
}

function mcsstable_showAllPageBtn(e,tableid)
{
    var mcsstable=mcsstable_getMCSSTable(tableid);
    $("#"+mcsstable.pagebarid).find("*").show();
    $(e).hide();

}
function mcsstable_firstpage(e,tableid)
{
    var table=mcsstable_getMCSSTable(tableid);
    if (table.currentpage==1 )
        mcdom.popup(lang.firstpages,{obj:e,width:150,fadeOut:true});
    else
    {
        mcsstable_gotoFirstPage(tableid);

    }
}

function gotofirstpage(tableid)
{
    mcsstable_gotoFirstPage(tableid);
}
function mcsstable_gotoFirstPage(tableid)
{
    var mcsstable=mcsstable_getMCSSTable(tableid);
    mcsstable.currentpage=1;
    var page=mcsstable.getAction("currentpage");
    if (page)
    {
        $(page).val(mcsstable.currentpage);
        setCookie(page.id,mcsstable.currentpage);
        mcsstable_loaddatarows(tableid);
    }
}

function mcsstable_lastpage(e,tableid)
{
    var table=mcsstable_getMCSSTable(tableid);
    if (table.currentpage==table.totalpages){
        mcdom.popup(lang.lastpages,{obj:e,width:150,fadeOut:true});
        setCookie(tableid+'_currentpage',table.currentpage);
    }else{
        var urlpath=table.params.homeUrl+ g_systempath+"/getTotalRows";
        /*数据库分表*///此处添加参数 usermodel_recordid
        var sql = table.params.sql;
        $.get(urlpath,{sql:sql,modelid:table.modelid,filter:table.filter,searchword:table.searchword,usermodel_recordid:table.params.usermodel_recordid}, function(data){
            var total =parseInt(data)/table.pagerows;
            if (total > parseInt(total))
                total =parseInt(total)+1;
            table.currentpage = total;
            $("#"+table.getAction("currentpage").id).val(table.currentpage);
            setCookie(table.getAction("currentpage").id,table.currentpage);
            mcsstable_loaddatarows(tableid);
        });
    }
}

function mcsstable_prepage(e,tableid)
{
    var table=mcsstable_getMCSSTable(tableid);
    if (table.currentpage==1 )
        mcdom.popup(lang.firstpages,{obj:e,width:150,fadeOut:true});
    else
    {
//		$("#"+tableid+"_checkbox_selectall").attr("checked",false);
        var currentPage=parseInt(table.currentpage) - 1;
        table.createHeader(table.modeldata);
        table.currentpage= currentPage ;
        $("#"+table.getAction("currentpage").id).val(table.currentpage);
        setCookie(table.getAction("currentpage").id,table.currentpage);
        mcsstable_loaddatarows(tableid);
    }
    setCookie(tableid+'_action_currentpage',table.currentpage);
}

function mcsstable_nextpage(e,tableid)
{
    var table=mcsstable_getMCSSTable(tableid);
    if (table.currentpage>=table.totalpages){
        mcdom.popup(lang.lastpages,{obj:e,width:150,fadeOut:true});
    }
    else
    {
        var currentPage=parseInt(table.currentpage) + 1;
        table.createHeader(table.modeldata);
//		$("#"+tableid+"_checkbox_selectall").attr("checked",false);
        table.currentpage= currentPage;
        $("#"+table.getAction("currentpage").id).val(table.currentpage);
        setCookie(table.getAction("currentpage").id,table.currentpage);

        mcsstable_loaddatarows(tableid,currentPage*table.pagerows);
    }
    table.showAction("currentpage,pagerows");
}

function mcsstable_changepagerows(select,tableid)
{
    var table = mcsstable_getMCSSTable(tableid);
    var usermodel_recordid = table.params.usermodel_recordid;
    var modelid = table.modelid;
    $.post(g_homeurl+"/System/ModelMaker/savePagerows",{usermodel_recordid:usermodel_recordid,modelid:modelid,pagerows:select.value},function(){
        table.pagerows = select.value;
        setCookie(usermodel_recordid+"_pagerows",select.value);
        table.currentpage = 1;
        $("#"+tableid+"_action_currentpage").val(1);
        mcsstable_loaddatarows(tableid,currentPage*table.pagerows);
    })
}

//重新从后台取得总记录数并显示
function mcsstable_getTotalRows(tableid,e)
{
    var mcsstable=mcsstable_getMCSSTable(tableid);
    if(mcsstable.params.data){
        mcdom.popup('总记录数为:'+mcsstable.params.data.length,{obj:e,title:lang.hint,fadeOut:"fadeOut"});
    }else{
        var urlpath=mcsstable.params.homeUrl+ g_systempath+"/getTotalRows";
        var filter=mcsstable_getLastFilter(mcsstable);
        /*数据库分表*///此处添加参数 usermodel_recordid
        $.get(urlpath,{sql:mcsstable.params.sql,modelid:mcsstable.modelid,filter:filter,searchword:mcsstable.searchword,usermodel_recordid:mcsstable.params.usermodel_recordid}, function(data){
            if (!data)
                mcdom.popup('无法获得记录数，可能是因为计算记录时遇到错误！',{obj:e,title:lang.hint});
            else
            {
                mcsstable_calculateTotalRecordCount(mcsstable,data);
                if (e)
                {
                    var pop=mcdom.popup(lang.totalrecords+":"+data+","+lang.totalpages+":"+mcsstable.totalpages,{obj:e,fadeOut:true,seconds:5,width:'180px',title:'记录统计'});
                    mcdom.moveToScreen(pop);
                }
            }
        });
    }
}

//根据总记录数计算页数，并显示
function mcsstable_calculateTotalRecordCount(mcsstable,recordCount)
{
    mcsstable.recordcount=recordCount;
    var total =parseInt(recordCount)/mcsstable.pagerows;
    if (total > parseInt(total))
        total =parseInt(total)+1;
    mcsstable.totalpages=total;
    var tableid=mcsstable.tableid;
    $("#"+tableid+"_action_RecordCount").attr("title",lang.totalrecords+":"+recordCount+","+lang.totalpages+":"+total);
    $("#"+tableid+"_action_totalpage").html(total);
    if (!total)
    {
        $("#"+tableid+"_action_currentpage").attr("value",'1');
        $("#"+tableid+"_action_totalpage").html('1');
    }
}
//决定字段显示什么内容，显示内容不一定与存在数据库的值一样
function mcsstable_getFieldDisplay(field,value,special_field_show,record,mcsstable)
{

    var r=value;
    if (special_field_show)
    {
        var special_field=special_field_show[field.id];
        if (special_field)//如果自定义了字段显示处理方法
        {
            record.mcsstable=mcsstable;
            r=special_field(value,record,field,mcsstable);
        }
        else
            r=mcsstable_getFieldText(field,value,record,mcsstable);
    }
    else
    {
        r=mcsstable_getFieldText(field,value,record,mcsstable);
        //r="<span>"+r+"</span>";//原来为什么加span呢？加了后会导致内容超长时显示不正常
    }
    if (r=="") r="<span></span>"; //加上这样一句，因为编辑对话框弹出的时候，要靠单元格内的子对象来打开
    return r;
}

//获得字段定义信息,例如，获得name字段的fieldtype
MCSSTable.prototype.getFieldInfo=function(fieldid,info){
    var fieldData=this.modeldata.fields;
    for(var i=0;i<fieldData.length;i++)
    {
        if (fieldData[i].id==fieldid)
        {
            return fieldData[i][info];
        }
    }
    return null;
}

//点击放大图片
function getEvent(){
    if(document.all){
        return window.event;//如果是ie
    }
    func=getEvent.caller;
    while(func!=null){
        var arg0=func.arguments[0];
        if(arg0){
            if((arg0.constructor==Event || arg0.constructor ==MouseEvent)||(typeof(arg0)=="object" && arg0.preventDefault && arg0.stopPropagation)){
                return arg0;
            }
        }
        func=func.caller;
    }
    return null;
}
//add by sunliangchao 这样一来就可以直接勾选，不用勾选两下了。
function mcsstable_clickCheckBox(tableid,fieldid,obj){

    var td=obj.parentNode;
    if (!td.parentNode) return;
    var tr=td.parentNode;
    
    var recordid=tr.id;
    var mcsstable=mcsstable_getMCSSTable(tableid);
    var field=mcsstable.modelDealer.getField(fieldid);
    value="0";
    if (obj.innerText!="√"){
        value="1";
    }
    mcsstable.setCellData(recordid,fieldid,value);
    
}
//获得字段值的显示字符串。因为有些字段值与显示不一样。
function mcsstable_getFieldText(field,value,record,mcsstable)
{
    if (!value)
        value="";
    var r=value;
    var openRecord='';
    if (!field)
        return r;
    //下面根据字段类别把字段值转化为显示值
    if (field && field.type=='checkbox'){
        r='<span style="width:100%;height:100%" inEdit="true" onclick="mcsstable_clickCheckBox(\''+mcsstable.tableid+'\',\''+field.id+'\',this);">';
        if (value==1 || value=="1" || value=="on")
            r+='√</span>';
        else
            r+='  </span>';
    }
    else if (field.type=='checkboxlist')
    {
        var params={owner:mcsstable,container_id:mcsstable.tableid,modelid:mcsstable.modelid,setInputChange:true};

        r=createMcssCheckboxlist(field,value,params);

        if (mcsstable.params.project_fulldata && mcsstable.params.project_fulldata.shareoption && mcsstable.params.project_fulldata.shareoption.indexOf("[editinnullcell]")>-1)
        {
            if (value!="" && value !=null){
                r = r.replace( eval("/'checkbox'/gi"), "'checkbox' disabled='disabled'")
            }
        }
    }
    else if (field && (field.type=='hidden' || field.type=='password'))
    {
        if (r!="")
            r="***";
    }

    else if (value && field && (field.type=='date' || field.type=='datetime' || field.type=='SYS_ADDTIME' || field.type=='SYS_EDITTIME' ))
    {
        if (value=="0000-00-00" || value=="0000-00-00 00:00:00")
            r="";
        else{
            r=value;
            if (field.type=='date' && value.length>10)
                r=r.substr(0,10);
            if (field.type=='SYS_ADDTIME' || field.type=='SYS_EDITTIME')
                r=r.substr(5,11);
        }
        r="<span>" + r + "<span>";
    }
    else if (field && field.type=='checkboxlist')
    {
        var enum_arr = field.data.split(",");
        if (value)
        {
            var value_arr=value.split(",");

            r="";
            for(var n=0;n<value_arr.length;n++)
            {
                for(var k=0;k<enum_arr.length;k++)
                {
                    var one_arr=enum_arr[k].split(":");
                    if (one_arr[0]==value_arr[n] ||one_arr[1]==value_arr[n])
                    {
                        var v1=one_arr[0];
                        if (one_arr.length==2)
                            v1=one_arr[1];
                        if (r=="")
                            r=v1;
                        else
                            r=r+","+v1;
                        if (lang[r]) r=lang[r];

                        break;
                    }
                }
            }
        }
    }
    else if (field && field.type=='radio')
    {
        var enum_arr = "";
        var fielddata = field.data;
        if(fielddata){
            if(value && fielddata && fielddata.indexOf(value)==-1)
                fielddata = fielddata.replace("otheroption",value);
            fielddata = fielddata.replace("otheroption","其它");
        }
        if(fielddata)
            enum_arr = fielddata.split(",");
        if (value != null)
        {
            var value_arr=value.split(",");
            r="";
            for(var n=0;n<value_arr.length;n++)
            {
                value_arr[n] = value_arr[n].trim();
                for(var k=0;k<enum_arr.length;k++)
                {
                    var one_arr=enum_arr[k].split(":");
                    if (one_arr[0]==value_arr[n]||one_arr[1]==value_arr[n])
                    {
                        var r=one_arr[0];
                        if (one_arr.length==2)
                        {
                            r=one_arr[1];
                            if (lang[r]) r=lang[r];
                        }
                    }
                }
            }
        }
    }
    else if (field && field.type=='image')
    {

        var imagepath =g_rooturl+"/Public/uploadfile/"; //图片路径路径
        if(uni_global.issaas && g_rooturl.indexOf("localhost")<0 )
            imagepath = uni_global.filepath;

        if(value)
        {
            //解析文件显示名与存储名
            var filename=value;
            var savename=value;
            var arr=value.split('~');
            if (arr.length>1)
            {
                filename=arr[0];
                savename=arr[1];
                savename = savename.replace(/\\\//g,'/');
                var savename1=savename.replace(/\[small\]/,"");

            }

            // var _style="";
            // if(field.width && field.height)
            // _style="width='"+field.width+"' height='"+field.height+"'";
            // else if(field.width)
            // _style="width='"+field.width+"' height='"+y+"'";
            // else if(field.height)
            // _style="width='"+x+"' height='"+field.height+"'";
            // else
            // _style="width:";
            // mcsstable.view==""
            // r="<img src='"+imagepath+savename+"' style='"+_style+"' ";
            // if(mcsstable.view =="" ||mcsstable.view=="table")
            // r+=" onclick='viewImage(this)' ";
            // r+="/>";
            var _style="";
            if(field.width && field.height)
                _style="width='"+field.width+"' height='"+field.height+"'";
            else if(field.width)
                _style="width='"+field.width+"' height='"+y+"'";
            else if(field.height)
                _style="width='"+x+"' height='"+field.height+"'";
            else
                _style="width:";
            mcsstable.view==""
            var img_str="<img  class='example-image' style='"+_style+"' src='"+imagepath+savename+"' onerror='errorimg(this)'  onmousedown='loadingscript()' />";
            if(mcsstable.view =="" ||mcsstable.view=="table")
                r="<a class='example-image-link' data-lightbox='example-set' href='"+imagepath+savename1+"'>"+img_str+"</a>";
            else
                r=img_str;
        }
    }
    else if(field && field.type=='mail'){
        if(value)
            r="<span>"+value+"</span>";
    }
    else if(field && field.type=='url'){
        var _urlValue = "",_urlName="",href="";
        if(value){
            if(value.indexOf("<~>")>-1 || value.indexOf("<|>")>-1){
                var _value = value.split("<~>");
                if(value.indexOf("<|>")>-1)
                    _value = value.split("<|>")
                _urlValue = _value[1];
                _urlName = _value[0];
            }else{
                //_urlValue = value;
                _urlName = value;
            }
            href = _urlValue;
            r="<span title='"+href+"'>"+_urlName+"</span>";
            if (_urlValue)
            {
                if(_urlValue.indexOf("http"))
                    _urlValue = "http://"+_urlValue;
                if (IsUrl(_urlValue))
                    r+="<a class='icon-newtab' onmousedown='uexOpenUrl(\""+_urlValue+"\")' style='font-size:15px' target='_blank' title='打开:"+href+"'>访问</a>";
            }

        }
    }
    else if(field && (field.type=='text' || !field.type)){
        var _urlValue = "";
        if(value){
            r="<span title='"+value+"'>"+value+"</span>";
            if(field.formate=='qq'){
                if (IsQQ(value))
                {
                    _urlValue = "tencent://Message/?Uin="+value+"&websiteName=qzone.qq.com&Menu=yes";
                    r+="<a style='float:right' href='"+_urlValue+"' title='与"+value+"聊天'>"
                        +"<img border='0' src='"+g_rooturl+"/Public/projects/oa/images/qq.jpg'/></a>";
                }
            }else if(field.formate=='phone' &&(uni_global.isMobile() || uni_global.isSmallScreen())){
                if (IsPhone(value))
                {
                    r+="<a style='float:right' href='tel:"+value+"'>"
                        +"<img border='0' src='"+g_rooturl+"/Public/projects/oa/images/phone.png'/></a>";
                }
            }

        }
    }
    else if(field && field.type=='site'){
        if(value){
            var initval = value;
            var arr = value.split("<->");
            var str = "";
            if(arr[0] && arr[0]!='区/县')
                str+=arr[0];
            if(arr[1] && arr[1]!='区/县')
                str+=arr[1];
            if(arr[2] && arr[2]!='区/县')
                str+=arr[2];
            if(arr[3] && arr[3]!='区/县')
                str+=arr[3];
            r="<div style='width:100%;height:100%'><span initval='"+initval+"' title='"+str+"'>"+str+"</span>";
            r+="</div>";
        }
    }
    else if (field && field.type=='file')
    {
        var imagepath =g_rooturl+"/Public/uploadfile/";
        if(uni_global.issaas && g_rooturl.indexOf("localhost")<0 )
            imagepath = uni_global.filepath;
        if(value)
        {
            var filename=value;
            var savename=value;
            var arr=value.split('~');
            if (arr.length>1)
            {
                filename=arr[0];
                savename=arr[1];
            }
            var nameEnd = filename.substring(filename.length-4).toLowerCase();
            if(nameEnd==".png" || nameEnd==".gif" ||nameEnd==".jpg"){
                r="<a onclick='inputer_viewFile(this)'>"+filename+"</a>";
                r+="<a><img src='"+imagepath+savename+"' style='display:none'/></a>";
                //r+="<a class='icon-arrow-down' style='color:gray' title='下载原图片' target='_blank' href='"+imagepath+savename+"' ></a>";
                //r+="<a class='icon-eye' style='color:gray' onclick='inputer_viewFile(this)'></a>";
            }else{
                r="<a target='_blank' href='"+imagepath+savename+"' >"+filename+"</a>";
                //r+="<a class='icon-arrow-down' style='color:gray' title='下载原文件'  target='_blank' href='"+imagepath+savename+"' ></a>";
                //r+="<a class='icon-eye' style='color:gray' onclick='viewOfficeDoc(\""+imagepath+savename+"\")'></a>";
            }
        }
    }
    else if (field && (field.type=='dropdown' || !field.type || field.type=='smartselect'))
    {
        if(field.data!=null)
        {
            r=mcss_getDropdownName(field.data,value);
        }

    }
    else if (field && field.type=='relatedrecords')
    {
        r=mcsstable_getRelatedrecordsBtn(mcsstable.tableid,record[mcsstable.keyfield],field.id);
    }else if (field){
        r="<span>"+r+"</span>";
    }

    if (field && field.prop && (field.prop).indexOf("OPENRECORD")!=-1)
    {
        openRecord=true;
    }

    if (openRecord)
    {
        var modeldata=mcsstable.modeldata;
        var tableid=mcsstable.tableid;
        var openurl = modeldata.openurl;
        if(openurl && (openurl.indexOf("/")>0))
            r=mcsstable_getOpenActionHTML(mcsstable,openurl,record[mcsstable.keyfield],lang.open,lang.open,r,true);
        else
            r= "<a style='cursor:pointer;' onclick=\"mcsstable_showPopupForm(this,'"+mcsstable.tableid+"','"+openurl+"','"+lang.open+"','"+record[mcsstable.keyfield]+"')\",\"VIEW\""
                + " title='"+ lang.open_title + "'>" + r + "</a>";
    }
    if(!r || r=="null")
        r='';
    return r;
}
function mcss_goUrl(url){
    var e = getEvent();
    e.stopPropagation();
    location.href = url;
}

//创建查看相关记录的按钮
function mcsstable_getRelatedrecordsBtn(tableid,recordid,fieldid)
{
    var h="<a class='icon-eye' fieldid='"+fieldid+"' recordid='"+recordid+"' tableid='"+tableid+"' onclick='mcsstable_openRelatedrecordsPage(this)'></a>";
    return h;
}

//打开“查看相关记录”页面。这个功能不是mcstable的通用功能，是超级表格专用的
function mcsstable_openRelatedrecordsPage(e)
{
    var modelid;
    var owner;
    var fieldid=$(e).attr('fieldid');
    var recordid=$(e).attr('recordid');
    var tableid=$(e).attr('tableid');
    var mcsstable=mcsstable_getMCSSTable(tableid);
    var field=mcsstable.modelDealer.getField(fieldid)

    var willopensheetid='',willopenfieldid='',thisfieldid='';
    if (field.data)
    {
        var arr=field.data.split('VIEW_FROMTABLE');
        if (arr.length==2)
        {
            var arr2=arr[1].split(":");
            if (arr2.length==3)
            {
                willopensheetid=arr2[1];
                var arr3=arr2[2].split("=");
                if (arr3.length==2)
                {
                    willopenfieldid=arr3[0];
                    thisfieldid=arr3[1];
                    var fieldvalue=mcsstable.mcssData.getFieldValue(recordid,thisfieldid);
                    var filter=willopenfieldid+"='"+fieldvalue+"'";
                    if(!willopenfieldid || !thisfieldid)
                        filter='1=1';
                    filter=uni_cleanSQL(filter);
                    var url=g_homeurl+"/Project/main/id/"+willopensheetid+"/filter/"+filter;
                    mcdom.showIframe(url,{width:'900px',height:'600px',title:'查看相关记录',openNewWindow:true});
                    return;
                }
            }
        }
    }
    alert('没有找到相关记录！');
}


//得到mcss模型下拉类型字段的显示值
function mcss_getDropdownName(fielddata,value)
{
    value+="";
    var r=value;
    if (fielddata)
    {
        var enum_arr = fielddata.split(",");

        if (value !=null && value !="undefined")
        {
            var value_arr=value.split(",");
            for(var n=0;n<value_arr.length;n++)
            {
                for(var k=0;k<enum_arr.length;k++)
                {
                    var one_arr=enum_arr[k].split(":");
                    if (one_arr[0]==value_arr[n] ||one_arr[1]==value_arr[n])
                    {
                        var v1=one_arr[0];
                        if (one_arr.length==2)
                            v1=one_arr[1];
                        r=v1;
                        if (lang[r]) r=lang[r];

                        break;
                    }
                }
            }
        }
    }
    if (r==null) r="";
    return r;
}


//使用某个视图显示数据
function mcsstable_setView(tableid){
    var mcsstable=mcsstable_getMCSSTable(tableid);
    if (mcsstable.viewtype=="calendar")
    {
        if ($("#"+tableid+"_view_calendar").size()==0)
        {
            $("#"+tableid).after("<div id='"+tableid+"_view_calendar"+"'>日历</div>");
            mcsstable_loadCalendarRows(mcsstable);
        }

        $("table[id='"+tableid+"'] tr").hide();
        $("#"+tableid+"_view_calendar").show();
        mcsstable.showAction("tableedit,del",false);
    }
    else
    if (mcsstable.viewtype=="kapian")
    {
        $("table[id='"+tableid+"'] tr").hide();
        $("#"+tableid+"_view_calendar").hide();
        mcsstable_loadKapian(mcsstable);
        mcsstable.showAction("tableedit,del",false);
    }
    else
    if (mcsstable.viewtype=="table" || mcsstable.viewtype=="")
    {
        $("table[id='"+tableid+"'] tr").show();
        $("#"+tableid+"_view_calendar").hide();
        $("#"+tableid+"_view_kapian").hide();
        mcsstable.showAction("tableedit,del",true);
    }

}

//切换视图
function mcsstable_swithView(tableid){
    var mcsstable=mcsstable_getMCSSTable(tableid);
    if (!mcsstable.viewtype || mcsstable.viewtype=="table"|| mcsstable.viewtype=="tableedit")
    {
        mcsstable.viewtype="calendar";
    }
    else
    if (mcsstable.viewtype=="calendar")
    {
        mcsstable.viewtype="kapian";
    }
    else
    if (mcsstable.viewtype=="kapian")
    {
        mcsstable.viewtype="table";
    }
    mcsstable_setView(tableid);
}

//用卡片方式显示表的内容
function mcsstable_loadKapian(mcsstable)
{
    var html="";
    var fields=mcsstable.fields;
    var direction="";//文本对齐方式
    var fieldtype="";
    var type='';
    var style='';
    var rowData;
    var line="1";
    var params=mcsstable.params;
    var last_td_func=mcsstable.last_td_func;
    for(var i=0;i<mcsstable.data.length;i++)
    {
        rowData=mcsstable.data[i];


        html+="<br /><table>";
        for ( var j = 0; j < fields.length; j++)
        {
            //还没整理好代码

        }
        if (
            ((params.showRecordActionAtLast || last_td_func) && !params.hideLastTd
            && (params.toolbar.indexOf('open') > -1 || params.toolbar.indexOf('edit') > -1 || params.toolbar.indexOf('del') > -1))
        ) //打开和编辑按钮
        {
            var td = '';
            if(last_td_func)
            {

                td = last_td_func(rowData[mcsstable.keyfield],rowData);
            }
            html += "<tr><td class='mcsstable_td"+line+" recordaction' style=''>"+mcsstable_getRecordAction(mcsstable,rowData)+td+"</td></tr>";
        }
        if (last_td_func && !params.hideLastTd && !(params.style=="simple") && (params.toolbar.indexOf('open') == -1 && params.toolbar.indexOf('edit') == -1 && params.toolbar.indexOf('del') == -1))
        {

            var td = last_td_func(rowData[mcsstable.keyfield],rowData);
            html += "<tr><td class='mcsstable_td"+line+"  recordaction' style='white-space:nowrap;'>" + td + "</td></tr>";
        }


        html+="</table>";
    }
    if ($("#"+mcsstable.tableid+"_view_kapian").size()==0)
        $("#"+mcsstable.tableid).after("<div id='"+mcsstable.tableid+"_view_kapian'></div>");
    $("#"+mcsstable.tableid+"_view_kapian").html(html).show();

}
//用日历方式显示表的内容
function mcsstable_loadCalendarRows(mctable)
{
    var fields=mctable.modeldata.fields;
    mctable.titleField=mctable.modeldata.calendar_title_field;
    mctable.timeField=mctable.modeldata.calendar_date_field;
    if (!mctable.titleField || !mctable.timeField)
    {
        for(var i=0;i<fields.length;i++)
        {
            if (!mctable.titleField)
            {
                if (fields[i].id=="name" || fields[i].id=="title" || fields[i].id=="subtitle" || fields[i].fieldtype=="string" || fields[i].type=="text")
                {
                    mctable.titleField=fields[i].id;
                }
            }
            if (!mctable.timeField)
            {
                if (fields[i].isvisible!="false" && (fields[i].type=="date" || fields[i].type=="datetime" || fields[i].fieldtype=="date" || fields[i].fieldtype=="datetime") )
                {
                    mctable.timeField=fields[i].id;
                }
            }
        }
    }
    if (!mctable.titleField)
    {
        alert("无法按日历方式查看，因为缺少标题字段。");
        return;
    }
    if (!mctable.timeField)
    {
        alert("无法按日历方式查看，因为缺少日期字段。");
        return;
    }
    var data=[];
    for(var i=0;i<mctable.data.length;i++)
    {
        if (mctable.data[i][mctable.timeField])
            data.push({date:mctable.data[i][mctable.timeField],work:mctable.data[i][mctable.titleField],id:mctable.data[i][mctable.keyfield]});
    }
    //var data=[{date:'2012-06-19',work:'我今天的工作1'},{date:'2012-06-19',work:'我今天的工作2'}];

    this.calendar=new McssCalendar({id:mctable.tableid+"_view_calendar",data:data,mostRows:20,mostWords:20,onShowDayData:canlendar_showData,calendartype:'monthweek',refObject:mctable,showCalendartype:true});
    this.calendar.run();

}

function canlendar_showData(calendar,date,day,refObject)
{
    var h="<div><span>"+day+"</span><span style='text-align:right;'></span></div>";
    var taskCount=0;
    var data=calendar.params.data;
    var mostRows=10;
    if (calendar.params.mostRows)
        mostRows=calendar.params.mostRows;
    var mostWords=20;
    if (calendar.params.mostWords)
        mostWords=calendar.params.mostWords;
    var mcsstable=refObject;
    var url;
    for(var i=0;i<data.length;i++)
    {
        if (data[i].date==date || data[i].date.length>10 && data[i].date.substr(0,10)==date)
        {
            if (taskCount<mostRows)
            {
                var temp=data[i].work;
                if (mostWords && data[i].work.length>mostWords)
                {
                    temp=data[i].work.substr(0,mostWords)+"..";
                }
                url=mcsstable.params.homeUrl
                    + mcsstable.modeldata.openurl
                    + "/"+mcsstable.keyfield+"/"
                    + data[i][mcsstable.keyfield];

                temp="<span style='cursor:pointer;color:blue;' onClick=\"mcsstable_openWindow('"+url
                    + "','"
                    + lang.open
                    + "' ,'"+mcsstable.modeldata.open_window_style+"')\" title="
                    + lang.open_title + ">" + temp+  "&nbsp;&nbsp;</span>";

                h+=temp+"<br />";
                taskCount++;
            }
            else
            {
                h+="<span>更多...</span>";
                break;
            }

        }
    }
    return h;
}
function calendar_openRecord()
{
    s += "<span style='cursor:pointer;color:blue;' onClick=\"mcsstable_openWindow('"+params.homeUrl
        + openurl
        + "/"+mcsstable.keyfield+"/"
        + v[mcsstable.keyfield]
        + "','"
        + lang.open
        + "' ,'"+modeldata.open_window_style+"')\" title="
        + lang.open_title + ">" + lang.open + "&nbsp;&nbsp;</span>";
    return s;
}
//将xls文件导入到数据库
function mcsstable_import_old(tableid,recordid,value){
    var mcsstable=mcsstable_getMCSSTable(tableid);

    //行数超限判断 ADD by chenli
    if(isOverLimitedRowCount(mcsstable.recordcount)){
        return;
    }
    //下面的escape方法在对应php中现在没有解码却正确，用了解码函数却不正确。
    //因此没有用。对应的代码是$filterFromClient=($_REQUEST['filter']);
    try
    {
        var source = "table";
        if(!recordid)
            source = "excel";
        var url=mcsstable.params.homeUrl+"System/Model/appendFromExcel_old/modelid/"+mcsstable.modelid+"/tableid/"+mcsstable.tableid+"/source/"+source+"/recordid/"+recordid+"/table_name/"+value;
        mcdom.showIframe(url,{width:"300",height:"250",title:"导入数据",tag:"importexcel",openNewWindow:true});
    }
    catch(ex)
    {
        alert(ex.message);
    }
}


//将xls文件导入到数据库
function mcsstable_import(tableid,recordid,value){
    var mcsstable=mcsstable_getMCSSTable(tableid);

    //行数超限判断 ADD by chenli
    if(isOverLimitedRowCount(mcsstable.recordcount)){
        return;
    }
    //下面的escape方法在对应php中现在没有解码却正确，用了解码函数却不正确。
    //因此没有用。对应的代码是$filterFromClient=($_REQUEST['filter']);
    try
    {
        var source = "table";
        if(!recordid) source = "excel";

        if (g_homeurl.indexOf("localhost")<0){
            var dmain=getdomain();
            dmain=dmain.replace("http://","");
            document.domain=dmain;
            var urlup = g_homeurl;//+"/Oa/Project/uploadexcel/modelid/sys_user/tableid/gantetu_table";
            urlup=urlup.replace("http://","http://up.");
            urlup=urlup.replace(".com/index",".com/uni/index");

        }else{
            var urlup=g_homeurl;
        }

        mcsstable.params.impurl=urlup+"/Oa/Project/xlstosql/projectid/"+mcsstable.params.project_fulldata.project_record.id+"/ownerid/"+mcsstable.params.model_ownerid;

        var url=urlup+"/System/Model/appendFromExcel/modelid/"+mcsstable.modelid+"/tableid/"+mcsstable.tableid+"/source/"+source+"/recordid/"+recordid+"/torecordid/"+mcsstable.params.usermodel_recordid +"/table_name/"+value;
        var pop=mcdom.showIframe(url,{width:"800",height:"250",title:"导入数据",tag:"importexcel",openNewWindow:true});
        var userName = getCookie("mcss_loginuser");
        $(pop).attr("username",userName);
    }
    catch(ex)
    {
        alert(ex.message);
    }
}

//导出到xls等格式的文件
function mcsstable_export(tableid,type,exporturl)
{
    if($("#progressBar").html())
        $("#progressBar").html('导出数据中,请稍等...');
    mcdom.confirm(lang.confirmtoexportexcel,function(b){
        if (!b) return;
        mcdom.alert(lang.isexporting,'','info');
        var mcsstable=mcsstable_getMCSSTable(tableid);
        var params=mcsstable.params;
        var fields=mcsstable.fields;
        var modeldata=mcsstable.modeldata;
        var first_td_func=mcsstable.first_td_func;
        var last_td_func=mcsstable.last_td_func;
        //下面的escape方法在对应php中现在没有解码却正确，用了解码函数却不正确。因此没有用。
        //对应的代码是$filterFromClient=($_REQUEST['filter']);
        try
        {
            var sql="";
            var url=mcsstable.params.homeUrl+g_systempath+"/exporttoxls";
            if (params.sql)
                sql=params.sql;
            if (exporturl)//额外的导出格式采用这个，必须有参数exporturl_ext
            {
                url = exporturl;
            }
            else
            if (mcsstable.params.exporturl)//跳到指定的Action中
                url = mcsstable.params.exporturl;
            //var lastfilter=mcsstable_getLastFilter(mcsstable);
            var lastfilter=mcsstable.lastfilter;

            if (mcsstable.selectedRowsID)
            {
                var srids=mcsstable.selectedRowsID;
                srids = srids.replace(/\,/g, "','");
                lastfilter += ' and ' + mcsstable.keyfield + " in ('"+srids+"')";
            }	//如果传递了数据，则后来直接根据这个数据导出，不需要sql语句
            if(!lastfilter)
                lastfilter = mcsstable.filter;
            lastfilter=uni_cleanSQL(lastfilter);
            $.post(url,{modelid:mcsstable.modelid,searchword:mcsstable.searchword,sql:sql,data:params.datastr,filter:lastfilter,
                filetype:type,usermodel_recordid:mcsstable.params.usermodel_recordid,
                model_ownerid:mcsstable.params.model_ownerid,viewid:mcsstable.view_id},function(data){
                mcdom.closePopup();
                if(data=="isbig"){
                    mcdom.alert("导出内容太多。您可以缩小数据范围后再导出","导出提示","info","fadeout",'250','auto');
                    return;
                }
                if (data && data.length>100){
                    alert(lang.failtoexport);
                    if (uni_global.loginuser=="admin")
                        mcdom.popup(data,{title:"错误信息"});
                    return;
                }
                var arr = data.split(';');
                if(arr[1]){
                    var filepath =mcsstable.rooturl+"/Public/temp/"+arr[1].replace(/[ ]/g,"");
                    mcdom.alert("导出成功,文件下载到浏览器指定位置","导出提示","info","fadeout",'250','auto');
                    if($("#progressBar").html())
                        $("#progressBar").html('数据加载中,请稍等...');
                    //window.location=filepath;

                    var a = document.createElement("a");
                    var a_attachement_id="attachement_"+mcsstable.params.usermodel_recordid;
                    var file_format= "." + filepath.replace(/.+\./, "");
                    var download_filename=mcsstable.params.project_fulldata.project[0].name+file_format;
                    a.id=a_attachement_id;
                    a.href =filepath;
                    a.download = download_filename;
                    document.body.appendChild(a);
                    a.click();

                }
                else alert(lang.failtoexport);
            });
        }
        catch(ex)
        {
            alert(ex.message);
        }
    });
}

//打印，王灿开发
function mcsstable_print(tableid)
{
    mcss_importJS("js/jqprint.js");
    setTimeout(function(){
        $("#"+tableid).find('caption').hide();
        $("#"+tableid).find(".mcsstable_th_actions,.mcsstable-action-col").hide();
        $("#"+tableid).jqprint({operaSupport:true});
        $("#"+tableid).find('caption').show();
        $("#"+tableid).find(".mcsstable_th_actions,.mcsstable-action-col").show();
    },500)
}

//切换把表格中内容的全部显示在一行，或者折行显示
//不折行，表格可能会很宽
//允许折行，表格宽度尽量等于屏幕宽度

function mcsstable_autowidth(tableid)
{
    if (!$("#"+tableid).attr('autowithstyle') || $("#"+tableid).attr('autowithstyle')==lang.nofoldlines)
    {
        //当前样式:100%，不折行
        $("#"+tableid).find('tr').css('white-space','nowrap');
        $("#"+tableid).attr('autowithstyle',lang.compact);
    }
    else
    {
        //当前样式:紧凑，可折行
        $("#"+tableid).find('tr').css('white-space','');
        $("#"+tableid).attr('autowithstyle',lang.nofoldlines);
    }
}

function mcsstable_clearAndSearch(tableid,obj){
    $(obj).prev().val('');
    $(obj).hide();
    mcsstable_search(tableid);
}

function mcsstable_onsearchkeyup(event,tableid,input)
{
    var obj = event.srcElement ? event.srcElement : event.target;
    window.setTimeout(function(){
        mcsstable_search(tableid);
    },200);    
    var val = $(input).val();
    if(!val)
        $(input).next().hide();
    else
        $(input).next().show();
    if ($("#"+tableid+"_searchnow").attr('checked')==true)
        mcsstable_search(tableid);
    //if (event.keyCode!=13)
    //    mcss_auto_search_list(tableid,input);
    //else
    //    dom_closePopupMenu();
}
function mcss_auto_search_list(tableid,input){
    var val = $(input).val();
    var h = "";
    if(val){
        var mcsstable=mcsstable_getMCSSTable(tableid);
        var fields = mcsstable.modeldata.fields;
        h+="<div style='max-height:200px;overflow-y:auto;overflow-x:hidden'>";
        for(var i=0;i<fields.length;i++){
            if (fields[i].name && fields[i].isvisible!='false'){
                var filter = fields[i].id+"='"+val+"'";
                var show_str = '搜索：'+fields[i].name+"=\""+val+"\"";
                h+="<a filter=\" "+filter+" \" title='"+show_str+"' onclick='mcsstable_search_auto(\""+tableid+"\",this)'>"
                    +"<div style='height:18px;width:180px;overflow:hidden'>"+show_str
                    +"</div></a>";
            }
        }
        h+="</div>";
        dom_showPopupMenu(h,input,{width:'200px'});
    }
}

//修改了以下方法 mcsstable_search_auto mcsstable_search mcss_auto_search_list mcsstable_onsearchkeyup
function mcsstable_search_auto(tableid,obj){
    var table=mcsstable_getMCSSTable(tableid);
    table.advancedFilter = $(obj).attr('filter');
    if(table.advancedFilter)
        mcsstable_loaddatarows(tableid);
}

function mcsstable_onsearchchange(tableid)
{
    if ($("#"+tableid+"_searchnow").prop('checked')==true)
        mcsstable_search(tableid);
}

function mcsstable_search(tableid)
{
    var mcsstable=mcsstable_getMCSSTable(tableid);
    var str=$(mcsstable.getAction('search')).find("input").val();
    str = str.replace(/^(\s|\u00A0)+/,'');
    for(var i=str.length-1; i>=0; i--){
        if(/\S/.test(str.charAt(i))){
            str = str.substring(0, i+1);
            break;
        }
    }

    mcsstable.searchword=str;
    //if(mcsstable.searchword && mcsstable.view=='tableedit' && mcsstable.recordcount==mcsstable.mcssData.data.length)

    if(mcsstable.params.data) //查询数据没问题，但是老孙做的固定列会显示空，等老孙解决
        mcsstable_searchInTable(tableid,str);
    else{
        mcsstable.advancedFilter = '';
        mcsstable.currentpage=1;
        mcsstable_loaddatarows(tableid);
    }

}
//在表格内搜索
function mcsstable_searchInTable(tableid,str){
    var tds = $("#"+tableid).find("td");
    tds.parent().hide();
    for(var i =0;i < tds.length;i++){
        var firstObj = tds.eq(i).children().eq(0);
        var val = firstObj.val();
        if(!val)
            val = firstObj.html();
        if(val && val.indexOf(str) > -1)
            tds.eq(i).parent().show();
    }
}

//权限检查
function checkaccess(data)
{
    if (data)
    {
        if (data.length>0 && data[0]['err'])
        {
            var err=data[0]['err'];
            if (err.indexOf('unaccessible:')>-1)
            {
                return err;
            }
        }
    }
    return '';
}


function getJSONValue(jsonArray,key)
{
    for(var i=0;i<jsonArray.length;i++)
    {
        if (jsonArray[i][key])
            return jsonArray[i][key];
    }
    return null;
}

MCSSTable.prototype.getSelectedRows=function(){
    var checkboxs=document.getElementsByName(this.tableid+"_checkbox");
    var ary=[];
    var selectedID=[];
    for(var i=0;i<checkboxs.length;i++)
    {
        if(checkboxs[i].checked==true)
        {
            selectedID.push(checkboxs[i].value);
        }
    }
    selectedID=selectedID.join(",")+",";
    var rows=this.data;
    var result=[];
    for(var i=0;i<rows.length;i++)
    {
        for(var j=0;j<selectedID.length;j++)
        {
            if (selectedID.indexOf(rows[i][this.keyfield]+",")>-1){
                result.push(rows[i]);
                break;
            }
        }
    }
    return result;

}

function returnSelectedRows(tableid)
{
    var mcsstable=mcsstable_getMCSSTable(tableid);
    var arr=mcsstable.getSelectedRows();

    var owner=this;
    if (mcsstable.params.popupFormID)
    {
        var formid=mcsstable.params.popupFormID;
        var form=owner.getAutoform(formid);
        var mcssmodel=mcsstable.modelDealer;
        var fieldsetting=mcssmodel.getFieldInfo(mcsstable.params.popupFieldID,'data');
        if (fieldsetting.indexOf("(")>-1)
        {
            fieldsetting=fieldsetting.substr(fieldsetting.indexOf("(")+1);
            fieldsetting=fieldsetting.substr(0,fieldsetting.length-1);
            fieldsetting=fieldsetting.split(",");
            var setarr;
            for(var i=0;i<fieldsetting.length;i++)
            {
                setarr=fieldsetting[i].split(":")
                if (setarr.length==2)
                {
                    var fieldobjid=form.getFieldID(setarr[1]);
                    var f=owner.document.getElementById(fieldobjid);
                    if (arr.length>0){
                        f.value=arr[0][setarr[0]];
                        if (form.params.afterPopupSelectOne)
                            form.params.afterPopupSelectOne(setarr[1],f.value);
                    }
                }
            }
        }
        else
        {
            var fieldobjid=form.getFieldID(mcsstable.params.popupFieldID);
            var f=owner.document.getElementById(fieldobjid);
            if (arr.length>0){
                f.value=arr[0][mcsstable.fields[0]];
            }
        }

    }
    else
    {
        if (parent.mcss_SetSelected)
            parent.mcss_SetSelected(arr,mcsstable.params.popupFieldID,mcsstable.params.popupFormID);//调用父窗体的方法来处理选中的记录。父窗体的方法名必须是mcss_SetSelected
        else
            alert("父窗体没有对应的方法（parent.mcss_SetSelected），因此现在无法调用。");
        parent.ClosePop();
    }
    //parent.ClosePop();

}

function mcsstable_tableEdit(tableid,e)
{
    var mctable=mcsstable_getMCSSTable(tableid);

    mctable.view='tableedit';
    var maxrecords=201;
    if (mctable.params.maxTableEditCount)
        maxrecords=mctable.params.maxTableEditCount;
    if ($("#"+tableid).find('tr').size()>maxrecords)
    {
        if(confirm(lang.areyousuretotableedit)==false)
        {
            mctable.view='celledit';
        }
    }

    //设置保存快捷键
    if (mctable.params.showShortcutKey)
    {
        //$(document).unbind("keydown");
        $(document).keydown(function(event){
            //ctrl+s
            if (event.ctrlKey == true && event.keyCode == 83 && mctable.view=='tableedit') {
                $(mctable.cellEditor.currentEditor).change();
                $(mctable.getAction('save')).click();
                return false;
            }
        })
    }


    if (!mctable.cellEditor)
    {
        //auto_fill_fields:需要提供自动收集值选项的字段列表
        var params={mcsstable:mctable,mcssData:mctable.mcssData,auto_fill_fields:mctable.params.auto_fill_fields,
            auto_fill_fields_data:mctable.params.auto_fill_fields_data,
            createSpecialFieldInputer:mctable.params.createSpecialFieldInputer
        };
        if (mctable.params.afterCreateAllEditor)
            params.afterCreateAllEditor=mctable.params.afterCreateAllEditor;
        var editor=new TableEditor(mctable.tableid,mctable.mcssData.data,mctable.modeldata,params);
        mctable.cellEditor=editor;
    }
    mctable.cellEditor.run();

    if (mctable.view=='tableedit')
    {
        mctable.showAction("tableedit",false);
        $(mctable.table).find(".mcsstable_record_edit").remove();//必须把编辑按钮移除。如果只是隐藏，将导致该按钮出现在全表编辑状态下
        $("#"+tableid+"_pagebar").hide();
    }
    if (mctable.params.afterRunTableEdit)
        mctable.params.afterRunTableEdit(mctable);
}

//进入单元格编辑状态
function mcsstable_cellEdit(tableid,e)
{
    var mctable=mcsstable_getMCSSTable(tableid);
    if (!mctable.cellEditor)
    {
        if (typeof(TableEditor)=='undefined')
        {
            mcdom.popup("抱歉，加载文件不完整，表格编辑功能无法使用。可能是因为网速过慢，请刷新页面试试。",{obj:e,title:"错误信息"});
            return;
        }
        else
        {
            var editor=new TableEditor(mctable.tableid,mctable.mcssData.data,mctable.modeldata,{mcsstable:mctable,mcssData:mctable.mcssData,auto_fill_fields:mctable.params.auto_fill_fields,auto_fill_fields_data:mctable.params.auto_fill_fields_data});
            mctable.cellEditor=editor;
        }
    }
    mctable.view='celledit';
    mctable.cellEditor.run();
}

function mcsstable_save(tableid,e)
{
    var mcsstable=mcsstable_getMCSSTable(tableid);
    if (mcsstable.params.beforeSave)
        mcsstable.params.beforeSave(mcsstable);
    if (!mcsstable.verifyInputer())
    {
        mcdom.popup('保存失败，因为校验未通过！',{title:lang.save,fadeOut:true,obj:e});
        return false;
    }
    mcsstable.cellEditor.mcssData.save(mcsstable_afterSave,{srcElement:e,refObject:mcsstable});
    return true;
}

//设置保存后的样式
function mcsstable_setStyleAfterSave(mcsstable)
{
    $(mcsstable.table).find(".mcss_editedData").removeClass(mcsstable.cellEditor.editedDataClassName).css("background-color","").css("color","");
    $(mcsstable.table).find("tr[mcss_recordtype='NEW']").attr("mcss_recordtype","");
    mcsstable_setSaveBtn(mcsstable.tableid);
}

function mcsstable_afterSave(mcssData,mcsstable)
{
    mcsstable_setStyleAfterSave(mcsstable);
    if (mcsstable.params.afterSave)
        mcsstable.params.afterSave(mcssData,mcsstable);
}
function mcsstable_cancel(tableid,e)
{
    var mcsstable=mcsstable_getMCSSTable(tableid);
    if (mcsstable.ifDataChanged())
    {
        var h="<div id='"+tableid+"_addTask' class='form-group'>"+lang.notsavedtoask+"</div>"
            +"<div class='fuceng_buttom'>"
            +"<input type='button' id='"+tableid+"_save' class='btn btn-primary btn-sm' value='"+lang.save+"' />"
            +" <span id='"+tableid+"_cancel' class='btn btn-default btn-sm'>"+lang.notsave+"</span>"
            +"</div>";

        mcpopup=mcdom.showPopup(e,h,null,null,null,'auto','auto',"返回");
        $("#"+tableid+"_save").focus();
        $("#"+tableid+"_save").click(function()
        {
            if (mcsstable_save(tableid))
            {
                mcsstable_gotoCelleditState(tableid,true);
            }
            $(mcpopup).remove();
        })
        $("#"+tableid+"_cancel").click(function()
        {
            mcsstable_gotoCelleditState(tableid,true);
            $(mcpopup).remove();
        })
    }else
    {
        mcsstable_gotoCelleditState(tableid,true);
    }
    if (mcsstable.params.aterExecuteAction)
        mcsstable.params.aterExecuteAction(mcsstable,e);
}

//恢复保存按钮样式
function mcsstable_setSaveBtn(tableid)
{
    var mctable=mcsstable_getMCSSTable(tableid);
    $(mctable.getAction('save')).css('border-color','');//数据修改后按钮颜色会变，这里保存后要恢复
}
//把表格置于浏览celledit状态
function mcsstable_gotoCelleditState(tableid,celledit)
{
    var mctable=mcsstable_getMCSSTable(tableid);
    mctable.view="celledit";
    if (mctable.cellEditor)
        mctable.cellEditor.mcssData.state="";
    mcsstable_setIconField(mctable);//设置图标
    mcsstable_loaddatarows(tableid);//从后台数据库取
    //mcsstable_loadRows(mctable,mctable.mcssData.data);//从前台的data中读取速度快.但因为data没有按排序字段排列，因此暂时不能用这个。有个解决办法：根据当前的tr顺序，逐行根据data数据重新生成tr。这样就保留了当前顺序

    mctable.showAllActions();
    mctable.showAction("insert",false);
    mctable.showAction("cancel",false);
    //mctable.showAction("save",false);
    mctable.showAction("batchedit",true);
    mctable.showAction("open",true);
    $("#"+mctable.params.tableid+"_pagebar").show();
    $("#addgroupbtn_atlast").remove();
    mcsstable_setSaveBtn(tableid);
}
//为图标自定义处理变量赋值
function mcsstable_setIconSpecial(mctable,field){
    if(mctable.params.notcelledit)
        mctable.params.notcelledit+=",";
    mctable.params.notcelledit+=field.id;
    if(field.type=="colorIcon")
        mctable.params.special_field_show[field.id] = mcsstable_getColorIcon;
    else if(field.type=="priorityIcon")
        mctable.params.special_field_show[field.id] = mcsstable_getPriorityIcon;
    else if(field.type=="statueIcon")
        mctable.params.special_field_show[field.id] = mcsstable_getStatusIcon;
}
//设置图标字段执行方法
function mcsstable_setIconField(mctable){
    if (!mctable.modeldata)
        return;
    var fields = mctable.modeldata.fields;
    if(!mctable.params.special_field_show)
        mctable.params.special_field_show = {};
    if(!mctable.params.notcelledit)
        mctable.params.notcelledit = "";
    for(var i = 0;i < fields.length;i++){
        if(fields[i].type=="colorIcon" || fields[i].type=="priorityIcon" || fields[i].type=="statueIcon")
            mcsstable_setIconSpecial(mctable,fields[i]);
    }
}
//处理优先级图标
function mcsstable_getPriorityIcon(value,record,field,mcsstable){
    if (!field)
        return '';
    var h='',css='';
    if(!value)value="";
    if (value=='高') css='icon-priority-high';
    else if (value=='中') css='icon-priority-middle';
    else if (value=='低') css='icon-priority-low';
    else css='icon-priority-other';
    h="<div title='"+value+"' inEdit='true' class='" +css+"' ";
    h+=" onclick='mcsstable_changePriority(this,\""+mcsstable.tableid+"\",\""+record.id+"\",\""+field.id+"\")' ";
    if(value=='高' || value=='中' || value=='低')
        h+="></div>";
    else
        h+=">"+value+"</div>";
    return h;
}
//点击更换优先级图标
function mcsstable_changePriority(e,id,recordid,fieldid)
{
    var mcsstable=mcsstable_getMCSSTable(id);
    var readonly=mcsstable.modelDealer.getFieldInfo(fieldid,"readonly");
    if (mcsstable.view=='table' || mcsstable.view=='' || readonly==true)
        return;
    var td=e.parentNode.parentNode;
    if (td.readonly || $(td).attr('readonly')) return;
    var value='';
    e.innerHTML = "";
    if (e.className.indexOf('icon-priority-high')>-1){
        e.className='icon-priority-middle';
        value='中';
    }else if (e.className.indexOf('icon-priority-middle')>-1){
        e.className='icon-priority-low';
        value='低';
    }else{
        e.className='icon-priority-high';
        value='高';
    }
    e.title=value;


    var oldValue=mcsstable.mcssData.getFieldValue(recordid,fieldid);if (oldValue==null) oldValue=''
    mcsstable.editField(recordid,fieldid,value,false);
    if(mcsstable.view=="celledit")
        mcsstable.mcssData.save(null,{alert:false,refObject:mcsstable,onlyonefield:fieldid,oldValue:oldValue},mcsstable.setFieldValueToTD);
}

//点击更换状态图标
function mcsstable_changeStatus(e,id,recordid,fieldid)
{
    var mcsstable=mcsstable_getMCSSTable(id);
    var readonly=mcsstable.modelDealer.getFieldInfo(fieldid,"readonly");
    if (mcsstable.view=='table' || mcsstable.view=='' || readonly==true)
        return;
    var td=e.parentNode.parentNode;
    if (td.readonly || $(td).attr('readonly')) return;
    var value='';
    e.innerHTML = "";
    if (e.className.indexOf('icon-status-plan')>-1){
        e.className='icon-status-execute';
        value='执行';
    }else if (e.className.indexOf('icon-status-execute')>-1){
        e.className='icon-status-done';
        value='完成';
    }else if (e.className.indexOf('icon-status-done')>-1){
        e.className='icon-status-cancel';
        value='取消';
    }else{
        e.className='icon-status-plan';
        value='计划';
    }

    e.title=value;
    var oldValue=mcsstable.mcssData.getFieldValue(recordid,fieldid);if (oldValue==null) oldValue=''
    mcsstable.editField(recordid,fieldid,value,false);
    $(e.parentNode.parentNode).css("background",mcsstable.cellEditor.editedDataColor);
    if(mcsstable.view=="celledit")
        mcsstable.mcssData.save(null,{alert:false,refObject:mcsstable,onlyonefield:fieldid,oldValue:oldValue},mcsstable.setFieldValueToTD);
//	if(mcsstable.params.gantetu_showTimeLines) //sssss
//		gantetu_showTimeLines(mcsstable);
}

//点击更换颜色
function mcsstable_changeColor(e,tableid,recordid,fieldid)
{
    var mcsstable=mcsstable_getMCSSTable(tableid);
    var readonly=mcsstable.modelDealer.getFieldInfo(fieldid,"readonly");
    if (mcsstable.view=='table' || mcsstable.view=='' || readonly==true)
        return;

    var td=e.parentNode.parentNode;
    if (td.readonly || $(td).attr('readonly')) return;

    var value='';
    e.innerHTML = "";
    if (e.className.indexOf('icon-color-red')>-1){
        e.className='icon-color-green';
        value='绿';
    }else if (e.className.indexOf('icon-color-green')>-1){
        e.className='icon-color-yellow';
        value='黄';
    }else{
        e.className='icon-color-red';
        value='红';
    }
    e.title=value;
    var oldValue=mcsstable.mcssData.getFieldValue(recordid,fieldid);if (oldValue==null) oldValue='';
    mcsstable.editField(recordid,fieldid,value,false);
    if(mcsstable.view=="celledit")
        mcsstable.mcssData.save(null,{alert:false,refObject:mcsstable,onlyonefield:fieldid,oldValue:oldValue},mcsstable.setFieldValueToTD);
}

//处理状态图标
function mcsstable_getStatusIcon(value,record,field,mcsstable){
    if (!field)
        return '';
    var h='',css='';
    if(!value) value="";
    if (value=='计划') css='icon-status-plan';
    else if (value=='执行') css='icon-status-execute';
    else if (value=='完成') css='icon-status-done';
    else if (value=='取消') css='icon-status-cancel';
    else css='icon-status-other';
    h="<div title='"+value+"' inEdit='true' class='" +css+"' ";
    h+=" onclick='mcsstable_changeStatus(this,\""+mcsstable.tableid+"\",\""+record.id+"\",\""+field.id+"\")' ";
    if(value=='计划' || value=='执行' || value=='完成' || value=='取消')
        h+="></div>";
    else
        h+=">"+value+"</div>";
    return h;
}

//处理三色图标
function mcsstable_getColorIcon(value,record,field,mcsstable){
    if (!field)
        return '';
    var h='',css='';
    if(!value) value="";
    if (value=='红') css='icon-color-red';
    else if (value=='绿') css='icon-color-green';
    else if (value=='黄') css='icon-color-yellow';
    else css='icon-color-other';
    h="<div title='"+value+"' inEdit='true' class='" +css+"' ";
    //if (mcsstable.view=='celledit')
    h+=" onclick='mcsstable_changeColor(this,\""+mcsstable.tableid+"\",\""+record.id+"\",\""+field.id+"\")' ";
    if(value=='红' || value=='黄' || value=='绿')
        h+="></div>";
    else
        h+=">"+value+"</div>";
    return h;
}


function mcsstable_canAddNewRecord(mcsstable)
{
    if (mcsstable.params.project_fulldata && mcsstable.params.project_fulldata.shareoption && mcsstable.params.project_fulldata.shareoption.indexOf("[notaddrecord]")>-1)
    {
        //mcdom.alert("表格共享时被设置为不允许添加记录。",lang.del,'info','fadeout','','',{'position':'middle',hideheader:true,seconds:3});
        return false;
    }
    return true;
}

//mctable的新增记录功能
function mcsstable_addnew(e,tableid,url,lang_add,isinsert)
{
    //chenli prof todo
    var mcsstable=mcsstable_getMCSSTable(tableid);
    if (!url) url=mcsstable.getAddUrl();
    if (!isinsert) isinsert=false;
    if (!lang_add)lang_add="添加";
    if (mcsstable_canAddNewRecord(mcsstable)==false)
        return;

    if (mcsstable.params.beforeAddNew){
        if (mcsstable.params.beforeAddNew(mcsstable)==false)
            return;
    }
    if (mcsstable.view=='tableedit') //表格编辑
        mcsstable_addNewEditRow(mcsstable,isinsert);
    else
    {
        var sort=0;
        if (mcsstable.modeldata.sortfieldid)
        {
            sort=mcsstable_getMaxRowSortIndex(mcsstable);
            mcsstable.modelDealer.setFieldProp(mcsstable.modeldata.sortfieldid,'defaultdata',sort+100);
        }
        if (mcsstable.params.getAutoId==true)
            mcsstable.modelDealer.setFieldProp(mcsstable.keyfield,'defaultdata',getTimeId());
        mcsstable_showPopupForm(e,tableid,url,lang_add,null,'ADD');
		//记录多少人通过微信添加按钮添加 
        
        $(".fuchuceng").css("position","static");

        
    }
}

//获得某个表的最大序号值
function mcsstable_getMaxRowSortIndex(mcsstable)
{
    var max=0;
    if (mcsstable.mcssData)
    {
        for(var i=0;i<mcsstable.mcssData.data.length;i++){
            if (mcsstable.mcssData.data[i][mcsstable.modeldata.sortfieldid]>max)
                max=mcsstable.mcssData.data[i][mcsstable.modeldata.sortfieldid];
        }
    }
    return parseFloat(max);
}

//获得时间随机数，从年份的最后2位，到毫秒，最后加5	位随机数，共20位。例如13112918400745636211
function getTimeId()
{
    return uni_getTimeId();
}


function mcsstable_addNewEditRow(mcsstable,isinsert,focus)
{

    var row = mcsstable_addNewEditRow2(mcsstable,isinsert,focus);
    var recordid=row.id;
    var rowData=row.data;
    //为支持撤销
    //var tableState={rowIndex:row.rowIndex-1,cellIndex:mcsstable.divSel.td.cellIndex};
    mcsstable.addUndoList(mcsstable,"deleteRow",[recordid],mcsstable,"addRow",[rowData,row.dataIndex]);

    mcsstable_autoSetSnField(mcsstable,row.id);
    return row;
}

//添加一条编辑状态的记录
//isinsert:是否插入到当前行前面。可选：true,after,false
//mcsstable_addNewEditRow会调用这个函数
function mcsstable_addNewEditRow2(mcsstable,isinsert,focus)
{
    if (typeof(mcsstable)=="string")
        mcsstable=mcsstable_getMCSSTable(mcsstable);

    if (mcsstable.table.rows.length>550)
    {
        mcdom.alert("表格每页行数不能超过550。",'警告','info');
        return;
    }
    if(mcsstable.params.canAppendRow==false)
        return;

    var recordid=getTimeId();
    if (mcsstable.params.getAutoId==true)
        mcsstable.modelDealer.setFieldProp(mcsstable.keyfield,'defaultdata',recordid);
    var fields=mcsstable.fields;
    var tableid=mcsstable.tableid;

    var row;

    var newdata=mcsstable_createNewData(mcsstable,recordid);
    var s=mcsstable_addonerow(newdata,tableid);
    var sfix=mcsstable_addfixrow(newdata,tableid);
    var rowid=mcsstable.lastRowId;
    if (!mcsstable.currentRow) mcsstable.setCurrentRow($(mcsstable.tableFixCol).get(0).rows[0]);
    var cur=mcsstable.tableData.rows[mcsstable.currentRow.rowIndex];
    var curfix=mcsstable.tableFixCol.rows[mcsstable.currentRow.rowIndex];

    if(isinsert)
    {
        if(cur && cur.rowIndex>=0)
        {
            if (cur.getAttribute("groupname") || isinsert=='after'){
                $(cur).after(s);
                $(curfix).after(sfix);
                row=mcsstable.tableData.rows[mcsstable.currentRow.rowIndex+1];
            }
            else{
                $(cur).before(s);
                $(curfix).before(sfix);
                row=mcsstable.tableData.rows[mcsstable.currentRow.rowIndex-1];
            }
        }
        else{
            $(mcsstable.tableData).append(s);
            $(mcsstable.tableFixCol).append(sfix);
            row=mcsstable.tableData.rows[mcsstable.tableData.rows.length-1];
        }
    }else{

        $(mcsstable.tableData).append(s);
        $(mcsstable.tableFixCol).append(sfix);
        row=mcsstable.tableData.rows[mcsstable.tableData.rows.length-1];
    }
    var rowfix=mcsstable.tableFixCol.rows[ row.rowIndex];
    mcsstable_showNewRowActions( rowfix,mcsstable.tableFixCol);

    if ($(mcsstable.table).attr("showAllRecordActions")=="true")
        $(mcsstable.table).find("a[tag='recordAction']").show();

    $(row).attr("mcss_recordtype",'NEW');
    $(rowfix).attr("mcss_recordtype",'NEW');
    mcsstable.setCurrentRow(rowfix);
    var tleedit=false;
    if (mcsstable.view=='celledit')
        tleedit=true;

    mcsstable.cellEditor.createRowEditor(row,null,tleedit);
    row.data=newdata;
    rowfix.data=newdata;

    //新增行的分组字段的值应该默认等于当前分组
    if (mcsstable.view=='tableedit' && mcsstable.groupFieldid && mcsstable.groupFieldid!='NONEGROUP'){
        var recordid=$(row).attr("recordid");
        var g=$(row).prevAll('.rowgroup').attr('groupname');
        if (g)
            mcsstable.editField(recordid,mcsstable.groupFieldid,g,true);
    }

    if (mcsstable.modeldata.sortfieldid)
    {
        mcsstable_setSortForNewRecord(mcsstable,row);
    }

    if (mcsstable.params.afterAppendRow)
    {
        mcsstable.params.afterAppendRow(mcsstable,row);
    }

    if(mcsstable.view=="celledit"){
        mcsstable.mcssData.save(mcsstable.params.afterSave,{alert:false,srcElement:row,refObject:mcsstable});
        $(row).attr('mcss_recordtype',"");
        $(rowfix).attr('mcss_recordtype',"");
    }
    mcsstable_bindMouseEventOnRows(row,mcsstable);
    mcsstable.resortLineIndex();
    return row;
}

//根据前一行的按钮显示情况，来决定新行显示哪些按钮
function mcsstable_showNewRowActions(row,table)
{
    if (table.rows.length<2) return;
    if ($(table.rows[1]).hasClass("rowgroup"))
        var row1cell=table.rows[2].cells[0];
    else
        var row1cell=table.rows[1].cells[0];
    $(row1cell).find("a[tag='recordAction']").each(function(i,item){
        var cls=item.className.split(" ")[0];
        if (item.style.display=="")
            $(row.cells[0]).find("."+cls).show();
    })
}

//给当前行设置排序值。要先保证currentRow是对的
function mcsstable_setSortForNewRecord(mcsstable,row)
{
    var sortfield=mcsstable.modeldata.sortfieldid;
    var recordid=$(row).attr('recordid');
    var newSortIndex=mcsstable_getSortIndexForNewRecord(mcsstable,row);
    mcsstable.mcssData.setFieldValue(recordid,sortfield,newSortIndex);
    mcsstable.cellEditor.showFieldValueInCell(recordid,sortfield);
    if(mcsstable.view=="celledit"){
        //mcsstable.mcssData.save(null,{alert:true});
        //$(mcsstable.table).find("tr[mcss_recordtype='NEW']").attr('mcss_recordtype',"");
    }
    return newSortIndex;
}

//给新增的行赋予顺序号的值，要先保证currentRow是对的
function mcsstable_getSortIndexForNewRecord(mcsstable,row)
{
    var sortfield=mcsstable.modeldata.sortfieldid;
    //给新增记录赋予正确的序号
    var pre_id=$(mcsstable.currentRow).prev().attr("recordid");
    var next_id=$(mcsstable.currentRow).next().attr("recordid");
    var recordid=$(row).attr('recordid');
    var newSortIndex=0;
    var maxSort = parseFloat(mcsstable.params.maxSort)+100;
    if (!pre_id && !next_id)//空记录时
    {
        // newSortIndex= 1;
        newSortIndex= maxSort;
    }
    else
    if (!pre_id)//第一条记录
    {
        var next_sort=mcsstable.mcssData.getFieldValue(next_id,sortfield);
        newSortIndex= parseFloat(next_sort)-100;
    }
    else
    if (!next_id)//最后一条记录
    {
        // var pre_sort=mcsstable.mcssData.getFieldValue(pre_id,sortfield);
        // newSortIndex= parseFloat(pre_sort)+100;
        newSortIndex= maxSort
    }
    else
    {
        //中间记录
        var pre_sort=mcsstable.mcssData.getFieldValue(pre_id,sortfield);
        var next_sort=mcsstable.mcssData.getFieldValue(next_id,sortfield);
        newSortIndex=getMiddleSort(parseFloat(pre_sort),parseFloat(next_sort));
    }
    if (isNaN(newSortIndex))
        newSortIndex=0;
    return newSortIndex;
}

//获得两个小数之间的一个数，这个新数的位数（包括小数点后面的数字）应该尽量少
function getMiddleSort(num1,num2)
{
    //计算小数点后面的位数
    var next=num2+"";
    var pos=next.indexOf(".");
    var u=1;
    if (pos>-1)
    {
        pos=next.length-pos-1;
    }
    else
        pos=0;
    var u=1;
    for(var i=0;i<pos;i++)
    {
        u=u/10;
    }

    num2-=u;
    num2=parseFloat(num2.toFixed(pos));
    if (num2>num1)
        return num2;
    num2=num2+u;
    var r=(num1+num2)/2;
    r1=num2+"";
    var pos=r1.length-r1.indexOf(".");
    if (pos==-1) pos=0;
    return r.toFixed(pos);
}
//生成随机字符串
function randomString(l)
{
    var   x="0123456789qwertyuioplkjhgfdsazxcvbnm";
    var   tmp="";
    if (l==undefined)	l=8;
    for(var   i=0;i<l;i++)   {
        tmp   +=   x.charAt(Math.ceil(Math.random()*100000000)%x.length);
    }
    return   tmp;
}

function mcsstable_createNewDataByRow(mcsstable,row)
{
    if (!mcsstable.cellEditor)
        return;
    //根据mcsstable的一条tr对象生成新的数据并更新到类的数据组中
    var mcssData=mcsstable.cellEditor.mcssData;
    var fields=mcssData.modeldata.fields;
    var recordid=$(row).attr("recordid");
    var newData={mcss_recordtype:"NEW",recordid:recordid,fieldid:mcsstable.keyfield,value:recordid};
    mcssData.editNewData(newData);
    for(var i=0;i<fields.length;i++)
    {
        if (fields[i].defaultdata && fields[i].id!=mcsstable.keyfield)//处理字段默认值，都要生成一条记录更新到数据组中
        {
            newData={mcss_recordtype:"NEW",recordid:recordid};
            newData.fieldid=fields[i].id;
            newData.value=fields[i].defaultdata;

            mcssData.editNewData(newData);
        }
    }
}

function mcsstable_createNewData(mcsstable,recordid)
{
    //根据mcsstable的一条tr对象生成新的数据并更新到类的数据组中
    var mcssData=mcsstable.cellEditor.mcssData;
    var fields=mcssData.modeldata.fields;

    var newData={mcss_recordtype:"NEW",recordid:recordid,fieldid:mcsstable.keyfield,value:recordid};
    mcssData.editNewData(newData);
    for(var i=0;i<fields.length;i++)
    {
        var v="";
        if (fields[i].id=="SYS_ADDUSER" || fields[i].id=="SYS_EDITUSER")
            v=loginuser;
        else
        if (fields[i].id=="SYS_ADDTIME" || fields[i].id=="SYS_EDITTIME")
            v=getGoodNow();

        if (fields[i].defaultdata && fields[i].id!=mcsstable.keyfield)//处理字段默认值，都要生成一条记录更新到数据组中
        {
            v=fields[i].defaultdata;
        }
        else if (mcsstable.groupField && mcsstable.groupField.id==fields[i].id)
            v=mcsstable.last_groupfield_value;

        if(getCookie("mcss_always_"+fields[i].id))
            v=getCookie("mcss_always_"+fields[i].id);

        if (v)
        {
            newData.fieldid=fields[i].id;
            newData.value=v;
            mcssData.editNewData(newData);
        }

    }
    return mcssData.getRow(recordid);
}

function mcsstable_setFieldValue(v,fields,i,newData,mcsstable)
{
    var v="";
    if (fields[i].id=="SYS_ADDUSER" || fields[i].id=="SYS_EDITUSER")
        v=loginuser;
    else
    if (fields[i].id=="SYS_ADDTIME" || fields[i].id=="SYS_EDITTIME")
        v=uni_global.today;


    if (fields[i].defaultdata && fields[i].id!=mcsstable.keyfield)//处理字段默认值，都要生成一条记录更新到数据组中
    {
        v=fields[i].defaultdata;
    }
    else if (mcsstable.groupField && mcsstable.groupField.id==fields[i].id)
        v=mcsstable.last_groupfield_value;

    if(getCookie("mcss_always_"+fields[i].id))
        v=getCookie("mcss_always_"+fields[i].id);

    if (v)
    {
        newData.fieldid=fields[i].id;
        newData.value=v;
        mcsstable.mcssData.editNewData(newData);
    }
}

//mctable的新增记录功能
function mcsstable_editRecord(e,tableid,url,lang_add,open_window_style,recordid)
{
    mcsstable_showPopupForm(e,tableid,url,lang_add,recordid,'ADD');
}

/*
 打开新增或查看窗口
 包括两种：自动创建表单、用新窗口打开. action:EDIT,VIEW,ADD
 */
function mcsstable_showPopupForm(e,tableid,url,title,recordid,action)
{
    if (!action)
        action=title;
    var mcsstable=mcsstable_getMCSSTable(tableid);
    var open_window_style=mcsstable.modeldata.open_window_style;
    if (open_window_style=='samewindow')
    {
        document.location.href=url;
        return;
    }
    else if (open_window_style=='newwindow')
    {
        window.open(url,"_blank");
        return;
    }

    //mcsstable.click_obj=e;//临时给mcsstable添加变量，目的是给关闭浮出层找到是哪个对象（如按钮）触发了浮出层
    //如果没有指定弹出窗口，则采用自动创建表单的方式
    if (!recordid
        && (!mcsstable.modeldata.addurl || (((mcsstable.modeldata.addurl).indexOf("/")<0) && mcsstable.modeldata.addurl!="default")) && title==lang.add
        ||  ((!mcsstable.modeldata.editurl || (((mcsstable.modeldata.editurl).indexOf("/")<0) && mcsstable.modeldata.editurl!="default")) && recordid && title==lang.edit)
        || ((!mcsstable.modeldata.openurl || (((mcsstable.modeldata.openurl).indexOf("/")<0) && mcsstable.modeldata.openurl!="default")) && recordid && title==lang.open)
    )
    {
        mcsstable_openAutoform(e,mcsstable,recordid,title,action);
    }
    else
    {
        //个性化页面用弹出iframe
        var u=url+"/tableid/"+tableid;
        if (mcsstable.params.defaultValue && url.indexOf('defaultValue') == -1)
            u+="/defaultValue/"+mcsstable.params.defaultValue;
        mcsstable_openWindow(u,title,open_window_style);
    }
}

//单击mcsstable的新增、编辑或打开按钮，创建表单，显示内容
//mcsstable:可以是tableid，也可以是mcsstable对象。action：VIEW或EDIT
function mcsstable_openAutoform(e,mcsstable,recordid,title,action)
{
    $(".actionSheet_wrap").fadeOut(100);
    var loginuser = getCookie("mcss_loginuser");
    if (!action)
        action=title;
    mcsstable=mcsstable_getMCSSTable(mcsstable);
    
    var open_window_style=mcsstable.modeldata.open_window_style;
    var tableid=mcsstable.tableid;

    var currentRow = "";
    if(mcdom.getObjByTagName(e,"TR"))
        currentRow = mcdom.getObjByTagName(e,"TR").rowIndex;

    var header="";

    var position="";
    var width='';

    if (recordid)
    {
        if(action=='EDIT' && mcsstable.modeldata.editurl)
            editmodelid=mcsstable.modeldata.editurl;
        else if(mcsstable.modeldata.openurl && action=='VIEW')
            editmodelid=mcsstable.modeldata.openurl;
        else
            editmodelid=mcsstable.params.modelid
    }
    else
    {
        if(mcsstable.modeldata.addurl)
            editmodelid=mcsstable.modeldata.addurl;
        else
            editmodelid=mcsstable.params.modelid;
    }


    var obj = $("#"+tableid+"_save");
    var formButtons="";
    if (action=='EDIT' || action=='ADD')
        formButtons="SAVENOTCLOSE,SAVE";
    if (action=='VIEW')
        formButtons="VIEW";
    var params={modelid:editmodelid,defaultValue:mcsstable.params.defaultValue,
        recordid:recordid,refObject:mcsstable,popupObj:e,
        usermodel_recordid:mcsstable.params.usermodel_recordid,
        afterSave:mcsstable_afterSaveAndClose,
        action:action,
        afterSaveAndNew:mcsstable_afterSaveAndNew,
        showFormButton:formButtons,moveToScreen:true,
        makeVisible:true,
        model_ownerid:mcsstable.params.model_ownerid,
        view_id:mcsstable.view_id,
    };
    if (mcsstable.params.beforeSaveForm)
        params.beforeSave=mcsstable.params.beforeSaveForm;
    var cols = getCookie(loginuser+"_"+mcsstable.params.usermodel_recordid+"_cols");
    if(cols){
        params.cols = cols;
        $("a[value = '"+cols+"']").css('color','red');
    }
    mcsstable.modelDealer.setFieldProp(mcsstable.keyfield,'defaultdata',getTimeId());

    //if (editmodelid==mcsstable.modelid) //注释这两行是为了让智能输入时带出字段对隐藏字段也有效
    //	params.model=mcsstable.modeldata;
    if (mcsstable.params.afterGetModel)
        params.afterGetModel=mcsstable.params.afterGetModel;

    mcsstable.modelDealer.setFieldProp(mcsstable.keyfield,'defaultdata',getTimeId());

    //var Mcsstableform=new Autoform(tableid+"_autoform",params);
    //Mcsstableform.run(mcsstable_afterRunForm);
    if (params.recordid==null) params.recordid="";
    if (params.recordid=="null") params.recordid="";
    if (!params.view_id) params.view_id="";
    if (!params.usermodel_recordid) params.usermodel_recordid="";
    localStorage.setItem("open_form_recordid",params.recordid);
    localStorage.setItem("open_form_fieldid",params.fieldid);
    localStorage.setItem("open_form_view_id",params.view_id);
    localStorage.setItem("open_form_modelid",params.modelid);
    localStorage.setItem("open_form_model_ownerid",params.model_ownerid);
    localStorage.setItem("open_form_defaultValue",params.defaultValue);
    localStorage.setItem("open_form_action",params.action);
    localStorage.setItem("open_form_usermodel_recordid",params.usermodel_recordid);

    localStorage.setItem("open_form_shareOptions",mcsstable.params.project_fulldata.shareoption);
    var allowAdd=mcsstable_canAddNewRecord(mcsstable);
    localStorage.setItem("open_form_allowAdd",allowAdd);
    uexWindow.open('open_form', '0', 'datarecord.html', '5', '', '', 0);
    
    
}

//重新生成一个autoform
function mcsstable_newAutoForm(obj)
{
    obj.run();
}

//单击新增弹出页面生成autoform后
function mcsstable_afterRunForm(autoform)
{
    mcsstable_autowithpopup(autoform);
    if (autoform.params.refObject.params.afterRunAutoform)
        autoform.params.refObject.params.afterRunAutoform(autoform);
    if (uni_global.isMobile())
    {
        //alert("调试中"+document.body.clientWidth);
        var popup=mcdom.getObjByClass(autoform.form,"fuchuceng");
        mcdom.moveToScreen(popup);
        $(".fuchuceng").show();

    }
    $(".fuchuceng").show();
}
//自动调整浮层的大小
function mcsstable_autowithpopup(form)
{
    if(form.isshow)
    {
        form.SetReadonly();
    }
    var mcsstable=form.params.refObject;
    if (mcsstable && mcsstable.params.afterOpenEditForm)
        mcsstable.params.afterOpenEditForm(form);
}

//
function mcsstable_afterSaveAndNew(recordid,hint,mcform)
{
    mcsstable_updateNewRow(recordid,hint,mcform);
    if (mcform.modeldata.sortfieldid)
    {
        var sort=mcsstable_getMaxRowSortIndex(mcform.params.refObject);
        mcform.modelDealer.setFieldProp(mcform.modeldata.sortfieldid,'defaultdata',sort+100);
    }
    //if (getAutoId)这里应该要设置条件
    mcform.modelDealer.setFieldProp(mcform.modeldata.keyfield,'defaultdata',getTimeId());
    mcform.run();
}


//把新增或更新的记录更新到mcsstable
function mcsstable_updateNewRow(recordid,hint,mcform)
{
    var mcsstable=mcform.params.refObject;
    if (!mcsstable)
        return;
    var row=mcsstable.mcssData.getRow(recordid);
    if (!row)//新增记录
    {

        var rows = mcsstable.table.rows;
        var dataIndex=1;
        for(var r=1;r<rows.length-1;r++)
        {
            var row=rows[r];
            if (!row.getAttribute("recordid") && !row.getAttribute("groupname") )
            {
                mcsstable.currentRow=row;
                break;
                dataIndex=row.dataIndex;
            }
        }
        mcsstable_onNewRow(mcsstable.currentRow,mcform.datarecord,recordid);
        //支持撤销
        mcsstable.addUndoList(mcsstable,"deleteRow",[recordid],mcsstable,"addRow",[mcform.datarecord,dataIndex]);

        if (mcsstable.params.afterAppendRow)
        {
            mcsstable_bindMouseEventOnRows(tr,mcsstable);
            mcsstable.params.afterAppendRow(mcsstable,mcsstable.table.rows[mcsstable.table.rows.length-1]);
            var tr=mcsstable.table.rows[mcsstable.table.rows.length-1];
            $(tr).find("input").focus();
            $(tr).fadeTo("slow",0.05);//动画效果
            $(tr).fadeTo("slow",1);
        }
    }
    else
    {
        mcform.updateFormToData();
        var row=mcform.datarecord;

        var oldrow=mcsstable.mcssData.getRow(recordid);
        var oldrowData = uni_copyJSON(oldrow);
        var newrowData=uni_copyJSON(row);
        //支持撤销
        mcsstable.addUndoList(mcsstable,"updaterowData",[oldrowData],mcsstable,"updaterowData",[newrowData]);

        mcsstable.mcssData.updateRow(recordid,row);
        mcsstable.updateRowByData(recordid,row);
    }
    mcsstable.resortLineIndex();
}

function mcsstable_afterSaveNotClose(recordid,hint,mcform)
{
    mcsstable_updateNewRow(recordid,hint,mcform);
}

//保存新增表单后刷新mctable
function mcsstable_afterSaveAndClose(recordid,hint,mcform)
{
    if (mcform.IsNewRecord){
        var mcsstable=mcform.params.refObject;
        var bFindRow=false;
        var rows = mcsstable.tableData.rows;
        var dataIndex=1;
        for(var r=1;r<rows.length-1;r++)
        {
            var row=rows[r];
            if (!row.getAttribute("recordid") && !row.getAttribute("groupname") )
            {
                mcsstable.currentRow=row;
                bFindRow=true;
                break;
            }
            dataIndex=row.dataIndex+1;
        }
        if (bFindRow==false) {
            dataIndex = mcsstable.tableData.rows[mcsstable.tableData.rows.length-1].dataIndex;
            mcsstable_appendEmptyrow(mcsstable.table,dataIndex,1);
            var row=mcsstable.tableData.rows[mcsstable.tableData.rows.length-1];
            mcsstable.currentRow=row;
        }
        //支持撤销
        mcsstable.addUndoList(mcsstable,"deleteRow",[recordid],mcsstable,"addRow",[mcform.datarecord,dataIndex]);

        mcsstable_onNewRow(mcsstable.currentRow,mcform.datarecord,recordid);
        mcsstable_bindMouseEventOnRows(row,mcsstable);

        if (mcsstable.params.afterAddRow)
            s=mcsstable.params.afterAddRow(mcsstable,s,rowData);
        if (mcsstable.params.afterAppendRow)
        {
            mcsstable.params.afterAppendRow(mcsstable,mcsstable.tableData.rows[mcsstable.tableData.rows.length-1]);
            var tr=mcsstable.tableData.rows[mcsstable.tableData.rows.length-1];
            $(tr).find("input").focus();
        }
    }
    else{
        mcsstable_updateNewRow(recordid,hint,mcform);
    }
    mcdom.closePopup(mcform.params.popup);
}

function mcsstable_row_onmouseover(e)
{
    var tr=$(e);
    var table=e.parentNode.parentNode;
    var mcsstable=mcsstable_getMCSSTable(table.id);
    var trf=mcsstable.tableFixCol.rows[e.rowIndex];
    var trd=mcsstable.tableData.rows[e.rowIndex];

    $(trd).addClass('mcsstable_tr_hover');
    $(trf).addClass('mcsstable_tr_hover');
    if (trf.cells.length>0){
        if (trf.cells[0].className=="linenumber")
            trf.cells[0].className="linenumber_hover";
    }
    $(trf).find("[tag='rowactionmenu']").show();

}
function mcsstable_row_onmouseout(e,tableid)
{
    var tr=$(e);
    var table=e.parentNode.parentNode;
    var mcsstable=mcsstable_getMCSSTable(table.id);
    var trf=mcsstable.tableFixCol.rows[e.rowIndex];
    var trd=mcsstable.tableData.rows[e.rowIndex];

    $(trd).removeClass('mcsstable_tr_hover');
    $(trf).removeClass('mcsstable_tr_hover');//.find("td").eq(0).css('border-left','solid 3px '+uni_global.highlight_color);
    if (trf.cells.length>0){
        if (trf.cells[0].className=="linenumber_hover") trf.cells[0].className="linenumber";
    }
    if(!mcsstable.params.alwaysShowRowActionMenu)
        $(trf).find("[tag='rowactionmenu']").hide();

}
//鼠标移动到td上时,已经废弃
function mcsstable_td_onmouseover(e,tableid)
{
    var mcsstable=mcsstable_getMCSSTable(tableid);
    var go=true;
    if (mcsstable.params.td_onmouseover)
        go=mcsstable.params.td_onmouseover(e);
    if (go==false)
        return;

}

//得到选中的所有记录ID
function mcsstable_whencheck(e,tableid)
{
    var mcsstable=mcsstable_getMCSSTable(tableid);
    var ids=mcsstable.selectedRowsID;
    var id=$(e).val();
    if (id + ""=="") return;
    var arr=new Array();
    if(ids)
        arr=ids.split(',');
    var t=false;
    for(var i=0;i<arr.length;i++)
    {
        if(id==arr[i])
        {
            arr.splice(i,1);
            t=true;
        }
    }
    ids=arr.join();
    if(!t){
        if (ids)
            ids+=",";
        ids+=id;
    }
    mcsstable.selectedRowsID=ids;
}
//全选得到选中的所有记录ID old
function mcsstable_whencheckall_old(e,tableid)
{
    var mcsstable=mcsstable_getMCSSTable(tableid);
    var ids=mcsstable.selectedRowsID;
    var id=$(e).val();
    var arr=new Array();
    arr=ids.split(',');
    for(var i=0;i<arr.length;i++)
    {
        if(id==arr[i])
        {
            arr.splice(i,1);
        }
    }
    ids=arr.join();
    if (ids)
        ids+=",";
    ids+=id;
    mcsstable.selectedRowsID=ids;
}
/*
 //选中已选择的记录
 function mcsstable_setSelectedRow_old(mcsstable,fvalue)
 {
 if (mcsstable.selectedRowsID)
 {
 var ids=mcsstable.selectedRowsID;
 var id=fvalue;
 var arr=new Array();
 arr=ids.split(',');
 for(var i=0;i<arr.length;i++)
 {
 if(id==arr[i])
 {
 return " checked='checked' ";
 }
 }
 }else{
 return '';
 }
 }
 */

//根据文本框输入的数据跳到指定的页面
function mcsstable_gotoPagex(tableid,page,event)
{
    //alert(event.keyCode);
    if (!event) { //火狐
        var event = window.event;
    }
    var e = event.keyCode;
    if (e==13||e==32)
    {
        var reg = /^(?:0|[1-9][0-9]?|100)$/;
        var table=mcsstable_getMCSSTable(tableid);
        if(!reg.test(page)||page==0)
        {
            //alert("请输入指定范围内的整数！");
            mcdom.popup("请输入指定范围内的整数!",{obj:event.currentTarget,fadeOut:true});
            $("#"+tableid+"_currentpage").attr("value",table.currentpage);
        }else{
            var maxpage = $("#"+tableid+"_action_totalpage").html();
            if(parseInt(page) > parseInt(maxpage)){
                mcdom.popup("超出了最大页数!",{obj:event.currentTarget,fadeOut:true});
                $("#"+tableid+"_action_currentpage").attr("value",table.currentpage);
                return;
            }
            table.currentpage=page;
            mcsstable_loaddatarows(tableid);
        }
    }

}

//根据模型获取数据，获取之后调用回调函数处理数据
function mcss_getDataByModel(callback,params)
{
    if (!params)
        params={};
    var url=g_homeurl+g_systempath+"/getData";
    $.getJSON(url, params,function(data) {
        callback(data,params.parentObject);
    });
}

function mcsstable_refresh(tableid,e)
{
    $("#func_checkRowFirst").remove();
    $("#func_checkColumnFirst").remove();
    var mcsstable=mcsstable_getMCSSTable(tableid);

    //mcsstable_getChangedDataInfo(mcsstable);

    if (mcsstable.divSelParent){
        mcsstable.divSelParent.innerHTML="";
        mcsstable.divSel = null;
        mcsstable.table.divCopy = null;
    }

    if(!mcsstable.params.noClousePopup)
        mcdom.closePopup();
    $(mcsstable.getAction("currentpage")).val(1);
    mcsstable.currentpage=1;
    mcsstable.tableData.fromRow=0;
    mcsstable.table.parentNode.parentNode.scrollTop = 0;
    mcsstable.table.style.marginTop="0px";
    mcsstable.selectedRowsID = "";
    mcsstable_clearSelect_td(mcsstable.table);
    mcsstable_loaddatarows(tableid);
    //var row = $(mcsstable.table).find("td").parent();
    //mcsstable.setTrMoveFunc(row);
}

//获取td中的显示值,考虑了td中的输入组件的类别。干特图用到
function mcsstable_getCellDisplayText(td)
{
    var text=td.value;
    if (text==undefined)
    {
        var editor=$(td).children().get(0);
        if (editor)
        {
            var typename=editor.__proto__.constructor.name;
            if (typename=="HTMLSelectElement")
                text=mcdom.getSelectedText($(td).children().get(0));
            else if (typename="HTMLCheckBox")
            {
                //还有其它各种情况未处理
            }
        }
        else
            text=td.innerHTML;
    }
    return text;
}




//显示批量处理菜单
function mcsstable_showBatchMenu(e,tableid,modelid){
    var mcsstable=mcsstable_getMCSSTable(tableid);
    var h = "";
    var actions=mcsstable.modeldata.actions+",";
    if (actions.indexOf("del,")>-1)
        h+="<a id='' onclick='mcsstable_deleterows(this,\""+tableid+"\")' title=''>删除(ctrl+d)</a>";
    // h+="<a id='' onclick='mcsstable_copyRows(this,\""+tableid+"\")' title='复制选中行并添加到末尾'>复制(ctrl+c)</a>";
    if (actions.indexOf("batchedit,")>-1)
        h+="<a id='' onclick='mcsstable_batchedit(this,\""+tableid+"\")' title='批量修改选中的行'>批量修改</a>";
    if (actions.indexOf("import,")>-1)
    {
        var projectdata=mcsstable.params.project_fulldata;
        if (!projectdata || (projectdata.project_record.SYS_ADDUSER==loginuser || projectdata.sharerecord  && projectdata.sharerecord.options && projectdata.sharerecord.options.indexOf('[canedit]')>-1))
        {
            if(uni_global.importstyle)
                h+="<a id='' onclick='mcsstable_import(\""+tableid+"\")' title='把Excel文件追加到表末'>插入Excel表格</a>";
            else
                h+="<a id='' onclick='mcsstable_import(\""+tableid+"\")' title='把Excel文件追加到表末'>插入Excel表格</a>";
            h+="<a id='' onclick='mcsstable_importFromTable(\""+tableid+"\")' title='把现有超级表格追加到表末'>插入已有的超级表格</a>";
        }
    }
    dom_showPopupMenu(h,e);
    $(mcsstable.getAction("batchedit")).removeClass("mcsstable-selected-action");
}

function mcsstable_importFromTable(tableid)
{
    var mcsstable=mcsstable_getMCSSTable(tableid);
    //行数超限判断 ADD by chenli
    if(isOverLimitedRowCount(mcsstable.recordcount)){
        return;
    }

    var h="<div style='height:320px'><div id='import_From_Table' style='height:320px'></div></div>";
    mcdom.popup(h,{width:"800px",height:"400px",title:"从超级表格文件导入",position:"middle",tag:"importfromtable"});
    var btn_action_show = {'name':mcsstable_importFromTable_specialname};
    var params={
        showfirst:false,
        pageposition:'top',
        pageoptions:"5,10,20,50,80,100",
        special_field_show:btn_action_show,
        mcsstableParentTableid:tableid,
        afterLoadRows:mcsstable_importTable_afterLoadRows,
        autoAddEmptyRow:false,
        filter:"SYS_ADDUSER=loginuser() and SYS_ADDUSER<>'' and id != '"+project_record.id+"'"
    };
    var mcsstable_poup=newMCSSTable('import_From_Table','oa_upload_table',params);
    mcsstable_poup.run();
}

function mcsstable_importFromTable_specialname(value,record,field,mcsstable){
    return "<a title='"+record.id+"' onclick='mcsstable_showSheetData(\""+record.id+"\",\""+value+"\",\""+mcsstable.params.mcsstableParentTableid+"\")'>"+value+"</a>";
}


function mcsstable_showSheetData(recordid,value,parentTableId)
{
    mcdom.closePopup();
    mcsstable_import(parentTableId,recordid,value);
}


function mcsstable_importTable_afterLoadRows(mcsstable)
{
    $(mcsstable.getAction("pagerows")).hide();
    $(mcsstable.table).find("td").removeAttr("onclick");
}


function mcsstable_setBold(e,tableid){
    var mcsstable=mcsstable_getMCSSTable(tableid);
    var params = {mcsstable:mcsstable};
    mcsstable.uniTableStyle.setStyleBold(params);
}

function mcsstable_setItalic(e,tableid){
    var mcsstable = mcsstable_getMCSSTable(tableid);
    var params = {mcsstable:mcsstable};
    mcsstable.uniTableStyle.setStyleItalic(params);
}

function mcsstable_setUnderline(e,tableid){
    var mcsstable = mcsstable_getMCSSTable(tableid);
    var params = {mcsstable:mcsstable};
    mcsstable.uniTableStyle.setUnderline(params);
}

function mcsstable_setFontSize(e,tableid){
    mcdom.closePopup();
    var mcsstable=mcsstable_getMCSSTable(tableid);
    var arr = new Array("默认","8px","9px","10px","12px","14px","16px","18px","20px","24px","28px");
    var h = "";
    for(var i = 0;i < arr.length;i++){
        h += "<a fontsize='a'>"+arr[i]+"</a>";
    }
    dom_showPopupMenu(h,e,{height:"auto",width:"120px"});
    $("a[fontsize='a']").unbind("click");
    $("a[fontsize='a']").click(function(){
        var value = $(this).html()
        if(value=="默认")
            value = "12px";
        var params = {value:$(this).html()};
        mcsstable.uniTableStyle.setFontSize(params);
        dom_closePopupMenu();
    })
}
function mcsstable_setFontFace(e,tableid){
    mcdom.closePopup();
    var mcsstable=mcsstable_getMCSSTable(tableid);
    var arr = new Array("黑体","宋体","隶书","楷体","华文彩云","微软雅黑","华文行楷","华文琥珀","幼圆","方正舒体");
    var h = "";
    for(var i = 0;i < arr.length;i++){
        h += "<a style='font-family:"+arr[i]+"' fontface='a'>"+arr[i]+"</a>";
    }
    dom_showPopupMenu(h,e,{height:"auto",width:"120px"});
    $("a[fontface='a']").unbind("click");
    $("a[fontface='a']").click(function(){
        var params = {value:$(this).html()};
        mcsstable.uniTableStyle.setFontFace(params);
        dom_closePopupMenu();
    })
}

function mcsstable_setFontPosition(e,tableid){
    mcdom.closePopup();
    var mcsstable=mcsstable_getMCSSTable(tableid);
    var arr = new Array("居左","居中","居右");
    var h = "";
    for(var i = 0;i < arr.length;i++){
        h += "<a fontposition='a'>"+arr[i]+"</a>";
    }
    dom_showPopupMenu(h,e,{width:'65px'});
    $("a[fontposition='a']").unbind("click");
    $("a[fontposition='a']").click(function(){
        var params = {value:$(this).html()};
        mcsstable.uniTableStyle.setFontPosition(params);
        dom_closePopupMenu();
    })
}

function mcsstable_createFontColorSelector(e,tableid){
    var mcsstable=mcsstable_getMCSSTable(tableid);
    var h = mcsstable_createColorDiv(mcsstable,"font");
    mcdom.showPopup(e,h,"","","",180,224,"字体颜色",{tag:"color_div"})
    $("div[fontcolor='div']").unbind("click");
    $("div[fontcolor='div']").click(function(){
        var params = {value:$(this).attr("color")};
        mcsstable.uniTableStyle.setFontColor(params);
        mcdom.closePopup();
        dom_closePopupMenu();
    })
}
function mcsstable_createBackgroundColorSelector(e,tableid){
    var mcsstable=mcsstable_getMCSSTable(tableid);
    var h = mcsstable_createColorDiv(mcsstable,"background");
    mcdom.showPopup(e,h,"","","",180,224,"背景色",{tag:"color_div"});
    $("div[fontcolor='div']").unbind("click");
    $("div[fontcolor='div']").click(function(){
        var params = {value:$(this).attr("color")};
        mcsstable.uniTableStyle.setBackgroundColor(params);
        mcdom.closePopup();
        dom_closePopupMenu();
    })
}

function mcsstable_createColorDiv(mcsstable,type){
    var colorArr = new Array("#f00","#ff0","#0f0","#0ff","#00f","#f0f","#fff","#ebebeb","#e1e1e1","#d7d7d7",
        "#cccccc","#c2c2c2","#b7b7b7","#acacac","#a0a0a0","#959595","#ee1d24","#fff100","#00a650","#00aeef",
        "#2f3192","#ed008c","#898989","#7d7d7d","#707070","#626262","#555","#464646","#363636","#262626","#111",
        "#000","#f7977a","#fbad82","#fdc68c","#fff799","#c6df9c","#a4d49d","#81ca9d","#7bcdc9","#6ccff7","#7ca6d8",
        "#8293ca","#8881be","#a286bd","#bc8cbf","#f49bc1","#f5999d","#f16c4d","#f68e54","#fbaf5a","#fff467","#acd372",
        "#7dc473","#39b778","#16bcb4","#00bff3","#438ccb","#5573b7","#5e5ca7","#855fa8","#a763a9","#ef6ea8","#f16d7e",
        "#ee1d24","#f16522","#f7941d","#fff100","#8fc63d","#37b44a","#00a650","#00a99e","#00aeef","#0072bc","#0054a5",
        "#2f3192","#652c91","#91278f","#ed008c","#ee105a","#9d0a0f","#a1410d","#a36209","#aba000","#588528","#197b30",
        "#007236","#00736a","#0076a4","#004a80","#003370","#1d1363","#450e61","#62055f","#9e005c","#9d0039","#790000",
        "#7b3000","#7c4900","#827a00","#3e6617","#045f20","#005824","#005951","#005b7e","#003562","#002056","#0c004b",
        "#30004a","#4b0048","#7a0045","#7a0026");
    var html = "<div style='cursor:pointer'>";
    for(var i = 0;i < colorArr.length;i++){
        html+="<div style='border:1px solid #003377;float:left;background:"+colorArr[i]+";width:12px;height:14px'"+
            " onmouseover='mcsstable_showCurrentColor(this,\""+type+"\")' color='"+colorArr[i]+"' fontcolor='div'></div>";
    }
    html+="</div><div style='text-align:center;margin-top:102px;width:192px;height:20px;border:1px solid #003377;font-weight:bold;'>--超级表格 AaBbCc--</div>"
    return html;
}

function mcsstable_showCurrentColor(e,type){
    var color = $(e).attr("color");
    var div = $(e).parent().next();
    if(type=="font")
        $(div).css("color",color);
    else if(type=="background")
        $(div).css("background",color);
}


function mcsstable_batchedit(e,tableid,modelid){
    var mcsstable=mcsstable_getMCSSTable(tableid);
    var ids=mcsstable.selectedRowsID;
    if(!ids){
        mcdom.alert("请先勾选要操作的记录。",lang.batchedit,'info','fadeout','','',{obj:e});
        //wx_alert('','请先勾选要操作的记录!', fadeOut());

        return;
    }
    var fields = mcsstable.modeldata.fields;
    var html = "<div class='form-group'><div class='popup-notes'>"+lang.batchedit_hint+"</div><select id='"+tableid+"_batchedit_select' class='form-control input-sm' style='width:200px'><option value=''>"+lang.batchedit_hint2+"</option>";
    if(fields.length > 0){
        var name1="";
        for(var i = 0;i < fields.length;i++){
            if(fields[i].isvisible!="false" && fields[i].readonly!='true' && fields[i].name && fields[i].type!='file' && fields[i].type!='image' && fields[i].type!='recordindex' && fields[i].type!='calculated' && fields[i].type!="hidden" && fields[i].type!="relatedrecords"){
                name1=fields[i].name;
                if (name1.length>10)
                    name1=name1.substr(0,5)+".."+name1.substr(name1.length-3);
                html+="<option value='"+fields[i].id;
                html+="~"+name1+"~"+fields[i].type;
                html+="~"+fields[i].fieldtype+"'>"+name1+"</option>";
            }
        }
    }
    html+="</select><br><br><div id='"+tableid+"_batchedit_fieldpanel' style='height:160px;overflow:auto;' ><table id='"+tableid+"_batchedit_fieldtable' style='line-height:30px'></table></div>";
    html+="<div  class='popup-bottom' style='margin-top:10px'><button id='batchsavetbn' class='btn btn-primary btn-sm'  onclick='mcsstable_batchedit_save(this,\""+tableid+"\",\""+ids+"\")' style='display:none' title='"+lang.verify+"'>确定</button></div></div>";
    mcdom.popup(html,{width:'450px',height:'350px',position:"middle",title:'批量修改',tag:'table_batchedit'});
    $("#"+tableid+"_batchedit_select").change(function(){
        var val = $(this).val();
        if(val){
            var arr = val.split('~');
            var html = "<tr onmouseover='mcsstable_batchedit_over(this)'  onmouseout='mcsstable_batchedit_out(this)' fieldid='"+arr[0]+"' fieldname='"+arr[1]+"' type='"+arr[2]+"' fieldtype='"+arr[3]+"'><td style='word-break:break-all;'>"+arr[1]+"</td><td></td>";
            html+= "<td class='batchedit_delbtn' style='padding-top:5px;'><a href='javascript:void(0)' onclick='mcsstable_batchedit_delfield(this,\""+tableid+"\")' title='移除该列'>"
//				+"<img src='"+g_rooturl+"/Public/themes/default/images/delete_default.png' />"
                +"<span class='icon-close'></span>"
                +"</a></td></tr>"
            $("#"+tableid+"_batchedit_fieldtable").append(html);
            var trArr = $("#"+tableid+"_batchedit_fieldtable").find('tr');
            var obj = trArr.eq(trArr.length-1).find('td').get(1);
            var type = arr[2],fieldtype = arr[3],field = "";
            for(var i = 0;i < fields.length;i++){
                if(fields[i].isvisible!="false"){
                    if(arr[0]==fields[i].id){
                        field = fields[i];
                        break;
                    }
                }
            }
            var params = {parentNode:obj,dom_id:field.id+"_"+getRandomStr(3)};
            if (!type || type=='text' || type=='date'  || type=='datetime' || type=='undefined' || type=='numtext' || type=='textarea' || type=='smartselect'){
                var dom = createMcssText(field,"",params);
                $(dom).attr("placeholder","输入新值");
                $(dom).css({border:"1px solid #ccc",width:"195px",marginBottom:"8px"});
                if(type=='date' || type=='datetime' || fieldtype=='date' || fieldtype=='datetime'){
                    $(dom).css({width:"139px",marginBottom:"8px"});
                    $(dom).attr('placeholder',"请选择日期");
                    $(dom).before("<select class='form-control input-sm' style='width:60px'><option value='equal'>=</option><option value='add'>+</option><option value='diff'>-</option></select> ");
                    $(dom).prev().change(function(){
                        if($(this).val()=="add" || $(this).val()=="diff"){
                            $(dom).next().remove();
                            $(dom).attr('action',$(this).val());
                            $(dom).attr('unit','day');
                            $(dom).hide();
                            $(dom).after("<span><input class='form-control input-sm' onkeyup='batchedit_checkNum(this)' style='width:70px;margin-bottom:8px' placeholder='请输入整数'> <select style='width:60px' class='form-control input-sm' onchange='batchedit_addUnit(this)' title='时间单位'><option value='day'>日</option><option value='month'>月</option><option value='year'>年</option></select></span>");
                        }else{
                            $(dom).show();
                            $(dom).removeAttr('action');
                            $(dom).removeAttr("num");
                            $(dom).removeAttr("unit");
                            $(dom).val('');
                            $(dom).next().remove();
                        }
                    })
                }else if(fieldtype=='int' || fieldtype=='float' || fieldtype=='real' || type=='numtext'){
                    $(dom).css({width:"139px",marginBottom:"8px"});
                    $(dom).attr('placeholder',"请输入数字");
                    $(dom).before("<select class='form-control input-sm' style='width:60px'><option value='equal'>=</option><option value='add'>+</option><option value='diff'>-</option></select> ");

                    $(dom).prev().change(function(){
                        if($(this).val()=="add" || $(this).val()=="diff"){
                            $(dom).attr('action',$(this).val());
                            $(dom).keyup(function(){
                                if(isNaN($(this).val())){
                                    alert("请输入数字!");
                                    $(this).val('');
                                    return;
                                }
                                if($(this).val())
                                    $(this).attr("num",$(this).val());
                            })
                        }else{
                            $(dom).removeAttr('action');
                            $(dom).removeAttr('num');
                            $(dom).unbind('keyup');
                        }
                    })
                }
            }else if (type=='checkbox'){
                var dom = createMcssCheckbox(field,"",params);
            }else if(type=='checkboxlist'){
                var dom = createMcssCheckboxlist(field,"",params);
            }else if (type=='colorIcon' || type=='statueIcon' || type=='priorityIcon' || type=='dropdown' || type=='radio'){
                if(type=='colorIcon')
                    field.data = '红:红,绿:绿,黄:黄';
                else if(type=='priorityIcon')
                    field.data = '低:低,中:中,高:高';
                else if(type=='statueIcon')
                    field.data = '计划:计划,执行:执行,完成:完成,取消:取消';
                var dom = createMcssSelect(field,"",params);
                $(dom).css({border:"1px solid #ccc",width:"195px"});
            }
            $("#batchsavetbn").show();
        }
        $(this).find("option:selected").remove();
    })
}
function batchedit_addUnit(obj){
    $(obj).parent().prev().attr("unit",$(obj).val());
}
function batchedit_checkNum(obj){
    var r = /^\+?[1-9][0-9]*$/;　　//正整数
    if($(obj).val()){
        if(!r.test($(obj).val())){
            alert("请输入正整数");
            $(obj).val('');
            $(obj).parent().prev().removeAttr("num");
            return;
        }
    }
    if($(obj).val())
        $(obj).parent().prev().attr("num",$(obj).val());
}
function mcsstable_batchedit_delfield(obj,tableid){
    var tr = $(obj).parent().parent();
    var html = "<option value='"+tr.attr("fieldid")+"~"+tr.attr("fieldname")+"~"+tr.attr("type")+"~"+tr.attr("fieldtype")+"'>"+tr.attr("fieldname")+"</option>";
    $("#"+tableid+"_batchedit_select").append(html);
    tr.remove();
}
function mcsstable_batchedit_save(obj,tableid,ids){
    if (confirm("执行批量修改后，您选择的那些行的数据将被修改，且不可恢复。确定吗？")==false)
        return;
    var mcsstable = mcsstable_getMCSSTable(tableid);
    var modelid = mcsstable.params.modelid;
    mcsstable_batchedit_saveToDataBase(obj,tableid,ids,modelid);
}
function mcsstable_batchedit_saveToDataBase(obj,tableid,ids,modelid){
    var values = "";
    var fields = "";
    var trArr = $("#"+tableid+"_batchedit_fieldtable").find("tr");
    if(trArr){
        for(var i = 0;i < trArr.length;i++){
            var fieldid = trArr.eq(i).attr('fieldid');
            var input = trArr.eq(i).find('td').eq(1).find("input[fieldid='"+fieldid+"'],select[fieldid='"+fieldid+"']");
            if(input.attr('action') && !input.attr('num') && !input.val()){
                alert(trArr.eq(i).find('td').eq(0).html()+"尚未填写,请确认填写所有的输入框");
                return;
            }else if(input.attr('action') && input.attr('num') && input.attr('unit')){
                input.val(input.attr('action')+'_'+input.attr('num')+'_'+input.attr('unit'));
            }else if(input.attr('action') && input.attr('num'))
                input.val(input.attr('action')+'_'+input.attr('num'));
            var value = "";
            value = input.val();
            if(input.length>1){
                value = "";
                for(var i = 0;i < input.length;i++){
                    if($(input[i]).prop("checked")){
                        if(value)
                            value+=",";
                        value+=$(input[i]).attr("value");
                    }
                }
            }
            if(values)
                values+='|';
            values+=value;
            if(fields)
                fields+=',';
            fields+=fieldid;
        }
        var mcsstable = mcsstable_getMCSSTable(tableid);
        /*数据库分表*///参数不明确，已添加 usermodel_recordid
        var _project_id = mcsstable.params.usermodel_recordid;
        var url = g_homeurl+"/System/Model/batcheditData";
        $.post(url,{ids:ids,fields:fields,values:values,modelid:modelid,usermodel_recordid:_project_id},function(){
            mcsstable_loaddatarows(tableid);
            mcdom.closePopup();
            mcdom.alert("批量修改成功",lang.batchedit,'info','fadeout');
        })
    }else{
        alert('请至少选择一个字段');
        //wx_alert('','请至少选择一个字段!', fadeOut());

    }
}
function mcsstable_batchedit_over(obj){
    $(obj).find('img').eq(0).attr('src',g_rooturl+'/Public/themes/default/images/delete.png');
}
function mcsstable_batchedit_out(obj){
    $(obj).find('img').eq(0).attr('src',g_rooturl+'/Public/themes/default/images/delete_default.png');
}





//获得预览office文档的链接
function openOffice(url,ext)
{
    var app_url,temp_url;
    switch (ext) {
        case 'doc':
        case 'docx':
        case 'docm':
        case 'dot':
            app_url ='http://sg1b-word-view.officeapps.live.com/wv/wordviewerframe.aspx?ui=zh-CN&rs=zh-CN&WOPISrc=';
            break;
        case 'ppt':
        case 'pptm':
        case 'pptx':
            app_url ='http://sg1b-powerpoint.officeapps.live.com/p/PowerPointFrame.aspx?PowerPointView=ReadingView&ui=zh-CN&rs=zh-CN&WOPISrc=';
            break;
        case 'xls':
        case 'xlsb':
        case 'xlsm':
        case 'xlsx':
            app_url = 'http://sg1b-excel.officeapps.live.com/x/_layouts/xlviewerinternal.aspx?ui=zh-CN&rs=zh-CN&WOPISrc=';
            break;
        default:break;
    }
    if (!app_url)
        return '';
    temp_url = 'http://sg1b-15-view-wopi.wopi.live.net:808/oh/wopi/files/@/wFileId?wFileId=';
    temp_url += encodeURIComponent(url);
    return app_url+encodeURIComponent(temp_url)+'&access_token=1&access_token_ttl=0';
}

//打开office文档预览页面
function viewOfficeDoc(url)
{
    var pos=url.lastIndexOf(".");
    var ext=url.substring(pos+1);
    var url=openOffice(url,ext);
    if (!url)
        mcdom.popup('无法预览此文件',{fateOut:true,position:'middle',fadeOut:true});
    else
        mcdom.showIframe(url,{width:'800px',height:'600px',openNewWindow:true});
}

//修改组名
function project_editGroup(newname)
{
    if (!newname)
    {
        mcdom.alert("不能为空！","提示",'info','fadeout');
        //wx_alert('','不能为空!', fadeOut());

        return;
    }

    var id=mcdom.temp_mcsstable.tableid;
    var mcsstable=mcsstable_getMCSSTable(id);
    var prevname=mcdom.temp_prevtasktype;
    if(newname==prevname)
    {
        mcdom.closePopup();
        mcdom.alert("没有可保存的数据","提示",'info','fadeout');
    }
    else
    if(!prevname || prevname=="(无分组)")
    {
        project_addGroup(newname);
    }else
    {
        if ((","+mcsstable.gantetu.params.project_data.tasktypes+",").indexOf(","+newname+",")>-1)

        {
            mcdom.alert("该分组已存在","修改组名",'info','fadeout');
            return;
        }else
        {
            var url=g_homeurl+"/Oa/Project/updateTaskTypeName/";
            $.getJSON(url,{newname:newname,prevname:prevname,projectid:mcdom.temp_project_id},function(data) {
                if(data==1)
                {
                    mcdom.closePopup();
                    mcdom.alert("修改成功","修改组名",'success','fadeout');
                    var tr=mcdom.temp_gantetu_current_row;
                    $(tr).find(".groupname").html(newname);
                    project_refreshTasktype(mcsstable,'edit',newname,prevname);
                }
                else
                    mcdom.alert("修改失败","修改组名",'fail','fadeout');
            });
        }
    }


}

//新增组1
function project_addGroup1(newname,isbs)
{
    var tableid=mcdom.temp_mcsstable.tableid;
    var mcsstable=mcsstable_getMCSSTable(tableid);
    if(!isbs)
    {
        if ((","+mcsstable.gantetu.params.project_data.tasktypes+",").indexOf(","+newname+",")>-1)
        {

            mcdom.alert("该分组已存在","添加组",'info','fadeout');
            return;
        }
    }
    var url=g_homeurl+"/Oa/Project/insertTaskType/";

    $.getJSON(url,{tasktype:newname,projectid:mcdom.temp_project_id},function(data) {
        if(data==1)
        {
            mcdom.alert("添加成功","添加组",'success','fadeout');
            gantetu_appenNewGroup(mcsstable,newname);

            project_refreshTasktype(mcsstable,'add',newname);
        }
        else
            mcdom.alert("添加成功","添加组",'fail','fadeout');
    });
    return true;
}
//新增组
function project_addGroup(newname)
{
    var id=mcdom.temp_mcsstable.tableid;
    var mcsstable=mcsstable_getMCSSTable(id);
    var tasktypes = mcss_getFieldProp(mcsstable.modeldata,'tasktype','data');
    if ((","+mcsstable.gantetu.params.project_data.tasktypes+",").indexOf(","+newname+",")>-1)
    {
        mcdom.alert("该分组已存在","添加组",'info','fadeout');
        return;
    }
    var url=g_homeurl+"/Oa/Project/insertTaskType/";

    $.getJSON(url,{tasktype:newname,projectid:mcdom.temp_project_id},function(data) {
        if(data==1)
        {
            mcdom.alert("添加成功","添加组",'success','fadeout');
            var newRow1=mcdom.temp_gantetu_current_row;
            var newRow=newRow1.cloneNode(true);
            newRow.id=$(newRow).attr('id')+"copy";
            $(newRow).find('.groupname').html(newname);
            $(mcdom.temp_mcsstable.table).find("tbody").append(newRow);
//			project_bindMouseEventOnRows($(newRow),mcdom.temp_mcsstable);
            project_refreshTasktype(mcsstable,'add',newname);
        }
        else
            mcdom.alert("添加失败","添加组",'fail','fadeout');


    });
    return true;
}

//在本组下插入新记录，在第一条记录前
function mcsstable_GroupInsertRow(mcsstable,tr)
{
    var name=tr.getAttribute("groupname");
    mcsstable.currentRow=tr;
    var tr=mcsstable_addNewEditRow(mcsstable,true);
    var tr=mcsstable.tableFixCol.rows[tr.rowIndex];
    for(var c=0;c<tr.cells.length;c++){
        var td=tr.cells[c];
        if (td.getAttribute("id")==mcsstable.groupField.id){
            setFocusToTd(td);
            mcsstable.editCell(td);
            $(tr).find("input,textarea[fieldid],select[fieldid]").val(name);
            break;
        }
    }
}
//浮动工具条：全选  edit by gang
function mcsstable_tablesetting(e,tableid)
{
    var table=mcdom.getObjByTagName(e,"TABLE");
    if (tableid) table=document.getElementById(tableid);
    if(g_showFloatTool){
        mcsstable_clearFloatToolBar();
    }
    var h = "";
    var mcsstable=mcsstable_getMCSSTable(table.id);

    //原来行操作按钮 Modified by Chen Li    {{{
    var tableid=table.id;
    var actions=mcsstable.actions;
    var projectdata=mcsstable.params.project_fulldata;

    if (mcsstable.params.showBatchEditBtns)
        actions=mcsstable.params.showBatchEditBtns(mcsstable);

    if (actions.indexOf("add,")>-1){
        var tmp=mcsstable.modeldata.addurl;
        if (!tmp || tmp=="default")
            tmp=g_systempath+"/addRecord";
        if (tmp)
            tmp+="/modelid/"+mcsstable.params.modelid;
        var url=mcsstable.params.homeUrl+tmp;
        if (mcsstable.params.defaultValue)
        {
            url+="/defaultValue/"+mcsstable.params.defaultValue;
        }
        var url=mcsstable.getAddUrl();
        
        h += "<a id=''  onclick=\"mcsstable_addnew($("+tableid+").get(0).rows[0].cells[0],'"+tableid+"','"+url+"','"+lang.add+"') \" >" + lang.add + "   ( Ctrl+I )</a>";
    }
    if (actions.indexOf("del,")>-1)
    {
        h+="<a id='' onclick='mcsstable_deleterows(this,\""+tableid+"\")' title=''>删除   ( Ctrl+D )</a>";
    }

    if (actions.indexOf("batchedit,")>-1)
    {
        h+="<a id='' onclick='mcsstable_batchedit(this,\""+tableid+"\")' title='批量修改选中的行'>批量修改</a>";
    }
    h+="<hr style='margin:5px 0'/>";


    if (actions.indexOf("import,")>-1)
    {
        if(uni_global.importstyle=="old")
            h+="<a id='' onclick='mcsstable_import_old(\""+tableid+"\")' title='把Excel文件追加到表末'>插入Excel表格</a>";
        else
            h+="<a id='' onclick='mcsstable_import(\""+tableid+"\")' title='把Excel文件追加到表末'>插入Excel表格</a>";
        h+="<a id='' onclick='mcsstable_importFromTable(\""+tableid+"\")' title='把现有超级表格追加到表末'>插入已有的超级表格</a>";
    }
    h+="<a  title='选中所有行' onclick='mcsstable_selectAllRows(this,\""+table.id+"\");'><span>全选</span></a>";
    if (actions.indexOf("cleardata,")>-1)
    {
        h+="<a  title='清空数据' onclick='mcsstable_clearTableData(\""+table.id+"\");' class='mcsstable_record_del'><span>清空数据</span></a>";
    }

    $(mcsstable.getAction("batchedit")).removeClass("mcsstable-selected-action");

    h+="<hr style='margin:5px 0'/>";
    if (mcsstable.actions.indexOf('tableedit')>-1 && (mcsstable.view=="tableedit" || mcsstable.view=="celledit"))
    {
        if ($("#a_mcsstable_tablesetting").attr("livesave")=="YES" || !$("#a_mcsstable_tablesetting").attr("livesave")){
            var css='icon-checkbox-checked2';
        }
        else{
            var css='icon-checkbox-unchecked';
        }
    }

    if (mcsstable.actions.indexOf('groupby')>-1 && mcsstable.groupFieldid )
    {
        var title="折叠分组";
        if ($("#a_mcsstable_tablesetting").attr("groupvisible")=="NO")
            title="展开分组";
        h+="<a id='mainsheet_table_expandAllGroup'  title='展开收缩所有分组' onclick='mcsstable_expandAllGroup(this);' class='icon-stack'><span>"+title+"</span></a>";
        if (mcsstable.groupFieldid)
            h+="<a id='mainsheet_table_cancelGroup'  title='不分组' onclick='mcsstable_cancelGroup(\""+table.id+"\");' class='icon-cross'><span>取消分组</span></a>";
    }
    //h+="<div id='mainsheet_table_setting'  title='表格样式设置' onclick='mcsstable_setTableStyle(this);'><span class='icon-cog' ></span><a style='display:inline-block' >设置表格样式</a></div>";

    if (actions.indexOf("advancedsetting,")>-1 || loginuser=='admin')
        h+="<a id='' onclick='tableset_allFields(this,\""+tableid+"\")' title='高级设置'>高级设置</a>";
    dom_showPopupMenu(h,e);

}

//设置是否自动保存
function mcsstable_setLiveSave(tableid)
{
    var mcsstable=mcsstable_getMCSSTable(tableid);
    if ($("#a_mcsstable_tablesetting").attr("livesave")=="NO")
    {
        $("#a_mcsstable_tablesetting").attr("livesave","YES");
        mcsstable.view="celledit";
        $(mcsstable.getAction("save")).hide();
        mcdom.popup("已设置为自动保存！",{fadeOut:true,hideheader:true});
    }
    else
    {
        $("#a_mcsstable_tablesetting").attr("livesave","NO");
        mcsstable.view="tableedit";
        $(mcsstable.getAction("save")).show();
        mcdom.popup("已设置为手动保存！",{fadeOut:true,hideheader:true});
    }
}

//清空表格中的数据
function mcsstable_clearTableData(tableid,e){
    var hint = "确认清空当前显示的所有数据";
    var h = "<div style='margin-left:60px'><input type='checkbox' id='input_cleardata' /> <label for='input_cleardata'>"+hint+"</label></div>";
    h+="<div style='margin-left:100px;margin-top:15px'>"
        +"<input id='btn_confirmclear' type='button' class='btn btn-primary btn-sm' value='确认'/>"
        +" <input id='btn_cancelclear' type='button' class='btn btn-primary btn-sm' value='取消'/></div>";
    var popup = mcdom.popup(h,{obj:e,title:"清空数据",width:"350px",height:"130px",position:"middle",tag:"delete_tag"});
    $("#btn_confirmclear").click(function(){
        var checked = $("#input_cleardata").prop("checked");
        if(checked){
            var mcsstable=mcsstable_getMCSSTable(tableid);
            var usermodel_recordid = mcsstable.params.usermodel_recordid;
            var tablename = mcsstable.params.project_fulldata.project_record.tablename;
            var lastfilter=mcsstable.lastfilter;
            lastfilter=uni_cleanSQL(lastfilter);
            $.post(g_homeurl+"/System/Model/clearTableData",{usermodel_recordid:usermodel_recordid,tablename:tablename,
                modelid:mcsstable.modelid,searchword:mcsstable.searchword,filter:lastfilter,model_ownerid:mcsstable.params.model_ownerid},function(data){
                if(data > 0)
                //wx_alert('','删除成功!', fadeOut());
                    alert("删除成功");

                else
                //wx_alert('','没有内容删除!', fadeOut());
                    alert("没有内容删除!");


                mcdom.closePopup(popup);

                mcsstable.reload();
            })
        }else{
            mcdom.alert("请勾选后确认","","","fadeout","","",{hideheader:true});
            //wx_alert('','请勾选后确认!', fadeOut());

        }
    })

    $("#btn_cancelclear").click(function(){
        mcdom.closePopup(popup);
    })
}


function mcsstable_showRowButtons(tableid)
{
    var mcsstable=mcsstable_getMCSSTable(tableid);
    if ($("#a_mcsstable_tablesetting").attr("rowactionvisible")=="YES"){
        $("a[tag='recordAction']").hide();
        $("#a_mcsstable_tablesetting").attr("rowactionvisible","NO");
        $("#"+mcsstable.rowaction_col_id).css("width","20px");

    }
    else{
        $("a[tag='recordAction']").show();
        $("#a_mcsstable_tablesetting").attr("rowactionvisible","YES");
        var btnCount=$(mcsstable.table).find("td.mcsstable-action-col").eq(0).parent().find("a[tag='recordAction']").size();
        $("#"+mcsstable.rowaction_col_id).css("width",(btnCount*19)+"px");
    }
}


function mcsstable_selectAllRows(e,tableid,selected) {
    var mcsstable=mcsstable_getMCSSTable(tableid);
    if(selected!="false"){
        mcsstable_linenum_onclick(e,event,{allselect:"select",table:mcsstable.table,trFloatShow:'hidden'});
    }else{
        mcsstable_linenum_onclick(e,event,{allselect:"notselect",table:mcsstable.table});
    }
}

//全选得到选中的所有记录ID
function mcsstable_selectRow(e,selected,mcsstable)
{
    if (!mcsstable){
        table=mcdom.getObjByTagName(e,"TABLE");
        mcsstable=mcsstable_getMCSSTable(table.id);
    }
    if (selected==false)
    {
        var currentId=$(e.parentNode).attr('recordid');
        var arr=mcsstable.selectedRowsID.split(',');
        for(var i=0;i<arr.length;i++)
        {
            if(arr[i]==currentId)
            {
                arr.splice(i,1);
            }
        }
        mcsstable.selectedRowsID=arr.join();
        mcsstable.selectedRows=[];
        mcsstable.selectedRows.push(e.parentNode);
    }
    else
    {
        if (!mcsstable.selectedRows) mcsstable.selectedRows=[];
        var bfind=false;
        for(var i=0;i<mcsstable.selectedRows.length;i++){
            if (e.parentNode== mcsstable.selectedRows[i])
            {
                bfind=true;
                break;
            }
        }
        if (bfind==false){

            if ($(e.parentNode).attr('recordid')){
                if (mcsstable.selectedRowsID)
                    mcsstable.selectedRowsID+=",";
                mcsstable.selectedRowsID+=$(e.parentNode).attr('recordid');

                mcsstable.selectedRows.push(e.parentNode);
            }
        }
    }

    if (mcsstable.selectedRowsID=="")
        $(mcsstable.getAction("batchedit")).removeClass("mcsstable-selected-action");
    else
        $(mcsstable.getAction("batchedit")).addClass("mcsstable-selected-action");

}

//选中已选择的记录
function mcsstable_setSelectedRow(mcsstable)
{
    if (mcsstable.selectedRowsID)
    {
        var ids=mcsstable.selectedRowsID+",";
        for(var i=1;i<mcsstable.table.rows.length;i++)
        {
            var recordid=$(mcsstable.table.rows[i]).attr("recordid");
            if (ids.indexOf(recordid+",")>-1)
            {
                var td = $(mcsstable.table).find("tr").eq(i).find(".linenumber").eq(0).get(0);
                mcsstable_linenum_onclick(td,event,{ctrlKey:"true"});
            }
        }
    }
}

function mcsstable_showStyleEditorBar(e,tableid)
{
    mcsstable=mcsstable_getMCSSTable(tableid);
    if ($(e).attr("StyleEditorBarIsVisible")=="YES")
    {
        mcsstable.showAction("styleedit",false);
        $(e).attr("StyleEditorBarIsVisible","NO");
        $(e).find("span").removeClass("icon-arrow-left5").addClass('icon-arrow-right4');

    }
    else
    {
        $(mcsstable.getAction("styleedit")).css('display','inline-block');
        $(e).attr("StyleEditorBarIsVisible","YES");
        $(e).find("span").addClass("icon-arrow-left5").removeClass('icon-arrow-right4');
    }
}

//点击视图按钮
function mcsstable_showViewMenu(e,tableid,modelid)
{
    var mcsstable=mcsstable_getMCSSTable(tableid);
    var projectid = mcsstable.params.usermodel_recordid;
    var select = $(mcsstable.getAction("view"));
    var current_view = select.val();
    var current_view_tag  = select.find('option:selected').attr("tag");
    $.post(g_homeurl+"/Oa/Project/mcsstable_getViews",{projectid:projectid},function(data){
        var h = "";
        if(!(current_view=="DEFAULT" || current_view_tag=="USEROPTION")){
            if (mcsstable.params.project_fulldata && mcsstable.params.project_fulldata.project_record.SYS_ADDUSER==loginuser)
            {
                h+="<a id='view_edit'  title='修改当前视图的条件'>修改当前视图</a>";
                h+="<a id='view_delete'  title='删除当前视图（不会删除数据）'>删除当前视图</a>";
            }
        }
        h+="<a id='view_new'  title='重新开始创建一个新视图'>新建普通视图</a>";
        if(modelid=='oa_sheet')
            h+="<a id='view_stat'  title='新建统计视图'>新建统计视图</a>";
        h+="<a id='view_saveas'  title='把当前表格结果保存为视图'>另存为新视图</a>";
        if(data>0)
            h+="<a id='view_manage'  title='整理自己创建的所有视图'>整理视图</a>";
        dom_showPopupMenu(h,e,{width:'120px'});
        $("#view_new").click(function(){
            mcsstable_editView(tableid,"new",current_view);
        })
        $("#view_stat").click(function(){
            mcsstable_statView(tableid,"new");
        })
        $("#view_saveas").click(function(){
            mcsstable_saveasview(tableid,this);
        })
        $("#view_manage").click(function(){
            var h="<div id='table_viewlist' style='width:100%;height:300px;table-layout:fixed;overflow:hidden;margin:5px'></div>";
            var filter="user='"+loginuser+"' and modelid='"+mcsstable.modelid+"' ";
            if (mcsstable.params.usermodel_recordid)
                filter+=" and usermodel_recordid='"+ mcsstable.params.usermodel_recordid+"'";
            mcdom.popup(h,{title:'视图整理',mouseoverclose:false,width:420,height:320,obj:e});
            var params={filter:filter,view:'tableedit',afterSave:manageview_aftersave,
                refObject:mcsstable,fieldAction:'none',canAppendRow:false,autoWidthField123:'name',
                afterLoadRows:mcsstable_view_afterLoadRows,width:'auto'
            };
            mcsstable_view=new MCSSTable('table_viewlist','sys_view_my',params);
            mcsstable_view.run(mcsstable_view_afterRun);
        })
        $("#view_edit").click(function(){
            var viewType = mcsstable.modeldata.viewType;
            if(viewType=='stat')
                mcsstable_statView(tableid,"edit",current_view);
            else
                mcsstable_editView(tableid,"edit",current_view);
        })
        
    })


}

function mcsstable_statView(tableid,status,current_view){
    var mcsstable=mcsstable_getMCSSTable(tableid);
    if (!mcsstable.params.project_fulldata){
        alert("缺少项目信息!");
        //wx_alert('','缺少项目信息!', fadeOut());

        return;
    }
    var query=mcsstable.advancedFilter;
    var _view_searchword=mcsstable.view_searchword;
    var title = "新建统计视图";
    if(status=="edit"){
        title = "修改统计视图";
        query=g_statview_query;
        _view_searchword=g_statview_searchWord;
    }
    if (!query)
        query="";
    query=uni_cleanSQL(query);
    searchword=uni_cleanSQL(_view_searchword);
    var project_id=mcsstable.params.project_fulldata.project_record.id;
    var modelid=mcsstable.params.modelid;
    var adduser=mcsstable.params.project_fulldata.project_record.SYS_ADDUSER;
    var url = g_homeurl+"/Oa/Project/statView/projectid/"+project_id+"/modelid/"+modelid
        +"/ownerid/"+adduser+"/tableid/"+tableid+"/viewid/"+current_view+"/status/"+status;
    if (query)
        url+="/query/"+query;
    if (searchword)
        url+="/searchword/"+searchword;
    var params = {title:title,width:'875',height:'500',openNewWindow:true};
    mcdom.showIframe(url,params);
}
function mcsstable_editView(tableid,status,current_view){
    var mcsstable=mcsstable_getMCSSTable(tableid);
    if (!mcsstable.params.project_fulldata){
        alert("缺少项目信息!");
        return;
    }
    var project_id=mcsstable.params.project_fulldata.project_record.id;
    var modelid=mcsstable.params.modelid;
    var adduser=mcsstable.params.project_fulldata.project_record.SYS_ADDUSER;
    var title = "新建视图";
    if(status=="edit")
        title = "修改视图";
    var url = g_homeurl+"/Oa/Project/editView/projectid/"+project_id+"/modelid/"+modelid+"/ownerid/"+adduser+"/tableid/"+tableid+"/viewid/"+current_view+"/status/"+status;
    var params = {title:title,width:'875',height:'640',openNewWindow:true};
    mcdom.showIframe(url,params);
}

function mcsstable_advancedSearch(tableid,e)
{
    var mcsstable=mcsstable_getMCSSTable(tableid);
    if ($(e).attr("AdvancedSearchIsVisible")=="YES")
    {
        var container_id=tableid+"_searchcondition";
        $("#"+container_id).remove();
        $(e).attr("AdvancedSearchIsVisible","NO");
        //$(e).addClass("icon-arrow-up2").removeClass('icon-arrow-down2');
    }
    else
    {
        mcsstable_search2(tableid,e);
        $(e).attr("AdvancedSearchIsVisible","YES");
        //$(e).removeClass("icon-arrow-up2").addClass('icon-arrow-down2');
    }
}

function mcsstable_showDiscussion(obj,tableid)
{
    var mcsstable=mcsstable_getMCSSTable(tableid);
    if (mcsstable.params.onShowDiscussion)
        mcsstable.params.onShowDiscussion(obj,mcsstable);
}

function mcsstable_setTableStyle(obj)
{
    var obj=$("#a_mcsstable_tablesetting").get(0);
    var table=mcdom.getObjByTagName(obj,"TABLE");
    var mcsstable=mcsstable_getMCSSTable(table.id);
    var h="<div>";
    h+="<div><div class='stylesetting-label'>行高：</div>"+" <div class='stylesetting-inputer'><input class='form-control input-sm'  type='number'  min='20' max='100' style='width:50px' value='20' onchange='mcsstable_changeRowHeight(this);' />像素</div></div>"
        +"<div><div class='stylesetting-label'>字体："+"</div></div>"
        +"<div><div class='stylesetting-label'>前景色："+"</div></div>"
        +"<div><div class='stylesetting-label'>背景色："+"</div></div>"
        +"</div>"
        +"<div><hr />"
        +"<a style='float:right;' class='btn btn-default btn-sm' title='采用默认' onclick='mcsstable_setTableDefaultStyle(\""+table.id+"\")'>默认</a>"
        +"<a style='float:right;' class='btn btn-default btn-sm' title='恢复原来' onclick='mcsstable_setTableDefaultStyle(\""+table.id+"\")'>恢复</a>"
        +"</div>"

    mcdom.popup(h,{obj:obj,title:'设置表格样式'});
}

function mcsstable_changeRowHeight(e)
{
    var obj=$("#a_mcsstable_tablesetting").get(0);
    var table=mcdom.getObjByTagName(obj,"TABLE");
    $(table).find("tr").css("height",e.value+"px");

}
//设置复制选中框
function mcsstable_setCopyBorder(mcsstable){

    var selTh = $(mcsstable.table).find("th[th_isselect='true']").length;
    var selTr = $(mcsstable.table).find("td[tr_isselect='true']").length;
    if(( selTh > 0 &&  selTr > 0) || selTh > 1 || selTr > 1){
        mcdom.alert("多选区域不能执行此操作",'提示','info','fadeout');
        //wx_alert('','多选区域不能执行此操作!', fadeOut());

        return;
    }

    var table = mcsstable.table;
    if (mcsstable.divSel) {
        //mcsstable.divSel.style.display="none";
        $(mcsstable.divSel.td).removeClass("selected-cell").removeAttr("isselect");

        if (!table.divCopy) {
            var divCopy = document.createElement('div');

            divCopy.setAttribute("class","dashed-box");
            divCopy.style.pointerEvents="none";
            var str='<div id="dashed-top" class="dashed-top"></div><div id="dashed-left" class="dashed-left"></div>'
            str+='<div id="dashed-right" class="dashed-right"></div><div id="dashed-bottom" class="dashed-bottom"></div>'
            divCopy.innerHTML = str;
            divCopy.setAttribute("onkeydown","divCoverOnkeydown(event);");
            table.divCopy = divCopy;
            mcsstable.divSelParent.appendChild(table.divCopy);
        }
        var divCopy = table.divCopy;

        if (mcsstable.cuttype=="cut")
            var color="#f55";
        else
            var color="#55f";
        divCopy.style.border="solid 1px #33f"
        divCopy.style.borderColor=color;
        divCopy.childNodes[0].style.borderColor=color;
        divCopy.childNodes[1].style.borderColor=color;
        divCopy.childNodes[2].style.borderColor=color;
        divCopy.childNodes[3].style.borderColor=color;
        divCopy.style.left = mcsstable.divSel.style.left;
        divCopy.style.top = mcsstable.divSel.style.top;
        divCopy.style.width = mcsstable.divSel.style.width;
        divCopy.style.height = mcsstable.divSel.style.height;
        divCopy.style.display="";
        table.divCopy.area = mcsstable.currentArea.area;
        table.divCopy.td=mcsstable.getCell(mcsstable.currentArea.area.beginRowIndex,mcsstable.currentArea.area.beginCellIndex) ;//table.rows[mcsstable.currentArea.area.beginRowIndex].cells[mcsstable.currentArea.area.beginCellIndex];
        if (table.divCopy.idTime) window.clearInterval(table.divCopy.idTime);
        table.divCopy.idTime = window.setInterval(function(){
            if (!document.getElementById("dashed-bottom"))
            {
                window.clearInterval(table.divCopy.idTime);
                return;
            }
            var $top=document.getElementById("dashed-bottom").style.left;
            var $left=document.getElementById("dashed-top").style.left;
            $left=parseInt($left);
            $top=parseInt($top);
            if($left<0){
                $left+=2;
            }else{
                $left=-1400;
            }

            if($top>-1000){
                $top-=2;
            }else{
                $top=0;
            }
            document.getElementById("dashed-bottom").style.left = $top + "px";
            document.getElementById("dashed-top").style.left = $left + "px";
            document.getElementById("dashed-right").style.top = $left + "px";
            document.getElementById("dashed-left").style.top = $top + "px";
        },60);

    }

    //window.setTimeout( function(){ mcsstable_clearSelect_td(mcsstable.table);},100);

}
//清除选中内容
function mcsstable_clearSelect_td(table,movie){

    var mcsstable=mcsstable_getMCSSTable(table.id);

    if (mcsstable.divSel) {
        if (!movie) mcsstable.divSel.style.display="none";
        $(mcsstable.divSel.td).removeClass("selected-cell").removeAttr("isselect");
    }

    if (mcsstable.selectAreas){
        for (var a=0;a<mcsstable.selectAreas.length;a++)
        {
            var area=mcsstable.selectAreas[a].area;
            if (!area) area = mcsstable.selectAreas[a];

            if (area.divSel){
                if (area.divSel!=mcsstable.divSel)	area.divSel.remove();
            };
            if (area){
                for(var r=area.beginRowIndex;r<=area.endRowIndex;r++)
                {
                    for(var c=area.beginCellIndex;c<=area.endCellIndex;c++)
                    {

                        if (mcsstable.tableData.rows.length>r){
                            var td=null;
                            if (c>=mcsstable.tableFixCol.rows[0].cells.length)
                                td=mcsstable.tableData.rows[r].cells[c-mcsstable.tableFixCol.rows[0].cells.length];
                            else
                                td=mcsstable.tableFixCol.rows[r].cells[c];

                            if (td) $(td).removeClass("selected-cell").removeAttr("isselect");
                        }
                    }
                }
            }
        }
        mcsstable.selectAreas=[];
    }

    if (mcsstable.selectedRows ){
        for(var r=0;r<mcsstable.selectedRows.length;r++)
        {
            var tr=mcsstable.selectedRows[r];
            var tr=mcsstable.tableFixRow.rows[tr.rowIndex];
            if (tr && tr.parentNode){
                $(tr).removeClass("mcsstable_tr_hover");

                $(tr.cells[0]).css({"color":"black"}).removeAttr("tr_isselect");
                $(tr.cells[0]).removeClass("unitable_focusedCell");
                $(tr.cells[0]).removeClass("selected-cell").css({"border":"1px solid rgb(204 204 204)"}).removeAttr("isselect");
                for(var c=0;c<tr.cells.length;c++)
                {
                    var cell = tr.cells[c];
                    if (cell){
                        $(cell).removeClass("selected-cell").css({"border":"1px solid rgb(204 204 204)"}).removeAttr("isselect");
                        cell.style.borderLeft="";
                        cell.style.borderTop="";
                        cell.style.borderRight="";
                        cell.style.borderBottom="";
                    }
                }
            }
        }
        mcsstable.selectedRowsID = "";
        mcsstable.selectedRows=null;
        $(table).find("div").removeClass("unitable_focusedCell");
    }

    if (mcsstable.selectedCols){

        $(mcsstable.table).find("th").removeAttr("th_isselect");
        $(mcsstable.table).find("th").find('.col-alpha');
        $(mcsstable.table).find("td,th,div").removeClass("unitable_focusedCell");

        for (var r=0;r<mcsstable.tableData.rows.length;r++)
        {
            var row=mcsstable.getRow(r);
            for(var j=0;j<mcsstable.selectedCols.length;j++)
            {
                var c=mcsstable.selectedCols[j];
                var td=row.cells[c];
                if (td){
                    $(td).removeClass("selected-cell").css({"border":"1px solid rgb(204 204 204)"});
                    $(td).removeAttr("isselect");
                    td.style.borderLeft="";
                    td.style.borderTop="";
                    td.style.borderRight="";
                    td.style.borderBottom="";
                }
            }
        }
        mcsstable.selectedCols=null;
    }

    return;

}

//清空选中行
function mcsstable_clear_selectrow(e,mcsstable){
    var before = $(e).parent().prev();
    var next = $(e).parent().next();

    $(e).parent().find("td[isselect='true']").css({"border":"1px solid rgb(204 204 204)","background":"white"}).removeAttr("isselect");
    $(e).css({"background":"rgb(222 222 222)","color":"#003377"}).removeAttr("tr_isselect");
    //把刚刚删除的记录的i的从mcsstable.selectedRowsID中移走
    var arr1=mcsstable.selectedRowsID.split(",");
    var ids="";
    var recordid = $(e).parent().attr("recordid");
    for(var j=0;j<arr1.length;j++)
    {
        if (arr1[j]!=recordid)
        {
            if (ids)
                ids+=",";
            ids+=arr1[j];
        }
    }
    mcsstable.selectedRowsID=ids;
    if (mcsstable.selectedRowsID=="")
        $(mcsstable.getAction("batchedit")).removeClass("mcsstable-selected-action");
    else
        $(mcsstable.getAction("batchedit")).addClass("mcsstable-selected-action");
}
function mcsstable_createMoveDiv(obj,event,mcsstable,type){

    var diffX=-50;var diffY=50;
    if(event.pageX==null&&event.clientX!=null)
    {var doc =document.documentElement,body =document.body;event.pageX=event.clientX+(doc &&doc.scrollLeft||body &&body.scrollLeft||0)-(doc &&doc.clientLeft||body &&body.clientLeft||0);event.pageY=event.clientY+(doc &&doc.scrollTop||body &&body.scrollTop||0)-(doc &&doc.clientTop||body &&body.clientTop||0);}
    e=event;
    var iDiffX = e.pageX -diffX;
    var iDiffY = e.pageY -diffY;
    var divid = mcsstable.tableid+"_movingbar";
    $("#"+divid).remove();
    var str = "行";
    var len = mcsstable.selectedRowsID.split(",").length;
    if(type){
        str = "列";
        len = $(mcsstable.table).find("th[th_isselect='true']").length;
    }
    var html = "";//"<span style='line-height:30px;margin-left:15px'>第<b>"+$(obj).html()+"</b>"+str+"被选中</span>";
    if(len > 1)
        html = "<span style='line-height:30px;margin-left:15px'>共<b>"+len+"</b>"+str+"被选中</span>";
    var moveDiv="<div id='"+divid+"' style='background:#CCDDFF;position:absolute;width:100px;height:30px;opacity:0.6;border:1px solid #BBFFEE'>"+html+"</div>";
    $(mcsstable.table).append(moveDiv);
    $("#"+divid).css({ "left": iDiffX+"px", "top": iDiffY+"px"});
}
//生成移动的div
function mcsstable_setRowMoveDiv(obj,event,mcsstable){
    var table = mcsstable.table;
    if(!$("#"+table.id+"_tableheader").attr("leftMouseDown"))
        return;
    var diffX=-50;var diffY=50;
    mcsstable_createMoveDiv(obj,event,mcsstable);
    var divid = mcsstable.tableid+"_movingbar";
    var checkval = $("#func_checkRowFirst").val();
    if(!checkval){
        $(mcsstable.table).after("<input type='hidden' value='true' id='func_checkRowFirst' />");
        // mcdom.popup("按住ctrl或shift可进行多选",{top:event.clientY+"px",left:(event.clientX+20)+"px",fadeOut:true,hideheader:true,seconds:2,tag:"ctrl"});
        if(!$("#func_checkColumnFirst").val())
        {
            $(document).mousemove(function (event)
            {

                if(event.pageX==null&&event.clientX!=null)

                {
                    vardoc =document.documentElement,body =document.body;event.pageX=event.clientX+(doc &&doc.scrollLeft||body &&body.scrollLeft||0)-(doc &&doc.clientLeft||body &&body.clientLeft||0);event.pageY=event.clientY+(doc &&doc.scrollTop||body &&body.scrollTop||0)-(doc &&doc.clientTop||body &&body.clientTop||0);
                }

                e=event;

                var iDiffX = e.pageX -diffX;

                var iDiffY = e.pageY -diffY;

                $("#"+divid).css({ "left": iDiffX-120+"px", "top": iDiffY+"px"});

            }).mouseup(function (event){

                $("#"+table.id+"_tableheader").removeAttr("leftMouseDown");

                $("#"+divid).remove();

            });
        }
        var tr = $(table).find("td[tag='cell']").parent();
        mcsstable.setTrMoveFunc(tr);
    }
}

//表格行点击事件
function mcsstable_linenum_onclick(td,event,params)
{
    if (params&&params.beforeRowClick){
        if (params.beforeRowClick()==false)
            return;
    }
    $(".actionSheet_wrap").fadeOut(100);
    var fix_th=$("#main_showFormStyle").get(0);
    var tb = mcdom.getObjByTagName(fix_th,"TABLE");
    var mcsstable=tb.mcsstable;
    var tableFixCol=mcsstable.tableFixCol;
    var tableid = tableFixCol.id;
    if (td.tagName=="TD"){
        var tr=td.parentNode;
        var rowIndex =tr.rowIndex;
    }else{
        var tr=null;
        var rowIndex =0;
    }
    var option_style='';
    if(is_share && is_share=='share')
        option_style = " style='display:none' ";
    
    clearSelStyle(mcsstable);
    td.style.backgroundColor='rgb(186, 225, 245)';
    
    var current_tr = tableFixCol.rows[rowIndex];
    
    var recordid=$(current_tr).attr('recordid');
    if (mcsstable.view!="celledit" && mcsstable.view!="tableedit") return;

    var menu_htm='';
    if (document.body && document.documentElement)
        var height3=document.body.clientHeight>document.documentElement.clientHeight?document.body.clientHeight:document.documentElement.clientHeight;
    else
        height3=document.documentElement.clientHeight;
    var current_link=window.location.href;
    if(current_link.indexOf("/")>-1){
        var current_link=current_link.split("/");
        var m_index=current_link.length-1;
        if(current_link[m_index]=='share')
            return;
    }
    
    var shareoptions=mcsstable.params.project_fulldata.shareoption;

    var menu_htm="";
    if (mcsstable.params.model_ownerid == loginuser || shareoptions.indexOf('[notaddrecord]')<0){
        menu_htm+="<div class='weui_actionsheet_cell ding_insert'>插入</div>";
    }
    if (recordid){
        if (mcsstable.params.model_ownerid == loginuser || shareoptions.indexOf('[editmyrecords]')<0 || (shareoptions.indexOf('[editmyrecords]')>=0 && mcsstable.mcssData.getFieldValue(recordid,'SYS_ADDUSER') == loginuser )){
            menu_htm+="<div class='weui_actionsheet_cell ding_editrecord' >编辑</div>";
        }
    }
    if (mcsstable.params.model_ownerid == loginuser || shareoptions.indexOf('[notdelrecord]')<0){
        menu_htm+="<div class='weui_actionsheet_cell ding_delrecord' >删除</div>";
    }
   
    if (menu_htm=="") return;
    dom_showPopupMenu(menu_htm,td,{position:"bottom"});
   
    $(".ding_insert").click(function(){
        //var current_tr=$(mcdom.getObjByTagName(_td,"TR")).get(0);
        mcsstable_insertRow(current_tr,tableid);
    })
    $(".ding_viewrecord").click(function(){
        mcsstable_openAutoform(current_tr,tableid,recordid,"查看","VIEW");
    })
    $(".ding_editrecord").click(function(){
        mcsstable_openAutoform(current_tr,tableid,recordid,"编辑","EDIT");
    })

    $(".ding_delrecord").click(function(){
        mcsstable_deleteOneRow(current_tr,tableid,{obj:this});
    })
    
    
}


//生成移动div
function mcsstable_setColumnMoveDiv(obj,event,mcsstable){
    var diffX=-50;var diffY=50;
    mcsstable_createMoveDiv(obj,event,mcsstable,"column");
    var table = mcsstable.tableFixRow;
    var divid = mcsstable.tableid+"_movingbar";
    var checkval = $("#func_checkColumnFirst").val();
    if(!checkval || checkval=="undefined"){
        $(table).after("<input type='hidden' value='true' id='func_checkColumnFirst' />");
        // mcdom.popup("按住ctrl或shift可进行多选",{top:(event.clientY+20)+"px",left:(event.clientX+20)+"px",fadeOut:true,hideheader:true,seconds:2,tag:"ctrl"});
        if(!$("#func_checkRowFirst").val() || $("#func_checkRowFirst").val()=="undefined"){
            $(document).mousemove(function (event) {
                if(event.pageX==null&&event.clientX!=null)
                {
                    var doc =document.documentElement;
                    body =document.body;
                    event.pageX=event.clientX+(doc &&doc.scrollLeft||body &&body.scrollLeft||0)-(doc &&doc.clientLeft||body &&body.clientLeft||0);
                    event.pageY=event.clientY+(doc &&doc.scrollTop||body &&body.scrollTop||0)-(doc &&doc.clientTop||body &&body.clientTop||0);
                }
                e=event;
                var iDiffX = e.pageX -diffX;
                var iDiffY = e.pageY -diffY;
                $("#"+divid).css({ "left": iDiffX-40+"px", "top": iDiffY-10+"px"});

            }).mouseup(function (event){

                $("#"+mcsstable.tableid+"_tableheader").removeAttr("leftMouseDown");
                $("#"+divid).remove();
                $("body").removeAttr("onselectstart");
            });
        }
        var row = $(table).find("tr");
        mcsstable.setTdMoveFunc(row);
    }
}
//选中整列
function mcsstable_header_alpha_click(obj,event,onlyPopMenu)
{
    mcsstable_setmousedown(obj);
    var cellIndex=obj.parentNode.cellIndex;
    var table=mcdom.getObjByTagName(obj,"TABLE");

    var mcsstable=mcsstable_getMCSSTable(table.id);
    var table2=null;
    if (table==mcsstable.tableFixRow)
    {
        cellIndex+=mcsstable.tableFixCol.rows[0].cells.length;
        table2=mcsstable.tableData;
    }else{
        table2=mcsstable.tableFixCol;
    }
    var tableData=mcsstable.tableData;

    mcsstable.setCurrentRow(null);
    var colFloatShow ='';
    var show_float_tool = false;
    if(mcsstable.params.setStyle && (mcsstable.view =="tableedit" || mcsstable.view=="celledit"))
        show_float_tool = true;

    var headerTr = $("#"+table.id+"_tableheader");
    var prevCellIndex = "";

    if((!event.ctrlKey && !event.shiftKey) )
    {
        mcsstable_clearSelect_td(table,true);
        mcsstable.selectedCols=[];
    }

    if(event.shiftKey){
        var th = $(table.rows[0]).find("th[th_isselect='true']").eq(0).get(0);
        if(th && th.id)
            prevCellIndex = $(th).get(0).cellIndex;
    }

    $(table).removeClass("mcsstable_notselect").addClass("mcsstable_notselect");
    if(prevCellIndex){
        var iniIndex = prevCellIndex;
        var maxIndex = cellIndex;
        if(prevCellIndex > maxIndex){
            iniIndex = cellIndex;
            maxIndex = prevCellIndex;
        }
        for(var i = prevCellIndex;i <= maxIndex;i++){
            //mcsstable_setColumnBorder(table,i,colFloatShow);
            table.rows[0].cells[i].setAttribute("th_isselect",'true');
            mcsstable.selectedCols.push[i];
        }
        ;
        for(var a=mcsstable.selectAreas.length-1;a>=0;a--)
        {
            var area=mcsstable.selectAreas[a];
            if (area.beginCellIndex>=prevCellIndex && area.endCellIndex<=maxIndex)
            {
                if (area.divSel ) {
                    if (area.divSel == mcsstable.divSel) mcsstable.divSel = null;
                    area.divSel.remove();
                }
                mcsstable.selectAreas.splice(a,1);
            }
        }
        var area={beginRowIndex:0,beginCellIndex:iniIndex,endRowIndex:tableData.rows.length,endCellIndex:maxIndex};
    }else{
        table.rows[0].cells[obj.parentNode.cellIndex].setAttribute("th_isselect",'true');
        var area={beginRowIndex:-1,beginCellIndex:cellIndex,endRowIndex:tableData.rows.length,endCellIndex:cellIndex};
    }

    if(colFloatShow!='hidden' && g_showFloatTool)
        mcsstable_setColFloatToolbar(table,obj.parentNode.cellIndex);
    else
        mcsstable_clearFloatToolBar();

    mcsstable.currentArea=area;
    mcsstable.currentArea.area=area;

    if (!mcsstable.selectAreas) mcsstable.selectAreas=[];

    if (!onlyPopMenu){

        var divS= getDivSel(mcsstable,(event.ctrlKey||event.shiftKey));
        //if (!mcsstable.divSel) mcsstable.divSel=divS;
        divS.style.backgroundColor='#c1e9ff';
        divS.style.display="";
        divS.style.opacity="0.5";
        divS.td=mcsstable.getCell(0,cellIndex);
        area.divSel = divS;
        mcsstable.selectAreas.push(area);
        mcsstable_SelectArea(mcsstable,divS,true);
    }

    table.parentNode.parentNode.scrollTop = 0;
    document.body.scrollTop = 0;
    if (!mcsstable.selectedCols) mcsstable.selectedCols=[];
    mcsstable.selectedCols.push(cellIndex);

    mcsstable_setColumnMoveDiv(obj,event,mcsstable);

}
//移动选中单元格
function mcsstable_selectTd_mouseover(td,event){

    var table=mcdom.getObjByTagName(td,"TABLE");
    var mcsstable=mcsstable_getMCSSTable(table.id);
    var fixcells=mcsstable.tableFixCol.rows[0].cells;

    mcsstable_clearSelect_td(mcsstable.table,true);
    mcsstable_clearFloatToolBar();
    mcsstable.currentArea.endRowIndex = td.parentNode.rowIndex;
    mcsstable.currentArea.endCellIndex = mcsstable.getCellIndex(td);

    //判断开始和结束的行列标志哪个更大，并整理其顺序。
    var area={beginRowIndex:mcsstable.currentArea.beginRowIndex,beginCellIndex:mcsstable.currentArea.beginCellIndex,endRowIndex:mcsstable.currentArea.endRowIndex,endCellIndex:mcsstable.currentArea.endCellIndex,table:table}
    if (area.beginRowIndex>area.endRowIndex){
        var rindex=area.beginRowIndex;
        area.beginRowIndex=area.endRowIndex;
        area.endRowIndex=rindex;
    }
    if (area.beginCellIndex>area.endCellIndex)
    {
        var cindex=area.beginCellIndex;
        area.beginCellIndex=area.endCellIndex;
        area.endCellIndex=cindex;
    }
    mcsstable.currentArea.area=area;
    mcsstable.selectAreas=[mcsstable.currentArea];
    mcsstable_SelectArea(mcsstable,mcsstable.divSel,true);

    for(var r=area.beginRowIndex;r<=area.endRowIndex;r++)
    {
        for(var c=area.beginCellIndex;c<=area.endCellIndex;c++)
        {
            var td=mcsstable.getCell(r,c);
            $(td).attr("isselect",true);
        }
    }

}
function mcsstable_SelectArea(mcsstable,divSelect,autoSelData)
{
    var table=mcsstable.table;
    var tableFixRow=mcsstable.tableFixRow;
    var tableFixCol=mcsstable.tableFixCol;
    var tableData=mcsstable.tableData;
    var tableFixRowCol=mcsstable.tableFixRowCol;
    var tableCur=null;
    var divSel = null;
    if (divSelect){
        divSel=divSelect;
    }else{
        if (!mcsstable.divSel) mcsstable.divSel = getDivSel(mcsstable);
        divSel= mcsstable.divSel;
    }
    if (divSel.divFill) divSel.divFill.style.display="none";
    area=mcsstable.currentArea.area;

    if (area.beginRowIndex<0) {
        if (area.beginCellIndex<tableFixCol.rows[0].cells.length){
            tableCur=tableFixRowCol;
            var tdSel=tableFixRowCol.rows[0].cells[area.beginCellIndex];
        }else{
            tableCur=tableFixRow;
            var tdSel=tableFixRow.rows[0].cells[area.beginCellIndex-tableFixCol.rows[0].cells.length];
        }
        divSel.style.top = tdSel.offsetTop + "px";
        divSel.style.left = tdSel.offsetLeft + tableCur.parentNode.parentNode.offsetLeft +  "px";
        area.beginRowIndex=0;
    }
    else{

        var tdSel=mcsstable.getCell(area.beginRowIndex,area.beginCellIndex);
        var tableCur=mcdom.getObjByTagName(tdSel,"TABLE");
        divSel.style.top = tdSel.offsetTop+ mcsstable.tableData.parentNode.parentNode.parentNode.offsetTop + mcsstable.tableData.offsetTop  -1 + "px";
        if (tableCur==mcsstable.tableData) {
            divSel.style.left = tdSel.offsetLeft - mcsstable.divSelBack.offsetLeft -1 + mcsstable.tableData.parentNode.parentNode.parentNode.offsetLeft + "px";
        }else{
            divSel.style.left = tdSel.offsetLeft - mcsstable.divSelBack.offsetLeft -1 + "px";
        }
    }
    divSel.style.display="";
    var width=2;
    var height=2;
    for(var r=area.beginRowIndex;r<=area.endRowIndex;r++)
    {
        if (r<tableData.rows.length)
            height+=tableData.rows[r].offsetHeight;
    }
    for(var c=area.beginCellIndex;c<=area.endCellIndex;c++){
        var r=0;
        if (mcsstable.tableData.rows[0].getAttribute("groupname")) r=1;
        var td=mcsstable.getCell(r,c);

        if (td) width+=td.offsetWidth;
    }
    divSel.style.width=width+"px";
    divSel.style.height=height+"px";
    mcsstable.divSelBack.style.width= mcsstable.table.clientWidth+"px";
    mcsstable.divSelBack.style.height=mcsstable.table.clientHeight+"px";;
    if (mcsstable.divSel.divFill){
        mcsstable.divSel.divFill.style.left = parseInt( mcsstable.divSel.style.width)-6+"px";
        mcsstable.divSel.divFill.style.top =parseInt(mcsstable.divSel.style.height)-6+"px";
        mcsstable.divSel.divFill.style.display="";
    }

    if (autoSelData)
    {
        mcsstable_SelTableData(mcsstable);
    }

}
//设置选中单元格的状态
function mcsstable_SetSelectTdStatus(e,event){

    var table=mcdom.getObjByTagName(e,"TABLE");
    if (!table) return;
    var headerTr = $("#"+table.id+"_tableheader");
    $(headerTr).attr("leftMouseDown",true);

    var mcsstable=mcsstable_getMCSSTable(table.id);

    if (mcsstable.divSel && mcsstable.divSel.td == e) {
        if (mcsstable.currentArea.beginCellIndex==mcsstable.currentArea.endCellIndex && mcsstable.currentArea.beginRowIndex == mcsstable.currentArea.endRowIndex)
            return;
    }

    if (!event.shiftKey)
    {
        mcsstable_clearSelect_td(table,true);
        mcsstable.selectAreas=[];
        mcsstable.currentArea = {};

        mcsstable.currentArea.beginRowIndex = e.parentNode.rowIndex;
        mcsstable.currentArea.beginCellIndex = mcsstable.getCellIndex( e);//.cellIndex;
        mcsstable.currentArea.endRowIndex = e.parentNode.rowIndex;
        mcsstable.currentArea.endCellIndex = mcsstable.getCellIndex( e);//.cellIndex;

        var area={beginRowIndex:mcsstable.currentArea.beginRowIndex,beginCellIndex:mcsstable.currentArea.beginCellIndex,endRowIndex:mcsstable.currentArea.endRowIndex,endCellIndex:mcsstable.currentArea.endCellIndex,table:table}
        mcsstable.currentArea.area=area;
        mcsstable.selectAreas.push(mcsstable.currentArea);
        $(e).attr("isselect",true);
        setFocusToTd(e,false);
        return;
    }
    $(e).attr("isselect",true);

    mcsstable_clearFloatToolBar();

    var td = $(table).find("td[isselect='true']").eq(0);
    if(td && td!="undefined"){
        var beginRowIndex = $(e).parent().get(0).rowIndex;
        var beginCellIndex = mcsstable.getCellIndex( e);
        var endRowIndex = $(td).parent().get(0).rowIndex;
        var endCellIndex = mcsstable.getCellIndex(td);

        if (table==mcsstable.tableData){
            //sunliangchao 还没改完，因为可能来自两个表
            beginCellIndex+=mcsstable.tableFixCol.rows[0].cells.length;
            endCellIndex+=mcsstable.tableFixCol.rows[0].cells.length;
        }

        if(beginRowIndex > endRowIndex){
            var tempRowIndex = beginRowIndex;
            beginRowIndex = endRowIndex;
            endRowIndex = tempRowIndex;
        }
        if(beginCellIndex > endCellIndex){
            var tempCellIndex = beginCellIndex;
            beginCellIndex = endCellIndex;
            endCellIndex = tempCellIndex;
        }
        //mcsstable_clearSelect_td(table);

        mcsstable.selectAreas=[];

        mcsstable.currentArea = {};
        mcsstable.currentArea.beginRowIndex = beginRowIndex;
        mcsstable.currentArea.beginCellIndex = beginCellIndex;
        mcsstable.currentArea.endRowIndex = endRowIndex;
        mcsstable.currentArea.endCellIndex = endCellIndex;

        mcsstable.selectAreas.push(mcsstable.currentArea);
        var area=mcsstable.currentArea;
        var tdSel=table.rows[area.beginRowIndex].cells[area.beginCellIndex];
        mcsstable.divSel.style.top = tdSel.offsetTop+ table.offsetTop - table.parentNode.parentNode.scrollTop -2 + "px";
        mcsstable.divSel.style.left = tdSel.offsetLeft+ table.offsetLeft - table.parentNode.parentNode.scrollLeft -2 + "px";
        mcsstable.divSel.style.display="";
        var width=5;
        var height=5;
        for(var r=area.beginRowIndex;r<=area.endRowIndex;r++)
            height+=table.rows[r].offsetHeight;
        for(var c=area.beginCellIndex;c<=area.endCellIndex;c++)
            width+=table.rows[area.endRowIndex].cells[c].offsetWidth;
        mcsstable.divSel.style.width=width+"px";
        mcsstable.divSel.style.height=height+"px";

        for(var r=area.beginRowIndex;r<=area.endRowIndex;r++)
        {
            for(var c=area.beginCellIndex;c<=area.endCellIndex;c++)
            {
                var td=table.rows[r].cells[c];
                $(td).attr("isselect",true);
            }
        }
        //var params = {beginRowIndex:beginRowIndex,beginCellIndex:beginCellIndex,endRowIndex:endRowIndex,endCellIndex:endCellIndex,table:table};
        //mcsstable_commonSetCellBorderStyle(params);
    }
}

//循环生成表格表格
function mcsstable_commonSetCellBorderStyle(params){
    var beginRowIndex = params.beginRowIndex;
    var endRowIndex = params.endRowIndex;
    var beginCellIndex = params.beginCellIndex;
    var endCellIndex = params.endCellIndex;
    var table = params.table;

    var type = params.type;
    var mcsstable=mcsstable_getMCSSTable(params.table.id);
    var boldobj = $(mcsstable.getAction("styleedit")).find("li[csstype='bold']").eq(0);
    $(boldobj).attr("selecttype","select");
    var italicobj = $(mcsstable.getAction("styleedit")).find("li[csstype='italic']").eq(0);
    $(italicobj).attr("selecttype","select");
    var underlineobj = $(mcsstable.getAction("styleedit")).find("li[csstype='underline']").eq(0);
    $(underlineobj).attr("selecttype","select");
    if(!type)
        type = "solid";
    for(var i = beginRowIndex;i <= endRowIndex;i++)
    {
        var tr = $(table).find("tr").eq(i);
        for(var k = beginCellIndex;k <= endCellIndex;k++){
            var td = $(tr).find("td").eq(k);
            if(i == beginRowIndex)
                $(td).css({"border-top":mcsstable.selected_border_css});
            if(i == endRowIndex)
                $(td).css({"border-bottom":mcsstable.selected_border_css});
            if(k == beginCellIndex)
                $(td).css({"border-left":mcsstable.selected_border_css});
            if(k == endCellIndex)
                $(td).css({"border-right":mcsstable.selected_border_css});

            if(type!="dashed"){
                if($(td).css("font-weight")!="bold")
                    $(boldobj).attr("selecttype","noselect");
                if($(td).css("font-style")!="italic")
                    $(italicobj).attr("selecttype","noselect");
                if($(td).css("text-decoration")!="underline")
                    $(underlineobj).attr("selecttype","noselect");
                $(td).attr("isselect",true);

            }else
            {
                $(td).attr("iscopy",true);
            }
        }
    }
}

function mcsstable_cleartd_right(column,mcsstable){
    var cellIndex = $(column).get(0).cellIndex;
    var trs = $(mcsstable.table).find("tr");
    for(var i = 0;i < trs.length;i++){
        $(trs[i]).find('td,th').eq(cellIndex).css('border-right','solid 1px #ccc');
    }
}
function mcsstable_cleartd_left(column,mcsstable){
    var cellIndex = $(column).get(0).cellIndex;
    var trs = $(mcsstable.table).find("tr");
    for(var i = 0;i < trs.length;i++){
        $(trs[i]).find('td,th').eq(cellIndex).css('border-left','solid 1px #ccc');
    }
}
function mcsstable_cleartd_top(row){
    var tds = $(row).find("td[tag='cell']");
    for(var i = 0;i < tds.length;i++){
        $(tds[i]).css('border-top','solid 1px #ccc');
    }
}
function mcsstable_cleartd_bottom(row){
    var tds = $(row).find("td[tag='cell']");
    for(var i = 0;i < tds.length;i++){
        $(tds[i]).css('border-bottom','solid 1px #ccc');
    }
}

function mcsstable_th_mouseout(e)
{
    $("#headernotes").remove();
}

/*
 function mcsstable_addtd_bottom(row){
 var tds = $(row).find("td[tag='cell']");
 for(var i = 0;i < tds.length;i++){
 $(tds[i]).css("border-bottom","solid 2px #003377");
 }
 }
 function mcsstable_addtd_top(row){
 var tds = $(row).find("td[tag='cell']");
 for(var i = 0;i < tds.length;i++){
 $(tds[i]).css("border-top","solid 2px #003377");
 }
 }
 */

//设置选中整行的边框格式
function mcsstable_setTrBorder(table,rowIndex,trFloatShow){

    var tr=table.rows[rowIndex];
    var tds = $(tr).find("td[tag='cell']");
    var title = $(tr).find("td[class*='linenumber']").eq(0);
    $(title).attr("tr_isselect",true);
    var mcsstable = mcsstable_getMCSSTable(table.id);
    var boldobj = $(mcsstable.getAction("styleedit")).find("li[csstype='bold']").eq(0);
    $(boldobj).attr("selecttype","select");
    var italicobj = $(mcsstable.getAction("styleedit")).find("li[csstype='italic']").eq(0);
    $(italicobj).attr("selecttype","select");
    var underlineobj = $(mcsstable.getAction("styleedit")).find("li[csstype='underline']").eq(0);
    $(underlineobj).attr("selecttype","select");
    if(tds){
        var topcondition = true;
        var botcondition = true;
        for(var i = 0;i < tds.length;i++){
            var td = tds[i];
            if(i==0){
                var before = $(td).parent().prev();
                var next = $(td).parent().next();
                if(before && $(before).find("td[tr_isselect='true']").length > 0){
                    topcondition = false;
                    mcsstable_cleartd_bottom(before);
                }
                if(next && $(next).find("td[tr_isselect='true']").length > 0){
                    botcondition = false;
                    mcsstable_cleartd_top(next);
                }
                $(td).css('border-left',mcsstable.selected_border_css);
            }
            if(topcondition)
                $(td).css("border-top",mcsstable.selected_border_css);
            if(botcondition)
                $(td).css("border-bottom",mcsstable.selected_border_css);
            $(td).attr("isselect",true);
            if(i==tds.length-1)
                $(td).css({'border-right':mcsstable.selected_border_css});
            if($(td).css("font-weight")!="bold")
                $(boldobj).attr("selecttype","noselect");
            if($(td).css("font-style")!="italic")
                $(italicobj).attr("selecttype","noselect");
            if($(td).css("text-decoration")!="underline")
                $(underlineobj).attr("selecttype","noselect");
        }
    }
}
//设置选中整列的边框格式
function mcsstable_setColumnBorder(table,cellIndex,colFloatShow){

    //浮动工具条：列选中  edit by gang
    var mcsstable = mcsstable_getMCSSTable(table.id);
    var boldobj = $(mcsstable.getAction("styleedit")).find("li[csstype='bold']").eq(0);
    $(boldobj).attr("selecttype","select");
    var italicobj = $(mcsstable.getAction("styleedit")).find("li[csstype='italic']").eq(0);
    $(italicobj).attr("selecttype","select");
    var underlineobj = $(mcsstable.getAction("styleedit")).find("li[csstype='underline']").eq(0);
    $(underlineobj).attr("selecttype","select");
    var leftcondition = true;
    var rightcondition = true;
    $(table.rows).each(function(i,item){
        var obj = $(item.cells[cellIndex]);
        if(i==0){
//			$(obj).find("div").eq(0).css("background","#003377").css("color","white");
            //$(obj).find("div").eq(0).css("background","#003377");
            $(obj).css({'border-top':mcsstable.selected_border_css});
            $(obj).attr("th_isselect",true);
            var before = $(item.cells[cellIndex-1]);
            var next =$(item.cells[parseInt(1)+parseInt(cellIndex)]);
            if(before && $(before).attr("th_isselect")){
                leftcondition = false;
                mcsstable_cleartd_right(before,mcsstable);
            }

            if(next && $(next).attr("th_isselect")){
                rightcondition = false;
                mcsstable_cleartd_left(next,mcsstable);
            }
        }
        /*
         if(leftcondition)
         $(obj).css('border-left',mcsstable.selected_border_css);
         if(rightcondition)
         $(obj).css('border-right',mcsstable.selected_border_css);
         */
        if(i!=0)
            $(obj).attr("isselect",true);
        /*
         if(i==table.rows.length-1)
         $(obj).css({'border-bottom':mcsstable.selected_border_css});
         if($(obj).css("font-weight")!="bold")
         $(boldobj).attr("selecttype","noselect");
         if($(obj).css("font-style")!="italic")
         $(italicobj).attr("selecttype","noselect");
         if($(obj).css("text-decoration")!="underline")
         $(underlineobj).attr("selecttype","noselect");
         */
    })
}

//从divEdit中得到粘贴过来的内容
function getPasteData(obj)
{
    var datarows=[];
    var content = obj.firstChild;
    if (content.tagName=="TABLE")
    {
        var table=content;
        //先复制一个表格结构，而后再往里添加数据
        for(var r=0;r<table.rows.length;r++)
        {
            var row=table.rows[0];
            var rdata={};
            rdata.cells=[];
            datarows.push(rdata);

            for(var c=0;c<row.cells.length;c++)
            {
                rdata.cells.push('[null]');
                var cell = row.cells[c];
                var csp = cell.getAttribute("cellspan")
                if (csp){
                    for(p=0;p<csp;p++)
                    {
                        rdata.cells.push('[null]');
                    }
                }
            }
        }

        for(var r=0;r<table.rows.length;r++)
        {
            var row=table.rows[r];
            var rdata=datarows[r];
            var cells = rdata.cells;
            for(var c=0;c< rdata.cells.length ;c++)
            {
                var cell = row.cells[c];
                if (cell){

                    var cindex=c;

                    while(rdata.cells[cindex]!='[null]')
                    {
                        cindex++;
                    }
                    if (cell.innerText)
                        var v=cell.innerText;
                    else if(cell.textContent)
                        var v=cell.textContent;
                    else
                        var v="";
                    var v=v.replace("\n","");
                    rdata.cells[cindex] = {value:v,style:cell.style};

                    var csp = cell.getAttribute("colspan");
                    var rsp = cell.getAttribute("rowspan");
                    if (csp) csp = parseInt(csp);else csp= 1;
                    if (rsp) rsp = parseInt(rsp);else rsp= 1;
                    if (csp>1 || rsp>1){
                        for(var r2=r;r2<r+rsp;r2++)
                        {
                            for(var c2=cindex;c2<cindex+csp;c2++)
                            {
                                if (cell.innerText)
                                    var v=cell.innerText;
                                else if(cell.textContent)
                                    var v=cell.textContent;
                                else
                                    var v="";
                                var v=v.replace("\n","");
                                var s = cell.style.cssText;
                                datarows[r2].cells[c2]= {value:v,style:s};
                            }
                        }
                    }
                }
            }
        }

        return datarows;
    }else{
        var rows=[];
        var rdata={};
        rdata.cells=[];
        rdata.cells.push({value:obj.innerText.replace("\n",""),style:""});
        rows.push(rdata);
        return rows;
    }
}
//清空所选区域中的数据
function mcsstable_clearSelectData(mcsstable,area,undo){

    if (!mcsstable.divSel) return;
    if (!mcsstable.divSel.td) return;

    if (!area)
        var area = mcsstable.currentArea.area;

    var oldrows=[];


    for(var r=area.beginRowIndex ;r<=area.endRowIndex;r++)
    {
        var oldrow={};
        oldrow.cells=[];
        oldrow.type="UPDATE";
        oldrows.push(oldrow);
        var tr = mcsstable.getRow(r);
        for(var c=area.beginCellIndex;c<=area.endCellIndex;c++)
        {
            var td = tr.cells[c];
            var recordid=tr.getAttribute("id");
            var fieldid = $(td).attr("id");
            var field=mcsstable.modelDealer.getField(fieldid);
            var value="";
            var oldcell={};
            oldcell.value=mcsstable.mcssData.getFieldValue(recordid,fieldid);
            oldrow.cells.push(oldcell);
            if (td.style.cssText!=""){
                oldcell.style=td.style.cssText;
                var params={mcsstable:mcsstable,type:'cell'};
                //设置格式的后台方法有问题，在插入的模式下一次能修改多个参数，但修改时一次只能修改一个参数，因此要随时保存
                mcsstable.uniTableStyle.setFieldFormate("font-weight", "normal" ,td,params);
                mcsstable.uniTableStyle.setFieldFormate("font-size","12px",td,params);
                mcsstable.uniTableStyle.setFieldFormate("color","",td,params);
                mcsstable.uniTableStyle.setFieldFormate("background-color","",td,params);
                mcsstable.uniTableStyle.setFieldFormate("text-align","left",td,params);
                mcsstable.uniTableStyle.setFieldFormate("font-style","normal",td,params);
                mcsstable.uniTableStyle.setFieldFormate("text-decoration","left",td,params);
                td.style.backgroundColor="#fff";
                td.style.fontSize="12px";
                td.style.fontWeight="normal";
                td.style.color="#000";
            }
            td.innerHTML="";
            if (field.type!="file" && field.type!="image")
                mcsstable.editField($(tr).attr("recordid"),fieldid,value,true);
        }


    }
    window.setTimeout(function(){mcsstable.fixRowHeight();},500);
    if (!undo) mcsstable.addUndoList(null,mcsstable_savePasteData,[mcsstable,oldrows,area.beginRowIndex,area.beginCellIndex,0,true],null,mcsstable_clearSelectData,[mcsstable,area,true]);

    mcsstable.uniTableStyle.save();
    mcsstable.mcssData.save(null,{refObject:mcsstable});

}

function showProcessMessage(tip,percent){

    if (!percent){
        var h = "<div style='margin-left:20px'><label id='divProcessTitle'>"+tip+"</label></div>";
        h+="<div style='margin-left:20px;margin-top:6px;width:330px;height:25px;border:solid 1px #bbb'><div id='divProcessValue' style='height:25px;width:1px:marin-left:0px;background-color:#99f'></div></div>"
        h+="<div style='margin-left:300px;margin-top:12px'>"
            +" <input id='btn_cancelProcess' type='button' class='btn btn-primary btn-sm' value='取消'/></div>";
        var popup = mcdom.popup(h,{title:"任务进度",width:"400px",height:"165px",position:"middle"});

        $("#btn_cancelProcess").click(function(){
            mcdom.closePopup(popup);
        })

    }else{

        if ($("#divProcessValue"))
        {
            var divV=$("#divProcessValue").get(0);
            var divT=$("#divProcessTitle").get(0);

            divV.style.width= parseInt ( divV.parentNode.offsetWidth * percent) + "px";
            divT.innerText= tip + ":" + parseInt( percent * 100 ) + "%";

        }else{
            return false;
        }
        if (percent>0.999) {
            var divT=$("#divProcessTitle").get(0);
            divT.innerText= tip
            var divB=$("#btn_cancelProcess").get(0);
            divB.value="确定";
            //mcdom.closePopup(popup);
        }
    }
    return true;

}

//从表格中粘贴数据
function mcsstable_pasteFromTable(mcsstable,datarows){

    if (!mcsstable.divSel) return;
    if (!mcsstable.divSel.td) return;
    var rowIndex = mcsstable.divSel.td.parentNode.rowIndex;
    var cellIndex = mcsstable.getCellIndex(mcsstable.divSel.td);

    var area = {beginRowIndex:rowIndex,beginCellIndex:cellIndex};

    area.endRowIndex = rowIndex + datarows.length-1;
    area.endCellIndex = cellIndex + datarows[0].cells.length-1;
    mcsstable.currentArea=area;
    mcsstable.currentArea.area=area;
    mcsstable_SelectArea(mcsstable);

    if (datarows.length>50) showProcessMessage("正在保存粘贴的数据");
    mcsstable.pasteOldRows=[];
    mcsstable.pasteFromRow=rowIndex;
    mcsstable.pasteFromCell=cellIndex;
    mcsstable_savePasteData(mcsstable,datarows,rowIndex,cellIndex,0)

}

function mcsstable_savePasteData(mcsstable,datarows,rowIndex,cellIndex,startRow,isundo)
{
    var startCell = cellIndex;
    var bDataChanged=false;
    var bSaveStyle=false;
    if (!startRow) startRow=0;
    for(var r=startRow;r<datarows.length;r++)
    {
        var cells=datarows[r].cells;

        var tr = mcsstable.getRow(rowIndex);

        var recordid="";
        if (tr) recordid=tr.id;

        if (!isundo) {
            var oldrow={};
            oldrow.cells=[];
            oldrow.id=recordid;
            oldrow.type="UPDATE";
            mcsstable.pasteOldRows.push(oldrow);
            datarows[r].type="UPDATE";
        }else if (datarows[r].type=="DELETE")
        {
            mcsstable.deleteRow( datarows[r].id);
            continue;
        }

        if (!recordid){
            recordid = getTimeId();
            oldrow.id=recordid;
            oldrow.type="DELETE";
            datarows[r].type="NEW";
            var rowData = mcsstable_createNewData(mcsstable, recordid);
            //新增行方法拆分出去

            mcsstable_onNewRow(mcsstable.tableFixCol.rows[rowIndex],rowData,recordid,fieldid);

            if (mcsstable.modeldata.sortfieldid) {
                var sortfield = mcsstable.modeldata.sortfieldid;
                mcsstable.params.maxSort = parseInt(mcsstable.params.maxSort) + 100;
                mcsstable.mcssData.setFieldValue(recordid,sortfield,mcsstable.params.maxSort);
                rowData[sortfield] = mcsstable.params.maxSort;
            }

        }

        for(var c=0;c<cells.length;c++)
        {
            var td = mcsstable.getCell(rowIndex,cellIndex);
            if (td) {
                var fieldid = $(td).attr("id");
            }
            var field=mcsstable.modelDealer.getField(fieldid);
            if (field){
                var value=cells[c].value;
                var style=cells[c].style;
                if (field.type=="numtext")
                {
                    value = parseFloat(value);
                    if (isNaN(value) ) value = '0';
                }else if(field.type=="date" || field.type=="datetime" || field.type=="time")
                {
                    svalue = value.replace(/-/g, "/");
                    var dvalue = new Date(svalue);
                    if (isNaN(dvalue)) value='';
                    var arr = svalue.split("/");
                    if (((parseInt(arr[0], 10) == dvalue.getFullYear()) && (parseInt(arr[1], 10) == (dvalue.getMonth() + 1)) && (parseInt(arr[2], 10) == dvalue.getDate())))
                    {
                        value = svalue.replace( /\//g,"-");
                    }else{
                        value='';
                    }
                }else if(field.type=="file" || field.type=="image")
                {
                    value = '';
                }
                if (value+""=="") value="";
                if (value == "undefined") value="";
                var oldvalue = mcsstable.mcssData.getFieldValue(recordid,fieldid);
                if (!isundo)
                {
                    var oldcell={value:oldvalue};
                    oldrow.cells.push(oldcell);
                }
                if(field.type!="file" && field.type!="image"){

                    if (oldvalue!=value){
                        if (td) {
                            td.innerHTML= "<span>" + value + "</span>" ;//var s=mcsstable_getFieldDisplay(field,value,mcsstable.params.special_field_show,rowData,this) ; value;
                            if (style+""!="") {
                                if (!isundo) oldcell.style=td.style.cssText;
                                td.style.cssText=style;
                                var params={mcsstable:mcsstable,type:'cell'};
                                if (td.style.fontWeight) mcsstable.uniTableStyle.setFieldFormate("font-weight", td.style.fontWeight ,td,params);
                                if (parseInt(td.style.fontSize)!=12) mcsstable.uniTableStyle.setFieldFormate("font-size",td.style.fontSize,td,params);
                                if (td.style.color!="") mcsstable.uniTableStyle.setFieldFormate("color",td.style.color,td,params);
                                if (td.style.backgroundColor!="") mcsstable.uniTableStyle.setFieldFormate("background-color",td.style.backgroundColor,td,params);
                                if (td.style.textAlign) mcsstable.uniTableStyle.setFieldFormate("text-align",td.style.textAlign,td,params);
                                if (td.style.fontStyle=="italic") mcsstable.uniTableStyle.setFieldFormate("font-style",td.style.fontStyle,td,params);
                                if (td.style.textDecoration=="underline") mcsstable.uniTableStyle.setFieldFormate("text-decoration",td.style.textDecoration,td,params);
                                bSaveStyle=true;
                            }
                        }
                        mcsstable.editField(recordid,fieldid,value,true);
                        bDataChanged=true;
                    }
                }
            }
            cellIndex++;
        }

        cellIndex=startCell;

        if (datarows.length>50) {

            if (r>startRow && parseInt( r /10 )*10 == r)
            {

                var b = showProcessMessage("正在保存粘贴的数据",r / datarows.length );
                mcsstable.tableData.parentNode.style.height=mcsstable.tableData.offsetTop + (mcsstable.tableData.rowHeight * mcsstable.tableData.rows.length) + "px";
                if (b==false) return;

                afterSave=function(){
                    window.setTimeout(function(){
                        startRow+=10;
                        mcsstable_savePasteData(mcsstable,datarows,rowIndex,cellIndex,startRow)
                    },10);
                }
                if (bDataChanged) {
                    mcsstable.mcssData.save(afterSave,{srcElement:mcsstable.divSel.td,refObject:mcsstable});
                    return;
                }
            }
        }
        rowIndex++;

    }

    if (datarows.length>50) showProcessMessage("所有数据保存成功",1);
    mcsstable.tableData.parentNode.style.height=mcsstable.tableData.offsetTop + (mcsstable.tableData.rowHeight * mcsstable.tableData.rows.length) + "px";

    var afterSaveAll=function(){
        if (datarows.length<100 && bSaveStyle ) {
            mcsstable.uniTableStyle.save();
        }
    }

    if (bDataChanged) mcsstable.mcssData.save(afterSaveAll,{srcElement:mcsstable.divSel.td,refObject:mcsstable,alert:(!isundo)});

    if (!isundo) {

        mcsstable.addUndoList(null,mcsstable_savePasteData,[mcsstable,mcsstable.pasteOldRows,mcsstable.pasteFromRow,mcsstable.pasteFromCell,0,true],null,mcsstable_savePasteData,[mcsstable,datarows,mcsstable.pasteFromRow,mcsstable.pasteFromCell,0,true]);
    }

}

function mcsstable_getTdArr(data){
    var arr = new Array();
    var firstNum = 0;var historyid = "";var secordNum = 0;
    for(var i = 0;i < data.length;i++){
        var recordid = $(data[i]).parent().attr("recordid");
        if(historyid!=recordid){
            secordNum = 0;
            if(historyid)
                firstNum++;
            arr[firstNum] = new Array();
            arr[firstNum][secordNum] = data[i];
            secordNum++;
            historyid = recordid;
        }else{
            arr[firstNum][secordNum] = data[i];
            secordNum++;
        }
    }
    return arr;
}
//鼠标下压事件
function mcsstable_setmousedown(e){
    document.body.leftMouseDown=true;
}
//鼠标弹起事件
function mcsstable_setmouseup(e){
    document.body.leftMouseDown=false;
    $("body").removeAttr("onselectstart");
}

/*
 MCSS格式化类
 newData=[{type:'cell',fielid:'string1',recordid:'23124134344',style:'font-size:16px;color:red'},
 {type:'row',recordid:'23124134344',style:'font-size:16px;color:red'},
 {type:'col',fielid:'string1',style:'font-size:16px;color:red'},
 {}]
 css('font-size','20px')
 set
 */
function UniTableStyle(container,params){
    if(!params)
        params = {};
    this.mcsstable = "";
    if(container.typename=='MCSSTable')
        this.mcsstable = container;
    this.newData = new Array();
    this.setStyleBold=function(params)
    {
        var prop = "font-weight";
        var value = "bold";
        if(params && params.mcsstable){

            //var obj = $(params.mcsstable.getAction("styleedit")).find("li[csstype='bold']").eq(0);
            var td=params.mcsstable.divSel.td;
            if(td.style.fontWeight=="bold"){
                value = "normal";
                //$(obj).attr("selecttype","noselect");
            }else{
                value = "bold";
                //$(obj).attr("selecttype","select");
            }

        }
        var types = this.getSaveFormateType();
        if(types.length==0){
            mcdom.alert("请至少选中一个格子!",'提示','info','fadeout');
            //wx_alert('','请至少选中一个格子!', fadeOut());

            return;
        }

        var condition = false;
        for(var i = 0;i < types.length;i++){
            if(types[i]=="column")
                condition = true;
            params["type"] = types[i];
            this.css(prop,value,params);
        }
        if(condition)
            this.save(unitablestyle_updateModelDealer);
        else
            this.save();
    }
    this.setStyleItalic=function(params)
    {
        var prop = "font-style";
        var value = "italic";
        if(params && params.mcsstable){
            //var obj = $(params.mcsstable.getAction("styleedit")).find("li[csstype='italic']").eq(0);
            var td=params.mcsstable.divSel.td;

            if(td.style.fontStyle=="italic"){
                value = "normal";
                //$(obj).attr("selecttype","noselect");
            }else{
                value = "italic";
                //$(obj).attr("selecttype","select");
            }
        }
        var types = this.getSaveFormateType();
        if(types.length==0){
            mcdom.alert("请至少选中一个格子!",'提示','info','fadeout');
            //wx_alert('','请至少选中一个格子!', fadeOut());

            return;
        }
        var condition = false;
        for(var i = 0;i < types.length;i++){
            if(types[i]=="column")
                condition = true;
            params["type"] = types[i];
            this.css(prop,value,params);
        }
        if(condition)
            this.save(unitablestyle_updateModelDealer);
        else
            this.save();
    }
    this.setUnderline=function(params)
    {
        var prop = "text-decoration";
        var value = "underline";
        if(params && params.mcsstable){
            var td=params.mcsstable.divSel.td;
            if(td.style.textDecoration=="underline"){
                value = "none";
                //$(obj).attr("selecttype","noselect");
            }else{
                value = "underline";
                //$(obj).attr("selecttype","select");
            }
        }
        var types = this.getSaveFormateType();
        if(types.length==0){
            mcdom.alert("请至少选中一个格子!",'提示','info','fadeout');
            //wx_alert('','请至少选中一个格子!', fadeOut());

            return;
        }
        var condition = false;
        for(var i = 0;i < types.length;i++){
            if(types[i]=="column")
                condition = true;
            params["type"] = types[i];
            this.css(prop,value,params);
        }
        if(condition)
            this.save(unitablestyle_updateModelDealer);
        else
            this.save();
    }
    this.setFontFace=function(params)
    {
        var prop = "font-family";
        var value = params.value;
        var types = this.getSaveFormateType();
        if(types.length==0){
            mcdom.alert("请至少选中一个格子!",'提示','info','fadeout');
            //wx_alert('','请至少选中一个格子!', fadeOut());

            return;
        }
        var condition = false;
        for(var i = 0;i < types.length;i++){
            if(types[i]=="column")
                condition = true;
            params["type"] = types[i];
            this.css(prop,value,params);
        }
        if(condition)
            this.save(unitablestyle_updateModelDealer);
        else
            this.save();
    }
    this.setFontSize=function(params)
    {
        var prop = "font-size";
        var value = params.value;
        var types = this.getSaveFormateType();
        if(types.length==0){
            mcdom.alert("请至少选中一个格子!",'提示','info','fadeout');
            //wx_alert('','请至少选中一个格子!', fadeOut());

            return;
        }
        var condition = false;
        for(var i = 0;i < types.length;i++){
            if(types[i]=="column")
                condition = true;
            params["type"] = types[i];
            this.css(prop,value,params);
        }
        if(condition)
            this.save(unitablestyle_updateModelDealer);
        else
            this.save();
    }
    this.setFontPosition=function(params)
    {
        var prop = "text-align";
        var value = params.value;
        if(value=="居左")
            value="left";
        else if(value=="居中")
            value="center";
        else
            value="right";
        var types = this.getSaveFormateType();
        if(types.length==0){
            mcdom.alert("请至少选中一个格子!",'提示','info','fadeout');
            //wx_alert('','请至少选中一个格子!', fadeOut());

            return;
        }
        var condition = false;
        for(var i = 0;i < types.length;i++){
            if(types[i]=="column")
                condition = true;
            params["type"] = types[i];
            this.css(prop,value,params);
        }
        if(condition)
            this.save(unitablestyle_updateModelDealer);
        else
            this.save();
    }
    this.setBackgroundColor=function(params)
    {
        var prop = "background-color";
        var value = params.value;
        var types = this.getSaveFormateType();
        if(types.length==0){
            mcdom.alert("请至少选中一个格子!",'提示','info','fadeout');
            //wx_alert('','请至少选中一个格子!', fadeOut());

            return;
        }
        var condition = false;
        for(var i = 0;i < types.length;i++){
            if(types[i]=="column")
                condition = true;
            params["type"] = types[i];
            this.css(prop,value,params);
        }
        if(condition)
            this.save(unitablestyle_updateModelDealer);
        else
            this.save();
    }
    this.setFontColor=function(params)
    {
        var prop = "color";
        var value = params.value;
        var types = this.getSaveFormateType();
        if(types.length==0){
            mcdom.alert("请至少选中一个格子!",'提示','info','fadeout');
            //wx_alert('','请至少选中一个格子!', fadeOut());

            return;
        }
        var condition = false;
        for(var i = 0;i < types.length;i++){
            if(types[i]=="column")
                condition = true;
            params["type"] = types[i];
            this.css(prop,value,params);
        }
        if(condition)
            this.save(unitablestyle_updateModelDealer);
        else
            this.save();
    }
    this.getSaveFormateType=function(){
        var types = new Array();
        var trLen = $(this.mcsstable.table).find("td[tr_isselect='true']").length;
        var thLen = $(this.mcsstable.table).find("th[th_isselect='true']").length;
        var tdLen = $(this.mcsstable.table).find("td[isselect='true']").length;
        if(trLen > 0)
            types.push("row");
        if(thLen > 0)
            types.push("column")
        if(tdLen > 0 && trLen == 0 && thLen == 0)
            types.push("cell");
        return types;
    }
    this.css=function(prop,value,params)
    {
        var type = params.type;
        if(type=="cell"){
            var cells = $(this.mcsstable.table).find("td[isselect='true']");
            for(var i = 0;i < cells.length;i++){
                this.setFieldFormate(prop,value,cells[i],params);
            }
        }else if(type=="row"){
            var tds = $(this.mcsstable.table).find("td[tr_isselect='true']");
            for(var i = 0;i < tds.length;i++){
                this.setFieldFormate(prop,value,tds[i],params);
            }
        }else if(type=="column"){
            var columns = $(this.mcsstable.table).find("th[th_isselect='true']");
            for(var i = 0;i < columns.length;i++){
                this.setFieldFormate(prop,value,columns[i],params);
            }
        }
    }
    this.getFieldFormate=function(obj,type)
    {
        var data = this.cssData;
        for(var i = 0;i < data.length;i++){
            var recordid = $(obj).parent().attr("recordid");
            if(type=="cell"){
                if(recordid == data[i].recordid && $(obj).attr("id") == data[i]["fielid"])
                    return this.data[i];
            }else if(type=="row"){
                if(recordid == data[i].recordid)
                    return this.data[i];
            }else if(type=="column"){
                if($(obj).attr("id") == data[i]["fielid"])
                    return this.data[i];
            }
        }
    }
    this.setFieldFormate=function(prop,value,obj,params)
    {
        var recordid = "";var fieldid = "";
        var type = params.type;
        if(type=="cell"){
            $(obj).css(prop,value);
            $(obj).children().eq(0).css(prop,value);
            fieldid = $(obj).attr("id");
            recordid = $(obj).parent().attr("recordid");
        }else if(type=="row"){
            $(obj).parent().find("td").css(prop,value);
            $(obj).children().eq(0).css(prop,value);
            recordid = $(obj).parent().attr("recordid");
        }else if(type=="column"){
            fieldid = $(obj).attr("fieldid");
            $(this.mcsstable.table).find("td[id='"+fieldid+"']").css(prop,value);
            $(this.mcsstable.table).find("td[id='"+fieldid+"']").children().eq(0).css(prop,value);
        }
        for(var i = 0;i < this.newData.length;i++){
            if(type=="cell"){
                if(recordid == this.newData[i].recordid && fieldid == this.newData[i].fieldid){
                    this.newData[i][prop] = value;
                    return;
                }
            }else if(type=="row"){
                if(recordid == this.newData[i].recordid){
                    this.newData[i][prop] = value;
                    return;
                }
            }else if(type=="column"){
                if(fieldid == this.newData[i].fieldid){
                    this.newData[i][prop] = value;
                    return;
                }
            }
        }
        var data = {type:type};
        if(type=="cell"){
            data.recordid = recordid;
            data.fieldid = fieldid
        }else if(type=="row"){
            data.recordid = recordid;
        }else if(type=="column"){
            data.projectid = this.mcsstable.params.project_fulldata.project_record.id;
            data.ownerid = this.mcsstable.params.project_fulldata.project_record.SYS_ADDUSER;
            data.fieldid = fieldid;
        }
        data[prop] = value;
        this.newData.push(data);
    }
    /*
     params.alert:false：不提醒保存结果，默认提醒
     params.onlyonefield:如果不为空，就表示保存成功后要立即更新第一条记录的该字段的值，即把this.initdata中该第一记录的该字段的值改为最新的。
     此情况用户celledit情况
     params.srcElement:触发此动作的html对象，如按钮
     params.refObject:引用词对象的对象，例如mcsstable对象
     */
    this.save=function(afterSave,params)
    {
        if (!params) params={};
        var showhint=true;
        if (params && params.alert==false)
            showhint=false;
        if (!this.newData || this.newData.length==0)
        {
            mcdom.popup(lang.nonetosave,{obj:params.srcElement,title:lang.save,fadeOut:true,width:'auto'});
            if (afterSave)
                afterSave(this);
            return;
        }
        var s="",s1="",recordidArr=[];
        for(i=0;i<this.newData.length;i++){
            recordidArr.push(this.newData[i]['mcss_recordid']);
            s1=JSON.stringify(this.newData[i]);
            if(s)
                s+="~|~";
            s+=s1;
        }
        if (!s)
        {
            if (showhint)
            {
                mcdom.popup(lang.nonetosave,{obj:params.srcElement,title:lang.save,fadeOut:true,width:'auto'});
                this.newData=[];
                if (afterSave)
                    afterSave(this);
                return;
            }
        }
        //alert(s);return;
        var _this=this;
        if(params.refObject && params.refObject.params.saveversion)
            params.refObject.params.saveversion(this);
        var urlpath =g_homeurl+g_systempath+"/saveFormateData";
        s=uni_cleanSQL(s);
        $.post(
            urlpath,
            {modelid:this.mcsstable.modeldata.id,usermodel_recordid:this.mcsstable.params.usermodel_recordid,tablename:this.mcsstable.modeldata.tablename,eidtedata:s},
            function(data){
                //alert(data);return;
                if (showhint)
                {
                    if (data=='0')
                    {
                        mcdom.alert(lang.noupdates,lang.save,'info','fadeout');
                    }
                    else{
                        if (params && params.onlyonefield){
                            _this.updateInitData(_this.newData[0]);
                        }
                        var pos={left:0}
                        if (params.srcElement)
                            pos=mcdom.getPosition(params.srcElement);
                        mcdom.popup(lang.successfullysaved,{top:"0px",left:pos.left+"px",fadeOut:true,hideheader:true,seconds:3,tag:"SAVEOK"});

                    }
                }
                _this.state="SAVED";
                _this.inputedByKey=false;
                //从mcssData中移除已经指定的记录。不能直接执行_this.newData=[];因为异步原因
                mcsstable_removeUpdatdNewData(_this.newData,recordidArr);
                if (afterSave)
                    afterSave(_this);
                if(params.refresh)
                    mcsstable_loaddatarows(_this.mcsstable.tableid);
            });
    }
}

function unitablestyle_updateModelDealer(uniTableStyle){
    var mcsstable=uniTableStyle.mcsstable;
    var	params={modelid:mcsstable.params.modelid,usermodel_recordid:mcsstable.params.usermodel_recordid,
        model_ownerid:mcsstable.params.model_ownerid,view_id:mcsstable.view_id,version_id:mcsstable.params.version_id};
    $.getJSON(mcsstable.getModelUrl,params,function(modeldata){
        //alert("控制器方法返回的模型数据是："+modeldata);
        mcsstable.setModelData(mcsstable,modeldata);
        mcsstable.modelDealer=new MCSSModel(modeldata);//模型操作类
        if (mcsstable.params.afterGetModel)
            mcsstable.params.afterGetModel(modeldata,mcsstable);
    })
}



/*
 UniSheet类
 UNISheet:铀尼表格类，类似于Excel的表格
 */
var UNISheets=new Array;
function getUNISheet(id){
    var r;
    for(var i=0;i<UNISheets.length;i++){
        if (UNISheets[i].id==id)
        {
            return UNISheets[i].data;
        }
    }
    return r;
}

function UNISheet(id,project_id,params)
{
    UNISheets.push({id:id,data:this});

    //获得模型数据后，进行一些操作。例如根据共享信息设置只读列
    this.UniSheet_afterGetModel=function(model,mcsstable)
    {

        if (mcsstable instanceof MCSSTable)
            var shareoptions=mcsstable.params.project_fulldata.shareoption;
        else
            var shareoptions=mcsstable.params.refObject.params.project_fulldata.shareoption;
        
        if (shareoptions)
        {
            var fields=shareoptions;
            var tag="editablefields:";
            if (fields.indexOf(tag)>-1)
            {
                fields=fields.substr(fields.indexOf(tag)+tag.length);
                fields=fields.substr(0,fields.indexOf("]"));
                if (fields)
                {
                    fields=","+fields+",";
                    for(var i=0;i<model.fields.length;i++)
                    {
                        if (fields.indexOf(","+model.fields[i].id+",")==-1)
                            model.fields[i].readonly=true;
                        else
                            model.fields[i].readonly=false;
                    }
                }
            }
        }
    }

    this.id=id;
    this.project_id=project_id;
    this.params=params;
    if (!this.params)
        this.params={};

    if (!this.params.modelid)
        this.params.modelid='oa_sheet';
    if (!this.params.pageposition)
        this.params.pageposition="leftdown";
    this.params.fieldAction='menu';
    this.params.usermodel_recordid=this.project_id;
    this.params.afterLoadRows=this.params.afterLoadRows;

    this.params.table_toolbar_id=this.params.table_toolbar_id;

    this.params.model=this.params.model;;
    this.params.afterSave=unisheet_afterSave;
    this.params.getAutoId=true;
    this.params.diffLineCss=false;
    this.params.model_ownerid=this.params.project_fulldata.project_record.SYS_ADDUSER;
    this.params.afterCreateHeader=unisheet_afterCreateHeader;
    this.params.beforeCreateTable=unisheet_beforeCreateTable;
    this.params.userAction=unisheet_recordAction;
    this.params.beforeBindMouseEventOnRows=unisheet_beforeBindMouseEventOnRows
    this.params.showRecordInsertAction=this.params.showRecordInsertAction;
    this.params.afterGetModel=this.UniSheet_afterGetModel;
    this.params.showRecordCount=true;
    this.params.showAddBtnAtBottom=true;
    this.params.afterRunTableEdit=unitable_afterRunTableEdit;
    this.params.defaultOrderby="sort";
    if (this.params.showRecordMenu==undefined)
        this.params.showRecordMenu=false;
    if (this.params.showTableSetting==undefined)
        this.params.showTableSetting=true;
    if (this.params.showRowActionMenu==undefined)
        this.params.showRowActionMenu=true;
    if (this.params.showLineNum==undefined)
        this.params.showLineNum=true;
    // if (this.params.showAlpha==undefined)
    // this.params.showAlpha=true;
    this.params.showAlpha=true;
    if (this.params.showAdvancedSetting==undefined)
        this.params.showAdvancedSetting=true;
//	this.params.showShareTools=true;

    this.mcsstable=new MCSSTable(this.id,this.params.modelid,this.params);
    this.mcsstable.unisheet=this;
    this.mcsstable.params.project_fulldata=this.params.project_fulldata;
   this.mcsstable.params.afterGetModel=this.UniSheet_afterGetModel;

    var _this=this;
    this.run=function()
    {
        this.mcsstable.run();
    }

    
}

function unisheet_afterLoadRows(mcsstable)
{
    var addBtn=mcsstable.getAction('add');
    if (mcsstable.view=='tableedit' && addBtn)
    {
        var n=1;
        if (mcsstable.table.rows.length<3 && mcsstable.unisheet.params.project_fulldata.project_record.status==null)
            n=3;
        for(var i=0;i<n;i++)
            $(addBtn).click();
        mcsstable.mcssData.state='';
    }
}

function unisheet_afterAppendRow(mcsstable,row)
{

}



function unisheet_afterSave(mcssData,mcsstable)
{
    mcsstable_setStyleAfterSave(mcsstable);
    if (mcsstable.params.project_fulldata.project_record.status=='')
    {
        var url=g_homeurl+"/Oa/Project/updataProjectStatus/";
        $.post(url,{projectid:mcsstable.params.project_fulldata.project_record.id,status:"doing"},function(data){
            mcsstable.params.project_fulldata.project_record.status='doing';
        });
    }

    //saveProjectPreview(mcsstable);
}

function unisheet_afterCreateHeader(mcsstable)
{
    $(mcsstable.table).css({"table-layout":"fixed"});//如果不指定最小宽度，则有很多列时，后面的列会非常拥挤在一起

    $(mcsstable.table.rows[0].cells).mouseover(function(){
        if ($(".adjustcolumnsize").size()==1)
        {
            tableset_setColumnDisplay($(this).attr('fieldid'),mcsstable.tableid,this);
        }
    })

    var projectdata=mcsstable.params.project_fulldata;
    if ((projectdata.project_record.SYS_ADDUSER==loginuser || projectdata.sharerecord  && projectdata.sharerecord.options && projectdata.sharerecord.options.indexOf('[canedit]')>-1))
    {
    }
    else
    {
        $(mcsstable.showAction("batchedit")).remove();
        mcsstable.removeActions("batchedit,styleedit");
    }
    if (projectdata.project_record.SYS_ADDUSER!=loginuser)
        mcsstable.removeActions("share,publish");

}


function unisheet_beforeCreateTable(mcsstable)
{
    mcsstable.unisheet.setTableWidth();
}

//根据字段数量和日期格子数量调整表格宽度
UNISheet.prototype.setTableWidth=function()
{
}

//没完成的功能
function unisheet_recordAction(html,recordid,record,mcsstable)
{
    return html;
    var show="none";
    var discus_css="gantetu_addDiscussion";
    var discus_css2="icon-bubble";
    if (mcsstable.params.project_fulldata.tasks_withdiscussion)
    {
        var arr = mcsstable.params.project_fulldata.tasks_withdiscussion;
        for(var i = 0;i < arr.length;i++){
            if(arr[i]==recordid){
                show="";
                discus_css="gantetu_addDiscussion_withdata";
                discus_css2="icon-bubble2";
                break;
            }else{
                continue;
            }
        }

    }
    var r="";
    if (record)
        r+="<a id='discussion_"+recordid+"' class='"+discus_css+" "+discus_css2+"'  onclick='project_addDiscussion(this,\""+recordid+"\",\""+mcsstable.tableid+"\");' onmouseover='unisheet_setDiscussTitle(this,\""+recordid+"\")' title='"+lang.comment+"' style='display123:"+show+";' >"
                //"<span class='"+discus_css2+"'></span>
            +"</a>";
    r=r+html;
    return r;
}

//移动到讨论上面浮现出内容
function unisheet_setDiscussTitle(obj,id){
    if($(obj).hasClass('gantetu_addDiscussion_withdata') && !$(obj).attr("hasloaddiscussion")){
        var url = g_homeurl+"/Oa/Project/getLastDiscuss";
        $.post(url,{id:id},function(data){
            var content = clearHTML(data).substr(0,200);
            $(obj).attr("title",content).attr("hasloaddiscussion",true);
        })
    }
}

function unitable_afterRunTableEdit(mcsstable)
{
    //if (common_getParam("tag")=='new' && $(mcsstable.getAction('add')))
    //	$(mcsstable.getAction('add')).click();
}
//添加评论（旧版本）
function project_addDiscussion2(e,recordid,tableid)
{
    var mctable =mcsstable_getMCSSTable(tableid);
    var record=mctable.mcssData.getRow(recordid);
    var s=""
    s+="<div id='"+recordid+"_commentlist' style='table-layout:fixed;word-wrap: break-word; word-break: break-all;'></div>";
    var w="550px";
    if (uni_global.isMobile())
        w=document.body.clientWidth;
    var pop=mcdom.popup(s,{obj:e,width:w,height:"auto",title:"评论",overflow:"auto"});
    var topicdealing={'content':showRecordDiscussionContent};
    var params={filter:"task_id='"+recordid+"'",defaultValue:"task_id:"+recordid+",project_id:"+mctable.params.project_fulldata.project_record.id
        ,pageposition:'rightdown',afterLoadRows:commentList_afterCreateHeader,userAction:discussion_userAction,
        special_field_show:topicdealing,
        showRecordMenu:true,refObject:mctable};
    newMCSSTable(recordid+"_commentlist",'oa_pm_taskcomment',params).run();
}
//显示讨论内容
function showRecordDiscussionContent(value,record)
{
    var content="";
    if (record.attach)
    {
        var imagepath =g_rooturl+"/Public/uploadfile/";
        if(uni_global.issaas && g_rooturl.indexOf("localhost")<0 )
            imagepath = uni_global.filepath;
        var filename=record.attach;
        var savename=record.attach;
        var arr=record.attach.split('~');
        if (arr.length>1)
        {
            filename=arr[0];
            savename=arr[1];
            var imgSrcLeft = savename.substring(0,savename.length-4);
            var imgSrcRight = savename.substring(savename.length-4);
            if(imgSrcLeft.substring(imgSrcLeft.length-7)=="[small]")
                imgSrcLeft = imgSrcLeft.substring(0,imgSrcLeft.length-7);
            savename = imgSrcLeft+imgSrcRight;
        }
        content+=lang.file+":<a target='_blank' href='"+imagepath+savename+"' >"+filename+"</a><br /><br />";
    }
    content+=(value+"");
    return content;
}
//讨论的自定义按钮
function discussion_userAction(html,recordid,record,mcsstable)
{
    var h1="";
    if (loginuser && (loginuser==record["SYS_ADDUSER"] || loginuser==mcsstable.params.refObject.params.project_fulldata.project_record.SYS_ADDUSER))
    {
        h1+='<a class="smallbut mcsstable_record_share" title="'+lang.share+'" tag="recordAction" onclick="shareComment('+recordid+')" ><span class="icon-share"></span></a>';
        h1+="<a class='mcsstable_record_edit icon-pencil'  tag='recordAction' onclick='mcsstable_openAutoform(this,\""+mcsstable.tableid+"\",\""+recordid+"\",\"编辑\",\"EDIT\")' ></a>";
        h1+="<a class='mcsstable_record_del icon-remove' tag='recordAction' onclick='mcsstable_deleteOneRow(this,\""+mcsstable.tableid+"\")' title='删除当前记录'></a>";
    }

    var h = h1;
    //	h= html+h1;
    if (record.SYS_ADDTIME)
        h+="<br />"+record.SYS_ADDUSER+"<br /><span title='added    on "+record.SYS_ADDTIME+"\r\nupdated on "+record.SYS_EDITTIME+"'>"+record.SYS_ADDTIME.substr(0,10)+"</span>";
    return h;
}

//分享评论
function shareComment(id,modelid)
{
    var url=g_homeurl+"/Project/viewContent/sharetype/RICHTEXT/id/"+id;
    common_publishPage(url);
}

function commentList_afterCreateHeader(mcsstable)
{
    $(mcsstable.getAction("del")).hide();
    mcsstable.showAction('firstpage,lastpage,pagerows,RecordCount',false);
    $(mcsstable.table).find("col").eq(0).css("width","30px");
    if (loginuser==mcsstable.params.refObject.params.project_fulldata.project_record.SYS_ADDUSER
        || (mcsstable.params.refObject.params.project_fulldata.shareoption && mcsstable.params.refObject.params.project_fulldata.shareoption.indexOf('[canedit]')>-1)
        || mcsstable.params.refObject.gantetutype=="SYSTEM_ALL_MYPROJECTS")
    {
        if (mcsstable.mcssData.initData.length==0)
            $(mcsstable.getAction("add")).click();
        else
            $(mcsstable.table).find(".mcsstable-action-col").css("vertical-align","top");
    }
    else
    {
        $(mcsstable.table).find("a[tag='recordAction']").remove();
    }

    var pop=mcdom.getObjByClass(mcsstable.table,"fuchuceng ");
    if (pop.offsetTop+pop.offsetHeight>document.documentElement.clientHeight)
        $(pop).css({top:pop.offsetTop-pop.offsetHeight-mcsstable.table.offsetHeight});
    if (pop.offsetLeft+pop.offsetWidth>document.documentElement.clientWidth)
        $(pop).css({left:pop.offsetLeft-pop.offsetWidth});

}

//return true表示调用者可以继续后续的程序
function unisheet_beforeBindMouseEventOnRows(mcsstable,rows)
{
    $(rows).mouseover(function(){
        //$(this).find(".gantetu_addDiscussion").show();
    }).mouseout(function(){
        //$(this).find(".gantetu_addDiscussion").hide();
    })
    return true;
}
//为某个元素设置动画效果
function setAction(obj,s)
{
    if (s!=0){
        obj.style["-webkit-transition"] ="all "+ s +"s ease-out";
        obj.style["-moz-transition"] ="all "+ s +"s ease-out";
        obj.style["-ms-transition"] ="all "+ s +"s ease-out";
        obj.style["-o-transition"] ="all "+ s +"s ease-out";
        obj.style["transition"] ="all "+ s +"s ease-out 0s";
    }else{
        obj.style["-webkit-transition"] ="";
        obj.style["-moz-transition"] ="";
        obj.style["-ms-transition"] ="";
        obj.style["-o-transition"] ="";
        obj.style["transition"] ="";
    }
}
//创建选择框
function getDivSel(mcsstable,bNew){

    if (!bNew && mcsstable.divSel) {
        return mcsstable.divSel;
    }
    var divSel = document.createElement('div');

    divSel.setAttribute("class","tableditor_cover");

    divSel.style.border=mcsstable.selected_border_css;
    divSel.style.position="absolute";
    divSel.style.left="0px";
    divSel.style.top="0px";
    divSel.style.width="1px";
    divSel.style.height="1px";
    if (!mcsstable.divSel ){
        mcsstable.divSel =divSel;
    }
    if (mcsstable.view!="celledit") divSel.style.opacity=0;
    //让divSel不处理鼠标事件
    divSel.style.pointerEvents="none";
    mcsstable.divSelParent.appendChild(divSel);

    divSel.divEdit = document.createElement('div');
    divSel.divEdit.setAttribute("contentEditable",'true');
    divSel.divEdit.setAttribute("readonly",'false');
    divSel.divEdit.style.marginTop = "0px";
    divSel.divEdit.style.width="1px";
    divSel.divEdit.style.height="1px";
    divSel.divEdit.style.overflow="hidden";
    divSel.divEdit.style.border="1 solid #fff";
    divSel.appendChild(divSel.divEdit);

    if (!bNew){
        divSel.divFill = document.createElement('div');

        divSel.divFill.style.width="6px";
        divSel.divFill.style.height="6px";
        divSel.divFill.style.border="5 solid #44f";
        divSel.divFill.style.backgroundColor="#44f";
        divSel.divFill.style.position="absolute";
        divSel.divFill.style.pointerEvents="auto";
        divSel.divFill.style.zIndex=1;
        divSel.divFill.style.cursor="crosshair";
        divSel.divFill.title="向下拖拽，自动填充。";
        divSel.divFill.onmousedown=function(){
            divSel.fillModel=true;
            var ev=getEvent();
            divSel.downX=ev.clientX;
            divSel.downY=ev.clientY;
            //setAction(divSel,0);
            document.body.setAttribute("onselectstart","return false;");

            document.body.onmousemove=function(){

                if (divSel.fillModel){

                    var ev=getEvent();

                    var x=ev.clientX;
                    var y=ev.clientY;

                    //divSel.style.width=parseInt(divSel.style.width) + x-divSel.downX + "px";
                    divSel.style.height=parseInt(divSel.style.height) + y-divSel.downY + "px";
                    divSel.divFill.style.top=parseInt(divSel.divFill.style.top) +  y-divSel.downY + "px";
                    divSel.downX=x;
                    divSel.downY=y;
                    //divSel.divFill.left=

                }
            }
            document.body.onmouseup=function(){

                divSel.fillModel=false;
                document.body.setAttribute("onselectstart","");
                document.body.onmousemove=null;
                //setAction(divSel,0.2);
                document.body.onmouseup=null;
                var fromArea=mcsstable.currentArea.area;

                var toArea={beginRowIndex:fromArea.endRowIndex+1,beginCellIndex:fromArea.beginCellIndex,endRowIndex:fromArea.endRowIndex,endCellIndex:fromArea.endCellIndex};
                var r1=fromArea.beginRowIndex;
                var top1=mcsstable.tableData.rows[r1].offsetTop;
                var hei=0;
                for(var r=r1;r<mcsstable.tableData.rows.length-1;r++)
                {
                    hei = mcsstable.tableData.rows[r].offsetTop - top1;
                    if (hei<divSel.offsetHeight && hei+ mcsstable.tableData.rows[r].offsetHeight > divSel.offsetHeight )
                    {
                        divSel.style.height = hei+"px";
                        divSel.divFill.style.top = hei-6+"px";
                        toArea.endRowIndex=r-1;
                        break;
                    }
                }
                mcsstable_autoFillData(mcsstable,fromArea,toArea);
            }


        }

        divSel.style.overflow="visible";
        divSel.appendChild(divSel.divFill);

    }

    return divSel;

}
function getNextValue(v1,v2)
{
    function isDate(date){
        date=date.replace("年","-");
        date=date.replace("月","-");
        date=date.replace("日","");
        return (new Date(date).getDate()==date.substring(date.length-2));
    }
    Date.prototype.Format = function (fmt) { //author: meizz
        var o = {
            "M+": this.getMonth() + 1, //月份
            "d+": this.getDate(), //日
            "h+": this.getHours(), //小时
            "m+": this.getMinutes(), //分
            "s+": this.getSeconds(), //秒
            "q+": Math.floor((this.getMonth() + 3) / 3), //季度
            "S": this.getMilliseconds() //毫秒
        };
        if (/(y+)/.test(fmt)) fmt = fmt.replace(RegExp.$1, (this.getFullYear() + "").substr(4 - RegExp.$1.length));
        for (var k in o)
            if (new RegExp("(" + k + ")").test(fmt)) fmt = fmt.replace(RegExp.$1, (RegExp.$1.length == 1) ? (o[k]) : (("00" + o[k]).substr(("" + o[k]).length)));
        return fmt;
    }
    function getNextStr(s1,s2)
    {

        var strlist=[];
        strlist.push("abcdefghijklmnopqrstuvwxyz");
        strlist.push("ABCDEFGHIJKLMNOPQRSTUVWXYZ");
        strlist.push("一二三四五六七八九十");
        strlist.push("甲乙丙丁戊己庚辛壬癸");
        strlist.push("子丑寅某辰巳午未申酉戌亥");
        strlist.push("１２３４５６７８９");
        for(var r=0;r<strlist.length;r++)
        {
            var str=strlist[r];
            var i1=str.indexOf(s1);
            var i2=str.indexOf(s2);
            if (i1>=0 && i2>0){
                var i3=i2*2-i1;
                if (i3<str.length)
                    return str.substring(i3,i3+1);
                else
                    return str.substring(0,1);
            }
        }

        var e1=s1.substring(s1.length-1,s1.length);
        var e2=s2.substring(s2.length-1,s2.length);
        var strt=s1.substring(0,s1.length-1);
        for(var r=0;r<strlist.length;r++)
        {
            var str=strlist[r];
            var i1=str.indexOf(e1);
            var i2=str.indexOf(e2);
            if (i1>=0 && i2>0){
                var i3=i2*2-i1;
                if (i3<str.length)
                    return strt + str.substring(i3,i3+1);
                else
                    return strt + str.substring(0,1);
            }
        }

        var b1=s1.substring(0,1);
        var b2=s2.substring(0,1);
        var stre=s1.substring(1,s1.length);
        for(var r=0;r<strlist.length;r++)
        {
            var str=strlist[r];
            var i1=str.indexOf(b1);
            var i2=str.indexOf(b2);
            if (i1>=0 && i2>0){
                var i3=i2*2-i1;
                if (i3<str.length)
                    return  str.substring(i3,i3+1) + stre;
                else
                    return  str.substring(0,1) + stre;
            }
        }

        return s2;
    }

    if (!isNaN(v1) && !isNaN(v2)){
        return v2*2-v1;
    }else if(isDate(v1) && isDate(v2))
    {
        var fmt="yyyy-MM-dd";

        if (v1.indexOf("年")>0 && v1.indexOf("月")>0 && v1.indexOf("日")>0) fmt="yyyy年MM月dd日";
        if (v1.indexOf("年")>0 && v1.indexOf("月")>0 && v1.indexOf("日")<=0) fmt="yyyy年MM月";
        if (v1.indexOf("年")<=0 && v1.indexOf("月")>0 && v1.indexOf("日")>0) fmt="MM月dd日";

        v1=v1.replace("年","-");
        v1=v1.replace("月","-");
        v1=v1.replace("日","");
        v2=v2.replace("年","-");
        v2=v2.replace("月","-");
        v2=v2.replace("日","");

        var v2=new Date(v2);
        var v1=new Date(v1);
        var iDays = parseInt(Math.abs(v2 - v1) / 1000 / 60 / 60 / 24);
        var v3 = new Date(v2 - 1 + 1 + iDays*24*60*60*1000)

        return v3.Format(fmt);
    }
    else if( !isNaN(parseFloat(v1)) && !isNaN(parseFloat(v2))  )
    {
        var str=v2.replace(parseFloat(v2)+"","");

        var v1=parseFloat(v1);
        var v2=parseFloat(v2);

        return (v2*2-v1) + str;

    }
    else{

        var v3 =  getNextStr(v1,v2);
        return v3;
        //return v2;
    }
}
function mcsstable_autoFillData(mcsstable,fromArea,toArea,isundo)
{
    if (!isundo){
        //填充前把旧数据备份下来，以便还原的时候使用
        mcsstable.fillOldRows=[];
        mcsstable.fillFromRow=toArea.beginRowIndex;
        mcsstable.fillFromCell=fromArea.beginCellIndex;
        for(var r=toArea.beginRowIndex;r<=toArea.endRowIndex;r++)
        {
            var oldRow={};
            oldRow.type="UPDATE";
            mcsstable.fillOldRows.push(oldRow);
            oldRow.cells=[];
            for(var c=fromArea.beginCellIndex;c<=fromArea.endCellIndex;c++)
            {
                var cell={};
                cell.style="";
                cell.value="";
                oldRow.cells.push(cell);
            }
        }
    }
    for(var c=fromArea.beginCellIndex;c<=fromArea.endCellIndex;c++)
    {
        var tableCur=null;
        var td=null;
        var cindex=c;
        if (c<mcsstable.tableFixCol.rows[0].cells.length){
            td=mcsstable.tableFixCol.rows[fromArea.beginRowIndex].cells[c];
            tableCur=mcsstable.tableFixCol;
        }
        else{
            cindex=c-mcsstable.tableFixCol.rows[0].cells.length;
            td=mcsstable.tableData.rows[fromArea.beginRowIndex].cells[cindex];
            tableCur=mcsstable.tableData;
        }
        var fieldid = $(td).attr("id");
        var recordid=mcsstable.tableFixCol.rows[fromArea.endRowIndex].id;
        var field=mcsstable.modelDealer.getField(fieldid);
        var v2 = mcsstable.mcssData.getFieldValue(recordid,fieldid);
        var autoAdd=false;
        if (fromArea.beginRowIndex<fromArea.endRowIndex) {
            var v1=mcsstable.mcssData.getFieldValue(mcsstable.tableFixCol.rows[fromArea.endRowIndex-1].id,fieldid);
            autoAdd=true;
        }else{
            v1=v2
        }

        for(var r=toArea.beginRowIndex;r<=toArea.endRowIndex;r++)
        {
            var tr=tableCur.rows[r];
            var recordid=tr.getAttribute('recordid');

            if (!recordid){
                recordid = getTimeId();
                mcsstable.fillOldRows[r-toArea.beginRowIndex].type="DELETE";

                var rowData = mcsstable_createNewData(mcsstable, recordid);

                mcsstable_onNewRow(tr,rowData,recordid,fieldid);

                if (mcsstable.modeldata.sortfieldid) {
                    var sortfield = mcsstable.modeldata.sortfieldid;
                    mcsstable.params.maxSort = parseInt(mcsstable.params.maxSort) + 100;
                    mcsstable.mcssData.setFieldValue(recordid,sortfield,mcsstable.params.maxSort);
                    rowData[sortfield] = mcsstable.params.maxSort;
                }

            }
            var value=v2;
            var oldValue = mcsstable.mcssData.getFieldValue(recordid,fieldid);
            if (!isundo)
            {
                mcsstable.fillOldRows[r-toArea.beginRowIndex].id=recordid;
                mcsstable.fillOldRows[r-toArea.beginRowIndex].cells[c-fromArea.beginCellIndex].value = oldValue;
            }
            if (autoAdd) value = getNextValue(v1,v2);

            if (value ==null || value ==undefined) value="";
            v1=v2;
            v2=value;


            mcsstable.editField(recordid,fieldid,value,true);
            if (tableCur.rows[r]){
                if (tableCur.rows[r].cells[cindex])
                    tableCur.rows[r].cells[cindex].innerHTML="<span>"+ value +"</span>";
            }

        }
    }

    if (!isundo) mcsstable.addUndoList(null,mcsstable_savePasteData,[mcsstable,mcsstable.fillOldRows,mcsstable.fillFromRow,mcsstable.fillFromCell,0,true],null,mcsstable_autoFillData,[mcsstable,fromArea,toArea,true]);

    mcsstable.mcssData.save(null,{srcElement:mcsstable.divSel.td,refObject:mcsstable});

}
function clearSelStyle(mcsstable){
    for(var i=0;i<mcsstable.tableFixRow.rows[0].cells.length;i++){
        var th=mcsstable.tableFixRow.rows[0].cells[i];
        th.style.backgroundColor="";
    }
    for(var r=0;r<mcsstable.tableFixCol.rows.length;r++)
    {
        var td=mcsstable.tableFixCol.rows[r].cells[0];
        td.style.backgroundColor="";
    }
}
//add by sunliangchao 增加浮动的选框
function setFocusToTd(td,focus)
{
  
    var event = window.event || arguments.callee.caller.arguments[0];
    var tdSel = "";
    if (!td)
        tdSel = event.srcElement?event.srcElement:event.target;
    else
        tdSel = td;

    if (tdSel.tagName!="TD" && tdSel.tagName!="TH") tdSel=tdSel.parentNode;
    if (tdSel.tagName!="TD" && tdSel.tagName!="TH") tdSel=tdSel.parentNode;
    if (tdSel.tagName!="TD" && tdSel.tagName!="TH") tdSel=tdSel.parentNode;

    var table=mcdom.getObjByTagName(tdSel,"TABLE");
    var mcsstable=mcsstable_getMCSSTable(table.id);
    //这句话会导致在只读模式下无法向外选择和复制数据
    //if (mcsstable.view!="celledit" && mcsstable.view!="tableedit")	return;

    var div=table.parentNode;
 
    if (!mcsstable.divSel) {
        mcsstable.divSel = getDivSel(mcsstable);
    }else{
        var fromTd=mcsstable.divSel.td;
    }
    mcsstable.divSel.style.opacity=1;
    mcsstable.divSel.style.background="none";
    if (mcsstable.divSel.divFill)
        mcsstable.divSel.divFill.style.display="none";
    
    clearSelStyle(mcsstable);
    
    mcsstable.divSel.td=tdSel;

    if (!tdSel) return;
    if (!tdSel.parentNode) return;

    if (mcsstable.divSel.style.display!="")	mcsstable.divSel.style.display="";

    var top = tdSel.offsetTop+ table.offsetTop + table.parentNode.parentNode.parentNode.offsetTop -1 ;
    var left = tdSel.offsetLeft+ table.offsetLeft + table.parentNode.parentNode.parentNode.offsetLeft -1 ;

    if (table==mcsstable.tableFixCol)
        left += mcsstable.tableData.parentNode.parentNode.scrollLeft;

    mcsstable.divSel.style.top = top + "px";
    mcsstable.divSel.style.left = left + "px";
    mcsstable.divSel.style.width=tdSel.clientWidth+3+"px";
    mcsstable.divSel.style.height=tdSel.clientHeight+3+"px";

    //window.setTimeout(function(){},100);
}
//add by sunliangchao 统一处理键盘按下的事件
function mcsstable_OnKeyDown(event)
{
 
    var obj = event.target;
    var td= mcdom.getObjByTagName(obj,'TD');
    if (!td) return;

    var tr=td.parentNode;
    var table=mcdom.getObjByTagName(obj,'TABLE');
    if (table.tagName!="TABLE") return;
    var mcsstable=mcsstable_getMCSSTable(table.id);
    var currentrowindex = $(obj).parent().parent().prop("rowIndex");
    var currentcellindex = $(obj).parent().prop("cellIndex");

    var maxrowindex = table.rows.length-1;
    var maxcellindex = $(obj).parent().parent().children("td").length;
    var inputobj;
    var currentRow;
    var inputerTagName='.tableditor_cover';//可用于输入的dom类别
    var cursorPos=getCursorPosition(obj);
    var totd=null;


    if(event.keyCode==37 || (event.shiftKey && event.keyCode==9))//左
    {
        if (obj.value=='' || cursorPos<=0 || event.keyCode==9)
        {
            
            var ttd=td.previousSibling;
            if (ttd && ttd.getAttribute("tag")=="cell") {
                totd=ttd;
            }
            else if (table==mcsstable.tableData) {
                
                var ttr=mcsstable.tableFixCol.rows[td.parentNode.rowIndex];
                var ttd = ttr.cells[ttr.cells.length-1];
                if (ttd.getAttribute("tag")=="cell") totd=ttd;
            }
            
            if (totd) currentRow=$(totd.parentNode);
            
        }
        
    }else if(event.keyCode==38)//上
    {
        if (table.divAuto && table.divAuto.style.display!="none") return;
        if(currentrowindex!=0)
        {
            currentRow=$(obj).parent().parent().prev();
            if(currentRow.attr("groupname"))//若果是分组行，则越过改行
                currentRow=currentRow.prev();
            if (td.parentNode.rowIndex>0)
                var tr=td.parentNode.previousSibling;
            else
                return;
            if (tr) totd=tr.cells[currentcellindex];
        }
    }else if(event.keyCode==39 || event.keyCode==9  ) //右
    {

        if (event.keyCode==13 && obj.tagName=="DIV")
        {
            if ($(obj).attr("fieldid")) return;
        }
        if ((cursorPos!=undefined && obj.tagName=='INPUT' && cursorPos==obj.value.length) || cursorPos==-1 || event.keyCode==9 )
        {
            totd=td.nextElementSibling;
            if (!totd && table==mcsstable.tableFixCol){
                totd = mcsstable.tableData.rows[td.parentNode.rowIndex].cells[0];
            }
            currentRow=$(td.parentNode);
            if (totd){
                while(!mcsstable.cellEditor.canEditCell(totd))
                {
                    totd=totd.nextElementSibling;
                    if (!totd) break;
                }
            }
            if (!totd &&  event.keyCode==13)
            {
                currentRow=$(td.parentNode.nextElementSibling);
                if (mcsstable.modeldata.fixFields.length>0)
                    totd=mcsstable.tableFixCol.rows[ td.parentNode.nextElementSibling.rowIndex].cells[2];
                else
                    totd=td.parentNode.nextElementSibling.cells[0];
            }               
        }
        if (event.keyCode==9) event.returnValue = false;


    }else if(event.keyCode==40 || event.keyCode==13 )//下
    {

        if (table.divAuto && table.divAuto.style.display!="none") return;
        if(currentrowindex < maxrowindex-1)
        {
            currentRow=$(obj).parent().parent().next();
            if(currentRow.attr("groupname"))
                currentRow=currentRow.next();
            var tr=td.parentNode.nextElementSibling;
            if (tr) totd=tr.cells[currentcellindex];
        }
        else
        {
            //默认在下方新增行
            if (mcsstable.params.addNewRowAfterInputEnterKey!=false){
                mcsstable_addNewEditRow(mcsstable);
            }
            if(table.rows[currentrowindex+1])
                inputobj=$(table.rows[currentrowindex+1].cells[currentcellindex]).find(inputerTagName).eq(0);
            currentRow=$(table.rows[currentrowindex+1]);
        }
    }
    //if (currentRow)
       // mcsstable.setCurrentRow(currentRow.get(0));

    if (totd) {
        //setFocusToTd(totd,true);
        mcsstable_edittd(mcsstable,totd);
        var area={beginRowIndex:totd.parentNode.rowIndex,beginCellIndex:mcsstable.getCellIndex(totd),endRowIndex:totd.parentNode.rowIndex,endCellIndex:mcsstable.getCellIndex(totd)};
        mcsstable.currentArea=area;
        mcsstable.currentArea.area=area;
        //mcsstable_SelTableData(mcsstable);
    }

}
//add by sunliangchao 统一处理鼠标落下的事件
function mcsstable_OnMouseDown(event)
{

    var ev = getEvent();
    window.isMouseDown=true;

    if (ev.changedTouches)
    {
        var pos = ev.changedTouches[0];
        window.mousedownX=pos.pageX;
        window.mousedownY=pos.pageY;
    }else{
        window.mousedownX=ev.clientX;
        window.mousedownY=ev.clientY;
    }
    var obj = ev.srcElement ? ev.srcElement : ev.target;
    if (obj.type=="checkbox") return;
    while(obj.tagName!="TD")
    {
        obj=obj.parentNode;
        if (obj==document.body) return;
        if (obj==null) return;
    }
    var td=obj;
    window.mousedownTd=td;
    var tr=td.parentNode;
    var table=tr.parentNode;
    if (table.tagName!="TABLE") table=table.parentNode;
    return;

}
//add by sunliangchao 统一处理鼠标抬起事件
function mcsstable_OnMouseUp(event)
{
    var ev = getEvent();
    window.isMouseDown=false;

    if (ev.changedTouches)
    {
        var pos = ev.changedTouches[0];
        window.mouseupX=pos.pageX;
        window.mouseupY=pos.pageY;
    }else{
        window.mouseupX=ev.clientX;
        window.mouseupY=ev.clientY;
    }

    var obj = ev.srcElement ? ev.srcElement : ev.target;
    if (obj.type=="checkbox" || obj.getAttribute('inEdit') =='true' ){
        obj.click();
        return;
    }
    while(obj.tagName!="TD")
    {
        obj=obj.parentNode;
        if (obj==document.body) return;
        if (obj==null) return;
    }
    var td=obj;

    var tr=td.parentNode;
    var recordid;
    recordid=tr.getAttribute("recordid");
    var table=tr.parentNode.parentNode;
    var mcsstable=mcsstable_getMCSSTable(table.id);

    var inDataTable=false;
    if (table==mcsstable.tableData || (table==mcsstable.tableFixCol && td.celIndex>1) ) inDataTable=true;

    $("body").removeAttr("onselectstart");
    if (table==mcsstable.tableFixRow) return;
    mcsstable_setmouseup(td);

    if (td.inEdit) return true;

    if (td.className=="linenumber" || td.className=="linenumber_hover" ){
        mcsstable_linenum_onclick(td,event);
        return;
    }
    //忽略填充事件和数据错误事件
    if (mcsstable.divSel)
    {
        if (mcsstable.divSel.fillModel) return;
        //if (mcsstable.divSel.td && mcsstable.divSel.td.isDataError) return;
    }
    //进入单元格编辑状态
    //if (mcsstable.currentArea==null || mcsstable.currentArea.beginRowIndex==mcsstable.currentArea.endRowIndex && mcsstable.currentArea.beginCellIndex==mcsstable.currentArea.endCellIndex){
    mcsstable_edittd(mcsstable,td);

    //}

}
function mcsstable_edittd(mcsstable,td){
    //setFocusToTd(td);
    //不加上这一句，多行文本框会自动变短
    //mcsstable.tableData.rows[td.parentNode.rowIndex].style.height=mcsstable.tableData.rows[td.parentNode.rowIndex].offsetHeight+"px";
    //以下这些语句的目的是为了把当前单元格快速移动到屏幕左上角的位置
    //否则由于输入焦点受到软键盘弹出的影响，整页就会向上滚动。
    
    var dv= mcsstable.tableData.parentNode.parentNode;

    var lf=td.offsetLeft-dv.scrollLeft-50;
    var tp=td.offsetTop+mcsstable.tableData.offsetTop-dv.scrollTop-2;     
    if (lf>0 || tp>0){
         tp=dv.scrollTop+tp;
         lf=dv.scrollLeft+lf;
         $(dv).animate({scrollTop: tp+'px',scrollLeft: lf+'px'}, 'fast','',function(){});
    }

    setFocusToTd(td);
    
    window.setTimeout(function(){
        var tr=td.parentNode;
        var recordid;
        recordid=tr.getAttribute("recordid");
        if (recordid){
            if (mcsstable.cellEditor && mcsstable.cellEditor.canEditCell(td)){
                if (!td.inEdit) {
                    mcsstable.cellEditor.clickCell(td);
                }
            }
        }else if (!tr.getAttribute("groupname")) {
            mcsstable_emptyRowClick(td,event);
        }
        var td0=mcsstable.tableFixCol.rows[td.parentNode.rowIndex].cells[0];
        var th0=mcsstable.tableFixRow.rows[0].cells[td.cellIndex];
        td0.style.backgroundColor="rgb(186, 225, 245)";
        th0.style.backgroundColor="rgb(186, 225, 245)";
    },200);

}
//add by sunliangchao 在表格选择区域后，把区域内的数据复制到divEdit中而后全选他们，这是为Ctrl+C做准备。
function mcsstable_SelTableData(mcsstable){

    if (!mcsstable.divSel) return;
    var str="";
    var divEdit = mcsstable.divSel.divEdit;
    var tableData=mcsstable.tableData;
    if (mcsstable.currentArea)
    {
        var area = mcsstable.currentArea.area;
        for (var r = area.beginRowIndex;r<=area.endRowIndex;r++)
        {
            if (r<tableData.rows.length){
                str+="<tr style='height:1px'>";
                for(var c=area.beginCellIndex;c<=area.endCellIndex;c++)
                {
                    var td=mcsstable.getCell(r,c);
                    if (td){
                        var val=td.innerText;
                        if (val==undefined){
                            if (td.textContent) val=td.textContent;
                        }
                        var input = $(td).find('input,select[fieldid]');
                        if (input && input.get(0)){
                            val = input.get(0).value;
                        }
                        if (val+""=="") val="";
                        str+="<td style='"+ td.style.cssText +"'>"+val+"</td>";
                    }
                }
                str+="</tr>";
            }
        }
    }

    //divEdit.focus();

    window.setTimeout(function (){
        divEdit.innerHTML = "<table>" + str + "</table>";
        if (document.selection) {
            var range = document.body.createTextRange();
            range.moveToElementText(divEdit.firstChild);
            range.select();
        } else if (window.getSelection) {
            var range = document.createRange();
            range.selectNode(divEdit.firstChild);
            window.getSelection().removeAllRanges();
            window.getSelection().addRange(range);
        }
    },10);

}

//以上是UniSheet类
function mcsstable_clearFloatToolBar(){
    $('.mcsstable_tr_float_toolbar_table').remove();
}

//这个方法过度，十天后删除
function clearFloatToolBar(){
    $('.mcsstable_tr_float_toolbar_table').remove();
}

//浮动工具条：列标题 add by gang
function mcsstable_setColFloatToolbar(table,cellIndex){
    mcsstable_clearFloatToolBar();
    var menu_show = false;
    var tableid=table.id;
    var current_col = $(table).find("th").eq(cellIndex).get(0);
    var fieldid=$(current_col).attr('fieldid');
    var margin_left = mcsstable_getColFloatTools_left(current_col,table);
    var mcsstable = mcsstable_getMCSSTable(tableid);
    var table_order = mcsstable.order;
    var table_group_field =mcsstable.groupFieldid;
    var table_group_type ='';
    var table_order_by='';
    var group_tag = false;
    if(table_group_field && table_group_field==fieldid){
        table_group_type =mcsstable.group_order_updown;
        group_tag = true;
    }
    if(table_order){
        var order_arr = table_order.split(' ');
        if(order_arr[0] && order_arr[0]==fieldid)
            table_order_by = order_arr[1];
    }
    if(mcsstable.params.project_fulldata && loginuser==mcsstable.params.project_fulldata.project_record.SYS_ADDUSER && $(mcsstable.getAction("view")).val()=="DEFAULT")
        menu_show = true;
    var h="<table id='mcsstable_tr_float_toolbar_' class='mcsstable_tr_float_toolbar_table' >";

    if(menu_show)
        h+="<tr><td  style='border-bottom:none;'  class='mcsstable_tr_float_toolbar_tr icon-null' onclick='talbeset_setColumnProperty(this,\""+fieldid+"\",\""+tableid+"\",\"update\")' onmouseover='fieldActionMenu_onmouseover(this)'>设置列属性</td></tr>";
    h+="<tr><td style='border-bottom:none;'  class='mcsstable_tr_float_toolbar_tr ";
    if(table_order_by=='asc' ||table_group_type=='asc')
        h+="icon-checkmark";
    else
        h+="icon-null";
    h+=" mcsstable_float_order_by' id='float_order_by_asc' orderby_tag='asc' onmouseover='fieldActionMenu_onmouseover(this)'>升序</td></tr>"
    h+="<tr><td style='border-top:none;border-bottom:none;'  class='mcsstable_tr_float_toolbar_tr ";
    if(table_order_by=='desc' ||table_group_type=='desc')
        h+="icon-checkmark";
    else
        h+="icon-null";
    h+=" mcsstable_float_order_by' orderby_tag='desc' onmouseover='fieldActionMenu_onmouseover(this)'>降序</td></tr>";
    h+="<tr><td style='border-top:none'  class='mcsstable_tr_float_toolbar_tr icon-null' onmouseover='tableset_showDataDiv(\""+tableid+"\",\""+fieldid+"\",this)'>快速过滤</td></tr>";
    if(menu_show){
        h+="<tr><td style='border-bottom:none;border-top:none'  class='mcsstable_tr_float_toolbar_tr icon-null' position='left' onclick='talbeset_setColumnProperty(this,\""+fieldid+"\",\""+tableid+"\",\"insert\")' onmouseover='fieldActionMenu_onmouseover(this)'>插入列</td></tr>"
            +"<tr><td  style='border-bottom:none;border-top:none' class='mcsstable_tr_float_toolbar_tr icon-null' onclick='tableset_hideField(\""+fieldid+"\",\""+tableid+"\")'  onmouseover='fieldActionMenu_onmouseover(this)'>隐藏列</td></tr>"
            +"<tr><td  style='border-top:none' class='mcsstable_tr_float_toolbar_tr icon-null' onclick='tableset_deleteColumn(\""+fieldid+"\",\""+tableid+"\")' onmouseover='fieldActionMenu_onmouseover(this)'>删除列</td></tr>"
    }

    if (table==mcsstable.tableFixRow)
        h+="<tr><td style='border-bottom:none;border-top:none'  class='mcsstable_tr_float_toolbar_tr icon-null' onclick='tableset_fixColumn(\""+fieldid+"\",\""+tableid+"\",true);' onmouseover='fieldActionMenu_onmouseover(this)'>冻结此列</td></tr>"
    else if(table==mcsstable.tableFixRowCol)
        h+="<tr><td  style='border-bottom:none;border-top:none' class='mcsstable_tr_float_toolbar_tr icon-null' onclick='tableset_fixColumn(\""+fieldid+"\",\""+tableid+"\",false);' onmouseover='fieldActionMenu_onmouseover(this)'>取消冻结</td></tr>"
    //+"<tr><td class='mcsstable_tr_float_toolbar_tr'>批量修改</td></tr>"
    h+="<tr><td  style='border-bottom:none;border-top:none' class='mcsstable_tr_float_toolbar_tr ";
    if(group_tag)
        h+="icon-checkmark";
    else
        h+="icon-null";
    h+="' id='set_float_group_by' onmouseover='fieldActionMenu_onmouseover(this)'>分组显示</td></tr>";

    //格式设置
    h+="<tr>"+mcsstable_createFormatTools(tableid)+"</tr>";
    h+="<tr><td style='border-top:none' class='mcsstable_tr_float_toolbar_tr icon-null' onclick='tableset_exportField(\""+fieldid+"\",\""+tableid+"\")' onmouseover='fieldActionMenu_onmouseover(this)'>导出列</td></tr>";
    // if(menu_show){
    // h+="<tr><td  style='border-bottom:none;border-top:none'  class='mcsstable_tr_float_toolbar_tr icon-null' onclick='talbeset_setColumnProperty(this,\""+fieldid+"\",\""+tableid+"\",\"update\")' onmouseover='fieldActionMenu_onmouseover(this)'>更改列类型</td></tr>";

    // h+="<tr><td style='border-top:none'  class='mcsstable_tr_float_toolbar_tr icon-null' onclick='tableset_allFields(this,\""+tableid+"\",\""+fieldid+"\")' onmouseover='fieldActionMenu_onmouseover(this)'>高级设置</td></tr>";
    // }
    //h+="<tr><td class='mcsstable_tr_float_toolbar_tr icon-null' onmouseover='tableset_showMoreMenu(\""+tableid+"\",\""+fieldid+"\",this)'>更多操作>></td></tr>";
    h+="</table>";
    $(table).before(h);
    $("#mcsstable_tr_float_toolbar_").css({'margin-left':(margin_left-25)+"px",'margin-top':"34px"});
    $('.mcsstable_tr_float_toolbar_tr').css('width','110px');
    $("#set_float_group_by").click(function(){
        if($(this).hasClass('icon-null')){
            $(this).removeClass('icon-null').addClass('icon-checkmark');
            $('.mcsstable_float_order_by').removeClass('icon-checkmark').addClass('icon-null');
            $('#float_order_by_asc').addClass('icon-checkmark');
            mcsstable_setFieldGroup(fieldid,tableid,'asc');
        }else{
            $(this).removeClass('icon-checkmark').addClass('icon-null');
            $('.mcsstable_float_order_by').removeClass('icon-checkmark').addClass('icon-null');
            mcsstable_cancelGroup(tableid,fieldid);
        }
        mcsstable_clearFloatToolBar();
    })
    $(".mcsstable_float_order_by").click(function(){
        var current_order = $(this).attr('orderby_tag');
        if($("#set_float_group_by").hasClass('icon-checkmark')){
            mcsstable_setFieldGroup(fieldid,tableid,current_order);
        }else{
            $('.mcsstable_float_order_by').removeClass('icon-checkmark').addClass('icon-null');
            $(this).addClass('icon-checkmark');
            tableset_setFieldOrder(fieldid,tableid,current_order)
        }
        mcsstable_clearFloatToolBar();
    })
}
//浮动工具条：单元格  add by gang
function mcsstable_setTdFloatToolbar(obj,table){
    mcsstable_clearFloatToolBar();
    var _td = $(obj).get(0);
    var tableid=table.id;
    var mcsstable = mcsstable_getMCSSTable(tableid);
    var recordid=$(mcdom.getObjByTagName(obj,"TR")).attr('recordid');
    var margin_top = mcsstable_getFloatTools_top(mcdom.getObjByTagName(obj,"TR"),table) +5 ;

    if (table==mcsstable.tableData)	 margin_top-=table.offsetTop;

    var margin_left = mcsstable_getFloatTools_left(_td,table,4) +5;
    var format_tool = mcsstable_createFormatTools(tableid,'horizontal');

    if (mcsstable.undoList.length<1) var undoStyle="mcsstable_tr_float_toolbar_td_disable"; else var undoStyle="mcsstable_tr_float_toolbar_td";
    if (mcsstable.redoList.length<1) var redoStyle="mcsstable_tr_float_toolbar_td_disable"; else var redoStyle="mcsstable_tr_float_toolbar_td";

    if (mcsstable.params.showBatchEditBtns)
        actions=mcsstable.params.showBatchEditBtns(mcsstable);
    else
        actions=mcsstable.params.toolbar + ",";

    var h="<table id='mcsstable_tr_float_toolbar_' class='mcsstable_tr_float_toolbar_table'><tr>";
    if (actions.indexOf("add")>-1)
        h+="<td class='mcsstable_tr_float_toolbar_td' id='"+table.id+"_td_float_toolbar_add' title='在这一行前面插入空行'>插入</td>";
    if (actions.indexOf("del")>-1 || actions.indexOf("edit")>-1)
    {
        h+="<td class='mcsstable_tr_float_toolbar_td' id='"+table.id+"_td_float_toolbar_del' title='删除这一行'>删除</td>";
    }
    if (actions.indexOf("edit")>-1){
        h+="<td class='mcsstable_tr_float_toolbar_td' id='"+table.id+"_td_float_toolbar_edit' title='用表单方式编辑这一行'>编辑</td>";
        h+="<td class='"+ undoStyle +"' id='"+table.id+"_td_float_toolbar_cancel' title='撤销前一个动作'>撤销</td>";
        h+="<td class='"+redoStyle+"' id='"+table.id+"_td_float_toolbar_redo' title='恢复前一个撤销的动作'>重做</td>";
        h+=format_tool;
    }
    h+="</tr></table>";

    $(table).before(h);

    $("#mcsstable_tr_float_toolbar_").css({'margin-top':margin_top+"px",'margin-left':margin_left+"px"});
    $("#"+table.id+"_td_float_toolbar_edit").click(function(){
        mcsstable_openAutoform(_td.childNodes[0] ,tableid,recordid,"编辑","EDIT");//modify by sunliangchao 第一个参数穿的不对，导致弹出窗口无法翻页
        mcsstable_clearFloatToolBar();
    })
    $("#"+table.id+"_td_float_toolbar_del").click(function(){
        mcsstable_deleteOneRow(_td,tableid,{obj:this});
        mcsstable_clearFloatToolBar();
    })
    $("#"+table.id+"_td_float_toolbar_add").click(function(){
        var current_tr=$(mcdom.getObjByTagName(_td,"TR")).get(0);
        mcsstable_insertRow(current_tr,tableid);
        mcsstable_clearFloatToolBar();
    })
    $("#"+table.id+"_td_float_toolbar_cancel").click(function(){
        if (mcsstable.undoList.length<1) return;
        mcsstable_clearFloatToolBar();
        mcsstable.undo();
    })
    $("#"+table.id+"_td_float_toolbar_redo").click(function(){
        if (mcsstable.redoList.length<1) return;
        mcsstable_clearFloatToolBar();
        mcsstable.redo();

    })
}
//确定列类型浮动工具条的 margin-left
function mcsstable_getColFloatTools_left(obj,table){
    var margin_left = 0;
    var obj_off_set_left = obj.offsetLeft;
    var obj_client_width =obj.clientWidth;
    var mcsstable=mcsstable_getMCSSTable(table.id);
    var div_scroll_left = $(mcsstable.tableData).parent().parent().scrollLeft();
    margin_left = obj_off_set_left +obj_client_width-div_scroll_left;
    return margin_left;
}
//确定浮动工具条的 margin-left
function mcsstable_getFloatTools_left(obj,table,num){
    var margin_left = 0;
    var tableid=table.id;
    var recordid=$(mcdom.getObjByTagName(obj,"TR")).attr('recordid');

    var obj_off_set_left = obj.offsetLeft;
    var window_width = $(window).width();

    var obj_client_width =obj.clientWidth;

    var mcsstable=mcsstable_getMCSSTable(table.id);
    var scroll_div = $(mcsstable.tableData).parent().parent();
    var div_scroll_left = $(scroll_div).scrollLeft();

    if(obj_client_width>300)
        margin_left = obj_off_set_left +(obj_client_width-300)/2;
    else if((obj_off_set_left-div_scroll_left)<window_width/2)
        margin_left = obj_off_set_left +obj_client_width;
    else
        margin_left = obj_off_set_left -(num*50)-obj.offsetWidth;
    margin_left -= div_scroll_left;

    return margin_left;
}
//确定浮动工具条的 margin-top
function mcsstable_getFloatTools_top(obj,table){

    var margin_top = 0;
    var tableid=table.id;
    var recordid=$(mcdom.getObjByTagName(obj,"TR")).attr('recordid');

    var table_off_set_top = $(table).get(0).offsetTop;
    var obj_off_set_top = obj.offsetTop;
    var obj_client_height = obj.offsetHeight;
    var window_height = $(window).height();

    var mcsstable=mcsstable_getMCSSTable(tableid);
    var div_scroll_top = $(mcsstable.tableData).parent().parent().scrollTop();
    var tablecontianer_offsetTop = $(mcsstable.tablecontainer).get(0).offsetTop;

    obj_off_set_top=obj_off_set_top - tablecontianer_offsetTop-div_scroll_top + mcsstable.tableData.offsetTop + mcsstable.table.rows[0].offsetHeight;

    if((obj_off_set_top-div_scroll_top)<window_height-120)
        margin_top = obj_off_set_top +obj_client_height;
    else
        margin_top = obj_off_set_top -30;

    return margin_top;
}
//浮动工具条：行号 add by gang
function mcsstable_setTrFloatToolbar(table,rowIndex){
    mcsstable_clearFloatToolBar();
    var current_tr = table.rows[rowIndex];
    var tableid=table.id;
    var recordid=$(current_tr).attr('recordid');
    var margin_top = mcsstable_getFloatTools_top(current_tr,table);
    var format_tool = mcsstable_createFormatTools(tableid,'horizontal');
    var h="<table id='mcsstable_tr_float_toolbar_"+rowIndex+"' class='mcsstable_tr_float_toolbar_table'><tr>"
        +"<td class='mcsstable_tr_float_toolbar_td' id='"+tableid+"_tr_float_toolbar_add' title='在这一行前面插入空行'>插入</td>"
            //+"<td class='mcsstable_tr_float_toolbar_td' id='"+tableid+"_tr_float_toolbar_view'>查看</td>"
        +"<td class='mcsstable_tr_float_toolbar_td' id='"+tableid+"_tr_float_toolbar_edit' title='以表单方式编辑这一行'>编辑</td>"
        +"<td class='mcsstable_tr_float_toolbar_td' id='"+tableid+"_tr_float_toolbar_del' title='删除这一行'>删除</td>"
        +"<td class='mcsstable_tr_float_toolbar_td' id='"+tableid+"_tr_float_toolbar_cut' title='剪切这一行'>剪切</td>"
        +"<td class='mcsstable_tr_float_toolbar_td' id='"+tableid+"_tr_float_toolbar_paste' title='黏贴这一行'>粘贴</td>"
        +"<td class='mcsstable_tr_float_toolbar_td' id='"+tableid+"_tr_float_toolbar_discuss' title='评论这一行'>评论</td>"
        +format_tool+"</tr></table>";
    $(table).before(h);
    $("#mcsstable_tr_float_toolbar_"+rowIndex).css({'margin-top':margin_top+5+"px",'margin-left':"30px"});
    $("#"+tableid+"_tr_float_toolbar_view").click(function(){
        mcsstable_openAutoform(current_tr,tableid,recordid,"查看","VIEW");
        mcsstable_clearFloatToolBar();
    })
    $("#"+tableid+"_tr_float_toolbar_edit").click(function(){
        mcsstable_openAutoform(current_tr,tableid,recordid,"编辑","EDIT");
        mcsstable_clearFloatToolBar();
    })
    $("#"+tableid+"_tr_float_toolbar_del").click(function(){
        mcsstable_deleteOneRow(current_tr,tableid,{obj:this});
        mcsstable_clearFloatToolBar();
    })
    $("#"+tableid+"_tr_float_toolbar_add").click(function(){
        mcsstable_insertRow(current_tr,tableid);
        mcsstable_clearFloatToolBar();
    })
    $("#"+tableid+"_tr_float_toolbar_cut").click(function(){
        mcsstable_cut(current_tr,tableid);
        mcsstable_clearFloatToolBar();
    })
    $("#"+tableid+"_tr_float_toolbar_paste").click(function(){
        mcsstable_paste(current_tr,tableid);
        mcsstable_clearFloatToolBar();
    })
    $("#"+tableid+"_tr_float_toolbar_discuss").click(function(){
        var obj = $(current_tr).find(".gantetu_addDiscussion");
        obj.click();
        mcsstable_clearFloatToolBar();
    })
}
function setTrFloatToolbar_format(obj){
    if($(obj).attr('tag')=='close'){
        $(obj).attr('tag','open').html('格式<');
        $('.mcsstable_tr_float_toolbar_format').show();
    }else{
        $(obj).attr('tag','close').html('格式>');
        $('.mcsstable_tr_float_toolbar_format').hide();
    }
}
//浮动工具条：选中区域  add by gang
function mcsstable_setAreaFloatToolbar(obj,table){
    mcsstable_clearFloatToolBar();
    var tableid = table.id;
    var mcsstable = mcsstable_getMCSSTable(tableid);
    var margin_top = mcsstable_getFloatTools_top(mcsstable.divSel.td,table)+ mcsstable.divSel.clientHeight -15 ;
    var margin_left = mcsstable_getFloatTools_left(mcsstable.divSel.td,table,4) -50;
    var _formatTool = mcsstable_createFormatTools(tableid);
    var h="<table id='mcsstable_tr_float_toolbar_' class='mcsstable_tr_float_toolbar_table'>"
        +"<tr>"+_formatTool+"</tr>"
            //+"<tr><td class='mcsstable_tr_float_toolbar_tr icon-null'>批量修改</td></tr>"
            //+"<tr><td class='mcsstable_tr_float_toolbar_tr icon-null'>导出</td></tr>"
        +"</table>";
    $(table).before(h);
    $("#mcsstable_tr_float_toolbar_").css({'margin-left':(margin_left)+"px",'margin-top':margin_top+"px"});
    $('.mcsstable_tr_float_toolbar_tr').css('width','90px');
    $('.mcsstable_tr_float_toolbar_format').show();

}
//浮动工具条-样式工具条的实现
function mcsstable_createFormatTools(tableid,position){
    var formatClass = "mcsstable_tr_float_toolbar_tr icon-null";
    if(position=='horizontal')
        formatClass = "mcsstable_tr_float_toolbar_td";
    var htm = "";
    htm+="<td class='"+formatClass+"' tag='close' onmouseover='fieldActionMenu_onmouseover(this)' onclick='setTrFloatToolbar_format(this)' title='给本行设置统一格式'>格式></td>";
    htm+="<td title='加粗' class='styleeditor styleeditor-Bold_float mcsstable_tr_float_toolbar_td mcsstable_tr_float_toolbar_format' onclick='mcsstable_setBold(this,\""+tableid+"\")'></td>";
    htm+="<td title='斜体' class='styleeditor styleeditor-Italic_float mcsstable_tr_float_toolbar_td mcsstable_tr_float_toolbar_format' onclick='mcsstable_setItalic(this,\""+tableid+"\")'></td>";
    htm+="<td title='下划线' class='styleeditor styleeditor-Underline_float mcsstable_tr_float_toolbar_td mcsstable_tr_float_toolbar_format' onclick='mcsstable_setUnderline(this,\""+tableid+"\")'></td>";
    //htm+="<td title='字体样式' class='styleeditor styleeditor-FontName_float mcsstable_tr_float_toolbar_td mcsstable_tr_float_toolbar_format' onclick='mcsstable_setFontFace(this,\""+tableid+"\")'></td>";
    htm+="<td title='字号' class='styleeditor styleeditor-FontSize_float mcsstable_tr_float_toolbar_td mcsstable_tr_float_toolbar_format' onclick='mcsstable_setFontSize(this,\""+tableid+"\")'></td>";
    htm+="<td title='前景色' class='styleeditor styleeditor-ForeColor_float mcsstable_tr_float_toolbar_td mcsstable_tr_float_toolbar_format' onclick='mcsstable_createFontColorSelector(this,\""+tableid+"\")'></td>";
    htm+="<td title='背景色' class='styleeditor styleeditor-BackColor_float mcsstable_tr_float_toolbar_td mcsstable_tr_float_toolbar_format' onclick='mcsstable_createBackgroundColorSelector(this,\""+tableid+"\")'></td>";
    htm+="<td title='对齐方式' class='styleeditor styleeditor-AlignMode_float mcsstable_tr_float_toolbar_td mcsstable_tr_float_toolbar_format' onclick='mcsstable_setFontPosition(this,\""+tableid+"\")'></td>";
    return htm;
}
//浮动工具条：全选  add by gang 暂时隐藏
function mcsstable_setAllFloatToolbar(obj,table){
    mcsstable_clearFloatToolBar();
    var tableid=table.id;
    var _obj = obj;
    var all_setting_tag = $(obj).attr('all_setting_tag');
    if(all_setting_tag=='close'){
        $(obj).attr('all_setting_tag','open');
        mcsstable_selectAllRows(obj,tableid);
    }else{
        $(obj).attr('all_setting_tag','close');
        mcsstable_selectAllRows(obj,tableid,'false');
        return;
    }

    var _all_table=mcdom.getObjByClass(obj,"mcsstable_th_line");
    var _topHeight =  _all_table.clientHeight;
    var _formatTool = mcsstable_createFormatTools(tableid);
    var h="<table id='mcsstable_tr_float_toolbar_' class='mcsstable_tr_float_toolbar_table'>"
            // +"<tr><td class='mcsstable_tr_float_toolbar_tr'>复制</td></tr>"
            // +"<tr><td class='mcsstable_tr_float_toolbar_tr'>粘贴</td></tr>"
            // +"<tr><td class='mcsstable_tr_float_toolbar_tr'>显示隐藏的列</td></tr>"
        +"<tr>"+_formatTool+"</tr>"
            //+"<tr><td class='mcsstable_tr_float_toolbar_tr'>取消所有分组</td></tr>"
            //+"<tr><td class='mcsstable_tr_float_toolbar_tr'>导出到Excel</td></tr>"
        +"<tr><td class='mcsstable_tr_float_toolbar_tr' id='appendDataFromExcel'>插入Excel中的数据</td></tr>"
        +"<tr><td class='mcsstable_tr_float_toolbar_tr' id='appendDataFromTable'>插入其他表格的数据</td></tr>"
        +"</table>";
    $(table).before(h);
    $("#mcsstable_tr_float_toolbar_").css({'margin-left':"0px",'margin-top':_topHeight+"px"});
    $('.mcsstable_tr_float_toolbar_tr').css('width','130px');
    $("#appendDataFromExcel").click(function(){
        mcsstable_selectAllRows(_obj,tableid,'false');
        mcsstable_import(tableid);
        mcsstable_clearFloatToolBar();
    })
    $("#appendDataFromTable").click(function(){
        mcsstable_selectAllRows(_obj,tableid,'false');
        mcsstable_importFromTable(tableid);
        mcsstable_clearFloatToolBar();
    })
}
function getStyleWidth(style){
    if (!style) return 100;
    if (style.indexOf('width')<0) return 100;
    strs=style.split(';');
    for(var i=0;i<strs.length;i++){
        var ss=strs[i];
        var sn=ss.split(":")[0];
        var sv=ss.split(":")[1];
        if (sn.toLowerCase()=="width"){
            var iw=parseInt(sv);
            if (isNaN(iw)) iw=100;
            return iw;
        }
    }
    return 100;
}
function setStyleWidth(style){
    if (!style) return "width:100px";
    if (style.indexOf('width')<0) return style+";width:100px";
    strs=style.split(';');
    var str2="";
    for(var i=0;i<strs.length;i++){
        var ss=strs[i];
        var sn=ss.split(":")[0];
        var sv=ss.split(":")[1];
        if (sn.toLowerCase()=="width"){
            var iw=parseInt(sv);
            if (isNaN(iw)) {
                str2+="width:100px;";
            }else{
                str2+="width:"+iw+"px;";
            }
        }else{
            str2+=ss+";";
        }
    }
    return str2;
}
//add by sunliangchao
//快速导出到xls文件（实际是html格式）
function mcsstable_quickexport(tableid,type,exporturl)
{
    var mcsstable=mcsstable_getMCSSTable(tableid);
    var params=mcsstable.params;
    var fields=mcsstable.fields;
    var modeldata=mcsstable.modeldata;
    var first_td_func=mcsstable.first_td_func;
    var last_td_func=mcsstable.last_td_func;
    //下面的escape方法在对应php中现在没有解码却正确，用了解码函数却不正确。因此没有用。

    var sql="";
    var url=mcsstable.params.homeUrl+g_systempath+"/quickExportToFile";
    if (params.sql)
        sql=params.sql;
    if (exporturl)//额外的导出格式采用这个，必须有参数exporturl_ext
    {
        url = exporturl;
    }
    else
    if (mcsstable.params.exporturl)//跳到指定的Action中
        url = mcsstable.params.exporturl;
    //var lastfilter=mcsstable_getLastFilter(mcsstable);
    var lastfilter=mcsstable.lastfilter;

    if (mcsstable.selectedRowsID)
    {
        var srids=mcsstable.selectedRowsID;
        srids = srids.replace(/\,/g, "','");
        lastfilter += ' and ' + mcsstable.keyfield + " in ('"+srids+"')";
    }	//如果传递了数据，则后来直接根据这个数据导出，不需要sql语句
    if(!lastfilter)
        lastfilter = mcsstable.filter;
    lastfilter=uni_cleanSQL(lastfilter);
    window.startTime=new Date().getTime();

    var str = "<div id='exportProBack' style='width:580px;height:160px;background-color:#fff;margin-top:0px;' >";
    str +="<div id='exportProTitle'>正在导出数据</div>";
    str +="<div id='exportProB' style='width:560px;height:25px;border:1px solid #ccc;background-color:#fff;margin-top:20px;overflow:hidden' >";
    str +="<div id='exportPro' style='margin-left:0px;height:23px;width:0px;background-color:#33f;' ></div>";
    str +="</div>";
    str +="<div id='exportProTip' style='margin-top:20px' >正在分析所需时间</div>";
    str +="</div>";

    mcdom.proform = mcdom.popup(str,{position:'middle',title:"快速导出",width:"600px",height:"200px",overflow:"hidden"});

    window.setTimeout(function(){
        $.post(url,{modelid:mcsstable.modelid,searchword:mcsstable.searchword,sql:sql,data:params.datastr,filter:lastfilter,
            filetype:type,usermodel_recordid:mcsstable.params.usermodel_recordid,
            model_ownerid:mcsstable.params.model_ownerid,viewid:mcsstable.view_id},function(data){
            var info = eval("info =" + data);
            info.mcsstable=mcsstable;
            exportMoreData(info);
            return;
        });
    },200);
}
//导出更多的数据
function exportMoreData(info){

    var mcsstable=info.mcsstable;
    function getDateDiff(){
        var now = new Date().getTime();
        var diffValue = now - window.startTime;//Date.parse(d.replace(/-/g,'/').replace(/：/g,":"));
        if(diffValue < 0){
            return "";
        }else{
            return parseInt(diffValue/1000);
        }
    }

    if (info.finished){
        var url=mcsstable.params.homeUrl.replace("index.php","") +"Public/temp/"+info.file;
        var downfile=mcsstable.params.project_fulldata.project[0].name+"."+info.filetype;
        downloadfile(mcsstable,url,downfile);
        mcdom.closePopup();
    }else{

        var data=info;
        var per = (data.rowIndex/data.rowCount);
        var divpro = document.getElementById( "exportPro");
        if (!divpro){
            return;
        }
        var divpar=divpro.parentNode.parentNode;
        divpar.style.display="";
        divpro.style.backgroundColor="#9f9";
        divpro.style.width =  parseInt( divpar.offsetWidth * per)+ "px";
        var divTip = document.getElementById( "exportProTip");
        divTip.innerHTML = "正在导出第"+data.rowIndex + "条,共"+data.rowCount+"条";
        var divTitle=document.getElementById("exportProTitle");
        divTitle.innerHTML="正在导出";

        var sec=getDateDiff();
        if (data.rowIndex>0){
            var dec=parseInt(sec/data.rowIndex * (data.rowCount - data.rowIndex));
            //var sc=parseInt(dec/60)
            var mc=parseInt(dec/60);
            var hc=parseInt(mc/60);
            var strc=dec+"秒";
            if (mc>0) strc=mc+"分"+(dec-(mc*60))+"秒";
            if (hc>0) strc=hc+":"+(mc-(hc*60))+":"+(dec-(mc*60));
            divTitle.innerHTML="正在导出，估计剩余时间:"+strc;
        }


        var url=mcsstable.params.homeUrl+g_systempath+"/exportMoreData";
        $.post(url,{fileName:info.file,filetype:info.filetype,rowIndex:info.rowIndex,rowCount:info.rowCount},function(mdata){
            var minfo = eval("minfo =" + mdata);
            minfo.mcsstable=mcsstable;
            exportMoreData(minfo);
        });
    }
    return;
}
//下载文件
function downloadfile(mcsstable, file_url,download_filename){
    
    var a = document.createElement("a");
    var a_attachement_id="attachement_"+mcsstable.params.usermodel_recordid;
    a.id=a_attachement_id;
    a.href =file_url;
    a.download = download_filename;
    document.body.appendChild(a);
    a.click();

}

function table_touchStart(event){
    
    
    mcsstable_OnMouseDown(event);
    var td=mcdom.getObjByTagName(event.target,'TD');
    if (td.inEdit) return;
    
    event.preventDefault();
    var table=mcdom.getObjByTagName(event.target,'TABLE');
    var mcsstable=table.mcsstable;
    var touches = event.touches;
    var x1= touches[0].pageX;
    var y1= touches[0].pageY;
    mcsstable.table.touchX1=x1;
    mcsstable.table.touchY1=y1;
    mcsstable.table.touchLength=1;
    
    if(touches.length!=2) return false;
    var x2= touches[1].pageX;
    var y2= touches[1].pageY;
    mcsstable.table.touchX1=(x1+x2)/2;
    mcsstable.table.touchY1=(y1+y2)/2;
    mcsstable.table.touchLength=2;
    mcsstable.table.baseTouch= Math.sqrt((x2-x1)*(x2-x1)+(y2-y1)*(y2-y1));
    return false;
}

function table_touchMove(event){
    
    var td=mcdom.getObjByTagName(event.target,'TD');
    if (td.inEdit) return;
    event.preventDefault();
    var table=mcdom.getObjByTagName(event.target,'TABLE');
    var mcsstable=table.mcsstable;

    var touches = event.touches;
    var x1= touches[0].pageX;
    var y1= touches[0].pageY;
    
    if(touches.length==1) {
        if (mcsstable.table.touchLength==1){
            var panel=mcsstable.tableData.parentNode.parentNode;
            panel.scrollLeft-=(x1-mcsstable.table.touchX1);
            panel.scrollTop-=(y1-mcsstable.table.touchY1);
            mcsstable.table.touchX1=x1;
            mcsstable.table.touchY1=y1;
            return false;
        }
    }
    
    var x2= touches[1].pageX;
    var y2= touches[1].pageY;
    
    var t2 = Math.sqrt((x2-x1)*(x2-x1)+(y2-y1)*(y2-y1));
    
    if (!mcsstable.table.baseTouch){
        mcsstable.table.baseTouch=t2;
        return;
    }
    var scn=t2/mcsstable.table.baseTouch;
    if (!mcsstable.scaleTo) mcsstable.scaleTo=1;
    sc=scn*mcsstable.scaleTo;
 
    mcsstable.scale(sc);
    mcsstable.temp_scaleTo=sc;
    var x=(x1+x2)/2;
    var y=(y1+y2)/2;
    
    var panel=mcsstable.tableData.parentNode.parentNode;
    panel.scrollLeft=(mcsstable.table.touchX1*(scn-1))/2;
    panel.scrollTop=(mcsstable.table.touchY1*(scn-1))/2;
    return false;
}
function table_touchEnd(event){
    var td=mcdom.getObjByTagName(event.target,'TD');
    if (td.inEdit) {
        return;
    }
    event.preventDefault();
    var touches = event.touches;
    var table=mcdom.getObjByTagName(event.target,'TABLE');
    var mcsstable=table.mcsstable;
    var x1= window.mousedownX;
    var y1= window.mousedownY;

    if (event.changedTouches)
    {
        var pos = event.changedTouches[0];
        var x2=pos.pageX;
        var y2=pos.pageY;
    }
    var t2 = Math.sqrt((x2-x1)*(x2-x1)+(y2-y1)*(y2-y1));

    if (t2<6)
    {
        if (table==mcsstable.tableData || table==mcsstable.tableFixCol)
            mcsstable_OnMouseUp(event);
    }else{
    }
    if (mcsstable.temp_scaleTo)
        mcsstable.scaleTo=mcsstable.temp_scaleTo;

    return false;

}
function mcsstable_popupViewMenu(tableid,obj){
    var mcsstable=mcsstable_getMCSSTable(tableid);
    var menu_htm = mcsstable.viewlist_HtmlString;
    dom_showPopupMenu(menu_htm,obj,{position:"bottom"});
    
    $('.weui_actionsheet_cell').click(function(){
        var id=this.title;//var text=this.textContent;

        if (id=='DEFAULT')
        {
            mcsstable.view_id='DEFAULT';
            mcsstable.run();
            return;
        }
        
        var today = uni_global.today;

        var dd = new Date();
        dd.setDate(dd.getDate()-1);//获取AddDayCount天后的日期
        var y = dd.getFullYear();
        var m = dd.getMonth()+1;//获取当前月份的日期
        if (m<10) m ="0"+"m";
        var d = dd.getDate();
        if (d<10) d="0"+d;
        var yestoday = y+"-"+m+"-"+d;
        
        var v=id;
        var filter='';
        if (v=='SHOW_CREATE_MY'){
            filter = "SYS_ADDUSER = '"+user+"'";
        }else if (v=='SHOW_EDIT_NEW'){
            filter = "SYS_EDITTIME > '"+mcsstable.lastReadtime+"'";
        }else if (v=='SHOW_UPDATE_MY'){
            filter = "SYS_EDITUSER = '"+user+"'";
        }else if (v=='SHOW_CREATE_TODAY'){
            filter = "SYS_ADDUSER = '"+user+"' and to_days(SYS_EDITTIME) = to_days('"+today+"')";
        }else if (v=='SHOW_UPDATE_TODAY'){
            filter = "SYS_EDITUSER = '"+user+"' and to_days(SYS_EDITTIME) = to_days('"+today+"')";
        }else if (v=='SHOW_EDIT_TODAY'){
            filter = "left(SYS_EDITTIME,10) = '"+today+"'";
        }else if (v=='SHOW_EDIT_YESTODAY'){
            filter = "left(SYS_EDITTIME,10) = '"+yestoday+"'";
        }else if (v=='SHOW_MYUPDATE_YESTODAY'){
            filter = "left(SYS_EDITTIME,10) = '"+yestoday+"' and SYS_EDITUSER = '"+user+"'";
        }else{
            
            if (id!='NONE')
            {
                mcsstable.openView(id);
            }          
            return;  
        }

        mcsstable.advancedFilter = "";
        mcsstable.autoFilter = filter;//"";
        mcsstable_loaddatarows(mcsstable.tableid);

        
    })
    
}
